@@ -337,6 +337,9 @@ sub parse_rpm_dist($$) {
         if ($dist_rpm =~ /system-release-2-17.amzn2/) {
                 $DISTRO = "AL2";
                 $rpm_distro = "amzn2";
+	} elsif ($dist_rpm =~ /system-release-2023/) {
+		$DISTRO = "AL23";
+		$rpm_distro = "amzn23";
         } elsif ($dist_rpm =~ /openSUSE-release-12.1/) {
 		$DISTRO = "openSUSE12.1";
 		$rpm_distro = "opensuse12sp1";
@@ -346,7 +349,7 @@ sub parse_rpm_dist($$) {
 	} elsif ($dist_rpm =~ /openSUSE-release-15.3/) {
                 $DISTRO = "openSUSE15.3";
                 $rpm_distro = "opensuse15sp3";
-	} elsif ($dist_rpm =~ /(?:sles|SLE_HPC|SLES_SAP)-release-(1[\d])(|\.([\d]))-/) {
+	} elsif ($dist_rpm =~ /(?:sles|SLES|SLE_HPC|SLES_SAP)-release-(1[\d])(|\.([\d]))-/) {
 		# example: dist_rpm: "sles-release-15.1-39.1"
 		#					"sles-release-12-1.377"
 		my $sp = $2? $3 : "0";
@@ -447,8 +450,8 @@ sub parse_rpm_dist($$) {
 		my ($major, $minor) = ($2, $3);
 		if (
 			(($major eq "8") and ($minor <= 10)) ||
-			(($major eq "9") and ($minor <= 6)) ||
-			(($major eq "10") and ($minor <= 0))
+			(($major eq "9") and ($minor <= 7)) ||
+			(($major eq "10") and ($minor <= 1))
 		) {
 			$DISTRO = "RHEL$2.$3";
 			$rpm_distro = "rhel$2u$3";
