@@ -1062,18 +1062,30 @@ err_cqb:
 
 int mlx5_ib_destroy_cq(struct ib_cq *cq, struct ib_udata *udata)
 {
-	struct mlx5_ib_dev *dev = to_mdev(cq->device);
-	struct mlx5_ib_cq *mcq = to_mcq(cq);
 	int ret;
 
-	ret = mlx5_core_destroy_cq(dev->mdev, &mcq->mcq);
+	ret = mlx5_ib_pre_destroy_cq(cq);
 	if (ret)
 		return ret;
 
 	if (udata)
-		destroy_cq_user(mcq, udata);
+		destroy_cq_user(to_mcq(cq), udata);
 	else
-		destroy_cq_kernel(dev, mcq);
+		mlx5_ib_post_destroy_cq(cq);
+	return 0;
+}
+
+int mlx5_ib_pre_destroy_cq(struct ib_cq *cq)
+{
+	struct mlx5_ib_dev *dev = to_mdev(cq->device);
+	struct mlx5_ib_cq *mcq = to_mcq(cq);
+
+	return mlx5_core_destroy_cq(dev->mdev, &mcq->mcq);
+}
+
+int mlx5_ib_post_destroy_cq(struct ib_cq *cq)
+{
+	destroy_cq_kernel(to_mdev(cq->device), to_mcq(cq));
 	return 0;
 }
 
