@@ -274,7 +274,7 @@ static int show_nic_node_guid(struct mlx5_core_dev *dev, u16 vf,
 {
 	int err;
 
-	err = mlx5_query_nic_vport_node_guid(dev, vf + 1, node_guid);
+	err = mlx5_query_nic_vport_node_guid(dev, vf + 1, true, node_guid);
 	if (!err)
 		*node_guid = cpu_to_be64(*node_guid);
 
@@ -1281,6 +1281,22 @@ static struct device_attribute *mlx5_class_attributes[] = {
 	&dev_attr_mlx5_num_vfs,
 };
 
+int mlx5_sriov_groups_sysfs_init(struct mlx5_core_dev *dev)
+{
+#ifdef CONFIG_MLX5_ESWITCH
+	struct mlx5_core_sriov *sriov = &dev->priv.sriov;
+
+	if (MLX5_CAP_QOS(dev, log_esw_max_sched_depth)) {
+		sriov->groups_config = kobject_create_and_add("groups",
+							      sriov->config);
+		if (!sriov->groups_config) {
+			return -ENOMEM;
+		}
+	}
+#endif
+	return 0;
+}
+
 int mlx5_sriov_sysfs_init(struct mlx5_core_dev *dev)
 {
 	struct mlx5_core_sriov *sriov = &dev->priv.sriov;
@@ -1292,17 +1308,6 @@ int mlx5_sriov_sysfs_init(struct mlx5_core_dev *dev)
 	if (!sriov->config)
 		return -ENOMEM;
 
-#ifdef CONFIG_MLX5_ESWITCH
-	if (MLX5_CAP_QOS(dev, log_esw_max_sched_depth)) {
-		sriov->groups_config = kobject_create_and_add("groups",
-							      sriov->config);
-		if (!sriov->groups_config) {
-			err = -ENOMEM;
-			goto err_groups;
-		}
-	}
-#endif
-
 	for (i = 0; i < ARRAY_SIZE(mlx5_class_attributes); i++) {
 		err = device_create_file(device, mlx5_class_attributes[i]);
 		if (err)
@@ -1312,17 +1317,23 @@ int mlx5_sriov_sysfs_init(struct mlx5_core_dev *dev)
 	return 0;
 
 err_attr:
-	if (sriov->groups_config) {
-		kobject_put(sriov->groups_config);
-		sriov->groups_config = NULL;
-	}
-
-err_groups:
+	for(i--; i >= 0; i--)
+		device_remove_file(device, mlx5_class_attributes[i]);
 	kobject_put(sriov->config);
 	sriov->config = NULL;
 	return err;
 }
 
+void mlx5_sriov_groups_sysfs_cleanup(struct mlx5_core_dev *dev)
+{
+#ifdef CONFIG_MLX5_ESWITCH
+	struct mlx5_core_sriov *sriov = &dev->priv.sriov;
+
+	if (MLX5_CAP_QOS(dev, log_esw_max_sched_depth))
+		kobject_put(sriov->groups_config);
+#endif
+}
+
 void mlx5_sriov_sysfs_cleanup(struct mlx5_core_dev *dev)
 {
 	struct mlx5_core_sriov *sriov = &dev->priv.sriov;
@@ -1332,8 +1343,6 @@ void mlx5_sriov_sysfs_cleanup(struct mlx5_core_dev *dev)
 	for (i = 0; i < ARRAY_SIZE(mlx5_class_attributes); i++)
 		device_remove_file(device, mlx5_class_attributes[i]);
 
-	if (MLX5_CAP_QOS(dev, log_esw_max_sched_depth))
-		kobject_put(sriov->groups_config);
 	kobject_put(sriov->config);
 	sriov->config = NULL;
 }
