@@ -5,6 +5,19 @@
 
 #include_next <linux/compiler-gcc.h>
 
+#ifndef __has_attribute
+# define __has_attribute(x) __GCC4_has_attribute_##x
+# define __GCC4_has_attribute___copy__                0
+#endif
+
+#ifndef __copy
+#if __has_attribute(__copy__)
+# define __copy(symbol)                 __attribute__((__copy__(symbol)))
+#else
+# define __copy(symbol)
+#endif
+#endif /* __copy__ */
+
 #ifndef fallthrough
 # define fallthrough                    do {} while (0)  /* fallthrough */
 #endif
@@ -15,12 +28,4 @@
 		     + __GNUC_PATCHLEVEL__)
 #endif
 
-#ifndef HAVE_LINUX_OVERFLOW_H
-
-#if GCC_VERSION >= 50100
-#define COMPILER_HAS_GENERIC_BUILTIN_OVERFLOW 1
-#endif
-
-#endif /* HAVE_LINUX_OVERFLOW_H */
-
 #endif /* _COMPAT_LINUX_COMPILER_GCC_H */
