@@ -7,6 +7,7 @@
 */
 
 #include "fuse_i.h"
+#include "dev_uring_i.h"
 
 #include <linux/pagemap.h>
 #include <linux/slab.h>
@@ -25,6 +26,11 @@
 #include <linux/pid_namespace.h>
 #include <uapi/linux/magic.h>
 
+#ifndef HAVE_FILEID_INO64
+#define FILEID_INO64_GEN 0x81
+#define FILEID_INO64_GEN_PARENT 0x82
+#endif
+
 MODULE_AUTHOR("Miklos Szeredi <miklos@szeredi.hu>");
 MODULE_DESCRIPTION("Filesystem in Userspace");
 MODULE_LICENSE("GPL");
@@ -34,6 +40,8 @@ DEFINE_MUTEX(fuse_mutex);
 
 static int set_global_limit(const char *val, const struct kernel_param *kp);
 
+unsigned int fuse_max_pages_limit = 256;
+
 unsigned max_user_bgreq;
 module_param_call(max_user_bgreq, set_global_limit, param_get_uint,
 		  &max_user_bgreq, 0644);
@@ -168,6 +176,14 @@ static void fuse_evict_inode(struct inode *inode)
 			fuse_cleanup_submount_lookup(fm, fi->submount_lookup);
 			fi->submount_lookup = NULL;
 		}
+		/*
+		 * Evict of non-deleted inode may race with outstanding
+		 * LOOKUP/READDIRPLUS requests and result in inconsistency when
+		 * the request finishes.  Deal with that here by bumping a
+		 * counter that can be compared to the starting value.
+		 */
+		if (inode->i_nlink > 0)
+			atomic64_inc(&fm->fc->evict_ctr);
 	}
 	if (S_ISREG(inode->i_mode) && !fuse_is_bad(inode)) {
 		WARN_ON(fi->iocachectr != 0);
@@ -190,17 +206,30 @@ static ino_t fuse_squash_ino(u64 ino64)
 
 void fuse_change_attributes_common(struct inode *inode, struct fuse_attr *attr,
 				   struct fuse_statx *sx,
-				   u64 attr_valid, u32 cache_mask)
+				   u64 attr_valid, u32 cache_mask,
+				   u64 evict_ctr)
 {
 	struct fuse_conn *fc = get_fuse_conn(inode);
 	struct fuse_inode *fi = get_fuse_inode(inode);
 
 	lockdep_assert_held(&fi->lock);
 
+	/*
+	 * Clear basic stats from invalid mask.
+	 *
+	 * Don't do this if this is coming from a fuse_iget() call and there
+	 * might have been a racing evict which would've invalidated the result
+	 * if the attr_version would've been preserved.
+	 *
+	 * !evict_ctr -> this is create
+	 * fi->attr_version != 0 -> this is not a new inode
+	 * evict_ctr == fuse_get_evict_ctr() -> no evicts while during request
+	 */
+	if (!evict_ctr || fi->attr_version || evict_ctr == fuse_get_evict_ctr(fc))
+		set_mask_bits(&fi->inval_mask, STATX_BASIC_STATS, 0);
+
 	fi->attr_version = atomic64_inc_return(&fc->attr_version);
 	fi->i_time = attr_valid;
-	/* Clear basic stats from invalid mask */
-	set_mask_bits(&fi->inval_mask, STATX_BASIC_STATS, 0);
 
 	inode->i_ino     = fuse_squash_ino(attr->ino);
 	inode->i_mode    = (inode->i_mode & S_IFMT) | (attr->mode & 07777);
@@ -214,10 +243,20 @@ void fuse_change_attributes_common(struct inode *inode, struct fuse_attr *attr,
 	attr->mtimensec = min_t(u32, attr->mtimensec, NSEC_PER_SEC - 1);
 	attr->ctimensec = min_t(u32, attr->ctimensec, NSEC_PER_SEC - 1);
 
+#ifdef HAVE_INODE_GET_MTIME
 	inode_set_atime(inode, attr->atime, attr->atimensec);
+#else
+	inode->i_atime.tv_sec   = attr->atime;
+	inode->i_atime.tv_nsec  = attr->atimensec;
+#endif
 	/* mtime from server may be stale due to local buffered write */
 	if (!(cache_mask & STATX_MTIME)) {
+#ifdef HAVE_INODE_GET_MTIME
 		inode_set_mtime(inode, attr->mtime, attr->mtimensec);
+#else
+		inode->i_mtime.tv_sec   = attr->mtime;
+		inode->i_mtime.tv_nsec  = attr->mtimensec;
+#endif
 	}
 	if (!(cache_mask & STATX_CTIME)) {
 		inode_set_ctime(inode, attr->ctime, attr->ctimensec);
@@ -279,9 +318,9 @@ u32 fuse_get_cache_mask(struct inode *inode)
 	return STATX_MTIME | STATX_CTIME | STATX_SIZE;
 }
 
-void fuse_change_attributes(struct inode *inode, struct fuse_attr *attr,
-			    struct fuse_statx *sx,
-			    u64 attr_valid, u64 attr_version)
+static void fuse_change_attributes_i(struct inode *inode, struct fuse_attr *attr,
+				     struct fuse_statx *sx, u64 attr_valid,
+				     u64 attr_version, u64 evict_ctr)
 {
 	struct fuse_conn *fc = get_fuse_conn(inode);
 	struct fuse_inode *fi = get_fuse_inode(inode);
@@ -300,12 +339,22 @@ void fuse_change_attributes(struct inode *inode, struct fuse_attr *attr,
 		attr->size = i_size_read(inode);
 
 	if (cache_mask & STATX_MTIME) {
+#ifdef HAVE_INODE_GET_MTIME
 		attr->mtime = inode_get_mtime_sec(inode);
 		attr->mtimensec = inode_get_mtime_nsec(inode);
+#else
+		attr->mtime = inode->i_mtime.tv_sec;
+		attr->mtimensec = inode->i_mtime.tv_nsec;
+#endif
 	}
 	if (cache_mask & STATX_CTIME) {
+#ifdef HAVE_INODE_GET_MTIME
 		attr->ctime = inode_get_ctime_sec(inode);
 		attr->ctimensec = inode_get_ctime_nsec(inode);
+#else
+		attr->ctime = inode_get_ctime(inode).tv_sec;
+		attr->ctimensec = inode_get_ctime(inode).tv_nsec;
+#endif
 	}
 
 	if ((attr_version != 0 && fi->attr_version > attr_version) ||
@@ -314,8 +363,13 @@ void fuse_change_attributes(struct inode *inode, struct fuse_attr *attr,
 		return;
 	}
 
+#ifdef HAVE_INODE_GET_MTIME
 	old_mtime = inode_get_mtime(inode);
-	fuse_change_attributes_common(inode, attr, sx, attr_valid, cache_mask);
+#else
+	old_mtime = inode->i_mtime;
+#endif
+	fuse_change_attributes_common(inode, attr, sx, attr_valid, cache_mask,
+				      evict_ctr);
 
 	oldsize = inode->i_size;
 	/*
@@ -356,6 +410,13 @@ void fuse_change_attributes(struct inode *inode, struct fuse_attr *attr,
 		fuse_dax_dontcache(inode, attr->flags);
 }
 
+void fuse_change_attributes(struct inode *inode, struct fuse_attr *attr,
+			    struct fuse_statx *sx, u64 attr_valid,
+			    u64 attr_version)
+{
+	fuse_change_attributes_i(inode, attr, sx, attr_valid, attr_version, 0);
+}
+
 static void fuse_init_submount_lookup(struct fuse_submount_lookup *sl,
 				      u64 nodeid)
 {
@@ -368,7 +429,12 @@ static void fuse_init_inode(struct inode *inode, struct fuse_attr *attr,
 {
 	inode->i_mode = attr->mode & S_IFMT;
 	inode->i_size = attr->size;
+#ifdef HAVE_INODE_GET_MTIME
 	inode_set_mtime(inode, attr->mtime, attr->mtimensec);
+#else
+	inode->i_mtime.tv_sec  = attr->mtime;
+	inode->i_mtime.tv_nsec = attr->mtimensec;
+#endif
 	inode_set_ctime(inode, attr->ctime, attr->ctimensec);
 	if (S_ISREG(inode->i_mode)) {
 		fuse_init_common(inode);
@@ -410,7 +476,8 @@ static int fuse_inode_set(struct inode *inode, void *_nodeidp)
 
 struct inode *fuse_iget(struct super_block *sb, u64 nodeid,
 			int generation, struct fuse_attr *attr,
-			u64 attr_valid, u64 attr_version)
+			u64 attr_valid, u64 attr_version,
+			u64 evict_ctr)
 {
 	struct inode *inode;
 	struct fuse_inode *fi;
@@ -471,8 +538,8 @@ retry:
 	fi->nlookup++;
 	spin_unlock(&fi->lock);
 done:
-	fuse_change_attributes(inode, attr, NULL, attr_valid, attr_version);
-
+	fuse_change_attributes_i(inode, attr, NULL, attr_valid, attr_version,
+				 evict_ctr);
 	return inode;
 }
 
@@ -554,14 +621,6 @@ static void fuse_umount_begin(struct super_block *sb)
 
 	if (fc->no_force_umount)
 		return;
-
-/*
-	fuse_abort_conn(fc);
-
-	// Only retire block-device-based superblocks.
-	if (sb->s_bdev != NULL)
-		retire_super(sb);
-*/
 }
 
 static void convert_fuse_statfs(struct kstatfs *stbuf, struct fuse_kstatfs *attr)
@@ -710,26 +769,6 @@ enum {
 	OPT_ERR
 };
 
-static const struct fs_parameter_spec fuse_fs_parameters[] = {
-	fsparam_string	("source",		OPT_SOURCE),
-	fsparam_u32	("fd",			OPT_FD),
-	fsparam_u32oct	("rootmode",		OPT_ROOTMODE),
-#ifdef HAVE_FSPARAM_UID
-	fsparam_uid	("user_id",		OPT_USER_ID),
-	fsparam_gid	("group_id",		OPT_GROUP_ID),
-#else
-	fsparam_u32     ("user_id",             OPT_USER_ID),
-	fsparam_u32     ("group_id",            OPT_GROUP_ID),
-#endif
-	fsparam_flag	("default_permissions",	OPT_DEFAULT_PERMISSIONS),
-	fsparam_flag	("allow_other",		OPT_ALLOW_OTHER),
-	fsparam_u32	("max_read",		OPT_MAX_READ),
-	fsparam_u32	("blksize",		OPT_BLKSIZE),
-	fsparam_string	("subtype",		OPT_SUBTYPE),
-	{}
-};
-
-
 static int fuse_show_options(struct seq_file *m, struct dentry *root)
 {
 	struct super_block *sb = root->d_sb;
@@ -766,11 +805,6 @@ static void fuse_iqueue_init(struct fuse_iqueue *fiq,
 			     void *priv)
 {
 	memset(fiq, 0, sizeof(struct fuse_iqueue));
-	//spin_lock_init(&fiq->lock);
-	//init_waitqueue_head(&fiq->waitq);
-	//INIT_LIST_HEAD(&fiq->pending);
-	//INIT_LIST_HEAD(&fiq->interrupts);
-	//fiq->forget_list_tail = &fiq->forget_list_head;
 	fiq->connected = 1;
 	fiq->ops = ops;
 	fiq->priv = priv;
@@ -789,8 +823,6 @@ void fuse_conn_init(struct fuse_conn *fc, struct fuse_mount *fm,
 	init_waitqueue_head(&fc->blocked_waitq);
 	if (fiq_ops)
 		fuse_iqueue_init(&fc->iq, fiq_ops, fiq_priv);
-	//INIT_LIST_HEAD(&fc->bg_queue);
-	//INIT_LIST_HEAD(&fc->entry);
 	INIT_LIST_HEAD(&fc->devices);
 	atomic_set(&fc->num_waiting, 0);
 	fc->max_background = FUSE_DEFAULT_MAX_BACKGROUND;
@@ -801,11 +833,12 @@ void fuse_conn_init(struct fuse_conn *fc, struct fuse_mount *fm,
 	fc->initialized = 0;
 	fc->connected = 1;
 	atomic64_set(&fc->attr_version, 1);
+	atomic64_set(&fc->evict_ctr, 1);
 	get_random_bytes(&fc->scramble_key, sizeof(fc->scramble_key));
 	fc->pid_ns = get_pid_ns(task_active_pid_ns(current));
 	fc->user_ns = get_user_ns(user_ns);
 	fc->max_pages = FUSE_DEFAULT_MAX_PAGES_PER_REQ;
-	fc->max_pages_limit = FUSE_MAX_MAX_PAGES;
+	fc->max_pages_limit = fuse_max_pages_limit;
 
 	if (IS_ENABLED(CONFIG_FUSE_PASSTHROUGH))
 		fuse_backing_files_init(fc);
@@ -819,6 +852,8 @@ static void delayed_release(struct rcu_head *p)
 {
 	struct fuse_conn *fc = container_of(p, struct fuse_conn, rcu);
 
+	fuse_uring_destruct(fc);
+
 	put_user_ns(fc->user_ns);
 	fc->release(fc);
 }
@@ -858,7 +893,7 @@ static struct inode *fuse_get_root_inode(struct super_block *sb, unsigned mode)
 	attr.mode = mode;
 	attr.ino = FUSE_ROOT_ID;
 	attr.nlink = 1;
-	return fuse_iget(sb, FUSE_ROOT_ID, 0, &attr, 0, 0);
+	return fuse_iget(sb, FUSE_ROOT_ID, 0, &attr, 0, 0, 0);
 }
 
 struct fuse_inode_handle {
@@ -1213,6 +1248,18 @@ static void process_init_reply(struct fuse_mount *fm, struct fuse_args *args,
 #ifdef HAVE_FUSE_NO_EXPORT_SUPPORT
 			if (flags & FUSE_NO_EXPORT_SUPPORT)
 				fm->sb->s_export_op = &fuse_export_fid_operations;
+#endif
+#ifdef HAVE_SB_I_NOIDMAP
+			if (flags & FUSE_ALLOW_IDMAP) {
+				if (fc->default_permissions)
+					fm->sb->s_iflags &= ~SB_I_NOIDMAP;
+				else
+					ok = false;
+			}
+#endif
+#ifdef CONFIG_FUSE_IO_URING
+			if (flags & FUSE_OVER_IO_URING && fuse_uring_enabled())
+				fc->io_uring = 1;
 #endif
 		} else {
 			ra_pages = fc->max_read / PAGE_SIZE;
@@ -1266,6 +1313,9 @@ void fuse_send_init(struct fuse_mount *fm)
 #endif
 #ifdef HAVE_FUSE_HAS_RESEND
 		FUSE_HAS_RESEND |
+#endif
+#ifdef HAVE_SB_I_NOIDMAP
+		FUSE_ALLOW_IDMAP |
 #endif
 		0;
 #ifdef CONFIG_FUSE_DAX
@@ -1280,6 +1330,14 @@ void fuse_send_init(struct fuse_mount *fm)
 	if (IS_ENABLED(CONFIG_FUSE_PASSTHROUGH))
 		flags |= FUSE_PASSTHROUGH;
 #endif
+	/*
+	 * This is just an information flag for fuse server. No need to check
+	 * the reply - server is either sending IORING_OP_URING_CMD or not.
+	 */
+#ifdef CONFIG_FUSE_IO_URING
+	if (fuse_uring_enabled())
+		flags |= FUSE_OVER_IO_URING;
+#endif
 
 	ia->in.flags = flags;
 	ia->in.flags2 = flags >> 32;
@@ -1352,21 +1410,11 @@ static int fuse_bdi_init(struct fuse_conn *fc, struct super_block *sb)
 struct fuse_dev *fuse_dev_alloc(void)
 {
 	struct fuse_dev *fud;
-	//struct list_head *pq;
 
 	fud = kzalloc(sizeof(struct fuse_dev), GFP_KERNEL);
 	if (!fud)
 		return NULL;
 
-	//pq = kcalloc(FUSE_PQ_HASH_SIZE, sizeof(struct list_head), GFP_KERNEL);
-	//if (!pq) {
-	//	kfree(fud);
-	//	return NULL;
-	//}
-	//
-	//fud->pq.processing = pq;
-	//fuse_pqueue_init(&fud->pq);
-
 	return fud;
 }
 
@@ -1401,15 +1449,19 @@ void fuse_dev_free(struct fuse_dev *fud)
 
 		fuse_conn_put(fc);
 	}
-	//kfree(fud->pq.processing);
 	kfree(fud);
 }
 
 static void fuse_fill_attr_from_inode(struct fuse_attr *attr,
 				      const struct fuse_inode *fi)
 {
+#ifdef HAVE_INODE_GET_MTIME
 	struct timespec64 atime = inode_get_atime(&fi->inode);
 	struct timespec64 mtime = inode_get_mtime(&fi->inode);
+#else
+	struct timespec64 atime = fi->inode.i_atime;
+	struct timespec64 mtime = fi->inode.i_mtime;
+#endif
 	struct timespec64 ctime = inode_get_ctime(&fi->inode);
 
 	*attr = (struct fuse_attr){
@@ -1440,6 +1492,9 @@ static void fuse_sb_defaults(struct super_block *sb)
 	sb->s_time_gran = 1;
 	sb->s_export_op = &fuse_export_operations;
 	sb->s_iflags |= SB_I_IMA_UNVERIFIABLE_SIGNATURE;
+#ifdef HAVE_SB_I_NOIDMAP
+	sb->s_iflags |= SB_I_NOIDMAP;
+#endif
 	if (sb->s_user_ns != &init_user_ns)
 		sb->s_iflags |= SB_I_UNTRUSTED_MOUNTER;
 	sb->s_flags &= ~(SB_NOSEC | SB_I_VERSION);
@@ -1471,7 +1526,8 @@ static int fuse_fill_super_submount(struct super_block *sb,
 		return -ENOMEM;
 
 	fuse_fill_attr_from_inode(&root_attr, parent_fi);
-	root = fuse_iget(sb, parent_fi->nodeid, 0, &root_attr, 0, 0);
+	root = fuse_iget(sb, parent_fi->nodeid, 0, &root_attr, 0, 0,
+			 fuse_get_evict_ctr(fm->fc));
 	/*
 	 * This inode is just a duplicate, so it is not looked up and
 	 * its nlookup should not be incremented.  fuse_iget() does
@@ -1480,7 +1536,13 @@ static int fuse_fill_super_submount(struct super_block *sb,
 	fi = get_fuse_inode(root);
 	fi->nlookup--;
 
+#ifdef HAVE_SET_DEFAULT_D_OP
+	/* Newer kernels (v6.17-rc1+): use set_default_d_op helper */
+	set_default_d_op(sb, &fuse_dentry_operations);
+#else
+	/* Older kernels: direct field assignment */
 	sb->s_d_op = &fuse_dentry_operations;
+#endif
 	sb->s_root = d_make_root(root);
 	if (!sb->s_root)
 		return -ENOMEM;
@@ -1614,24 +1676,30 @@ int fuse_fill_super_common(struct super_block *sb, struct fuse_fs_context *ctx)
 
 	err = -ENOMEM;
 	root = fuse_get_root_inode(sb, ctx->rootmode);
+#ifdef HAVE_SET_DEFAULT_D_OP
+	/* Newer kernels (v6.17-rc1+): use set_default_d_op helper */
+	set_default_d_op(sb, &fuse_root_dentry_operations);
+#else
+	/* Older kernels: direct field assignment */
 	sb->s_d_op = &fuse_root_dentry_operations;
+#endif
 	root_dentry = d_make_root(root);
 	if (!root_dentry)
 		goto err_dev_free;
 	/* Root dentry doesn't have .d_revalidate */
+#ifdef HAVE_SET_DEFAULT_D_OP
+	/* Newer kernels (v6.17-rc1+): use set_default_d_op helper */
+	set_default_d_op(sb, &fuse_dentry_operations);
+#else
+	/* Older kernels: direct field assignment */
 	sb->s_d_op = &fuse_dentry_operations;
+#endif
 
 	mutex_lock(&fuse_mutex);
 	err = -EINVAL;
 	if (ctx->fudptr && *ctx->fudptr)
 		goto err_unlock;
 
-	//err = fuse_ctl_add_conn(fc);
-	//if (err)
-	//	goto err_unlock;
-	//
-	//list_add_tail(&fc->entry, &fuse_conn_list);
-
 	sb->s_root = root_dentry;
 	if (ctx->fudptr)
 		*ctx->fudptr = fud;
@@ -1651,6 +1719,7 @@ int fuse_fill_super_common(struct super_block *sb, struct fuse_fs_context *ctx)
 	return err;
 }
 
+
 bool fuse_mount_remove(struct fuse_mount *fm)
 {
 	struct fuse_conn *fc = fm->fc;
