@@ -1031,7 +1031,7 @@ static struct iscsi_transport iscsi_iser_transport = {
 	/* connection management */
 	.create_conn            = iscsi_iser_conn_create,
 	.bind_conn              = iscsi_iser_conn_bind,
-#ifdef HAVE_ISCSI_CONN_UNBIND
+#ifdef HAVE_ISCSI_TRANSPORT_UNBIND_CONN
 	.unbind_conn		= iscsi_conn_unbind,
 #endif
 	.destroy_conn           = iscsi_conn_teardown,
