@@ -1,6 +1,7 @@
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/semaphore.h>
+#include <linux/version.h>
 #include <linux/uaccess.h>
 #include <linux/kernel.h>
 #include "nnt_ioctl.h"
@@ -9,6 +10,8 @@
 #include "nnt_ioctl_defs.h"
 #include "nnt_pci_conf_access.h"
 #include "mst_pciconf_bc.h"
+#include <linux/kthread.h>
+#include <linux/completion.h>
 
 MODULE_AUTHOR("Itay Avraham <itayavr@nvidia.com>");
 MODULE_DESCRIPTION("NNT Linux driver (NVIDIA® networking tools driver), this is the backward compatibility driver");
@@ -33,6 +36,19 @@ static char* name = "mst_pciconf";
 #define GET_DMA_PAGES PCICONF_GET_DMA_PAGES
 #define RELEASE_DMA_PAGES PCICONF_RELEASE_DMA_PAGES
 #define READ_DWORD_FROM_CONFIG_SPACE PCICONF_READ_DWORD_FROM_CONFIG_SPACE
+#define RESCAN PCICONF_RESCAN
+
+struct reset_thread_data
+{
+    struct pci_dev* pdev;
+    int error;
+    struct completion done;
+};
+
+/* Forward declarations */
+int nnt_pci_reset_bus_sd(struct pci_dev* pci_device_prime, struct pci_dev* pci_device_aux);
+int hot_reset_pcie_switch(struct hot_reset_pcie_switch* info);
+static int hot_reset_thread_fn(void* data);
 
 static int mst_pciconf_bc_open(struct inode* inode, struct file* file)
 {
@@ -131,6 +147,125 @@ ReturnOnFinished:
     return count;
 }
 
+int nnt_pci_reset_bus_sd(struct pci_dev* pci_device_prime, struct pci_dev* pci_device_aux)
+{
+    int error = 0;
+    struct task_struct* thread1 = NULL;
+    struct task_struct* thread2 = NULL;
+    struct reset_thread_data data1 = {0};
+    struct reset_thread_data data2 = {0};
+
+    init_completion(&data1.done);
+    init_completion(&data2.done);
+    data1.pdev = pci_device_prime;
+    data2.pdev = pci_device_aux;
+
+    thread1 = kthread_create(hot_reset_thread_fn, &data1, "pci_device_prime");
+    if (IS_ERR(thread1))
+    {
+        printk(KERN_ERR "mst_pciconf | Failed to create reset thread 1 (pci_device_prime)\n");
+        error = PTR_ERR(thread1);
+        goto cleanup;
+    }
+
+    thread2 = kthread_create(hot_reset_thread_fn, &data2, "pci_device_aux");
+    if (IS_ERR(thread2))
+    {
+        printk(KERN_ERR "mst_pciconf | Failed to create reset thread 2 (pci_device_aux)\n");
+        error = PTR_ERR(thread2);
+        goto cleanup;
+    }
+    wake_up_process(thread1);
+    wake_up_process(thread2);
+
+    wait_for_completion(&data1.done);
+    wait_for_completion(&data2.done);
+
+    if (data1.error || data2.error)
+    {
+        error = 1;
+    }
+
+cleanup:
+    return error;
+}
+
+static int nnt_pci_reset_bus(struct pci_dev* pci_device)
+{
+    int error = 0;
+    printk(KERN_INFO "mst_pciconf | Resetting the PCIe device: %4.4x:%2.2x:%2.2x.%1.1x\n",
+           pci_domain_nr(pci_device->bus), pci_device->bus->number, PCI_SLOT(pci_device->devfn),
+           PCI_FUNC(pci_device->devfn));
+
+#ifdef PCI_DEVICE_DATA
+    error = pci_reset_bus(pci_device);
+#else
+    error = pci_reset_bus(pci_device->bus);
+#endif
+
+    if (error)
+    {
+        printk(KERN_ERR "mst_pciconf | pci_reset_bus failed with error code: %d, pci device: %4.4x:%2.2x:%2.2x.%1.1x\n",
+               error, pci_domain_nr(pci_device->bus), pci_device->bus->number, PCI_SLOT(pci_device->devfn),
+               PCI_FUNC(pci_device->devfn));
+    }
+    else
+    {
+        printk(KERN_INFO "mst_pciconf | pci_reset_bus succeeded, pci device: %4.4x:%2.2x:%2.2x.%1.1x\n",
+               pci_domain_nr(pci_device->bus), pci_device->bus->number, PCI_SLOT(pci_device->devfn),
+               PCI_FUNC(pci_device->devfn));
+    }
+
+    return error;
+}
+
+static int hot_reset_thread_fn(void* data)
+{
+    struct reset_thread_data* thread_data = (struct reset_thread_data*)data;
+    struct pci_dev* pci_device = thread_data->pdev;
+    int error = 0;
+
+    error = nnt_pci_reset_bus(pci_device);
+
+    thread_data->error = error;
+    complete(&thread_data->done);
+    return 0;
+}
+
+int hot_reset_pcie_switch(struct hot_reset_pcie_switch* info)
+{
+    struct pci_dev* pdev_bus_prime;
+    struct pci_dev* pdev_bus_aux;
+
+    // Retrieve the PCI device corresponding to info.bus
+    pdev_bus_prime = pci_get_domain_bus_and_slot(info->prime.domain, info->prime.bus,
+                                                 PCI_DEVFN(info->prime.device, info->prime.function));
+    if (!pdev_bus_prime)
+    {
+        printk(KERN_ERR "mst_pciconf | Failed to get PCI device: %4.4x:%2.2x:%2.2x.%1.1x\n", info->prime.domain,
+               info->prime.bus, info->prime.device, info->prime.function);
+        return -ENODEV;
+    }
+
+    if (info->is_socket_direct)
+    {
+        pdev_bus_aux =
+          pci_get_domain_bus_and_slot(info->aux.domain, info->aux.bus, PCI_DEVFN(info->aux.device, info->aux.function));
+        if (!pdev_bus_aux)
+        {
+            printk(KERN_ERR "mst_pciconf | Failed to get PCI device: %4.4x:%2.2x:%2.2x.%1.1x\n", info->aux.domain,
+                   info->aux.bus, info->aux.device, info->aux.function);
+            return -ENODEV;
+        }
+
+        return nnt_pci_reset_bus_sd(pdev_bus_prime, pdev_bus_aux);
+    }
+    else
+    {
+        return nnt_pci_reset_bus(pdev_bus_prime);
+    }
+}
+
 static long ioctl(struct file* file, unsigned int command, unsigned long argument)
 {
     void* user_buffer = (void*)argument;
@@ -171,6 +306,25 @@ static long ioctl(struct file* file, unsigned int command, unsigned long argumen
         }
     }
 
+    if (nnt_device)
+    {
+        if (nnt_device->pci_device->error_state != pci_channel_io_normal)
+        {
+            /* The PCI device is in a bad state, need to obtain a new PCI device pointer */
+            printk(KERN_ERR "Device error state: %d. BDF: %04x:%02x:%02x.%d. Updating PCI device pointer...\n",
+                   nnt_device->pci_device->error_state, pci_domain_nr(nnt_device->pci_device->bus),
+                   nnt_device->pci_device->bus->number, PCI_SLOT(nnt_device->pci_device->devfn),
+                   PCI_FUNC(nnt_device->pci_device->devfn));
+
+            error = rescan();
+            if (error != 0)
+            {
+                printk(KERN_ERR "Rescan failed with error code: %d\n", error);
+                goto ReturnOnFinished;
+            }
+        }
+    }
+
     switch (command)
     {
         case INIT:
@@ -486,7 +640,8 @@ static long ioctl(struct file* file, unsigned int command, unsigned long argumen
             }
 
             /* Read the dword. */
-            if (read_dword(&nnt_read_from_cspace, nnt_device))
+            error = read_dword(&nnt_read_from_cspace, nnt_device);
+            if (error)
             {
                 goto ReturnOnFinished;
             }
@@ -557,7 +712,7 @@ static long ioctl(struct file* file, unsigned int command, unsigned long argumen
             mst_params.vendor = nnt_parameters.vendor;
             mst_params.subsystem_device = nnt_parameters.subsystem_device;
             mst_params.subsystem_vendor = nnt_parameters.subsystem_vendor;
-            mst_params.vendor_specific_cap = nnt_parameters.vendor_specific_capability;
+            mst_params.functional_vsc_offset = nnt_parameters.functional_vsc_offset;
             mst_params.multifunction = nnt_parameters.multifunction;
             mst_params.vsec_cap_mask = nnt_parameters.vsec_capability_mask;
 
@@ -575,6 +730,24 @@ static long ioctl(struct file* file, unsigned int command, unsigned long argumen
             error = destroy_nnt_device_bc(nnt_device);
             break;
         }
+        case RESCAN:
+        {
+            error = rescan();
+            break;
+        }
+        case PCICONF_HOT_RESET:
+        {
+            struct hot_reset_pcie_switch info = {0};
+
+            if (copy_from_user(&info, user_buffer, sizeof(struct hot_reset_pcie_switch)))
+            {
+                error = -EFAULT;
+                goto ReturnOnFinished;
+            }
+
+            error = hot_reset_pcie_switch(&info);
+            break;
+        }
         case PCICONF_DMA_PROPS:
         case PCICONF_MEM_ACCESS:
         case MODIFY:
@@ -600,6 +773,11 @@ int with_unknown = 0;
 
 module_param(with_unknown, int, S_IRUSR | S_IWUSR);
 
+int debug = 0;
+module_param(debug, int, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH); // Create "debug" entry under
+                                                                 // /sys/module/<module_name>/parameters/
+MODULE_PARM_DESC(debug, "Enable MFT kernel verbose debug prints");
+
 static int __init mst_pciconf_init_module(void)
 {
     dev_t device_number = -1;
@@ -617,7 +795,7 @@ static int __init mst_pciconf_init_module(void)
     /* Create device files for MFT. */
     error = create_nnt_devices(device_number, is_alloc_chrdev_region, &fop, NNT_PCICONF_DEVICES,
                                NNT_MELLANOX_PCI_VENDOR, with_unknown) ||
-            create_nnt_devices(device_number, is_alloc_chrdev_region, &fop, NNT_PCI_DEVICES, NNT_NVIDIA_PCI_VENDOR,
+            create_nnt_devices(device_number, is_alloc_chrdev_region, &fop, NNT_PCICONF_DEVICES, NNT_NVIDIA_PCI_VENDOR,
                                with_unknown);
 
     return error;
