@@ -38,12 +38,30 @@ int find_all_vendor_devices(unsigned int vendor_id);
      since it could be removed by hotplug event. */
 LIST_HEAD(nnt_device_list);
 
-unsigned short supported_rma_device_ids[] = {QUANTUM3_RMA_PCI_ID,  QUANTUM4_RMA_PCI_ID,   QUANTUM5_RMA_PCI_ID,
-                                             CONNECTX8_RMA_PCI_ID, ARCUSEE_RMA_PCI_ID,    QUANTUM2_RMA_PCI_ID,
-                                             CONNECTX7_RMA_PCI_ID, BLUEFIELD3_RMA_PCI_ID, SPECTRUM4_RMA_PCI_ID,
-                                             SPECTRUM5_RMA_PCI_ID, SPECTRUM6_RMA_PCI_ID};
+unsigned short supported_rma_device_ids[] = {
+  QUANTUM3_RMA_PCI_ID,  QUANTUM4_RMA_PCI_ID,   QUANTUM5_RMA_PCI_ID,
+  CONNECTX8_RMA_PCI_ID, ARCUSEE_RMA_PCI_ID,    QUANTUM2_RMA_PCI_ID,
+  CONNECTX7_RMA_PCI_ID, BLUEFIELD3_RMA_PCI_ID, SPECTRUM4_RMA_PCI_ID,
+  SPECTRUM5_RMA_PCI_ID, SPECTRUM6_RMA_PCI_ID,  CONNECTX8_RMA_PCI_ID,
+  CONNECTX9_RMA_PCI_ID, CONNECTX10_RMA_PCI_ID, CONNECTX9_PURE_PCIE_SWITCH_RMA_PCI_ID,
+  ARCUS2_RMA_PCI_ID};
 const unsigned int SUPPORTED_RMA_DEVICE_IDS_TABLE_SIZE = sizeof(supported_rma_device_ids) / sizeof(unsigned short);
 
+u_int16_t VERA_PCI_IDS[] = {VERA1_PCI_ID, VERA2_PCI_ID, VERA3_PCI_ID, VERA4_PCI_ID, VERA5_PCI_ID};
+
+int is_vera_pci_device(u_int16_t pci_device_id)
+{
+    unsigned int i;
+    for (i = 0; i < sizeof(VERA_PCI_IDS) / sizeof(u_int16_t); i++)
+    {
+        if (VERA_PCI_IDS[i] == pci_device_id)
+        {
+            return 1;
+        }
+    }
+    return 0;
+}
+
 int get_nnt_device(struct file* file, struct nnt_device** nnt_device)
 {
     int error_code = 0;
@@ -55,6 +73,12 @@ int get_nnt_device(struct file* file, struct nnt_device** nnt_device)
     else
     {
         *nnt_device = file->private_data;
+
+        if (!(*nnt_device)->pci_device)
+        {
+            printk(KERN_ERR "mst_pciconf driver: pci_device pointer is NULL\n");
+            error_code = -EINVAL;
+        }
     }
 
     return error_code;
@@ -297,6 +321,17 @@ int is_rma_device(unsigned short pci_device_id)
     return 0;
 }
 
+int is_livefish_device_with_supported_bar_access(unsigned short pci_device_id)
+{
+    switch (pci_device_id)
+    {
+        case SPECTRUM6_RECOVERY_ID:
+            return 1;
+        default:
+            return 0;
+    }
+}
+
 int is_livefish_device(unsigned short pci_device_id)
 {
     return pci_device_id >= CONNECTX3_LIVEFISH_ID && pci_device_id < CONNECTX3_PCI_ID &&
@@ -319,15 +354,22 @@ int is_toolspf(unsigned short pci_device_id)
     return is_nic(pci_device_id - 4000) || is_switch(pci_device_id - 4000);
 }
 
+int is_cpu_pci_device(u_int16_t pci_device_id)
+{
+    return is_vera_pci_device(pci_device_id);
+}
+
 int is_pciconf_device(unsigned short pci_device_id)
 {
     int result = is_nic(pci_device_id) || is_toolspf(pci_device_id) || is_livefish_device(pci_device_id) ||
-                 is_rma_device(pci_device_id) || is_switch(pci_device_id) || is_gpu_pci_device(pci_device_id);
+                 is_rma_device(pci_device_id) || is_switch(pci_device_id) || is_gpu_pci_device(pci_device_id) ||
+                 is_cpu_pci_device(pci_device_id);
 
     DEBUG_PRINTK(
-      "is_pciconf_device check for device 0x%x: result=%d (is_nic: %d, is_toolspf: %d, is_livefish: %d, is_rma: %d, is_switch: %d, is_gpu: %d)\n",
+      "is_pciconf_device check for device 0x%x: result=%d (is_nic: %d, is_toolspf: %d, is_livefish: %d, is_rma: %d, is_switch: %d, is_gpu: %d, is_cpu: %d)\n",
       pci_device_id, result, is_nic(pci_device_id), is_toolspf(pci_device_id), is_livefish_device(pci_device_id),
-      is_rma_device(pci_device_id), is_switch(pci_device_id), is_gpu_pci_device(pci_device_id));
+      is_rma_device(pci_device_id), is_switch(pci_device_id), is_gpu_pci_device(pci_device_id),
+      is_cpu_pci_device(pci_device_id));
 
     return result;
 }
@@ -336,12 +378,12 @@ int is_pcicr_device(unsigned short pci_device_id)
 {
     int result = (is_gpu_pci_device(pci_device_id) || is_switch(pci_device_id) || is_toolspf(pci_device_id) ||
                   is_connectx3(pci_device_id)) &&
-                 (!is_livefish_device(pci_device_id));
+                 !(is_livefish_device(pci_device_id) && !is_livefish_device_with_supported_bar_access(pci_device_id));
 
     DEBUG_PRINTK(
-      "is_pcicr_device check for device 0x%x: result=%d (is_gpu: %d, is_toolspf: %d, is_connectx3: %d, !is_livefish: %d)\n",
+      "is_pcicr_device check for device 0x%x: result=%d (is_gpu: %d, is_toolspf: %d, is_connectx3: %d, !is_livefish: %d, !is_livefish_device_with_supported_bar_access: %d)\n",
       pci_device_id, result, is_gpu_pci_device(pci_device_id), is_toolspf(pci_device_id), is_connectx3(pci_device_id),
-      !is_livefish_device(pci_device_id));
+      !is_livefish_device(pci_device_id), !is_livefish_device_with_supported_bar_access(pci_device_id));
 
     return result;
 }
@@ -528,7 +570,8 @@ int create_devices(dev_t device_number, struct file_operations* fop, int is_allo
           pci_find_capability(current_nnt_device->pci_device, PCI_CAP_ID_VPD);
 
         /* NNT_PCICONF_RECOVERY is for LF and late LF. */
-        if (!current_nnt_device->pciconf_device.functional_vsc_offset)
+        if (!current_nnt_device->pciconf_device.functional_vsc_offset &&
+            current_nnt_device->device_type != NNT_PCI_MEMORY)
         {
             current_nnt_device->device_type = NNT_PCICONF_RECOVERY;
             DEBUG_PRINTK(
