@@ -26,5 +26,8 @@ void mutex_unlock_nnt(struct file* file);
 int rescan(void);
 unsigned int get_vsc_address_by_type(struct nnt_device* current_nnt_device, int vsc_type);
 int is_vsc_type(struct nnt_device* current_nnt_device, unsigned int vsec_address, unsigned int vsc_type);
+int is_cpu_pci_device(u_int16_t pci_device_id);
+int is_vera_pci_device(u_int16_t pci_device_id);
+int is_livefish_device_with_supported_bar_access(unsigned short pci_device_id);
 
 #endif
