@@ -55,12 +55,16 @@
 #define CONNECTX6LX_PCI_ID 4127
 #define CONNECTX7_PCI_ID 4129
 #define CONNECTX7_RMA_PCI_ID 537
-#define CONNECTX8_PCI_ID 4131 // also the ID for CX8 PCIe switch with eHCA
+#define CONNECTX8_PCI_ID 4131
 #define CONNECTX8_RMA_PCI_ID 543
 #define CONNECTX8_BRIDGE_PCI_ID 6525
-#define CONNECTX9_BRIDGE_PCI_ID 6526
+#define CONNECTX8_PURE_PCIE_SWITCH_RMA_PCI_ID 547
 #define CONNECTX9_PCI_ID 4133
+#define CONNECTX9_RMA_PCI_ID 549
+#define CONNECTX9_BRIDGE_PCI_ID 6526
+#define CONNECTX9_PURE_PCIE_SWITCH_RMA_PCI_ID 553
 #define CONNECTX10_PCI_ID 4135
+#define CONNECTX10_RMA_PCI_ID 551
 #define SCHRODINGER_PCI_ID 6518
 #define FREYSA_PCI_ID 6521
 #define BLUEFIELD_PCI_ID 41682
@@ -91,8 +95,15 @@
 #define GR100_PCI_ID 12288
 #define GR150_PCI_ID 13440
 #define ARCUSEE_RMA_PCI_ID 643
+#define ARCUS2_RMA_PCI_ID 649
 #define SPECTRUM6_PCI_ID 53124
+#define SPECTRUM6_RECOVERY_ID 628
 #define SPECTRUM6_RMA_PCI_ID 629
+#define VERA1_PCI_ID 0x2F95
+#define VERA2_PCI_ID 0x2F96
+#define VERA3_PCI_ID 0x2F97
+#define VERA4_PCI_ID 0x2F98
+#define VERA5_PCI_ID 0x2F99
 
 enum nnt_device_type_flag
 {
