@@ -7,6 +7,21 @@
 #include <linux/delay.h>
 #include <linux/uaccess.h>
 
+/* Forward declarations */
+int clear_vsec_semaphore(struct nnt_device* nnt_device);
+int get_semaphore_ticket(struct nnt_device* nnt_device, unsigned int* lock_value, unsigned int* counter);
+int lock_vsec_semaphore(struct nnt_device* nnt_device);
+int wait_on_flag(struct nnt_device* nnt_device, u8 expected_val);
+int check_syndrome(struct nnt_device* nnt_device);
+int set_address_space(struct nnt_device* nnt_device, unsigned int address_space);
+int set_rw_address(unsigned int* offset, unsigned int rw);
+int read(struct nnt_device* nnt_device, unsigned int offset, unsigned int* data);
+int write(struct nnt_device* nnt_device, unsigned int offset, unsigned int data);
+int address_space_to_capability(u_int16_t address_space);
+int get_space_support_status(struct nnt_device* nnt_device, u_int16_t address_space);
+int init_vsec_capability_mask(struct nnt_device* nnt_device);
+void check_vsec_minimum_support(struct nnt_device* nnt_device);
+
 int clear_vsec_semaphore(struct nnt_device* nnt_device)
 {
     /* Clear the semaphore. */
@@ -50,6 +65,9 @@ int lock_vsec_semaphore(struct nnt_device* nnt_device)
     {
         if (retries > SEMAPHORE_MAX_RETRIES)
         {
+            DEBUG_PRINTK("Timed out trying to lock VSEC semaphore for device: %04x:%02x:%02x.%d. \n",
+                         pci_domain_nr(nnt_device->pci_device->bus), nnt_device->pci_device->bus->number,
+                         PCI_SLOT(nnt_device->pci_device->devfn), PCI_FUNC(nnt_device->pci_device->devfn));
             return -EINVAL;
         }
 
@@ -146,7 +164,8 @@ int check_syndrome(struct nnt_device* nnt_device)
         syndrome_code = EXTRACT(syndrome_code, PCI_SYNDROME_CODE_BIT_OFFSET, PCI_SYNDROME_CODE_BIT_LEN);
         if (syndrome_code == ADDRESS_OUT_OF_RANGE)
         {
-            error = -1;
+            error = syndrome_code;
+            DEBUG_PRINTK("check_syndrome: syndrome is set. syndrome_code is ADDRESS_OUT_OF_RANGE.\n");
         }
     }
 ReturnOnFinished:
@@ -184,16 +203,19 @@ int set_address_space(struct nnt_device* nnt_device, unsigned int address_space)
     /* Check if the space written is indeed the space we attempted to write */
     if (actual_value != expected_value)
     {
+        // DEBUG_PRINTK(
+        //   KERN_DEBUG
+        //   "actual_space_value != expected_space_value. expected_space_value: 0x%x actual_space_value: 0x%x. Meaning
+        //   space: 0x%x is not supported.\n", expected_value, actual_value, expected_value);
         error = -EINVAL;
     }
 
     /* Read status and make sure address_space is supported. */
     if (EXTRACT(read_value, PCI_STATUS_BIT_OFFSET, PCI_STATUS_BIT_LEN) == 0)
     {
-        // printk(
-        //   KERN_DEBUG
-        //   "failed to set VSC address_space to: %u. status bit is 0 indicating this VSC address_space is not
-        //   supported.\n", address_space);
+        DEBUG_PRINTK(
+          "failed to set VSC address_space to: %u. status bit is 0 indicating this VSC address_space is not supported.\n",
+          address_space);
         error = -EINVAL;
     }
 
@@ -288,11 +310,12 @@ int read(struct nnt_device* nnt_device, unsigned int offset, unsigned int* data)
 {
     int error = set_rw_address(&offset, READ_OPERATION);
     CHECK_ERROR(error);
-    // printk(KERN_DEBUG "in read() - set_rw_address succesfull. offset: %u.\n", offset);
 
     /* Write address. */
     error = pci_write_config_dword(nnt_device->pci_device, nnt_device->pciconf_device.address_offset, offset);
     CHECK_PCI_WRITE_ERROR(error, nnt_device->pciconf_device.address_offset, offset);
+    // DEBUG_PRINTK("in read() - pci_write_config_dword succesfull. nnt_device->pciconf_device.address_offset:%u.\n ",
+    //              nnt_device->pciconf_device.address_offset);
 
     error = wait_on_flag(nnt_device, 1);
     CHECK_ERROR(error);
@@ -300,6 +323,10 @@ int read(struct nnt_device* nnt_device, unsigned int offset, unsigned int* data)
     /* Read data. */
     error = pci_read_config_dword(nnt_device->pci_device, nnt_device->pciconf_device.data_offset, data);
     CHECK_PCI_READ_ERROR(error, nnt_device->pciconf_device.data_offset);
+    // DEBUG_PRINTK("in read() - pci_read_config_dword succesfull. nnt_device->pciconf_device.data_offset: %u. data:
+    // %u\n",
+    //              nnt_device->pciconf_device.data_offset,
+    //              *data);
 
     if (nnt_device->pciconf_device.pci_vsec_space_fully_supported == 1)
     {
@@ -328,22 +355,30 @@ int read_pciconf(struct nnt_device* nnt_device, struct nnt_rw_operation* read_op
     /* Is CR Space address space supported ?*/
     error = set_address_space(nnt_device, read_operation->address_space);
     CHECK_ERROR(error);
-    // printk(KERN_DEBUG "in read_pciconf() - set_address_space succesfull. read_operation->address_space: %u.\n",
-    //        read_operation->address_space);
 
     for (counter = 0; counter < read_operation->size; counter += 4)
     {
-        if (read(nnt_device, read_operation->offset + counter, &read_operation->data[counter >> 2]))
+        int read_result = read(nnt_device, read_operation->offset + counter, &read_operation->data[counter >> 2]);
+
+        if (read_result != 0)
         {
-            error = counter;
-            goto ReturnOnFinished;
+            if (read_result == ADDRESS_OUT_OF_RANGE) // Support PCI space
+            {
+                DEBUG_PRINTK("read_pciconf: address_space: 0x%x. read_result is ADDRESS_OUT_OF_RANGE.\n",
+                             read_operation->address_space);
+                error = ADDRESS_OUT_OF_RANGE; // On valid flows, counter will never be equal to 3 as its an odd number,
+                                              // so we can propagate the error back up.
+            }
+            else
+            {
+                error = counter; // Indicate where the error occurred
+            }
+            break;
         }
     }
 
 ReturnOnFinished:
-    /* Clear semaphore. */
     clear_vsec_semaphore(nnt_device);
-
     return error;
 }
 
@@ -392,17 +427,25 @@ int write_pciconf(struct nnt_device* nnt_device, struct nnt_rw_operation* write_
 
     for (counter = 0; counter < write_operation->size; counter += 4)
     {
-        if (write(nnt_device, write_operation->offset + counter, write_operation->data[counter >> 2]))
+        int write_result = write(nnt_device, write_operation->offset + counter, write_operation->data[counter >> 2]);
+
+        if (write_result != 0)
         {
-            error = counter;
-            goto ReturnOnFinished;
+            if (write_result == ADDRESS_OUT_OF_RANGE) // Support PCI space
+            {
+                error = ADDRESS_OUT_OF_RANGE; // On valid flows, counter will never be equal to 3 as its an odd number,
+                                              // so we can propagate the error back up.
+            }
+            else
+            {
+                error = counter; // Indicate where the error occurred
+            }
+            break;
         }
     }
 
 ReturnOnFinished:
-    /* Clear semaphore. */
     clear_vsec_semaphore(nnt_device);
-
     return error;
 }
 
@@ -456,6 +499,8 @@ int get_space_support_status(struct nnt_device* nnt_device, u_int16_t address_sp
 
     nnt_device->pciconf_device.vsec_capability_mask |= (status << address_space_to_capability(address_space));
 
+    DEBUG_PRINTK("status of address space: 0x%x is: %d\n", address_space, status);
+
     return status;
 }
 
@@ -515,6 +560,7 @@ void check_vsec_minimum_support(struct nnt_device* nnt_device)
              (nnt_device->pciconf_device.vsec_capability_mask & (1 << NNT_VSEC_PCI_GLOBAL_SEMAPHORE_SPACE_SUPPORTED))))
         {
             nnt_device->pciconf_device.pci_vsec_space_fully_supported = 1;
+            DEBUG_PRINTK("nnt_device->pciconf_device.pci_vsec_space_fully_supported = 1\n");
         }
         if (((nnt_device->pciconf_device.vsec_capability_mask & (1 << NNT_VSEC_INITIALIZED)) &&
              (nnt_device->pciconf_device.vsec_capability_mask & (1 << NNT_VSEC_ICMD_SPACE_SUPPORTED)) &&
