@@ -314,11 +314,7 @@ static struct rpcrdma_mr_seg *rpcrdma_mr_prepare(struct rpcrdma_xprt *r_xprt,
 	}
 
 	rpcrdma_mr_push(*mr, &req->rl_registered);
-#ifdef HAVE_RPC_XPRT_OPS_FREE_SLOT
 	return frwr_map(r_xprt, seg, nsegs, writing, req->rl_slot.rq_xid, *mr);
-#else
-	return frwr_map(r_xprt, seg, nsegs, writing, req->rl_xid, *mr);
-#endif
 
 out_getmr_err:
 #ifdef HAVE_TRACE_RPCRDMA_H
@@ -974,14 +970,6 @@ rpcrdma_marshal_req(struct rpcrdma_xprt *r_xprt, struct rpc_rqst *rqst)
 		rtype = rpcrdma_areadch;
 	}
 
-#if !defined(HAVE_RPC_XPRT_OPS_FREE_SLOT) || !defined(HAVE_XPRT_PIN_RQST)
-	req->rl_xid = rqst->rq_xid;
-#endif
-
-#ifndef HAVE_XPRT_PIN_RQST
-	rpcrdma_insert_req(&r_xprt->rx_buf, req);
-#endif
-
 	/* This implementation supports the following combinations
 	 * of chunk lists in one RPC-over-RDMA Call message:
 	 *
@@ -1417,16 +1405,13 @@ rpcrdma_decode_error(struct rpcrdma_xprt *r_xprt, struct rpcrdma_rep *rep,
  */
 void rpcrdma_unpin_rqst(struct rpcrdma_rep *rep)
 {
-#ifdef HAVE_XPRT_PIN_RQST
 	struct rpc_xprt *xprt = &rep->rr_rxprt->rx_xprt;
-#endif
 	struct rpc_rqst *rqst = rep->rr_rqst;
 	struct rpcrdma_req *req = rpcr_to_rdmar(rqst);
 
 	req->rl_reply = NULL;
 	rep->rr_rqst = NULL;
 
-#ifdef HAVE_XPRT_PIN_RQST
 #ifdef HAVE_XPRT_QUEUE_LOCK
 	spin_lock(&xprt->queue_lock);
 #else
@@ -1438,7 +1423,6 @@ void rpcrdma_unpin_rqst(struct rpcrdma_rep *rep)
 #else
 	spin_unlock(&xprt->recv_lock);
 #endif
-#endif
 }
 
 /**
@@ -1452,9 +1436,7 @@ void rpcrdma_unpin_rqst(struct rpcrdma_rep *rep)
 void rpcrdma_complete_rqst(struct rpcrdma_rep *rep)
 {
 	struct rpcrdma_xprt *r_xprt = rep->rr_rxprt;
-#ifdef HAVE_XPRT_PIN_RQST
 	struct rpc_xprt *xprt = &r_xprt->rx_xprt;
-#endif
 	struct rpc_rqst *rqst = rep->rr_rqst;
 	int status;
 
@@ -1475,17 +1457,13 @@ void rpcrdma_complete_rqst(struct rpcrdma_rep *rep)
 		goto out_badheader;
 
 out:
-#ifdef HAVE_XPRT_PIN_RQST
 #ifdef HAVE_XPRT_QUEUE_LOCK
 	spin_lock(&xprt->queue_lock);
 #else
 	spin_lock(&xprt->recv_lock);
 #endif
-#endif /* HAVE_XPRT_PIN_RQST */
 
 	xprt_complete_rqst(rqst->rq_task, status);
-
-#ifdef HAVE_XPRT_PIN_RQST
 	xprt_unpin_rqst(rqst);
 
 #ifdef HAVE_XPRT_QUEUE_LOCK
@@ -1493,7 +1471,6 @@ out:
 #else
 	spin_unlock(&xprt->recv_lock);
 #endif
-#endif /* HAVE_XPRT_PIN_RQST */
 
 	return;
 
@@ -1507,7 +1484,6 @@ out_badheader:
 	goto out;
 }
 
-#ifdef HAVE_XPRT_PIN_RQST
 static void rpcrdma_reply_done(struct kref *kref)
 {
 	struct rpcrdma_req *req =
@@ -1515,7 +1491,6 @@ static void rpcrdma_reply_done(struct kref *kref)
 
 	rpcrdma_complete_rqst(req->rl_reply);
 }
-#endif
 
 /**
  * rpcrdma_reply_handler - Process received RPC/RDMA messages
@@ -1565,7 +1540,6 @@ void rpcrdma_reply_handler(struct rpcrdma_rep *rep)
 	/* Match incoming rpcrdma_rep to an rpcrdma_req to
 	 * get context for handling any incoming chunks.
 	 */
-#ifdef HAVE_XPRT_PIN_RQST
 #ifdef HAVE_XPRT_QUEUE_LOCK
 	spin_lock(&xprt->queue_lock);
 #else
@@ -1582,48 +1556,24 @@ void rpcrdma_reply_handler(struct rpcrdma_rep *rep)
 	spin_unlock(&xprt->recv_lock);
 #endif
 
-	req = rpcr_to_rdmar(rqst);
-#else /* HAVE_XPRT_PIN_RQST */
-	spin_lock(&buf->rb_lock);
-	req = rpcrdma_lookup_req_locked(&r_xprt->rx_buf, rep->rr_xid);
-	if (!req) {
-		spin_unlock(&buf->rb_lock);
-		goto out;
-	}
-
-	/* Avoid races with signals and duplicate replies
-	 * by marking this req as matched.
-	 */
-#endif /* HAVE_XPRT_PIN_RQST */
-
 	if (credits == 0)
 		credits = 1;	/* don't deadlock */
 	else if (credits > r_xprt->rx_ep->re_max_requests)
 		credits = r_xprt->rx_ep->re_max_requests;
-	rpcrdma_post_recvs(r_xprt, credits + (buf->rb_bc_srv_max_requests << 1),
-			   false);
+	rpcrdma_post_recvs(r_xprt, credits + (buf->rb_bc_srv_max_requests << 1));
 	if (buf->rb_credits != credits)
 		rpcrdma_update_cwnd(r_xprt, credits);
 
+	req = rpcr_to_rdmar(rqst);
 	if (unlikely(req->rl_reply))
-#ifdef HAVE_XPRT_PIN_RQST
 		rpcrdma_rep_put(buf, req->rl_reply);
-#else
-		rpcrdma_recv_buffer_put_locked(req->rl_reply);
-#endif
 	req->rl_reply = rep;
-
-#ifdef HAVE_XPRT_PIN_RQST
 	rep->rr_rqst = rqst;
-#else
-	spin_unlock(&buf->rb_lock);
-#endif
 
 #ifdef HAVE_TRACE_RPCRDMA_H
 	trace_xprtrdma_reply(rqst->rq_task, rep, credits);
 #endif
 
-#ifdef HAVE_XPRT_PIN_RQST
 	if (rep->rr_wc_flags & IB_WC_WITH_INVALIDATE)
 		frwr_reminv(rep, &req->rl_registered);
 	if (!list_empty(&req->rl_registered))
@@ -1631,32 +1581,6 @@ void rpcrdma_reply_handler(struct rpcrdma_rep *rep)
 		/* LocalInv completion will complete the RPC */
 	else
 		kref_put(&req->rl_kref, rpcrdma_reply_done);
-#else
-#ifdef HAVE_RPC_XPRT_RECV_LOCK
-	spin_lock(&xprt->recv_lock);
-#else
-	spin_lock_bh(&xprt->transport_lock);
-#endif
-
-	rqst = xprt_lookup_rqst(xprt, rep->rr_xid);
-	if (!rqst) {
-#ifdef HAVE_RPC_XPRT_RECV_LOCK
-	spin_unlock(&xprt->recv_lock);
-#else
-	spin_unlock_bh(&xprt->transport_lock);
-#endif
-		goto out;
-	}
-
-	rep->rr_rqst = rqst;
-	rpcrdma_complete_rqst(rep);
-#ifdef HAVE_RPC_XPRT_RECV_LOCK
-	spin_unlock(&xprt->recv_lock);
-#else
-	spin_unlock_bh(&xprt->transport_lock);
-#endif
-#endif /* HAVE_XPRT_PIN_RQST */
-
 	return;
 
 out_badversion:
@@ -1665,7 +1589,6 @@ out_badversion:
 #endif
 	goto out;
 
-#ifdef HAVE_XPRT_PIN_RQST
 out_norqst:
 #ifdef HAVE_XPRT_QUEUE_LOCK
 	spin_unlock(&xprt->queue_lock);
@@ -1676,7 +1599,6 @@ out_norqst:
 	trace_xprtrdma_reply_rqst_err(rep);
 #endif
 	goto out;
-#endif
 
 out_shortreply:
 #ifdef HAVE_TRACE_RPCRDMA_H
