@@ -27,7 +27,7 @@ inline static void free_lwip_pbuf(struct pbuf *lwip_pbuf)
 {
     mem_buf_desc_t *p_desc = reinterpret_cast<mem_buf_desc_t *>(lwip_pbuf);
 
-    if (lwip_pbuf->desc.attr == PBUF_DESC_MDESC || lwip_pbuf->desc.attr == PBUF_DESC_NVME_TX) {
+    if (lwip_pbuf->desc.attr == PBUF_DESC_MDESC) {
         mem_desc *mdesc = reinterpret_cast<mem_desc *>(lwip_pbuf->desc.mdesc);
         mdesc->put();
     }
@@ -53,7 +53,7 @@ public:
     void register_memory(ib_ctx_handler *p_ib_ctx_h);
     void print_val_tbl();
     void print_report(vlog_levels_t log_level = VLOG_DEBUG);
-    static void print_full_report(vlog_levels_t log_level);
+    static void print_full_report(vlog_levels_t log_level, bool print_only_critical = false);
 
     uint32_t find_lkey_by_ib_ctx_thread_safe(ib_ctx_handler *p_ib_ctx_h);
 
