@@ -23,6 +23,8 @@
 #include "hw_queue_rx.h"
 #include "ring_simple.h"
 
+#include "core/dev/net_device_table_mgr.h"
+
 #define MODULE_NAME "cq_mgr_rx"
 
 #define cq_logpanic   __log_info_panic
@@ -125,10 +127,6 @@ cq_mgr_rx::~cq_mgr_rx()
 {
     cq_logdbg("Destroying Rx CQ");
 
-    if (m_rx_buffs_rdy_for_free_head) {
-        reclaim_recv_buffers(m_rx_buffs_rdy_for_free_head);
-    }
-
     if (m_rx_queue.size() + m_rx_pool.size()) {
         cq_logdbg("Returning %lu buffers to global Rx pool (ready queue %lu, free pool %lu))",
                   m_rx_queue.size() + m_rx_pool.size(), m_rx_queue.size(), m_rx_pool.size());
@@ -148,6 +146,8 @@ cq_mgr_rx::~cq_mgr_rx()
     ENDIF_VERBS_FAILURE;
     VALGRIND_MAKE_MEM_UNDEFINED(m_p_ibv_cq, sizeof(ibv_cq));
 
+    g_p_net_device_table_mgr->increase_closed_rings_rx_cq_drop_counter(
+        m_p_cq_stat->n_rx_hw_pkt_drops);
     statistics_print();
     xlio_stats_instance_remove_cq_block(m_p_cq_stat);
 
@@ -156,10 +156,13 @@ cq_mgr_rx::~cq_mgr_rx()
 
 void cq_mgr_rx::statistics_print()
 {
-    if (m_p_cq_stat->n_rx_pkt_drop || m_p_cq_stat->n_rx_sw_queue_len ||
-        m_p_cq_stat->n_rx_drained_at_once_max || m_p_cq_stat->n_buffer_pool_len) {
+    if (m_p_cq_stat->n_rx_sw_pkt_drops || m_p_cq_stat->n_rx_hw_pkt_drops ||
+        m_p_cq_stat->n_rx_sw_queue_len || m_p_cq_stat->n_rx_drained_at_once_max ||
+        m_p_cq_stat->n_buffer_pool_len) {
         cq_logdbg_no_funcname("Packets dropped: %12llu",
-                              (unsigned long long int)m_p_cq_stat->n_rx_pkt_drop);
+                              (unsigned long long int)m_p_cq_stat->n_rx_sw_pkt_drops);
+        cq_logdbg_no_funcname("HW RX Packets dropped: %12llu",
+                              (unsigned long long int)m_p_cq_stat->n_rx_hw_pkt_drops);
         cq_logdbg_no_funcname("Drained max: %17u", m_p_cq_stat->n_rx_drained_at_once_max);
         cq_logdbg_no_funcname("CQE errors: %18llu",
                               (unsigned long long int)m_p_cq_stat->n_rx_cqe_error);
@@ -251,7 +254,7 @@ void cq_mgr_rx::lro_update_hdr(struct xlio_mlx5_cqe *cqe, mem_buf_desc_t *p_rx_w
         // CQE indicates IPv4 in the l3_hdr_type field
         struct iphdr *p_ip_h = (struct iphdr *)(p_rx_wc_buf_desc->p_buffer + transport_header_len);
 
-        assert(p_ip_h->version == IPV4_VERSION);
+        assert(p_ip_h->version == XLIO_IPV4_VERSION);
         assert(p_ip_h->protocol == IPPROTO_TCP);
 
         p_ip_h->ttl = cqe->lro_min_ttl;
@@ -325,11 +328,6 @@ mem_buf_desc_t *cq_mgr_rx::cqe_process_rx(mem_buf_desc_t *p_mem_buf_desc, enum b
     /* Assume locked!!! */
     cq_logfuncall("");
 
-    /* we use context to verify that on reclaim rx buffer path we return the buffer to the right CQ
-     */
-    p_mem_buf_desc->rx.is_xlio_thr = false;
-    p_mem_buf_desc->rx.context = nullptr;
-
     if (unlikely(status != BS_OK)) {
         m_p_next_rx_desc_poll = nullptr;
         reclaim_recv_buffer_helper(p_mem_buf_desc);
@@ -360,7 +358,7 @@ bool cq_mgr_rx::compensate_qp_poll_success(mem_buf_desc_t *buff_cur)
         m_debt -= buffers;
         m_p_cq_stat->n_buffer_pool_len = m_rx_pool.size();
     } else if (m_b_sysvar_cq_keep_qp_full || m_debt >= (int)m_hqrx_ptr->m_rx_num_wr) {
-        m_p_cq_stat->n_rx_pkt_drop++;
+        m_p_cq_stat->n_rx_sw_pkt_drops++;
         m_hqrx_ptr->post_recv_buffer(buff_cur);
         --m_debt;
         return true;
@@ -419,10 +417,6 @@ void cq_mgr_rx::mem_buf_desc_return_to_owner(mem_buf_desc_t *p_mem_buf_desc,
 
 bool cq_mgr_rx::reclaim_recv_buffers(mem_buf_desc_t *rx_reuse_lst)
 {
-    if (m_rx_buffs_rdy_for_free_head) {
-        reclaim_recv_buffer_helper(m_rx_buffs_rdy_for_free_head);
-        m_rx_buffs_rdy_for_free_head = m_rx_buffs_rdy_for_free_tail = nullptr;
-    }
     reclaim_recv_buffer_helper(rx_reuse_lst);
     return_extra_buffers();
 
@@ -438,23 +432,6 @@ bool cq_mgr_rx::reclaim_recv_buffers_no_lock(mem_buf_desc_t *rx_reuse_lst)
     return false;
 }
 
-int cq_mgr_rx::reclaim_recv_single_buffer(mem_buf_desc_t *rx_reuse)
-{
-    int ret_val = 0;
-
-    ret_val = rx_reuse->lwip_pbuf_dec_ref_count();
-    if ((ret_val == 0) && (rx_reuse->get_ref_count() <= 0)) {
-        if (!m_rx_buffs_rdy_for_free_head) {
-            m_rx_buffs_rdy_for_free_head = m_rx_buffs_rdy_for_free_tail = rx_reuse;
-        } else {
-            m_rx_buffs_rdy_for_free_tail->p_next_desc = rx_reuse;
-            m_rx_buffs_rdy_for_free_tail = rx_reuse;
-        }
-        m_rx_buffs_rdy_for_free_tail->p_next_desc = nullptr;
-    }
-    return ret_val;
-}
-
 bool cq_mgr_rx::reclaim_recv_buffers(descq_t *rx_reuse)
 {
     cq_logfuncall("");
