@@ -83,11 +83,10 @@ public:
 
     /**
      * Poll RX CQ. Each CQE processed directly.
-     * @return True if CQ was drained.
+     * @return Zero - Not drained, Positive - Drained with packets, Negative - No packets.
      */
-    virtual bool poll_and_process_element_rx(uint64_t *p_cq_poll_sn,
-                                             void *pv_fd_ready_array = nullptr) = 0;
-    virtual mem_buf_desc_t *poll_and_process_socketxtreme() { return nullptr; };
+    virtual int poll_and_process_element_rx(uint64_t *p_cq_poll_sn,
+                                            void *pv_fd_ready_array = nullptr) = 0;
 
     /**
      * This will check if the cq was drained, and if it wasn't it will drain it.
@@ -112,9 +111,9 @@ public:
     bool reclaim_recv_buffers(descq_t *rx_reuse);
     bool reclaim_recv_buffers(mem_buf_desc_t *rx_reuse_lst);
     bool reclaim_recv_buffers_no_lock(mem_buf_desc_t *rx_reuse_lst);
-    int reclaim_recv_single_buffer(mem_buf_desc_t *rx_reuse);
 
     void get_cq_event(int count = 1) { xlio_ib_mlx5_get_cq_event(&m_mlx5_cq, count); };
+    uint64_t get_n_rx_hw_pkt_drops() { return m_p_cq_stat->n_rx_hw_pkt_drops; }
 
 protected:
     /**
@@ -165,13 +164,6 @@ protected:
     const uint32_t m_n_sysvar_rx_num_wr_to_post_recv;
     descq_t m_rx_pool;
 
-    /* This fields are needed to track internal memory buffers
-     * represented as struct xlio_buff_t
-     * from user application by special XLIO extended API
-     */
-    mem_buf_desc_t *m_rx_buffs_rdy_for_free_head = nullptr;
-    mem_buf_desc_t *m_rx_buffs_rdy_for_free_tail = nullptr;
-
 private:
     struct ibv_comp_channel *m_comp_event_channel;
     bool m_b_notification_armed = false;
