@@ -117,7 +117,9 @@ void cq_mgr_rx_regrq::cqe_to_mem_buff_desc(struct xlio_mlx5_cqe *cqe,
         p_rx_wc_buf_desc->rx.tls_decrypted = (cqe->pkt_info >> 3) & 0x3;
 #endif /* DEFINED_UTLS */
         p_rx_wc_buf_desc->rx.timestamps.hw_raw = ntohll(cqe->timestamp);
-        p_rx_wc_buf_desc->rx.flow_tag_id = ntohl((uint32_t)(cqe->sop_drop_qpn));
+        uint32_t sop_rxdrop_qpn_flowtag_h_byte = ntohl(cqe->sop_rxdrop_qpn_flowtag);
+        p_rx_wc_buf_desc->rx.flow_tag_id = sop_rxdrop_qpn_flowtag_h_byte & 0x00FFFFFF;
+        m_p_cq_stat->n_rx_hw_pkt_drops += sop_rxdrop_qpn_flowtag_h_byte >> 24;
         p_rx_wc_buf_desc->rx.is_sw_csum_need =
             !(m_b_is_rx_hw_csum_on && (cqe->hds_ip_ext & MLX5_CQE_L4_OK) &&
               (cqe->hds_ip_ext & MLX5_CQE_L3_OK));
@@ -186,8 +188,6 @@ int cq_mgr_rx_regrq::drain_and_proccess(uintptr_t *p_recycle_buffers_last_wr_id
      * Internal thread:
      *   Frequency of real polling can be controlled by
      *   PROGRESS_ENGINE_INTERVAL and PROGRESS_ENGINE_WCE_MAX.
-     * socketxtreme:
-     *   User does socketxtreme_poll()
      * Cleanup:
      *   QP down logic to release rx buffers should force polling to do this.
      *   Not null argument indicates one.
@@ -204,14 +204,13 @@ int cq_mgr_rx_regrq::drain_and_proccess(uintptr_t *p_recycle_buffers_last_wr_id
 
         if (cqe_process_rx(buff, status)) {
             if (p_recycle_buffers_last_wr_id) {
-                m_p_cq_stat->n_rx_pkt_drop++;
+                m_p_cq_stat->n_rx_sw_pkt_drops++;
                 reclaim_recv_buffer_helper(buff);
             } else {
                 bool procces_now = is_eth_tcp_frame(buff);
 
                 /* We process immediately all non udp/ip traffic.. */
                 if (procces_now) {
-                    buff->rx.is_xlio_thr = true;
                     if ((++m_debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
                         !compensate_qp_poll_success(buff)) {
                         process_recv_buffer(buff, NULL);
@@ -247,28 +246,7 @@ int cq_mgr_rx_regrq::drain_and_proccess(uintptr_t *p_recycle_buffers_last_wr_id
     return ret_total;
 }
 
-mem_buf_desc_t *cq_mgr_rx_regrq::poll_and_process_socketxtreme()
-{
-    buff_status_e status = BS_OK;
-    mem_buf_desc_t *buff_wqe = poll(status);
-
-    if (buff_wqe) {
-        if (cqe_process_rx(buff_wqe, status)) {
-            if ((++m_debt < (int)m_n_sysvar_rx_num_wr_to_post_recv) ||
-                !compensate_qp_poll_success(buff_wqe)) {
-                return buff_wqe;
-            }
-        } else if (++m_debt >= (int)m_n_sysvar_rx_num_wr_to_post_recv) {
-            compensate_qp_poll_failed();
-        }
-    } else {
-        compensate_qp_poll_failed();
-    }
-
-    return nullptr;
-}
-
-bool cq_mgr_rx_regrq::poll_and_process_element_rx(uint64_t *p_cq_poll_sn, void *pv_fd_ready_array)
+int cq_mgr_rx_regrq::poll_and_process_element_rx(uint64_t *p_cq_poll_sn, void *pv_fd_ready_array)
 {
     cq_logfuncall("");
 
@@ -294,7 +272,7 @@ bool cq_mgr_rx_regrq::poll_and_process_element_rx(uint64_t *p_cq_poll_sn, void *
                     process_recv_buffer(buff, pv_fd_ready_array);
                 }
             } else {
-                m_p_cq_stat->n_rx_pkt_drop++;
+                m_p_cq_stat->n_rx_sw_pkt_drops++;
                 if (++m_debt >= (int)m_n_sysvar_rx_num_wr_to_post_recv) {
                     compensate_qp_poll_failed();
                 }
@@ -308,11 +286,11 @@ bool cq_mgr_rx_regrq::poll_and_process_element_rx(uint64_t *p_cq_poll_sn, void *
 
     if (likely(rx_polled > 0)) {
         m_p_ring->m_gro_mgr.flush_all(pv_fd_ready_array);
-    } else {
-        compensate_qp_poll_failed();
+        return static_cast<int>(m_n_sysvar_cq_poll_batch_max - rx_polled);
     }
 
-    return (rx_polled < m_n_sysvar_cq_poll_batch_max);
+    compensate_qp_poll_failed();
+    return -1;
 }
 
 #endif /* DEFINED_DIRECT_VERBS */
