@@ -44,16 +44,19 @@ cq_mgr_rx_strq::cq_mgr_rx_strq(ring_simple *p_ring, ib_ctx_handler *p_ib_ctx_han
     return_stride(next_stride()); // Fill _stride_cache
 }
 
+/**
+ * @brief Destructor for cq_mgr_rx_strq class
+ *
+ * Cleans up all resources associated with the CQ STRQ
+ *
+ * @note The coverity[UNCAUGHT_EXCEPT] was added as it's a False Positive
+ */
+// coverity[UNCAUGHT_EXCEPT]
 cq_mgr_rx_strq::~cq_mgr_rx_strq()
 {
     cq_logfunc("");
     cq_logdbg("destroying CQ STRQ");
 
-    if (m_rx_buffs_rdy_for_free_head) {
-        reclaim_recv_buffer_helper(m_rx_buffs_rdy_for_free_head);
-        m_rx_buffs_rdy_for_free_head = m_rx_buffs_rdy_for_free_tail = nullptr;
-    }
-
     if (m_rx_queue.size()) {
         cq_logdbg("Clearing %zu stride objects)", m_rx_queue.size());
 
@@ -224,7 +227,9 @@ inline bool cq_mgr_rx_strq::strq_cqe_to_mem_buff_desc(struct xlio_mlx5_cqe *cqe,
         _current_wqe_consumed_bytes += _hot_buffer_stride->sz_buffer;
 
         _hot_buffer_stride->rx.timestamps.hw_raw = ntohll(cqe->timestamp);
-        _hot_buffer_stride->rx.flow_tag_id = ntohl((uint32_t)(cqe->sop_drop_qpn));
+        uint32_t sop_rxdrop_qpn_flowtag_h_byte = ntohl(cqe->sop_rxdrop_qpn_flowtag);
+        _hot_buffer_stride->rx.flow_tag_id = sop_rxdrop_qpn_flowtag_h_byte & 0x00FFFFFF;
+        m_p_cq_stat->n_rx_hw_pkt_drops += sop_rxdrop_qpn_flowtag_h_byte >> 24;
         _hot_buffer_stride->rx.is_sw_csum_need =
             !(m_b_is_rx_hw_csum_on && (cqe->hds_ip_ext & MLX5_CQE_L4_OK) &&
               (cqe->hds_ip_ext & MLX5_CQE_L3_OK));
@@ -320,14 +325,13 @@ int cq_mgr_rx_strq::drain_and_proccess_helper(mem_buf_desc_t *buff, mem_buf_desc
         ++ret_total;
         if (process_strq_cq_element_rx(buff, status)) {
             if (p_recycle_buffers_last_wr_id) {
-                m_p_cq_stat->n_rx_pkt_drop++;
+                m_p_cq_stat->n_rx_sw_pkt_drops++;
                 reclaim_recv_buffer_helper(buff);
             } else {
                 bool procces_now = is_eth_tcp_frame(buff);
 
                 // We process immediately all non udp/ip traffic..
                 if (procces_now) {
-                    buff->rx.is_xlio_thr = true;
                     process_recv_buffer(buff, nullptr);
                 } else { // udp/ip traffic we just put in the cq's rx queue
                     m_rx_queue.push_back(buff);
@@ -391,11 +395,6 @@ mem_buf_desc_t *cq_mgr_rx_strq::process_strq_cq_element_rx(mem_buf_desc_t *p_mem
     /* Assume locked!!! */
     cq_logfuncall("");
 
-    /* we use context to verify that on reclaim rx buffer path we return the buffer to the right CQ
-     */
-    p_mem_buf_desc->rx.is_xlio_thr = false;
-    p_mem_buf_desc->rx.context = nullptr;
-
     if (unlikely(status != BS_OK)) {
         reclaim_recv_buffer_helper(p_mem_buf_desc);
         return nullptr;
@@ -410,20 +409,7 @@ mem_buf_desc_t *cq_mgr_rx_strq::process_strq_cq_element_rx(mem_buf_desc_t *p_mem
     return p_mem_buf_desc;
 }
 
-mem_buf_desc_t *cq_mgr_rx_strq::poll_and_process_socketxtreme()
-{
-    buff_status_e status = BS_OK;
-    mem_buf_desc_t *buff = nullptr;
-    mem_buf_desc_t *buff_wqe = poll(status, buff);
-
-    if ((buff_wqe && (++m_debt >= (int)m_n_sysvar_rx_num_wr_to_post_recv)) || !buff) {
-        compensate_qp_poll_failed(); // Reuse this method as success.
-    }
-
-    return (buff && cqe_process_rx(buff, status) ? buff : nullptr);
-}
-
-bool cq_mgr_rx_strq::poll_and_process_element_rx(uint64_t *p_cq_poll_sn, void *pv_fd_ready_array)
+int cq_mgr_rx_strq::poll_and_process_element_rx(uint64_t *p_cq_poll_sn, void *pv_fd_ready_array)
 {
     cq_logfuncall("");
 
@@ -461,11 +447,11 @@ bool cq_mgr_rx_strq::poll_and_process_element_rx(uint64_t *p_cq_poll_sn, void *p
 
     if (likely(rx_polled > 0)) {
         m_p_ring->m_gro_mgr.flush_all(pv_fd_ready_array);
-    } else {
-        compensate_qp_poll_failed();
+        return static_cast<int>(m_n_sysvar_cq_poll_batch_max - rx_polled);
     }
 
-    return (rx_polled < m_n_sysvar_cq_poll_batch_max);
+    compensate_qp_poll_failed();
+    return -1;
 }
 
 void cq_mgr_rx_strq::add_hqrx(hw_queue_rx *hqrx)
