@@ -47,7 +47,7 @@ net_device_table_mgr::net_device_table_mgr()
     m_num_devices = 0;
     m_global_ring_epfd = 0;
     m_max_mtu = 0;
-
+    m_closed_rings_rx_cq_drop_counter = 0;
     ndtm_logdbg("");
 
     m_global_ring_epfd = SYSCALL(epoll_create, 48);
@@ -98,7 +98,6 @@ net_device_table_mgr::net_device_table_mgr()
     g_p_netlink_handler->register_event(nlgrpLINK, this);
     ndtm_logdbg("Registered to g_p_netlink_handler");
 
-#ifndef DEFINED_NO_THREAD_LOCK
     if (safe_mce_sys().progress_engine_interval_msec != MCE_CQ_DRAIN_INTERVAL_DISABLED &&
         safe_mce_sys().progress_engine_wce_max != 0) {
         ndtm_logdbg("registering timer for ring draining with %d msec intervales",
@@ -115,7 +114,6 @@ net_device_table_mgr::net_device_table_mgr()
                                                         PERIODIC_TIMER,
                                                         (void *)RING_ADAPT_CQ_MODERATION_TIMER);
     }
-#endif // DEFINED_NO_THREAD_LOCK
 
     ndtm_logdbg("Done");
 }
@@ -222,7 +220,7 @@ void net_device_table_mgr::update_tbl()
                 /* Add new interfaces */
                 switch (nl_msgdata->ifi_type) {
                 case ARPHRD_ETHER:
-                    p_net_device_val = new net_device_val_eth(&desc);
+                    p_net_device_val = new net_device_val(&desc);
                     break;
                 default:
                     goto next;
@@ -247,7 +245,14 @@ void net_device_table_mgr::update_tbl()
                                                sa_family_t family) {
                     auto it = net_device_map.find(ip->local_addr);
                     if (it != net_device_map.end()) {
-                        duplicate_ips.insert(ip->local_addr.to_str(family));
+                        // Exclude IPv6 link-local addresses from duplicate IP warnings.
+                        // Logical interfaces can legitimately have duplicate IPv6 link-local
+                        // addresses. The warning is not relevant because: (1) users need to specify
+                        // interface along with link-local address, and (2) XLIO doesn't support
+                        // IPv6 link-local at the moment RM #4544719.
+                        if (!(family == AF_INET6 && ip->local_addr.is_linklocal(family))) {
+                            duplicate_ips.insert(ip->local_addr.to_str(family));
+                        }
                     }
                     if (it == net_device_map.end() ||
                         p_net_device_val->get_state() != net_device_val::DOWN) {
@@ -347,25 +352,6 @@ net_device_val *net_device_table_mgr::get_net_device_val(int if_index)
                 goto out;
             }
         }
-        /* Check if interface is new netvsc slave */
-        if (net_dev->get_is_bond() == net_device_val::NETVSC) {
-            char if_name[IFNAMSIZ] = {0};
-            char sys_path[256] = {0};
-            int ret = 0;
-            if (if_indextoname(if_index, if_name)) {
-                ret = snprintf(sys_path, sizeof(sys_path), NETVSC_DEVICE_UPPER_FILE, if_name,
-                               net_dev->get_ifname());
-                if (ret > 0 && (size_t)ret < sizeof(sys_path)) {
-                    ret = errno; /* to suppress errno */
-                    int fd = SYSCALL(open, sys_path, O_RDONLY);
-                    if (fd >= 0) {
-                        SYSCALL(close, fd);
-                        goto out;
-                    }
-                    errno = ret;
-                }
-            }
-        }
     }
 
     ndtm_logdbg("Can't find net_device for index: %d", if_index);
@@ -457,6 +443,29 @@ int net_device_table_mgr::global_ring_epfd_get()
     return m_global_ring_epfd;
 }
 
+uint64_t net_device_table_mgr::get_rx_drop_counter()
+{
+    // coverity[missing_lock:FALSE] /*Turn off coverity missing_lock check*/
+    uint64_t accumulator = m_closed_rings_rx_cq_drop_counter;
+    std::for_each(m_net_device_map_index.begin(), m_net_device_map_index.end(),
+                  [&accumulator](const auto &net_dev_map_iter) {
+                      accumulator += net_dev_map_iter.second->get_accumulative_rx_cq_drop_counter();
+                  });
+    return accumulator;
+}
+
+void net_device_table_mgr::print_report(vlog_levels_t log_level,
+                                        bool print_only_critical /*=false*/)
+{
+    uint64_t accumulator = get_rx_drop_counter();
+    if (print_only_critical && !accumulator) {
+        return;
+    }
+    vlog_printf(log_level, "*********************************\n");
+    vlog_printf(log_level, "Total HW RX drop counter: %lu\n", accumulator);
+    vlog_printf(log_level, "*********************************\n");
+}
+
 void net_device_table_mgr::global_ring_wait_for_notification_and_process_element(
     uint64_t *p_poll_sn, void *pv_fd_ready_array /*=NULL*/)
 {
@@ -568,6 +577,7 @@ void net_device_table_mgr::get_net_devices(local_dev_vector &vec)
 void net_device_table_mgr::del_link_event(const netlink_link_info *info)
 {
     ndtm_logdbg("netlink event: RTM_DELLINK if_index: %d", info->ifindex);
+    NOT_IN_USE(info);
 
     /* This flow is actual when interface is removed quickly
      * w/o moving it in DOWN state.
@@ -575,48 +585,17 @@ void net_device_table_mgr::del_link_event(const netlink_link_info *info)
      * that puts it in DOWN state. In this case XLIO has more time to release
      * resources correctly.
      */
-    if (info->flags & IFF_SLAVE) {
-        net_device_val *net_dev = nullptr;
-        int if_index = info->ifindex;
-
-        ndtm_logdbg("netlink event: if_index: %d state: %s", info->ifindex,
-                    (info->flags & IFF_RUNNING ? "Up" : "Down"));
-
-        net_dev = get_net_device_val(if_index);
-        if (net_dev && (if_index != net_dev->get_if_idx()) &&
-            (net_dev->get_is_bond() == net_device_val::NETVSC) && (net_dev->get_slave(if_index))) {
-            ndtm_logdbg("found entry [%p]: if_index: %d : %s", net_dev, net_dev->get_if_idx(),
-                        net_dev->get_ifname());
-            net_dev->update_netvsc_slaves(info->ifindex, info->flags);
-        }
-    }
 }
 
 void net_device_table_mgr::new_link_event(const netlink_link_info *info)
 {
     ndtm_logdbg("netlink event: RTM_NEWLINK if_index: %d", info->ifindex);
+    NOT_IN_USE(info);
 
     /* This flow is used to process interface UP and DOWN scenarios.
      * It is important that interface can be removed w/o putting it into
      * DOWN state (see RTM_DELLINK).
      */
-    if (info->flags & IFF_SLAVE) {
-        net_device_val *net_dev = nullptr;
-        int if_index = info->ifindex;
-
-        ndtm_logdbg("netlink event: if_index: %d state: %s", info->ifindex,
-                    (info->flags & IFF_RUNNING ? "Up" : "Down"));
-
-        net_dev = get_net_device_val(if_index);
-        if (net_dev && (if_index != net_dev->get_if_idx()) &&
-            (net_dev->get_is_bond() == net_device_val::NETVSC) &&
-            ((net_dev->get_slave(if_index) && !(info->flags & IFF_RUNNING)) ||
-             (!net_dev->get_slave(if_index) && (info->flags & IFF_RUNNING)))) {
-            ndtm_logdbg("found entry [%p]: if_index: %d : %s", net_dev, net_dev->get_if_idx(),
-                        net_dev->get_ifname());
-            net_dev->update_netvsc_slaves(info->ifindex, info->flags);
-        }
-    }
 }
 
 void net_device_table_mgr::notify_cb(event *ev)
