@@ -72,6 +72,13 @@ public:
 
     int global_ring_epfd_get();
 
+    /*
+     * This will get accumlated RX out of buffer drops
+     * for all net devices.
+     */
+    uint64_t get_rx_drop_counter(void);
+    void print_report(vlog_levels_t log_level, bool print_only_critical = false);
+
     void handle_timer_expired(void *user_data);
 
     uint32_t get_max_mtu() const { return m_max_mtu; }
@@ -80,6 +87,13 @@ public:
 
     void get_net_devices(local_dev_vector &vec);
 
+    void increase_closed_rings_rx_cq_drop_counter(uint64_t count)
+    {
+        m_closed_rings_rx_cq_drop_counter_lock.lock();
+        m_closed_rings_rx_cq_drop_counter += count;
+        m_closed_rings_rx_cq_drop_counter_lock.unlock();
+    }
+
 private:
     void del_link_event(const netlink_link_info *info);
     void new_link_event(const netlink_link_info *info);
@@ -98,6 +112,8 @@ private:
     int m_global_ring_pipe_fds[2];
 
     uint32_t m_max_mtu;
+    lock_mutex m_closed_rings_rx_cq_drop_counter_lock;
+    uint64_t m_closed_rings_rx_cq_drop_counter;
 };
 
 extern net_device_table_mgr *g_p_net_device_table_mgr;
