@@ -26,7 +26,6 @@
 #include "event/event_handler_manager.h"
 #include "proto/L2_address.h"
 #include "dev/ib_ctx_handler_collection.h"
-#include "dev/ring_tap.h"
 #include "dev/ring_simple.h"
 #include "dev/ring_slave.h"
 #include "dev/ring_bond.h"
@@ -202,8 +201,6 @@ net_device_val::net_device_val(struct net_device_val_desc *desc)
     /* Identify device type */
     if ((get_flags() & IFF_MASTER) || check_bond_device_exist(get_ifname_link())) {
         verify_bonding_mode();
-    } else if (check_netvsc_device_exist(get_ifname_link())) {
-        m_bond = NETVSC;
     } else {
         m_bond = NO_BOND;
     }
@@ -213,16 +210,6 @@ net_device_val::net_device_val(struct net_device_val_desc *desc)
     valid = false;
     ib_ctx = g_p_ib_ctx_handler_collection->get_ib_ctx(get_ifname_link());
     switch (m_bond) {
-    case NETVSC:
-        if (get_type() == ARPHRD_ETHER) {
-            char slave_ifname[IFNAMSIZ] = {0};
-            unsigned int slave_flags = 0;
-            /* valid = true; uncomment it is valid flow to operate w/o SRIOV */
-            if (get_netvsc_slave(get_ifname_link(), slave_ifname, slave_flags)) {
-                valid = verify_qp_creation(slave_ifname, IBV_QPT_RAW_PACKET);
-            }
-        }
-        break;
     case LAG_8023ad:
     case ACTIVE_BACKUP:
         // this is a bond interface (or a vlan/alias over bond), find the slaves
@@ -269,6 +256,12 @@ net_device_val::net_device_val(struct net_device_val_desc *desc)
     } else {
         nd_logdbg("%s ==> none", get_ifname());
     }
+
+    set_transport_type(XLIO_TRANSPORT_ETH);
+    if (INVALID != get_state()) {
+        set_slave_array();
+        configure();
+    }
 }
 
 net_device_val::~net_device_val()
@@ -491,9 +484,6 @@ const std::string net_device_val::to_str_ex() const
 
     rc += " (";
     switch (m_bond) {
-    case NETVSC:
-        rc += "netvsc";
-        break;
     case LAG_8023ad:
         rc += "lag 8023ad";
         break;
@@ -552,20 +542,11 @@ void net_device_val::set_slave_array()
 
     nd_logdbg("");
 
-    if (m_bond == NETVSC) {
-        slave_data_t *s = nullptr;
-        unsigned int slave_flags = 0;
-        if (get_netvsc_slave(get_ifname_link(), active_slave, slave_flags)) {
-            if ((slave_flags & IFF_UP) && verify_qp_creation(active_slave, IBV_QPT_RAW_PACKET)) {
-                s = new slave_data_t(if_nametoindex(active_slave));
-                m_slaves.push_back(s);
-            }
-        }
-    } else if (m_bond == NO_BOND) {
+    if (m_bond == NO_BOND) {
         slave_data_t *s = new slave_data_t(if_nametoindex(get_ifname()));
         m_slaves.push_back(s);
     } else {
-        // bond device
+        // bond devices
 
         // get list of all slave devices
         char slaves_list[IFNAMSIZ * MAX_SLAVES] = {0};
@@ -624,10 +605,6 @@ void net_device_val::set_slave_array()
             }
         }
 
-        if (m_bond == NETVSC) {
-            m_slaves[i]->active = true;
-        }
-
         if (m_bond == NO_BOND) {
             m_slaves[i]->active = true;
         }
@@ -650,7 +627,7 @@ void net_device_val::set_slave_array()
         }
     }
 
-    if (m_slaves.empty() && NETVSC != m_bond) {
+    if (m_slaves.empty()) {
         m_state = INVALID;
         nd_logpanic("No slave found.");
     }
@@ -905,71 +882,15 @@ bool net_device_val::update_active_slaves()
     }
     return 0;
 }
-
-void net_device_val::update_netvsc_slaves(int if_index, int if_flags)
+uint64_t net_device_val::get_accumulative_rx_cq_drop_counter()
 {
-    slave_data_t *s = nullptr;
-    bool found = false;
-    ib_ctx_handler *ib_ctx = nullptr, *up_ib_ctx = nullptr;
-    char if_name[IFNAMSIZ] = {0};
-
-    m_lock.lock();
-
-    if (if_indextoname(if_index, if_name) && (if_flags & IFF_UP) && (if_flags & IFF_RUNNING)) {
-        nd_logdbg("slave %d is up", if_index);
-
-        g_p_ib_ctx_handler_collection->update_tbl(if_name);
-        if ((up_ib_ctx = g_p_ib_ctx_handler_collection->get_ib_ctx(if_name))) {
-            s = new slave_data_t(if_index);
-            s->active = true;
-            s->p_ib_ctx = up_ib_ctx;
-            s->p_L2_addr = create_L2_address(if_name);
-            s->port_num = get_port_from_ifname(if_name);
-            m_slaves.push_back(s);
-
-            up_ib_ctx->set_ctx_time_converter_status(
-                g_p_net_device_table_mgr->get_ctx_time_conversion_mode());
-            g_buffer_pool_rx_rwqe->register_memory(s->p_ib_ctx);
-            g_buffer_pool_tx->register_memory(s->p_ib_ctx);
-            found = true;
-        }
-    } else {
-        if (!m_slaves.empty()) {
-            s = m_slaves.back();
-            m_slaves.pop_back();
-
-            nd_logdbg("slave %d is down ", s->if_index);
-
-            ib_ctx = s->p_ib_ctx;
-            delete s;
-            found = true;
-        }
-    }
-
-    m_lock.unlock();
-
-    if (!found) {
-        nd_logdbg("Unable to detect any changes for interface %d. ignoring", if_index);
-        return;
-    }
-
-    /* restart if status changed */
-    m_p_L2_addr = create_L2_address(get_ifname());
-    // restart rings
-    rings_hash_map_t::iterator ring_iter;
-    for (ring_iter = m_h_ring_map.begin(); ring_iter != m_h_ring_map.end(); ring_iter++) {
-        THE_RING->restart();
-    }
-
-    if (ib_ctx) {
-        g_p_ib_ctx_handler_collection->del_ib_ctx(ib_ctx);
-    }
-}
-
-const std::string net_device_val::to_str() const
-{
-    return std::string("Net Device: " + m_name);
-}
+    uint64_t accumaltor = 0;
+    std::for_each(m_h_ring_map.begin(), m_h_ring_map.end(),
+                  [&accumaltor](const auto &m_h_ring_map_iter) {
+                      accumaltor += m_h_ring_map_iter.second.first->get_rx_cq_out_of_buffer_drop();
+                  });
+    return accumaltor;
+};
 
 ring *net_device_val::reserve_ring(resource_allocation_key *key)
 {
@@ -1006,10 +927,6 @@ ring *net_device_val::reserve_ring(resource_allocation_key *key)
             BULLSEYE_EXCLUDE_BLOCK_END
         }
 
-        if (key->get_ring_alloc_logic() == RING_LOGIC_ISOLATE) {
-            // Keep isolated rings until termination. Destructor will delete the ring.
-            ADD_RING_REF_CNT;
-        }
         g_p_net_device_table_mgr->global_ring_wakeup();
     }
 
@@ -1153,14 +1070,17 @@ bool net_device_val::global_ring_poll_and_process_element(uint64_t *p_poll_sn_rx
                                                           void *pv_fd_ready_array /*=NULL*/)
 {
     nd_logfuncall("");
-    bool all_drained = true;
+    int all_drained = 1;
     std::lock_guard<decltype(m_lock)> lock(m_lock);
     rings_hash_map_t::iterator ring_iter;
     for (ring_iter = m_h_ring_map.begin(); ring_iter != m_h_ring_map.end(); ring_iter++) {
-        all_drained &= THE_RING->poll_and_process_element_rx(p_poll_sn_rx, pv_fd_ready_array);
-        all_drained &= (THE_RING->poll_and_process_element_tx(p_poll_sn_tx) >= 0);
+        all_drained = std::min(
+            all_drained,
+            std::abs(THE_RING->poll_and_process_element_rx(p_poll_sn_rx, pv_fd_ready_array)));
+        all_drained =
+            std::min(all_drained, std::abs(THE_RING->poll_and_process_element_tx(p_poll_sn_tx)));
     }
-    return all_drained;
+    return !!all_drained;
 }
 
 int net_device_val::global_ring_request_notification(uint64_t poll_sn_rx, uint64_t poll_sn_tx)
@@ -1201,6 +1121,9 @@ int net_device_val::ring_drain_and_proccess()
     std::lock_guard<decltype(m_lock)> lock(m_lock);
     rings_hash_map_t::iterator ring_iter;
     for (ring_iter = m_h_ring_map.begin(); ring_iter != m_h_ring_map.end(); ring_iter++) {
+        if (THE_RING->is_ultra_ring()) {
+            continue;
+        }
         int ret = THE_RING->drain_and_proccess();
         if (ret < 0) {
             return ret;
@@ -1220,6 +1143,9 @@ void net_device_val::ring_adapt_cq_moderation()
     std::lock_guard<decltype(m_lock)> lock(m_lock);
     rings_hash_map_t::iterator ring_iter;
     for (ring_iter = m_h_ring_map.begin(); ring_iter != m_h_ring_map.end(); ring_iter++) {
+        if (THE_RING->is_ultra_ring()) {
+            continue;
+        }
         THE_RING->adapt_cq_moderation();
     }
 }
@@ -1265,7 +1191,7 @@ void net_device_val::unregister_to_ibverbs_events(event_handler_ibverbs *handler
     }
 }
 
-void net_device_val_eth::configure()
+void net_device_val::configure()
 {
     m_p_L2_addr = create_L2_address(get_ifname());
 
@@ -1311,7 +1237,7 @@ uint32_t net_device_val::get_priority_by_tc_class(uint32_t tc_class)
     return it->second;
 }
 
-void net_device_val_eth::parse_prio_egress_map()
+void net_device_val::parse_prio_egress_map()
 {
     int len, ret;
     nl_cache *cache = nullptr;
@@ -1356,35 +1282,31 @@ out:
     }
 }
 
-ring *net_device_val_eth::create_ring(resource_allocation_key *key)
+ring *net_device_val::create_ring(resource_allocation_key *key)
 {
     ring *ring = nullptr;
 
     try {
         switch (m_bond) {
         case NO_BOND:
-            ring = new ring_eth(get_if_idx(), nullptr, RING_ETH, true,
-                                (key ? key->get_use_locks() : true));
+            ring = new ring_simple(get_if_idx(), nullptr, (key ? key->get_use_locks() : true));
             break;
         case ACTIVE_BACKUP:
         case LAG_8023ad:
-            ring = new ring_bond_eth(get_if_idx());
-            break;
-        case NETVSC:
-            ring = new ring_bond_netvsc(get_if_idx());
+            ring = new ring_bond(get_if_idx());
             break;
         default:
             nd_logdbg("Unknown ring type");
             break;
         }
     } catch (xlio_error &error) {
-        nd_logdbg("failed creating ring %s", error.message);
+        nd_logdbg("failed creating ring %s", error.message.c_str());
     }
 
     return ring;
 }
 
-L2_address *net_device_val_eth::create_L2_address(const char *ifname)
+L2_address *net_device_val::create_L2_address(const char *ifname)
 {
     if (m_p_L2_addr) {
         delete m_p_L2_addr;
@@ -1395,7 +1317,7 @@ L2_address *net_device_val_eth::create_L2_address(const char *ifname)
     return new ETH_addr(hw_addr);
 }
 
-void net_device_val_eth::create_br_address(const char *ifname)
+void net_device_val::create_br_address(const char *ifname)
 {
     if (m_p_br_addr) {
         delete m_p_br_addr;
@@ -1411,9 +1333,9 @@ void net_device_val_eth::create_br_address(const char *ifname)
     }
     BULLSEYE_EXCLUDE_BLOCK_END
 }
-const std::string net_device_val_eth::to_str() const
+const std::string net_device_val::to_str() const
 {
-    return std::string("ETH: " + net_device_val::to_str());
+    return std::string("ETH Net Device: " + m_name);
 }
 
 bool net_device_val::verify_bond_or_eth_qp_creation()
@@ -1479,8 +1401,7 @@ bool net_device_val::verify_bond_or_eth_qp_creation()
                                           guid_iter->second.front().c_str()) &&
                 check_bond_roce_lag_exist(bond_roce_lag_path, sizeof(bond_roce_lag_path),
                                           guid_iter->second.back().c_str())) {
-                print_roce_lag_warnings(get_ifname_link(), bond_roce_lag_path,
-                                        guid_iter->second.front().c_str(),
+                print_roce_lag_warnings(get_ifname_link(), guid_iter->second.front().c_str(),
                                         guid_iter->second.back().c_str());
             }
         }
@@ -1532,7 +1453,7 @@ bool net_device_val::verify_qp_creation(const char *ifname, enum ibv_qp_type qp_
         nd_logdbg("Cant find ib_ctx for interface %s", base_ifname);
         if (qp_type == IBV_QPT_RAW_PACKET && m_bond != NO_BOND) {
             if (check_bond_roce_lag_exist(bond_roce_lag_path, sizeof(bond_roce_lag_path), ifname)) {
-                print_roce_lag_warnings(get_ifname_link(), bond_roce_lag_path);
+                print_roce_lag_warnings(get_ifname_link());
             } else if ((p_ib_ctx = g_p_ib_ctx_handler_collection->get_ib_ctx(get_ifname_link())) &&
                        strstr(p_ib_ctx->get_ibname(), "bond")) {
                 print_roce_lag_warnings(get_ifname_link());
