@@ -36,8 +36,9 @@ inline bool ipv6_check(const struct ip6_hdr &p_ip6_h)
 }
 
 rfs_uc_tcp_gro::rfs_uc_tcp_gro(flow_tuple *flow_spec_5t, ring_slave *p_ring,
-                               rfs_rule_filter *rule_filter, uint32_t flow_tag_id)
-    : rfs_uc(flow_spec_5t, p_ring, rule_filter, flow_tag_id)
+                               rfs_rule_filter *rule_filter, uint32_t flow_tag_id,
+                               int steering_index)
+    : rfs_uc(flow_spec_5t, p_ring, rule_filter, flow_tag_id, steering_index)
     , m_b_active(false)
     , m_b_reserved(false)
     , m_pcb(nullptr)
@@ -212,7 +213,6 @@ void rfs_uc_tcp_gro::flush_gro_desc(void *pv_fd_ready_array)
         m_gro_desc.p_first->lwip_pbuf.ref = 1;
         m_gro_desc.p_first->lwip_pbuf.payload =
             (u8_t *)(m_gro_desc.p_first->p_buffer + m_gro_desc.p_first->rx.n_transport_header_len);
-        m_gro_desc.p_first->rx.is_xlio_thr = m_gro_desc.p_last->rx.is_xlio_thr;
 
         for (mem_buf_desc_t *p_desc = m_gro_desc.p_last; p_desc != m_gro_desc.p_first;
              p_desc = p_desc->p_prev_desc) {
