@@ -42,7 +42,8 @@ class gro_mgr;
 class rfs_uc_tcp_gro : public rfs_uc {
 public:
     rfs_uc_tcp_gro(flow_tuple *flow_spec_5t, ring_slave *p_ring,
-                   rfs_rule_filter *rule_filter = nullptr, uint32_t flow_tag_id = 0);
+                   rfs_rule_filter *rule_filter = nullptr, uint32_t flow_tag_id = 0,
+                   int steering_index = -1);
 
     virtual bool rx_dispatch_packet(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd_ready_array);
 
