@@ -51,9 +51,6 @@ uint64_t ring_allocation_logic::calc_res_key_by_logic()
     switch (m_res_key.get_ring_alloc_logic()) {
     case RING_LOGIC_PER_INTERFACE:
         res_key = 0;
-        if (tcp_ctl_thread_on(safe_mce_sys().tcp_ctl_thread)) {
-            res_key = 1;
-        }
         break;
     case RING_LOGIC_PER_IP:
         res_key = m_source.m_ip.hash();
@@ -75,9 +72,6 @@ uint64_t ring_allocation_logic::calc_res_key_by_logic()
     case RING_LOGIC_PER_OBJECT:
         res_key = reinterpret_cast<uint64_t>(m_source.m_object);
         break;
-    case RING_LOGIC_ISOLATE:
-        res_key = 0;
-        break;
     default:
         // not suppose to get here
         ral_logdbg("Non-valid ring logic = %d", m_res_key.get_ring_alloc_logic());
