@@ -5,10 +5,8 @@
  */
 
 #include "ring_bond.h"
-
 #include "sock/sockinfo.h"
 #include "dev/ring_simple.h"
-#include "dev/ring_tap.h"
 
 #undef MODULE_NAME
 #define MODULE_NAME "ring_bond"
@@ -45,6 +43,12 @@ ring_bond::ring_bond(int if_index)
     m_max_send_sge = 0;
 
     print_val();
+
+    const slave_data_vector_t &slaves = p_ndev->get_slave_array();
+    update_cap();
+    for (size_t i = 0; i < slaves.size(); i++) {
+        slave_create(slaves[i]->if_index);
+    }
 }
 
 ring_bond::~ring_bond()
@@ -91,7 +95,7 @@ bool ring_bond::attach_flow(flow_tuple &flow_spec_5t, sockinfo *sink, bool force
     return ret;
 }
 
-bool ring_bond::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink)
+bool ring_bond::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink, rfs_rule **rule_extract)
 {
     bool ret = true;
     struct flow_sink_t value = {flow_spec_5t, sink};
@@ -108,7 +112,7 @@ bool ring_bond::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink)
     }
 
     for (uint32_t i = 0; i < m_recv_rings.size(); i++) {
-        bool step_ret = m_recv_rings[i]->detach_flow(flow_spec_5t, sink);
+        bool step_ret = m_recv_rings[i]->detach_flow(flow_spec_5t, sink, rule_extract);
         ret = ret && step_ret;
     }
 
@@ -129,191 +133,80 @@ void ring_bond::restart()
     m_lock_ring_rx.lock();
     m_lock_ring_tx.lock();
 
-    if (p_ndev->get_is_bond() == net_device_val::NETVSC) {
-        ring_bond_netvsc *p_ring_bond_netvsc = dynamic_cast<ring_bond_netvsc *>(this);
-        if (p_ring_bond_netvsc) {
-            ring_tap *p_ring_tap = dynamic_cast<ring_tap *>(p_ring_bond_netvsc->m_tap_ring);
-            if (p_ring_tap) {
-                size_t num_ring_rx_fds = 0;
-                int *ring_rx_fds_array = nullptr;
-                int epfd = -1;
-                int fd = -1;
-                int rc = 0;
-                size_t i, j, k;
+    /* for active-backup mode
+     * It is guaranteed that the first slave is active by popup_active_rings()
+     */
+    ring_simple *previously_active = dynamic_cast<ring_simple *>(m_xmit_rings[0]);
 
-                if (slaves.empty()) {
-                    ring_rx_fds_array =
-                        p_ring_bond_netvsc->m_vf_ring->get_rx_channel_fds(num_ring_rx_fds);
+    for (uint32_t i = 0; i < m_bond_rings.size(); i++) {
+        ring_simple *tmp_ring = dynamic_cast<ring_simple *>(m_bond_rings[i]);
 
-                    for (k = 0; k < num_ring_rx_fds; k++) {
-                        epfd = g_p_net_device_table_mgr->global_ring_epfd_get();
-                        if (epfd > 0) {
-                            fd = ring_rx_fds_array[k];
-                            rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_DEL, fd, nullptr);
-                            ring_logdbg("Remove fd=%d from epfd=%d rc=%d errno=%d", fd, epfd, rc,
-                                        errno);
-                        }
-                    }
-                    for (j = 0; j < m_rx_flows.size(); j++) {
-                        sockinfo *si = static_cast<sockinfo *>(m_rx_flows[j].sink);
-                        for (k = 0; k < num_ring_rx_fds; k++) {
-                            epfd = si->get_rx_epfd();
-                            if (epfd > 0) {
-                                fd = ring_rx_fds_array[k];
-                                rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_DEL, fd, NULL);
-                                ring_logdbg("Remove fd=%d from epfd=%d rc=%d errno=%d", fd, epfd,
-                                            rc, errno);
-                            }
-                            epfd = si->get_epoll_context_fd();
-                            if (epfd > 0) {
-                                fd = ring_rx_fds_array[k];
-                                rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_DEL, fd, NULL);
-                                ring_logdbg("Remove fd=%d from epfd=%d rc=%d errno=%d", fd, epfd,
-                                            rc, errno);
-                            }
-                        }
-                    }
-
-                    p_ring_tap->m_active = true;
-                    p_ring_tap->inc_vf_plugouts();
-                    p_ring_bond_netvsc->slave_destroy(
-                        p_ring_bond_netvsc->m_vf_ring->get_if_index());
-                    p_ring_bond_netvsc->m_vf_ring = nullptr;
-                    p_ring_tap->set_vf_ring(nullptr);
-                } else {
-                    for (i = 0; i < slaves.size(); i++) {
-                        if (slaves[i]->if_index != p_ring_tap->get_if_index()) {
-                            p_ring_tap->m_active = false;
-                            slave_create(slaves[i]->if_index);
-                            p_ring_tap->set_vf_ring(p_ring_bond_netvsc->m_vf_ring);
-
-                            ring_rx_fds_array =
-                                p_ring_bond_netvsc->m_vf_ring->get_rx_channel_fds(num_ring_rx_fds);
-
-                            for (k = 0; k < num_ring_rx_fds; k++) {
-                                epfd = g_p_net_device_table_mgr->global_ring_epfd_get();
-                                if (epfd > 0) {
-                                    epoll_event ev = {0, {nullptr}};
-                                    fd = ring_rx_fds_array[k];
-                                    ev.events = EPOLLIN;
-                                    ev.data.fd = fd;
-                                    rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_ADD, fd, &ev);
-                                    ring_logdbg("Add fd=%d from epfd=%d rc=%d errno=%d", fd, epfd,
-                                                rc, errno);
-                                }
-                            }
-                            for (j = 0; j < m_rx_flows.size(); j++) {
-                                sockinfo *si = static_cast<sockinfo *>(m_rx_flows[j].sink);
-                                p_ring_bond_netvsc->m_vf_ring->attach_flow(m_rx_flows[j].flow,
-                                                                           m_rx_flows[j].sink);
-                                for (k = 0; k < num_ring_rx_fds; k++) {
-                                    epfd = si->get_rx_epfd();
-                                    if (epfd > 0) {
-                                        epoll_event ev = {0, {0}};
-                                        fd = ring_rx_fds_array[k];
-                                        ev.events = EPOLLIN;
-                                        ev.data.fd = fd;
-                                        rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_ADD, fd, &ev);
-                                        ring_logdbg("Add fd=%d from epfd=%d rc=%d errno=%d", fd,
-                                                    epfd, rc, errno);
-                                    }
-                                    epfd = si->get_epoll_context_fd();
-                                    if (epfd > 0) {
-#define CQ_FD_MARK 0xabcd /* see sockinfo */
-                                        epoll_event ev = {0, {0}};
-                                        fd = ring_rx_fds_array[k];
-                                        ev.events = EPOLLIN | EPOLLPRI;
-                                        ev.data.u64 = (((uint64_t)CQ_FD_MARK << 32) | fd);
-                                        rc = SYSCALL(epoll_ctl, epfd, EPOLL_CTL_ADD, fd, &ev);
-                                        ring_logdbg("Add fd=%d from epfd=%d rc=%d errno=%d", fd,
-                                                    epfd, rc, errno);
-                                    }
-                                }
-                            }
-                            break;
-                        }
-                    }
-                }
-                NOT_IN_USE(rc); // Suppress --enable-opt-log=high warning
-            }
+        if (!tmp_ring) {
+            continue;
         }
-    } else {
-        /* for active-backup mode
-         * It is guaranteed that the first slave is active by popup_active_rings()
-         */
-        ring_simple *previously_active = dynamic_cast<ring_simple *>(m_xmit_rings[0]);
 
-        for (uint32_t i = 0; i < m_bond_rings.size(); i++) {
-            ring_simple *tmp_ring = dynamic_cast<ring_simple *>(m_bond_rings[i]);
+        for (uint32_t j = 0; j < slaves.size(); j++) {
 
-            if (!tmp_ring) {
+            if (slaves[j]->if_index != m_bond_rings[i]->get_if_index()) {
                 continue;
             }
 
-            for (uint32_t j = 0; j < slaves.size(); j++) {
-
-                if (slaves[j]->if_index != m_bond_rings[i]->get_if_index()) {
-                    continue;
+            /* For RoCE LAG device income data is processed by single ring only
+             * Consider using ring related slave with lag_tx_port_affinity = 1
+             * even if slave is not active.
+             * Always keep this ring active for RX
+             * but keep common logic for TX
+             */
+            if (slaves[j]->active) {
+                ring_logdbg("ring %d active", i);
+                if (slaves[j]->lag_tx_port_affinity != 1) {
+                    tmp_ring->start_active_queue_tx();
+                    /* coverity[sleep] */
+                    tmp_ring->start_active_queue_rx();
                 }
-
-                /* For RoCE LAG device income data is processed by single ring only
-                 * Consider using ring related slave with lag_tx_port_affinity = 1
-                 * even if slave is not active.
-                 * Always keep this ring active for RX
-                 * but keep common logic for TX
-                 */
-                if (slaves[j]->active) {
-                    ring_logdbg("ring %d active", i);
-                    if (slaves[j]->lag_tx_port_affinity != 1) {
-                        tmp_ring->start_active_queue_tx();
-                        /* coverity[sleep] */
-                        tmp_ring->start_active_queue_rx();
-                    }
-                    m_bond_rings[i]->m_active = true;
-                } else {
-                    ring_logdbg("ring %d not active", i);
-                    if (slaves[j]->lag_tx_port_affinity != 1) {
-                        /* coverity[sleep] */
-                        tmp_ring->stop_active_queue_tx();
-                        /* coverity[sleep] */
-                        tmp_ring->stop_active_queue_rx();
-                    }
-                    m_bond_rings[i]->m_active = false;
+                m_bond_rings[i]->m_active = true;
+            } else {
+                ring_logdbg("ring %d not active", i);
+                if (slaves[j]->lag_tx_port_affinity != 1) {
+                    /* coverity[sleep] */
+                    tmp_ring->stop_active_queue_tx();
+                    /* coverity[sleep] */
+                    tmp_ring->stop_active_queue_rx();
                 }
-                break;
+                m_bond_rings[i]->m_active = false;
             }
+            break;
         }
-        popup_xmit_rings();
+    }
+    popup_xmit_rings();
 
-        int ret = 0;
-        uint64_t poll_sn = cq_mgr_rx::m_n_global_sn_rx;
-        ret = request_notification(CQT_RX, poll_sn);
-        if (ret < 0) {
-            ring_logdbg("failed arming cq_mgr_rx (errno=%d %m)", errno);
-        }
-        ret = request_notification(CQT_TX, poll_sn);
-        if (ret < 0) {
-            ring_logdbg("failed arming cq_mgr_tx (errno=%d %m)", errno);
-        }
+    int ret = 0;
+    uint64_t poll_sn = cq_mgr_rx::m_n_global_sn_rx;
+    ret = request_notification(CQT_RX, poll_sn);
+    if (ret < 0) {
+        ring_logdbg("failed arming cq_mgr_rx (errno=%d %m)", errno);
+    }
+    ret = request_notification(CQT_TX, poll_sn);
+    if (ret < 0) {
+        ring_logdbg("failed arming cq_mgr_tx (errno=%d %m)", errno);
+    }
 
-        if (m_type == net_device_val::ACTIVE_BACKUP) {
-            ring_simple *currently_active = dynamic_cast<ring_simple *>(m_xmit_rings[0]);
-            if (currently_active && safe_mce_sys().cq_moderation_enable) {
-                if (likely(previously_active)) {
-                    currently_active->m_cq_moderation_info.period =
-                        previously_active->m_cq_moderation_info.period;
-                    currently_active->m_cq_moderation_info.count =
-                        previously_active->m_cq_moderation_info.count;
-                } else {
-                    currently_active->m_cq_moderation_info.period =
-                        safe_mce_sys().cq_moderation_period_usec;
-                    currently_active->m_cq_moderation_info.count =
-                        safe_mce_sys().cq_moderation_count;
-                }
-
-                currently_active->modify_cq_moderation(safe_mce_sys().cq_moderation_period_usec,
-                                                       safe_mce_sys().cq_moderation_count);
+    if (m_type == net_device_val::ACTIVE_BACKUP) {
+        ring_simple *currently_active = dynamic_cast<ring_simple *>(m_xmit_rings[0]);
+        if (currently_active && safe_mce_sys().cq_moderation_enable) {
+            if (likely(previously_active)) {
+                currently_active->m_cq_moderation_info.period =
+                    previously_active->m_cq_moderation_info.period;
+                currently_active->m_cq_moderation_info.count =
+                    previously_active->m_cq_moderation_info.count;
+            } else {
+                currently_active->m_cq_moderation_info.period =
+                    safe_mce_sys().cq_moderation_period_usec;
+                currently_active->m_cq_moderation_info.count = safe_mce_sys().cq_moderation_count;
             }
+
+            currently_active->modify_cq_moderation(safe_mce_sys().cq_moderation_period_usec,
+                                                   safe_mce_sys().cq_moderation_count);
         }
     }
 
@@ -339,12 +232,13 @@ void ring_bond::adapt_cq_moderation()
 }
 
 mem_buf_desc_t *ring_bond::mem_buf_tx_get(ring_user_id_t id, bool b_block, pbuf_type type,
-                                          int n_num_mem_bufs /* default = 1 */)
+                                          int n_num_mem_bufs /* default = 1 */,
+                                          bool tx_skip_poll /* default = false */)
 {
     mem_buf_desc_t *ret = nullptr;
 
     std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
-    ret = m_xmit_rings[id]->mem_buf_tx_get(id, b_block, type, n_num_mem_bufs);
+    ret = m_xmit_rings[id]->mem_buf_tx_get(id, b_block, type, n_num_mem_bufs, tx_skip_poll);
 
     return ret;
 }
@@ -436,68 +330,47 @@ int ring_bond::send_lwip_buffer(ring_user_id_t id, xlio_ibv_send_wr *p_send_wqe,
     return -1;
 }
 
-bool ring_bond::get_hw_dummy_send_support(ring_user_id_t id, xlio_ibv_send_wr *p_send_wqe)
-{
-    mem_buf_desc_t *p_mem_buf_desc = (mem_buf_desc_t *)(p_send_wqe->wr_id);
-
-    std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
-
-    if (is_active_member(p_mem_buf_desc->p_desc_owner, id)) {
-        return m_xmit_rings[id]->get_hw_dummy_send_support(id, p_send_wqe);
-    } else {
-        if (likely(p_mem_buf_desc->p_desc_owner == m_bond_rings[id])) {
-            return m_bond_rings[id]->get_hw_dummy_send_support(id, p_send_wqe);
-        }
-    }
-
-    return false;
-}
-
-bool ring_bond::poll_and_process_element_rx(uint64_t *p_cq_poll_sn,
-                                            void *pv_fd_ready_array /*NULL*/)
+int ring_bond::poll_and_process_element_rx(uint64_t *p_cq_poll_sn, void *pv_fd_ready_array /*NULL*/)
 {
     if (m_lock_ring_rx.trylock()) {
         return false;
     }
 
-    bool all_drained = true;
+    int all_drained = 1;
+    int all_empty = -1;
 
     for (uint32_t i = 0; i < m_recv_rings.size(); i++) {
         if (m_recv_rings[i]->is_up()) {
             // TODO consider returning immediately after finding something, continue next time from
             // next ring
-            all_drained &=
-                m_recv_rings[i]->poll_and_process_element_rx(p_cq_poll_sn, pv_fd_ready_array);
+            int rc = m_recv_rings[i]->poll_and_process_element_rx(p_cq_poll_sn, pv_fd_ready_array);
+            all_drained = std::min(all_drained, std::abs(rc));
+            all_empty = std::max(all_empty, std::min(rc, 1));
         }
     }
     m_lock_ring_rx.unlock();
 
-    return all_drained;
+    return all_drained * all_empty;
 }
 
 int ring_bond::poll_and_process_element_tx(uint64_t *p_cq_poll_sn)
 {
     if (m_lock_ring_tx.trylock()) {
-        errno = EAGAIN;
         return 0;
     }
 
-    int temp = 0;
-    int ret = 0;
+    int all_drained = 1;
+    int all_empty = -1;
     for (uint32_t i = 0; i < m_bond_rings.size(); i++) {
         if (m_bond_rings[i]->is_up()) {
-            temp = m_bond_rings[i]->poll_and_process_element_tx(p_cq_poll_sn);
-            if (temp > 0) {
-                ret += temp;
-            }
+            int rc = m_bond_rings[i]->poll_and_process_element_tx(p_cq_poll_sn);
+            all_drained = std::min(all_drained, std::abs(rc));
+            all_empty = std::max(all_empty, std::min(rc, 1));
         }
     }
     m_lock_ring_tx.unlock();
-    if (ret > 0) {
-        return ret;
-    } else {
-        return temp;
-    }
+
+    return all_drained * all_empty;
 }
 
 int ring_bond::drain_and_proccess()
@@ -750,13 +623,8 @@ void ring_bond::popup_recv_rings()
      * - For RoCE LAG device (lag_tx_port_affinity > 0) income data is processed by single ring only
      * Consider using ring related slave with lag_tx_port_affinity = 1
      * even if slave is not active.
-     * - For NETVSC device all rings (vf and tap) should be ready for receive.
      */
     for (uint32_t i = 0; i < m_bond_rings.size(); i++) {
-        if (p_ndev->get_is_bond() == net_device_val::NETVSC) {
-            m_recv_rings.push_back(m_bond_rings[i]);
-            continue;
-        }
         for (uint32_t j = 0; j < slaves.size(); j++) {
             if (slaves[j]->if_index != m_bond_rings[i]->get_if_index()) {
                 continue;
@@ -896,34 +764,20 @@ bool ring_bond::is_tso(void)
     return false;
 }
 
-int ring_bond::socketxtreme_poll(struct xlio_socketxtreme_completion_t *, unsigned int, int)
+uint64_t ring_bond::get_rx_cq_out_of_buffer_drop()
 {
-    return 0;
-}
-
-void ring_bond::slave_destroy(int if_index)
-{
-    ring_slave *cur_slave = nullptr;
-    ring_slave_vector_t::iterator iter;
-
-    for (iter = m_bond_rings.begin(); iter != m_bond_rings.end(); iter++) {
-        cur_slave = *iter;
-        if (cur_slave->get_if_index() == if_index) {
-            delete cur_slave;
-            m_bond_rings.erase(iter);
-            popup_xmit_rings();
-            popup_recv_rings();
-            update_rx_channel_fds();
-            break;
-        }
+    uint64_t accumulator = 0;
+    for (uint32_t i = 0; i < m_recv_rings.size(); i++) {
+        accumulator += m_recv_rings[i]->get_rx_cq_out_of_buffer_drop();
     }
+    return accumulator;
 }
 
-void ring_bond_eth::slave_create(int if_index)
+void ring_bond::slave_create(int if_index)
 {
     ring_slave *cur_slave;
 
-    cur_slave = new ring_eth(if_index, this);
+    cur_slave = new ring_simple(if_index, this, true);
     if (!cur_slave) {
         ring_logpanic("Error creating bond ring: memory allocation error");
     }
@@ -940,33 +794,3 @@ void ring_bond_eth::slave_create(int if_index)
     popup_recv_rings();
     update_rx_channel_fds();
 }
-
-void ring_bond_netvsc::slave_create(int if_index)
-{
-    ring_slave *cur_slave = nullptr;
-    net_device_val *p_ndev = nullptr;
-
-    p_ndev = g_p_net_device_table_mgr->get_net_device_val(m_parent->get_if_index());
-    if (!p_ndev) {
-        ring_logpanic("Error creating bond ring");
-    }
-
-    if (if_index == p_ndev->get_if_idx()) {
-        cur_slave = new ring_tap(if_index, this);
-        m_tap_ring = cur_slave;
-    } else {
-        cur_slave = new ring_eth(if_index, this);
-        m_vf_ring = cur_slave;
-        update_cap(cur_slave);
-    }
-
-    m_bond_rings.push_back(cur_slave);
-
-    if (m_bond_rings.size() > 2) {
-        ring_logpanic("Error creating bond ring with more than %d resource", 2);
-    }
-
-    popup_xmit_rings();
-    popup_recv_rings();
-    update_rx_channel_fds();
-}
