@@ -58,8 +58,8 @@ inline void ring_simple::send_status_handler(int ret, xlio_ibv_send_wr *p_send_w
     BULLSEYE_EXCLUDE_BLOCK_END
 }
 
-ring_simple::ring_simple(int if_index, ring *parent, ring_type_t type, bool use_locks)
-    : ring_slave(if_index, parent, type, use_locks)
+ring_simple::ring_simple(int if_index, ring *parent, bool use_locks)
+    : ring_slave(if_index, parent, use_locks)
     , m_lock_ring_tx_buf_wait("ring:lock_tx_buf_wait")
     , m_gro_mgr(safe_mce_sys().gro_streams_max, MAX_GRO_BUFS)
 {
@@ -75,9 +75,6 @@ ring_simple::ring_simple(int if_index, ring *parent, ring_type_t type, bool use_
 
     ring_logdbg("new ring_simple()");
 
-    /* m_p_ib_ctx, m_tx_lkey should be initialized to be used
-     * in ring_eth_direct, ring_eth_cb constructors
-     */
     BULLSEYE_EXCLUDE_BLOCK_START
     m_p_ib_ctx = p_slave->p_ib_ctx;
     if (!m_p_ib_ctx) {
@@ -99,6 +96,9 @@ ring_simple::ring_simple(int if_index, ring *parent, ring_type_t type, bool use_
     memset(&m_tls, 0, sizeof(m_tls));
 #endif /* DEFINED_UTLS */
     memset(&m_lro, 0, sizeof(m_lro));
+
+    m_vlan = p_ndev->get_vlan();
+    create_resources();
 }
 
 ring_simple::~ring_simple()
@@ -380,7 +380,7 @@ int ring_simple::request_notification(cq_type_t cq_type, uint64_t poll_sn)
     if (likely(CQT_RX == cq_type)) {
         m_lock_ring_rx.lock();
         ret = m_p_cq_mgr_rx->request_notification(poll_sn);
-        ++m_p_ring_stat->simple.n_rx_interrupt_requests;
+        ++m_p_ring_stat->n_rx_interrupt_requests;
         m_lock_ring_rx.unlock();
     } else {
         m_lock_ring_tx.lock();
@@ -391,10 +391,10 @@ int ring_simple::request_notification(cq_type_t cq_type, uint64_t poll_sn)
     return ret;
 }
 
-bool ring_simple::poll_and_process_element_rx(uint64_t *p_cq_poll_sn,
-                                              void *pv_fd_ready_array /*NULL*/)
+int ring_simple::poll_and_process_element_rx(uint64_t *p_cq_poll_sn,
+                                             void *pv_fd_ready_array /*NULL*/)
 {
-    bool ret = false; // CQ was not drained.
+    int ret = 0; // CQ was not drained.
     if (!m_lock_ring_rx.trylock()) {
         ret = m_p_cq_mgr_rx->poll_and_process_element_rx(p_cq_poll_sn, pv_fd_ready_array);
         m_lock_ring_rx.unlock();
@@ -404,61 +404,12 @@ bool ring_simple::poll_and_process_element_rx(uint64_t *p_cq_poll_sn,
 
 int ring_simple::poll_and_process_element_tx(uint64_t *p_cq_poll_sn)
 {
-    int ret = 0;
-    RING_TRY_LOCK_RUN_AND_UPDATE_RET(m_lock_ring_tx,
-                                     m_p_cq_mgr_tx->poll_and_process_element_tx(p_cq_poll_sn));
-    return ret;
-}
-
-int ring_simple::socketxtreme_poll(struct xlio_socketxtreme_completion_t *xlio_completions,
-                                   unsigned int ncompletions, int flags)
-{
-    int i = 0;
-    unsigned int pkts = 0U;
-    bool do_poll = true;
-
-    if (likely(xlio_completions) && ncompletions) {
-        // coverity[missing_lock:FALSE] /*Turn off coverity missing_lock check*/
-        if ((flags & SOCKETXTREME_POLL_TX) && !m_socketxtreme.ec_sock_list_start) {
-            uint64_t poll_sn = 0;
-            const std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
-            m_p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
-        }
-
-        const std::lock_guard<decltype(m_lock_ring_rx)> lock(m_lock_ring_rx);
-        while (!g_b_exit && (i < (int)ncompletions)) {
-            if (m_socketxtreme.ec_sock_list_start) {
-                if (socketxtreme_ec_pop_completion(xlio_completions)) {
-                    xlio_completions++;
-                    i++;
-                }
-            } else if (likely(do_poll)) {
-                mem_buf_desc_t *desc = m_p_cq_mgr_rx->poll_and_process_socketxtreme();
-                if (likely(desc)) {
-                    rx_process_buffer(desc, NULL);
-
-                    // To avoid too many unflushed GRO sessions, We flush GRO and continue polling.
-                    // Otherwise this loop may continue for too long and result in way more
-                    // completions than ncompletions, what is not optimal for performance.
-                    // Not each packet results in a real completion but this check is good enough.
-                    if (++pkts >= ncompletions) {
-                        m_gro_mgr.flush_all(nullptr);
-                        pkts = 0U;
-                    }
-                } else {
-                    m_gro_mgr.flush_all(nullptr);
-                    do_poll = false;
-                }
-            } else {
-                break;
-            }
-        }
-    } else {
-        i = -1;
-        errno = EINVAL;
+    int ret = 0; // CQ was not drained - If trylock fails.
+    if (!m_lock_ring_tx.trylock()) {
+        ret = m_p_cq_mgr_tx->poll_and_process_element_tx(p_cq_poll_sn);
+        m_lock_ring_tx.unlock();
     }
-
-    return i;
+    return ret;
 }
 
 void ring_simple::wait_for_notification_and_process_element(uint64_t *p_cq_poll_sn,
@@ -466,7 +417,7 @@ void ring_simple::wait_for_notification_and_process_element(uint64_t *p_cq_poll_
 {
     m_lock_ring_rx.lock();
     m_p_cq_mgr_rx->wait_for_notification_and_process_element(p_cq_poll_sn, pv_fd_ready_array);
-    ++m_p_ring_stat->simple.n_rx_interrupt_received;
+    ++m_p_ring_stat->n_rx_interrupt_received;
     m_lock_ring_rx.unlock();
 }
 
@@ -490,11 +441,6 @@ bool ring_simple::reclaim_recv_buffers_no_lock(mem_buf_desc_t *rx_reuse_lst)
     return m_p_cq_mgr_rx->reclaim_recv_buffers_no_lock(rx_reuse_lst);
 }
 
-int ring_simple::reclaim_recv_single_buffer(mem_buf_desc_t *rx_reuse)
-{
-    return m_p_cq_mgr_rx->reclaim_recv_single_buffer(rx_reuse);
-}
-
 void ring_simple::mem_buf_desc_return_to_owner_rx(mem_buf_desc_t *p_mem_buf_desc,
                                                   void *pv_fd_ready_array /*NULL*/)
 {
@@ -542,10 +488,10 @@ int ring_simple::drain_and_proccess()
 }
 
 mem_buf_desc_t *ring_simple::mem_buf_tx_get(ring_user_id_t id, bool b_block, pbuf_type type,
-                                            int n_num_mem_bufs /* default = 1 */)
+                                            int n_num_mem_bufs /* default = 1 */,
+                                            bool tx_skip_poll /* default = false */)
 {
     NOT_IN_USE(id);
-    int ret = 0;
     mem_buf_desc_t *buff_list = nullptr;
     uint64_t poll_sn = 0;
 
@@ -554,19 +500,15 @@ mem_buf_desc_t *ring_simple::mem_buf_tx_get(ring_user_id_t id, bool b_block, pbu
     m_lock_ring_tx.lock();
     buff_list = get_tx_buffers(type, n_num_mem_bufs);
     while (!buff_list) {
-
+        int ret = -1;
         // Try to poll once in the hope that we get a few freed tx mem_buf_desc
-        ret = m_p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
-        if (ret < 0) {
-            ring_logdbg("failed polling on cq_mgr_tx (hqtx=%p, cq_mgr_tx=%p) (ret=%d %m)", m_hqtx,
-                        m_p_cq_mgr_tx, ret);
-            /* coverity[double_unlock] TODO: RM#1049980 */
-            m_lock_ring_tx.unlock();
-            return nullptr;
-        } else if (ret > 0) {
-            ring_logfunc("polling succeeded on cq_mgr_tx (%d wce)", ret);
+        if (!tx_skip_poll) {
+            ret = m_p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
+        }
+        if (ret > 0) {
+            ring_logfunc("polling succeeded on cq_mgr_tx");
             buff_list = get_tx_buffers(type, n_num_mem_bufs);
-        } else if (b_block) { // (ret == 0)
+        } else if (b_block) {
             // Arm & Block on tx cq_mgr_tx notification channel
             // until we get a few freed tx mem_buf_desc & data buffers
 
@@ -624,18 +566,7 @@ mem_buf_desc_t *ring_simple::mem_buf_tx_get(ring_user_id_t id, bool b_block, pbu
                         p_cq_mgr_tx->reset_notification_armed();
 
                         // Perform a non blocking event read, clear the fd channel
-                        ret = p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
-                        if (ret < 0) {
-                            ring_logdbg("failed handling cq_mgr_tx channel (hqtx=%p, "
-                                        "cq_mgr_tx=%p) (errno=%d %m)",
-                                        m_hqtx, m_p_cq_mgr_tx, errno);
-                            /* coverity[double_unlock] TODO: RM#1049980 */
-                            m_lock_ring_tx.unlock();
-                            m_lock_ring_tx_buf_wait.unlock();
-                            return nullptr;
-                        }
-                        ring_logfunc("polling/blocking succeeded on cq_mgr_tx (we got %d wce)",
-                                     ret);
+                        p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
                     }
                 }
                 buff_list = get_tx_buffers(type, n_num_mem_bufs);
@@ -702,19 +633,11 @@ inline int ring_simple::send_buffer(xlio_ibv_send_wr *p_send_wqe, xlio_wr_tx_pac
         ring_logdbg("Silent packet drop, SQ is full!");
         ret = -1;
         reinterpret_cast<mem_buf_desc_t *>(p_send_wqe->wr_id)->p_next_desc = nullptr;
-        ++m_p_ring_stat->simple.n_tx_dropped_wqes;
+        ++m_p_ring_stat->n_tx_dropped_wqes;
     }
     return ret;
 }
 
-bool ring_simple::get_hw_dummy_send_support(ring_user_id_t id, xlio_ibv_send_wr *p_send_wqe)
-{
-    NOT_IN_USE(id);
-    NOT_IN_USE(p_send_wqe);
-
-    return m_hqtx->get_hw_dummy_send_support();
-}
-
 void ring_simple::send_ring_buffer(ring_user_id_t id, xlio_ibv_send_wr *p_send_wqe,
                                    xlio_wr_tx_packet_attr attr)
 {
@@ -747,20 +670,13 @@ int ring_simple::send_lwip_buffer(ring_user_id_t id, xlio_ibv_send_wr *p_send_wq
 bool ring_simple::is_available_qp_wr(bool b_block, unsigned credits)
 {
     bool granted;
-    int ret;
     uint64_t poll_sn = 0;
 
     // TODO credits_get() does TX polling. Call current method only for bocking mode?
 
     do {
         // Try to poll once in the hope that we get space in SQ
-        ret = m_p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
-        if (ret < 0) {
-            ring_logdbg("failed polling on cq_mgr_tx (hqtx=%p, cq_mgr_tx=%p) (ret=%d %m)", m_hqtx,
-                        m_p_cq_mgr_tx, ret);
-            /* coverity[missing_unlock] */
-            return false;
-        }
+        m_p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
         granted = m_hqtx->credits_get(credits);
         if (granted) {
             break;
@@ -777,7 +693,7 @@ bool ring_simple::is_available_qp_wr(bool b_block, unsigned credits)
             m_lock_ring_tx.lock();
 
             // TODO Resolve race window between previous polling and request_notification
-            ret = m_p_cq_mgr_tx->request_notification(poll_sn);
+            int ret = m_p_cq_mgr_tx->request_notification(poll_sn);
             if (ret < 0) {
                 // this is most likely due to cq_poll_sn out of sync, need to poll_cq again
                 ring_logdbg("failed arming cq_mgr_tx (hqtx=%p, cq_mgr_tx=%p) (errno=%d %m)", m_hqtx,
@@ -816,19 +732,7 @@ bool ring_simple::is_available_qp_wr(bool b_block, unsigned credits)
                     p_cq_mgr_tx->reset_notification_armed();
 
                     // Perform a non blocking event read, clear the fd channel
-                    ret = p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
-                    if (ret < 0) {
-                        ring_logdbg("failed handling cq_mgr_tx channel (hqtx=%p "
-                                    "cq_mgr_tx=%p) (errno=%d %m)",
-                                    m_hqtx, m_p_cq_mgr_tx, errno);
-                        /* coverity[double_unlock] TODO: RM#1049980 */
-                        m_lock_ring_tx.unlock();
-                        m_lock_ring_tx_buf_wait.unlock();
-                        /* coverity[double_lock] TODO: RM#1049980 */
-                        m_lock_ring_tx.lock();
-                        return false;
-                    }
-                    ring_logfunc("polling/blocking succeeded on cq_mgr_tx (we got %d wce)", ret);
+                    p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
                 }
             }
 
@@ -848,7 +752,7 @@ void ring_simple::init_tx_buffers(uint32_t count)
 {
     request_more_tx_buffers(PBUF_RAM, count, m_tx_lkey);
     m_tx_num_bufs = m_tx_pool.size();
-    m_p_ring_stat->simple.n_tx_num_bufs = m_tx_num_bufs;
+    m_p_ring_stat->n_tx_num_bufs = m_tx_num_bufs;
 }
 
 void ring_simple::inc_cq_moderation_stats()
@@ -868,14 +772,14 @@ mem_buf_desc_t *ring_simple::get_tx_buffers(pbuf_type type, uint32_t n_num_mem_b
             /*
              * TODO Unify request_more_tx_buffers so ring_slave
              * keeps number of buffers instead of reinventing it in
-             * ring_simple and ring_tap.
+             * ring_simple.
              */
             if (type == PBUF_ZEROCOPY) {
                 m_zc_num_bufs += count;
-                m_p_ring_stat->simple.n_zc_num_bufs = m_zc_num_bufs;
+                m_p_ring_stat->n_zc_num_bufs = m_zc_num_bufs;
             } else {
                 m_tx_num_bufs += count;
-                m_p_ring_stat->simple.n_tx_num_bufs = m_tx_num_bufs;
+                m_p_ring_stat->n_tx_num_bufs = m_tx_num_bufs;
             }
         }
 
@@ -910,14 +814,14 @@ void ring_simple::return_to_global_pool()
                  m_tx_num_bufs >= RING_TX_BUFS_COMPENSATE * 2)) {
         int return_bufs = m_tx_pool.size() / 2;
         m_tx_num_bufs -= return_bufs;
-        m_p_ring_stat->simple.n_tx_num_bufs = m_tx_num_bufs;
+        m_p_ring_stat->n_tx_num_bufs = m_tx_num_bufs;
         g_buffer_pool_tx->put_buffers_thread_safe(&m_tx_pool, return_bufs);
     }
     if (unlikely(m_zc_pool.size() > (m_zc_num_bufs / 2) &&
                  m_zc_num_bufs >= RING_TX_BUFS_COMPENSATE * 2)) {
         int return_bufs = m_zc_pool.size() / 2;
         m_zc_num_bufs -= return_bufs;
-        m_p_ring_stat->simple.n_zc_num_bufs = m_zc_num_bufs;
+        m_p_ring_stat->n_zc_num_bufs = m_zc_num_bufs;
         g_buffer_pool_zc->put_buffers_thread_safe(&m_zc_pool, return_bufs);
     }
 }
@@ -927,6 +831,7 @@ void ring_simple::return_tx_pool_to_global_pool()
     return_to_global_pool();
 }
 
+// Returns number of freed buffers.
 int ring_simple::put_tx_buffer_helper(mem_buf_desc_t *buff)
 {
     if (buff->tx.dev_mem_length) {
@@ -946,7 +851,6 @@ int ring_simple::put_tx_buffer_helper(mem_buf_desc_t *buff)
         buff->p_next_desc = nullptr;
         free_lwip_pbuf(&buff->lwip_pbuf);
         pool.push_back(buff);
-        // Return number of freed buffers
         return 1;
     }
     return 0;
@@ -1002,13 +906,18 @@ void ring_simple::modify_cq_moderation(uint32_t period, uint32_t count)
     m_cq_moderation_info.period = period;
     m_cq_moderation_info.count = count;
 
-    m_p_ring_stat->simple.n_rx_cq_moderation_period = period;
-    m_p_ring_stat->simple.n_rx_cq_moderation_count = count;
+    m_p_ring_stat->n_rx_cq_moderation_period = period;
+    m_p_ring_stat->n_rx_cq_moderation_count = count;
 
     // todo all cqs or just active? what about HA?
     priv_ibv_modify_cq_moderation(m_p_cq_mgr_rx->get_ibv_cq_hndl(), period, count);
 }
 
+uint64_t ring_simple::get_rx_cq_out_of_buffer_drop()
+{
+    return m_p_cq_mgr_rx->get_n_rx_hw_pkt_drops();
+}
+
 void ring_simple::adapt_cq_moderation()
 {
     if (m_lock_ring_rx.trylock()) {
