@@ -34,21 +34,18 @@ struct cq_moderation_info {
  */
 class ring_simple : public ring_slave {
 public:
-    ring_simple(int if_index, ring *parent, ring_type_t type, bool use_locks);
+    ring_simple(int if_index, ring *parent, bool use_locks);
     virtual ~ring_simple();
 
     int request_notification(cq_type_t cq_type, uint64_t poll_sn) override;
-    bool poll_and_process_element_rx(uint64_t *p_cq_poll_sn,
-                                     void *pv_fd_ready_array = nullptr) override;
+    int poll_and_process_element_rx(uint64_t *p_cq_poll_sn,
+                                    void *pv_fd_ready_array = nullptr) override;
     int poll_and_process_element_tx(uint64_t *p_cq_poll_sn) override;
     void adapt_cq_moderation() override;
     bool reclaim_recv_buffers(descq_t *rx_reuse) override;
     bool reclaim_recv_buffers(mem_buf_desc_t *rx_reuse_lst) override;
     bool reclaim_recv_buffers_no_lock(mem_buf_desc_t *rx_reuse_lst) override; // No locks
-    int reclaim_recv_single_buffer(mem_buf_desc_t *rx_reuse) override; // No locks
     void mem_buf_rx_release(mem_buf_desc_t *p_mem_buf_desc) override;
-    int socketxtreme_poll(struct xlio_socketxtreme_completion_t *xlio_completions,
-                          unsigned int ncompletions, int flags) override;
     int drain_and_proccess() override;
     void wait_for_notification_and_process_element(uint64_t *p_cq_poll_sn,
                                                    void *pv_fd_ready_array = nullptr) override;
@@ -63,7 +60,8 @@ public:
     void stop_active_queue_tx();
     void stop_active_queue_rx();
     mem_buf_desc_t *mem_buf_tx_get(ring_user_id_t id, bool b_block, pbuf_type type,
-                                   int n_num_mem_bufs = 1) override;
+                                   int n_num_mem_bufs = 1 /* default = 1 */,
+                                   bool tx_skip_poll = false) override;
     int mem_buf_tx_release(mem_buf_desc_t *p_mem_buf_desc_list, bool b_accounting,
                            bool trylock = false) override;
     void send_ring_buffer(ring_user_id_t id, xlio_ibv_send_wr *p_send_wqe,
@@ -75,7 +73,6 @@ public:
                                               unsigned ref) override;
     void mem_buf_desc_return_single_locked(mem_buf_desc_t *buff);
     void return_tx_pool_to_global_pool();
-    bool get_hw_dummy_send_support(ring_user_id_t id, xlio_ibv_send_wr *p_send_wqe) override;
     inline void convert_hw_time_to_system_time(uint64_t hwtime, struct timespec *systime)
     {
         m_p_ib_ctx->convert_hw_time_to_system_time(hwtime, systime);
@@ -107,10 +104,12 @@ public:
 
     void update_tso_stats(uint64_t bytes)
     {
-        ++m_p_ring_stat->simple.n_tx_tso_pkt_count;
-        m_p_ring_stat->simple.n_tx_tso_byte_count += bytes;
+        ++m_p_ring_stat->n_tx_tso_pkt_count;
+        m_p_ring_stat->n_tx_tso_byte_count += bytes;
     }
 
+    virtual uint64_t get_rx_cq_out_of_buffer_drop() override;
+
 #ifdef DEFINED_UTLS
     bool tls_tx_supported(void) override { return m_tls.tls_tx; }
     bool tls_rx_supported(void) override { return m_tls.tls_rx; }
@@ -161,6 +160,7 @@ public:
     {
         std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
         m_hqtx->tls_context_resync_tx(info, tis, skip_static);
+        ++(m_p_ring_stat->n_tx_tls_resyncs);
 
         uint64_t dummy_poll_sn = 0;
         m_p_cq_mgr_tx->poll_and_process_element_tx(&dummy_poll_sn);
@@ -169,6 +169,7 @@ public:
     {
         std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
         m_hqtx->tls_resync_rx(tir, info, hw_resync_tcp_sn);
+        ++(m_p_ring_stat->n_rx_tls_resyncs);
     }
     void tls_get_progress_params_rx(xlio_tir *tir, void *buf, uint32_t lkey) override
     {
@@ -208,31 +209,6 @@ public:
         std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
         return m_hqtx->create_tis(flags);
     }
-    int get_supported_nvme_feature_mask() const override
-    {
-        dpcp::adapter_hca_capabilities caps {};
-        auto adapter = m_p_ib_ctx->get_dpcp_adapter();
-
-        if (!adapter || (dpcp::DPCP_OK != adapter->get_hca_capabilities(caps)) ||
-            !caps.nvmeotcp_caps.enabled) {
-            return 0;
-        }
-        return (NVME_CRC_TX * caps.nvmeotcp_caps.crc_tx) |
-            (NVME_CRC_RX * caps.nvmeotcp_caps.crc_rx) |
-            (NVME_ZEROCOPY * caps.nvmeotcp_caps.zerocopy);
-    }
-
-    void nvme_set_static_context(xlio_tis *tis, uint32_t config) override
-    {
-        std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
-        m_hqtx->nvme_set_static_context(tis, config);
-    }
-
-    void nvme_set_progress_context(xlio_tis *tis, uint32_t tcp_seqno) override
-    {
-        std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
-        m_hqtx->nvme_set_progress_context(tis, tcp_seqno);
-    }
 
     void post_nop_fence(void) override
     {
@@ -377,22 +353,4 @@ private:
     } m_lro;
 };
 
-class ring_eth : public ring_simple {
-public:
-    ring_eth(int if_index, ring *parent = nullptr, ring_type_t type = RING_ETH,
-             bool call_create_res = true, bool use_locks = true)
-        : ring_simple(if_index, parent, type, use_locks)
-    {
-        net_device_val_eth *p_ndev = dynamic_cast<net_device_val_eth *>(
-            g_p_net_device_table_mgr->get_net_device_val(m_parent->get_if_index()));
-        if (p_ndev) {
-            m_vlan = p_ndev->get_vlan();
-
-            if (call_create_res) {
-                create_resources();
-            }
-        }
-    }
-};
-
 #endif // RING_SIMPLE_H
