@@ -11,7 +11,8 @@
 #include "dev/rfs_mc.h"
 #include "dev/rfs_uc_tcp_gro.h"
 #include "sock/fd_collection.h"
-#include "sock/sockinfo.h"
+#include "sock/sockinfo_tcp.h"
+#include "proto/tls.h"
 
 #undef MODULE_NAME
 #define MODULE_NAME "ring_slave"
@@ -21,16 +22,16 @@
 // AF_INET address 0.0.0.0:0, used for 3T flow spec keys.
 static const sock_addr s_sock_addrany;
 
-static thread_local lock_dummy t_lock_dummy_ring;
+static padded_lock_dummy g_lock_dummy_ring;
 
 static lock_base *get_new_lock(const char *name, bool real_lock)
 {
     return (real_lock
                 ? static_cast<lock_base *>(multilock::create_new_lock(MULTILOCK_RECURSIVE, name))
-                : static_cast<lock_base *>(&t_lock_dummy_ring));
+                : static_cast<lock_base *>(&g_lock_dummy_ring.lock));
 }
 
-ring_slave::ring_slave(int if_index, ring *parent, ring_type_t type, bool use_locks)
+ring_slave::ring_slave(int if_index, ring *parent, bool use_locks)
     : ring()
     , m_steering_ipv4(*this)
     , m_steering_ipv6(*this)
@@ -41,7 +42,6 @@ ring_slave::ring_slave(int if_index, ring *parent, ring_type_t type, bool use_lo
     , m_flow_tag_enabled(false)
     , m_b_sysvar_eth_mc_l2_only_rules(safe_mce_sys().eth_mc_l2_only_rules)
     , m_b_sysvar_mc_force_flowtag(safe_mce_sys().mc_force_flowtag)
-    , m_type(type)
 {
     net_device_val *p_ndev = nullptr;
     const slave_data_t *p_slave = nullptr;
@@ -61,15 +61,11 @@ ring_slave::ring_slave(int if_index, ring *parent, ring_type_t type, bool use_lo
     /* Configure ring_slave() fields */
     m_transport_type = p_ndev->get_transport_type();
 
-    /* Set the same ring active status as related slave has for all ring types
-     * excluding ring with type RING_TAP that does not have related slave device.
-     * So it is marked as active just in case related netvsc device is absent.
-     */
+    /* Set the same ring active status as related slave has for all ring types. */
     m_active = p_slave ? p_slave->active : p_ndev->get_slave_array().empty();
 
     // use local copy of stats by default
     memset(m_p_ring_stat.get(), 0, sizeof(ring_stats_t));
-    m_p_ring_stat->n_type = m_type;
     if (m_parent != this) {
         m_p_ring_stat->p_ring_master = m_parent;
     }
@@ -97,9 +93,8 @@ ring_slave::~ring_slave()
 
 void ring_slave::print_val()
 {
-    ring_logdbg("%d: %p: parent %p type %s", m_if_index, this,
-                ((uintptr_t)this == (uintptr_t)m_parent ? nullptr : m_parent),
-                ring_type_str[m_type]);
+    ring_logdbg("%d: %p: parent %p", m_if_index, this,
+                ((uintptr_t)this == (uintptr_t)m_parent ? nullptr : m_parent));
 }
 
 void ring_slave::restart()
@@ -139,19 +134,18 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
 {
     rfs *p_rfs;
     rfs *p_tmp_rfs = nullptr;
-    sockinfo *si = static_cast<sockinfo *>(sink);
 
-    if (!si) {
+    if (!sink) {
         return false;
     }
 
-    uint32_t flow_tag_id = si->get_flow_tag_val(); // spec will not be attached to rule
+    uint32_t flow_tag_id = sink->get_flow_tag_val(); // spec will not be attached to rule
     if (!m_ring.m_flow_tag_enabled) {
         flow_tag_id = 0;
     }
     ring_logdbg("flow: %s, with sink (%p), flow tag id %d "
                 "m_flow_tag_enabled: %d",
-                flow_spec_5t.to_str().c_str(), si, flow_tag_id, m_ring.m_flow_tag_enabled);
+                flow_spec_5t.to_str().c_str(), sink, flow_tag_id, m_ring.m_flow_tag_enabled);
 
     /* Get the appropriate hash map (tcp, uc or mc) from the 5t details
      * TODO: Consider unification of following code.
@@ -172,11 +166,11 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
             }
         }
 
-        if (flow_tag_id && si->flow_in_reuse()) {
+        if (flow_tag_id && sink->flow_in_reuse()) {
             flow_tag_id = FLOW_TAG_MASK;
             ring_logdbg("UC flow tag for socketinfo=%p is disabled: SO_REUSEADDR or SO_REUSEPORT "
                         "were enabled",
-                        si);
+                        sink);
         }
 
         auto itr = m_flow_udp_uc_map.find(rfs_key);
@@ -193,7 +187,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
                 p_tmp_rfs =
                     new (std::nothrow) rfs_uc(&flow_spec_5t, &m_ring, dst_port_filter, flow_tag_id);
             } catch (xlio_exception &e) {
-                ring_logerr("%s", e.message);
+                ring_logerr("%s", e.message.c_str());
                 return false;
             }
             BULLSEYE_EXCLUDE_BLOCK_START
@@ -217,16 +211,16 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
         KEY2T key_udp_mc(flow_spec_5t.get_dst_ip(), flow_spec_5t.get_dst_port());
         sock_addr rule_key(flow_spec_5t.get_family(), &flow_spec_5t.get_dst_ip(), 0U);
         if (flow_tag_id) {
-            if (m_ring.m_b_sysvar_mc_force_flowtag || !si->flow_in_reuse()) {
+            if (m_ring.m_b_sysvar_mc_force_flowtag || !sink->flow_in_reuse()) {
                 ring_logdbg("MC flow tag ID=%d for socketinfo=%p is enabled: force_flowtag=%d, "
                             "SO_REUSEADDR | SO_REUSEPORT=%d",
-                            flow_tag_id, si, m_ring.m_b_sysvar_mc_force_flowtag,
-                            si->flow_in_reuse());
+                            flow_tag_id, sink, m_ring.m_b_sysvar_mc_force_flowtag,
+                            sink->flow_in_reuse());
             } else {
                 flow_tag_id = FLOW_TAG_MASK;
                 ring_logdbg("MC flow tag for socketinfo=%p is disabled: force_flowtag=0, "
                             "SO_REUSEADDR or SO_REUSEPORT were enabled",
-                            si);
+                            sink);
             }
         }
         // Note for CX3:
@@ -256,7 +250,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
             try {
                 p_tmp_rfs = new rfs_mc(&flow_spec_5t, &m_ring, l2_mc_ip_filter, flow_tag_id);
             } catch (xlio_exception &e) {
-                ring_logerr("%s", e.message);
+                ring_logerr("%s", e.message.c_str());
                 return false;
             } catch (const std::bad_alloc &e) {
                 NOT_IN_USE(e);
@@ -292,7 +286,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
             (flow_spec_5t.is_3_tuple() || (!force_5t && safe_mce_sys().tcp_3t_rules) ||
              safe_mce_sys().tcp_2t_rules)) {
             ring_logdbg("flow tag id = %d is disabled for socket fd = %d to be processed on RFS!",
-                        flow_tag_id, si->get_fd());
+                        flow_tag_id, sink->get_fd());
             flow_tag_id = FLOW_TAG_MASK;
         }
 
@@ -313,15 +307,21 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
                     new rfs_rule_filter(m_ring.m_tcp_dst_port_attach_map, rule_key, tcp_3t_only);
             }
             try {
-                if (safe_mce_sys().gro_streams_max && m_ring.is_simple()) {
-                    p_tmp_rfs = new (std::nothrow)
-                        rfs_uc_tcp_gro(&flow_spec_5t, &m_ring, dst_port_filter, flow_tag_id);
+                sockinfo_tcp *tcp_si = dynamic_cast<sockinfo_tcp *>(sink);
+                int steering_index = -1;
+                if (tcp_si && tcp_si->get_listen_context()) {
+                    steering_index = tcp_si->get_listen_context()->get_steering_index();
+                }
+
+                if (safe_mce_sys().gro_streams_max) {
+                    p_tmp_rfs = new (std::nothrow) rfs_uc_tcp_gro(
+                        &flow_spec_5t, &m_ring, dst_port_filter, flow_tag_id, steering_index);
                 } else {
-                    p_tmp_rfs = new (std::nothrow)
-                        rfs_uc(&flow_spec_5t, &m_ring, dst_port_filter, flow_tag_id);
+                    p_tmp_rfs = new (std::nothrow) rfs_uc(&flow_spec_5t, &m_ring, dst_port_filter,
+                                                          flow_tag_id, steering_index);
                 }
             } catch (xlio_exception &e) {
-                ring_logerr("%s", e.message);
+                ring_logerr("%s", e.message.c_str());
                 return false;
             }
             BULLSEYE_EXCLUDE_BLOCK_START
@@ -332,7 +332,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
             BULLSEYE_EXCLUDE_BLOCK_END
 
             p_rfs = p_tmp_rfs;
-            si->set_rfs_ptr(p_rfs);
+            sink->set_rfs_ptr(p_rfs);
 #if defined(DEFINED_NGINX) || defined(DEFINED_ENVOY)
             if (g_p_app->type == APP_NONE || !g_p_app->add_second_4t_rule)
 #endif
@@ -354,7 +354,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::attach_flow(flow_tuple &flow_spec_5t,
         if (flow_tag_id && (flow_tag_id != FLOW_TAG_MASK)) {
             // A flow with FlowTag was attached succesfully, check stored rfs for fast path be
             // tag_id
-            si->set_flow_tag(flow_tag_id);
+            sink->set_flow_tag(flow_tag_id);
             ring_logdbg("flow_tag: %d registration is done!", flow_tag_id);
         }
     } else {
@@ -382,7 +382,8 @@ bool ring_slave::attach_flow(flow_tuple &flow_spec_5t, sockinfo *sink, bool forc
 }
 
 template <typename KEY4T, typename KEY2T, typename HDR>
-bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink)
+bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink,
+                                                      rfs_rule **rule_extract)
 {
     rfs *p_rfs = nullptr;
 
@@ -415,7 +416,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t,
         BULLSEYE_EXCLUDE_BLOCK_END
 
         p_rfs = itr->second;
-        p_rfs->detach_flow(sink);
+        p_rfs->detach_flow(sink, rule_extract);
         if (!keep_in_map) {
             m_ring.m_udp_uc_dst_port_attach_map.erase(
                 m_ring.m_udp_uc_dst_port_attach_map.find(rule_key));
@@ -450,7 +451,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t,
         }
         BULLSEYE_EXCLUDE_BLOCK_END
         p_rfs = itr->second;
-        p_rfs->detach_flow(sink);
+        p_rfs->detach_flow(sink, rule_extract);
         if (!keep_in_map) {
             m_ring.m_l2_mc_ip_attach_map.erase(m_ring.m_l2_mc_ip_attach_map.find(rule_key));
         }
@@ -486,7 +487,7 @@ bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t,
         BULLSEYE_EXCLUDE_BLOCK_END
 
         p_rfs = itr->second;
-        p_rfs->detach_flow(sink);
+        p_rfs->detach_flow(sink, rule_extract);
         if (!keep_in_map) {
             m_ring.m_tcp_dst_port_attach_map.erase(m_ring.m_tcp_dst_port_attach_map.find(rule_key));
         }
@@ -506,12 +507,13 @@ bool steering_handler<KEY4T, KEY2T, HDR>::detach_flow(flow_tuple &flow_spec_5t,
     return true;
 }
 
-bool ring_slave::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink)
+bool ring_slave::detach_flow(flow_tuple &flow_spec_5t, sockinfo *sink, rfs_rule **rule_extract)
 {
     std::lock_guard<decltype(m_lock_ring_rx)> lock(m_lock_ring_rx);
 
-    return (flow_spec_5t.get_family() == AF_INET ? m_steering_ipv4.detach_flow(flow_spec_5t, sink)
-                                                 : m_steering_ipv6.detach_flow(flow_spec_5t, sink));
+    return (flow_spec_5t.get_family() == AF_INET
+                ? m_steering_ipv4.detach_flow(flow_spec_5t, sink, rule_extract)
+                : m_steering_ipv6.detach_flow(flow_spec_5t, sink, rule_extract));
 }
 
 #ifdef DEFINED_UTLS
@@ -573,6 +575,9 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
 
     m_p_ring_stat->n_rx_byte_count += sz_data;
     ++m_p_ring_stat->n_rx_pkt_count;
+#ifdef DEFINED_UTLS
+    m_p_ring_stat->n_rx_tls_auth_fail += !!(p_rx_wc_buf_desc->rx.tls_decrypted == TLS_RX_AUTH_FAIL);
+#endif /* DEFINED_UTLS */
 
     // This is an internal function (within ring and 'friends'). No need for lock mechanism.
     if (likely(m_flow_tag_enabled && p_rx_wc_buf_desc->rx.flow_tag_id &&
@@ -584,6 +589,12 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
         assert(g_p_fd_collection);
         si = static_cast<sockinfo *>(
             g_p_fd_collection->get_sockfd(p_rx_wc_buf_desc->rx.flow_tag_id - 1));
+        if (likely(si) && si->is_xlio_socket() &&
+            unlikely(si->get_poll_group() == nullptr ||
+                     si->get_poll_group() != this->get_poll_group())) {
+            m_p_ring_stat->n_rx_zc_migiration_drop++;
+            return false;
+        }
 
         if (likely(si)) {
             // will process packets with set flow_tag_id and enabled for the socket
@@ -602,7 +613,7 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
 
             struct iphdr *p_ip_h =
                 (struct iphdr *)(p_rx_wc_buf_desc->p_buffer + transport_header_len);
-            if (likely(p_ip_h->version == IPV4_VERSION)) { // IPv4
+            if (likely(p_ip_h->version == XLIO_IPV4_VERSION)) { // IPv4
                 ip_hdr_len = IP_HLEN; //(int)(p_ip_h->ihl)*4;
                 ip_payload_len = ntohs(p_ip_h->tot_len) - ip_hdr_len;
                 protocol = p_ip_h->protocol;
@@ -735,12 +746,12 @@ bool ring_slave::rx_process_buffer(mem_buf_desc_t *p_rx_wc_buf_desc, void *pv_fd
     // Get the ip header pointer
     struct iphdr *p_ip_h = (struct iphdr *)(p_rx_wc_buf_desc->p_buffer + transport_header_len);
 
-    if (likely(p_ip_h->version == IPV4_VERSION)) {
+    if (likely(p_ip_h->version == XLIO_IPV4_VERSION)) {
         return m_steering_ipv4.rx_process_buffer_no_flow_id(p_rx_wc_buf_desc, pv_fd_ready_array,
                                                             p_ip_h);
     }
 
-    if (likely(p_ip_h->version == IPV6_VERSION)) {
+    if (likely(p_ip_h->version == XLIO_IPV6_VERSION)) {
         return m_steering_ipv6.rx_process_buffer_no_flow_id(p_rx_wc_buf_desc, pv_fd_ready_array,
                                                             reinterpret_cast<ip6_hdr *>(p_ip_h));
     }
