@@ -105,23 +105,6 @@ void event_handler_manager::register_socket_timer_event(sockinfo_tcp *sock_tcp)
     post_new_reg_action(reg_action);
 }
 
-void event_handler_manager::wakeup_timer_event(timer_handler *handler, void *node)
-{
-    evh_logdbg("timer handler '%p'", handler);
-    BULLSEYE_EXCLUDE_BLOCK_START
-    if (!handler) {
-        evh_logwarn("bad handler (%p)", handler);
-        return;
-    }
-    BULLSEYE_EXCLUDE_BLOCK_END
-    reg_action_t reg_action;
-    memset(&reg_action, 0, sizeof(reg_action));
-    reg_action.type = WAKEUP_TIMER;
-    reg_action.info.timer.handler = handler;
-    reg_action.info.timer.node = node;
-    post_new_reg_action(reg_action);
-}
-
 void event_handler_manager::unregister_timer_event(timer_handler *handler, void *node)
 {
     evh_logdbg("timer handler '%p'", handler);
@@ -156,11 +139,21 @@ void event_handler_manager::unregister_timers_event_and_delete(timer_handler *ha
     post_new_reg_action(reg_action);
 }
 
-void event_handler_manager::unregister_socket_timer_and_delete(sockinfo_tcp *sock_tcp)
+void event_handler_manager::unregister_socket_timer_event(sockinfo_tcp *sock_tcp)
 {
     evh_logdbg("Unregistering TCP socket timer: %p", sock_tcp);
     reg_action_t reg_action;
     memset(&reg_action, 0, sizeof(reg_action));
+    reg_action.type = UNREGISTER_TCP_SOCKET_TIMER;
+    reg_action.info.timer.user_data = sock_tcp;
+    post_new_reg_action(reg_action);
+}
+
+void event_handler_manager::unregister_socket_timer_and_delete(sockinfo_tcp *sock_tcp)
+{
+    evh_logdbg("Unregistering TCP socket timer and destroying: %p", sock_tcp);
+    reg_action_t reg_action;
+    memset(&reg_action, 0, sizeof(reg_action));
     reg_action.type = UNREGISTER_TCP_SOCKET_TIMER_AND_DELETE;
     reg_action.info.timer.user_data = sock_tcp;
     post_new_reg_action(reg_action);
@@ -227,12 +220,10 @@ void event_handler_manager::register_command_event(int fd, command *cmd)
 
 event_handler_manager::event_handler_manager(bool internal_thread_mode)
     : m_reg_action_q_lock("reg_action_q_lock")
-    , m_b_sysvar_internal_thread_arm_cq_enabled(safe_mce_sys().internal_thread_arm_cq_enabled)
     , m_n_sysvar_timer_resolution_msec(safe_mce_sys().timer_resolution_msec)
 {
     evh_logfunc("");
 
-    m_cq_epfd = 0;
     m_epfd = -1;
     m_event_handler_tid = 0;
 
@@ -421,6 +412,8 @@ const char *event_handler_manager::reg_action_str(event_action_type_e reg_action
     switch (reg_action_type) {
     case REGISTER_TCP_SOCKET_TIMER:
         return "REGISTER_TCP_SOCKET_TIMER";
+    case UNREGISTER_TCP_SOCKET_TIMER:
+        return "UNREGISTER_TCP_SOCKET_TIMER";
     case UNREGISTER_TCP_SOCKET_TIMER_AND_DELETE:
         return "UNREGISTER_TCP_SOCKET_TIMER_AND_DELETE";
     case REGISTER_TIMER:
@@ -477,11 +470,6 @@ void event_handler_manager::priv_register_timer_handler(timer_reg_info_t &info)
                           info.user_data, info.req_type);
 }
 
-void event_handler_manager::priv_wakeup_timer_handler(timer_reg_info_t &info)
-{
-    m_timer.wakeup_timer((timer_node_t *)info.node);
-}
-
 void event_handler_manager::priv_unregister_timer_handler(timer_reg_info_t &info)
 {
     m_timer.remove_timer((timer_node_t *)info.node, info.handler);
@@ -703,6 +691,10 @@ void event_handler_manager::handle_registration_action(reg_action_t &reg_action)
         sock = reinterpret_cast<sockinfo_tcp *>(reg_action.info.timer.user_data);
         sock->get_tcp_timer_collection()->add_new_timer(sock);
         break;
+    case UNREGISTER_TCP_SOCKET_TIMER:
+        sock = reinterpret_cast<sockinfo_tcp *>(reg_action.info.timer.user_data);
+        sock->get_tcp_timer_collection()->remove_timer(sock);
+        break;
     case UNREGISTER_TCP_SOCKET_TIMER_AND_DELETE:
         sock = reinterpret_cast<sockinfo_tcp *>(reg_action.info.timer.user_data);
         sock->get_tcp_timer_collection()->remove_timer(sock);
@@ -711,9 +703,6 @@ void event_handler_manager::handle_registration_action(reg_action_t &reg_action)
     case REGISTER_TIMER:
         priv_register_timer_handler(reg_action.info.timer);
         break;
-    case WAKEUP_TIMER:
-        priv_wakeup_timer_handler(reg_action.info.timer);
-        break;
     case UNREGISTER_TIMER:
         priv_unregister_timer_handler(reg_action.info.timer);
         break;
@@ -932,31 +921,6 @@ void *event_handler_manager::thread_loop()
             continue;
         }
 
-        if (m_b_sysvar_internal_thread_arm_cq_enabled && m_cq_epfd == 0 &&
-            g_p_net_device_table_mgr) {
-            m_cq_epfd = g_p_net_device_table_mgr->global_ring_epfd_get();
-            if (m_cq_epfd > 0) {
-                epoll_event evt = {0, {nullptr}};
-                evt.events = EPOLLIN | EPOLLPRI;
-                evt.data.fd = m_cq_epfd;
-                SYSCALL(epoll_ctl, m_epfd, EPOLL_CTL_ADD, m_cq_epfd, &evt);
-            }
-        }
-
-        uint64_t poll_sn_rx = 0;
-        uint64_t poll_sn_tx = 0;
-        if (m_b_sysvar_internal_thread_arm_cq_enabled && m_cq_epfd > 0 &&
-            g_p_net_device_table_mgr) {
-            g_p_net_device_table_mgr->global_ring_poll_and_process_element(&poll_sn_rx, &poll_sn_tx,
-                                                                           nullptr);
-            int ret =
-                g_p_net_device_table_mgr->global_ring_request_notification(poll_sn_rx, poll_sn_tx);
-            if (ret > 0) {
-                g_p_net_device_table_mgr->global_ring_poll_and_process_element(
-                    &poll_sn_rx, &poll_sn_tx, nullptr);
-            }
-        }
-
         // Make sure we sleep for a minimum of X milli seconds
         if (timeout_msec > 0) {
             if ((int)m_n_sysvar_timer_resolution_msec > timeout_msec) {
@@ -974,11 +938,7 @@ void *event_handler_manager::thread_loop()
 
         // check pipe
         for (int idx = 0; (idx < ret) && (m_b_continue_running); ++idx) {
-            if (m_b_sysvar_internal_thread_arm_cq_enabled && p_events[idx].data.fd == m_cq_epfd &&
-                g_p_net_device_table_mgr) {
-                g_p_net_device_table_mgr->global_ring_wait_for_notification_and_process_element(
-                    &poll_sn_rx, nullptr);
-            } else if (is_wakeup_fd(p_events[idx].data.fd)) {
+            if (is_wakeup_fd(p_events[idx].data.fd)) {
                 // a request for registration was sent
                 m_reg_action_q_lock.lock();
                 reg_action_q_t *temp = m_p_reg_action_q_to_push_to;
@@ -1005,15 +965,7 @@ void *event_handler_manager::thread_loop()
 
         // process ready event channels
         for (int idx = 0; (idx < ret) && (m_b_continue_running); ++idx) {
-            // if (p_events[idx].events & (EPOLLERR|EPOLLHUP))
-            //	evh_logdbg("error in fd %d",p_events[idx].data.fd );
-
             int fd = p_events[idx].data.fd;
-
-            if (m_b_sysvar_internal_thread_arm_cq_enabled && fd == m_cq_epfd) {
-                continue;
-            }
-
             evh_logfunc("Processing fd %d", fd);
 
             if (is_wakeup_fd(fd)) { // the pipe was already handled
@@ -1023,9 +975,7 @@ void *event_handler_manager::thread_loop()
             event_handler_map_t::iterator i = m_event_handler_map.find(fd);
             if (i == m_event_handler_map.end()) {
                 // No event handler - this is probably a poll_os event!
-                if (!g_p_fd_collection->set_immediate_os_sample(fd)) {
-                    evh_logdbg("No event handler (fd=%d)", fd);
-                }
+                evh_logdbg("No event handler (fd=%d)", fd);
                 continue;
             }
 
