@@ -28,9 +28,9 @@ typedef std::map<void * /*event_handler_id*/, event_handler_rdma_cm * /*p_event_
 
 typedef enum {
     REGISTER_TCP_SOCKET_TIMER,
+    UNREGISTER_TCP_SOCKET_TIMER,
     UNREGISTER_TCP_SOCKET_TIMER_AND_DELETE,
     REGISTER_TIMER,
-    WAKEUP_TIMER, /* NOT AVAILABLE FOR GROUPED TIMERS */
     UNREGISTER_TIMER,
     UNREGISTER_TIMERS_AND_DELETE,
     REGISTER_IBVERBS,
@@ -162,11 +162,11 @@ public:
 
     void *register_timer_event(int timeout_msec, timer_handler *handler, timer_req_type_t req_type,
                                void *user_data);
-    void wakeup_timer_event(timer_handler *handler, void *node);
     void unregister_timer_event(timer_handler *handler, void *node);
     void unregister_timers_event_and_delete(timer_handler *handler);
 
     void register_socket_timer_event(sockinfo_tcp *sock_tcp);
+    void unregister_socket_timer_event(sockinfo_tcp *sock_tcp);
     void unregister_socket_timer_and_delete(sockinfo_tcp *sock_tcp);
 
     void register_ibverbs_event(int fd, event_handler_ibverbs *handler, void *channel,
@@ -190,7 +190,6 @@ public:
 protected:
     pthread_t m_event_handler_tid;
     bool m_b_continue_running = false;
-    int m_cq_epfd;
     int m_epfd;
 
     // pipe for the event registration handling
@@ -201,13 +200,11 @@ protected:
     lock_spin m_reg_action_q_lock;
     timer m_timer;
 
-    const bool m_b_sysvar_internal_thread_arm_cq_enabled;
     const uint32_t m_n_sysvar_timer_resolution_msec;
 
     event_handler_map_t m_event_handler_map;
 
     void priv_register_timer_handler(timer_reg_info_t &info);
-    void priv_wakeup_timer_handler(timer_reg_info_t &info);
     void priv_unregister_timer_handler(timer_reg_info_t &info);
     void priv_unregister_all_handler_timers(timer_reg_info_t &info);
     void priv_register_ibverbs_events(ibverbs_reg_info_t &info);
