@@ -29,11 +29,12 @@
 static std::vector<poll_group *> s_poll_groups;
 static lock_spin s_poll_groups_lock;
 
-poll_group::poll_group(const struct xlio_poll_group_attr *attr)
-    : m_socket_event_cb(attr->socket_event_cb)
-    , m_socket_comp_cb(attr->socket_comp_cb)
-    , m_socket_rx_cb(attr->socket_rx_cb)
-    , m_group_flags(attr->flags)
+poll_group::poll_group(const struct xlio_poll_group_attr &attr)
+    : m_socket_event_cb(attr.socket_event_cb)
+    , m_socket_comp_cb(attr.socket_comp_cb)
+    , m_socket_rx_cb(attr.socket_rx_cb)
+    , m_socket_accept_cb(attr.socket_accept_cb)
+    , m_group_flags(attr.flags)
 {
     /*
      * In the best case, we expect a single ring per group. Reserve two elements for a scenario
@@ -53,8 +54,23 @@ poll_group::poll_group(const struct xlio_poll_group_attr *attr)
     grp_logdbg("Polling group %p created", this);
 }
 
+/**
+ * @brief Destructor for poll_group class
+ *
+ * Cleans up all resources associated with the poll_group object
+ *
+ * @note The coverity[UNCAUGHT_EXCEPT] was added as it's a False Positive
+ */
+// coverity[UNCAUGHT_EXCEPT]
 poll_group::~poll_group()
 {
+    //destroy all pending to remove sockets
+    while (!m_pending_to_remove_lst.empty()) {
+        sockinfo_tcp *si = m_pending_to_remove_lst.front();
+        m_pending_to_remove_lst.pop_front();
+        si->clean_socket_obj();
+    }
+
     s_poll_groups_lock.lock();
     auto iter = std::find(s_poll_groups.begin(), s_poll_groups.end(), this);
     if (iter != std::end(s_poll_groups)) {
@@ -89,6 +105,14 @@ void poll_group::destroy_all_groups()
     }
 }
 
+/*static*/
+void poll_group::fork_nullify()
+{
+    s_poll_groups_lock.lock();
+    s_poll_groups.clear();
+    s_poll_groups_lock.unlock();
+}
+
 int poll_group::update(const struct xlio_poll_group_attr *attr)
 {
     if (m_group_flags != attr->flags) {
@@ -100,19 +124,48 @@ int poll_group::update(const struct xlio_poll_group_attr *attr)
     m_socket_event_cb = attr->socket_event_cb;
     m_socket_comp_cb = attr->socket_comp_cb;
     m_socket_rx_cb = attr->socket_rx_cb;
+    m_socket_accept_cb = attr->socket_accept_cb;
 
     return 0;
 }
 
-void poll_group::poll()
+bool poll_group::poll()
 {
+    if (unlikely(m_is_slow_path)) {
+        slow_path_run();
+        m_is_slow_path = false;
+    }
+
+    int empty_poll = -1;
     for (ring *rng : m_rings) {
         uint64_t sn = 0;
-        rng->poll_and_process_element_tx(&sn);
+        empty_poll = std::max(empty_poll, rng->poll_and_process_element_tx(&sn));
         sn = 0;
-        rng->poll_and_process_element_rx(&sn);
+        empty_poll = std::max(empty_poll, rng->poll_and_process_element_rx(&sn));
     }
     m_event_handler->do_tasks();
+
+    return !!(empty_poll + 1);
+}
+
+void poll_group::slow_path_run()
+{
+    for (auto &iter : m_slow_path_sockets) {
+        sockinfo_tcp *si = iter.second;
+
+        switch (iter.first) {
+        case POLL_GROUP_SOCKET_CLOSE:
+            close_socket(si);
+            break;
+        case POLL_GROUP_SOCKET_DESTROY:
+            m_pending_to_remove_lst.remove(si);
+            si->clean_socket_obj();
+            break;
+        default:
+            grp_logerr("Unknown operation in the slow path: %d.", iter.first);
+        }
+    }
+    m_slow_path_sockets.clear();
 }
 
 void poll_group::add_dirty_socket(sockinfo_tcp *si)
@@ -136,6 +189,7 @@ void poll_group::add_ring(ring *rng, ring_alloc_logic_attr *attr)
     if (std::find(m_rings.begin(), m_rings.end(), rng) == std::end(m_rings)) {
         grp_logdbg("New ring %p in group %p", rng, this);
         m_rings.push_back(rng);
+        rng->set_poll_group(this);
 
         /*
          * Take reference to the ring. This avoids a race between socket destruction and buffer
@@ -160,34 +214,61 @@ void poll_group::add_ring(ring *rng, ring_alloc_logic_attr *attr)
     }
 }
 
-void poll_group::add_socket(sockinfo_tcp *si)
+void poll_group::add_socket_helper(sockinfo_tcp *si)
 {
     m_sockets_list.push_back(si);
+}
+
+void poll_group::add_socket(sockinfo_tcp *si)
+{
+    add_socket_helper(si);
     // For the flow_tag fast path support.
     g_p_fd_collection->set_socket(si->get_fd(), si);
 }
 
-void poll_group::close_socket(sockinfo_tcp *si, bool force /*=false*/)
+void poll_group::remove_socket(sockinfo_tcp *si)
 {
-    g_p_fd_collection->clear_socket(si->get_fd());
     m_sockets_list.erase(si);
-
     auto iter = std::find(m_dirty_sockets.begin(), m_dirty_sockets.end(), si);
     if (iter != std::end(m_dirty_sockets)) {
         m_dirty_sockets.erase(iter);
     }
+}
 
-    bool closed = si->prepare_to_close(force);
-    if (closed) {
-        /*
-         * Current implementation forces TCP reset, so the socket is expected to be closable.
-         * Do a polling iteration to increase the chance that all the relevant WQEs are completed
-         * and XLIO emitted all the TX completion before the XLIO_SOCKET_EVENT_TERMINATED event.
-         *
-         * TODO Implement more reliable mechanism of deferred socket destruction if there are
-         * not completed TX operations.
-         */
-        poll();
+void poll_group::reuse_sockfd(int fd, sockinfo_tcp *si)
+{
+    m_pending_to_remove_lst.remove(si);
+    g_p_fd_collection->set_socket(fd, si);
+    m_sockets_list.push_back(si);
+}
+
+void poll_group::close_socket(sockinfo_tcp *si, bool force /*=false*/)
+{
+    int fd = si->get_fd();
+    close_socket_helper(si, force);
+    g_p_fd_collection->clear_socket(fd);
+}
+
+void poll_group::close_socket_helper(sockinfo_tcp *si, bool force /*=false*/)
+{
+    remove_socket(si);
+    if (si->prepare_to_close(force)) {
+        // The socket is already closable.
         si->clean_socket_obj();
+    } else {
+        // The socket is not ready for close.
+        m_pending_to_remove_lst.push_back(si);
     }
 }
+
+void poll_group::mark_socket_to_close(sockinfo_tcp *si)
+{
+    m_slow_path_sockets.push_back(std::make_pair(POLL_GROUP_SOCKET_CLOSE, si));
+    m_is_slow_path = true;
+}
+
+void poll_group::mark_socket_to_destroy(sockinfo_tcp *si)
+{
+    m_slow_path_sockets.push_back(std::make_pair(POLL_GROUP_SOCKET_DESTROY, si));
+    m_is_slow_path = true;
+}
