@@ -52,6 +52,7 @@
 
 tcp_tx_pbuf_alloc_fn external_tcp_tx_pbuf_alloc;
 tcp_tx_pbuf_free_fn external_tcp_tx_pbuf_free;
+tcp_rx_pbuf_free_fn external_tcp_rx_pbuf_free;
 tcp_seg_alloc_fn external_tcp_seg_alloc;
 tcp_seg_free_fn external_tcp_seg_free;
 /* allow user to be notified upon tcp_state changes */
@@ -67,6 +68,11 @@ void register_tcp_tx_pbuf_free(tcp_tx_pbuf_free_fn fn)
     external_tcp_tx_pbuf_free = fn;
 }
 
+void register_tcp_rx_pbuf_free(tcp_rx_pbuf_free_fn fn)
+{
+    external_tcp_rx_pbuf_free = fn;
+}
+
 void register_tcp_seg_alloc(tcp_seg_alloc_fn fn)
 {
     external_tcp_seg_alloc = fn;
@@ -87,8 +93,6 @@ enum cc_algo_mod lwip_cc_algo_module = CC_MOD_LWIP;
 u16_t lwip_tcp_mss = CONST_TCP_MSS;
 u8_t enable_push_flag = 1;
 u8_t enable_ts_option = 0;
-u32_t lwip_tcp_snd_buf = 0;
-u32_t lwip_zc_tx_size = 0;
 u32_t lwip_tcp_nodelay_treshold = 0;
 
 /* slow timer value */
@@ -145,7 +149,7 @@ static err_t tcp_close_shutdown(struct tcp_pcb *pcb, u8_t rst_on_unacked_data)
 
     if (rst_on_unacked_data &&
         ((get_tcp_state(pcb) == ESTABLISHED) || (get_tcp_state(pcb) == CLOSE_WAIT))) {
-        if ((pcb->refused_data != NULL) || (pcb->rcv_wnd != pcb->rcv_wnd_max)) {
+        if (pcb->rcv_wnd != pcb->rcv_wnd_max) {
             /* Not all data received by application, send RST to tell the remote
                side about this. */
             LWIP_ASSERT("pcb->flags & TF_RXCLOSED", pcb->flags & TF_RXCLOSED);
@@ -279,11 +283,6 @@ err_t tcp_shutdown(struct tcp_pcb *pcb, int shut_rx, int shut_tx)
             /* shutting down the tx AND rx side is the same as closing for the raw API */
             return tcp_close_shutdown(pcb, 1);
         }
-        /* ... and free buffered data */
-        if (pcb->refused_data != NULL) {
-            pbuf_free(pcb->refused_data);
-            pcb->refused_data = NULL;
-        }
     }
     if (shut_tx) {
         /* This can't happen twice since if it succeeds, the pcb's state is changed.
@@ -321,7 +320,7 @@ void tcp_abandon(struct tcp_pcb *pcb, int reset)
     if (get_tcp_state(pcb) == TIME_WAIT) {
         tcp_pcb_remove(pcb);
     } else {
-        int send_rst = reset && (get_tcp_state(pcb) != CLOSED);
+        int send_rst = reset && (get_tcp_state(pcb) != CLOSED) && (get_tcp_state(pcb) != SYN_RCVD);
         void *errf_arg = pcb->my_container;
         tcp_err_fn errf = pcb->errf;
 
@@ -438,6 +437,7 @@ u32_t tcp_update_rcv_ann_wnd(struct tcp_pcb *pcb)
                     pcb->rcv_ann_right_edge + LWIP_MIN((pcb->rcv_wnd_max / 2), pcb->mss))) {
         /* we can advertise more window */
         pcb->rcv_ann_wnd = pcb->rcv_wnd;
+        // coverity[overflow:FALSE] /* Turn off coverity check, value is non-negative */
         return new_right_edge - pcb->rcv_ann_right_edge;
     } else {
         if (TCP_SEQ_GT(pcb->rcv_nxt, pcb->rcv_ann_right_edge)) {
@@ -462,7 +462,7 @@ u32_t tcp_update_rcv_ann_wnd(struct tcp_pcb *pcb)
  * @param pcb the tcp_pcb for which data is read
  * @param len the amount of bytes that have been read by the application
  */
-void tcp_recved(struct tcp_pcb *pcb, u32_t len)
+bool tcp_recved(struct tcp_pcb *pcb, u32_t len, bool do_output)
 {
     u32_t wnd_inflation;
 
@@ -485,18 +485,23 @@ void tcp_recved(struct tcp_pcb *pcb, u32_t len)
 
     wnd_inflation = tcp_update_rcv_ann_wnd(pcb);
 
+    LWIP_DEBUGF(TCP_DEBUG,
+                ("tcp_recved: recveived %" U16_F " bytes, wnd %" U16_F " (%" U16_F ").\n", len,
+                 pcb->rcv_wnd, TCP_WND_SCALED(pcb) - pcb->rcv_wnd));
+
     /* If the change in the right edge of window is significant (default
      * watermark is TCP_WND/4), then send an explicit update now.
      * Otherwise wait for a packet to be sent in the normal course of
      * events (or more window to be available later) */
     if (wnd_inflation >= TCP_WND_UPDATE_THRESHOLD) {
         tcp_ack_now(pcb);
-        tcp_output(pcb);
+        if (do_output) {
+            tcp_output(pcb);
+        }
+        return true;
     }
 
-    LWIP_DEBUGF(TCP_DEBUG,
-                ("tcp_recved: recveived %" U16_F " bytes, wnd %" U16_F " (%" U16_F ").\n", len,
-                 pcb->rcv_wnd, TCP_WND_SCALED(pcb) - pcb->rcv_wnd));
+    return false;
 }
 
 /**
@@ -547,7 +552,7 @@ err_t tcp_connect(struct tcp_pcb *pcb, const ip_addr_t *ipaddr, u16_t port, bool
     pcb->snd_wnd = TCP_WND;
 
     pcb->advtsd_mss = tcp_send_mss(pcb);
-    UPDATE_PCB_BY_MSS(pcb, pcb->advtsd_mss);
+    pcb->mss = pcb->advtsd_mss;
     pcb->cwnd = 1;
     pcb->ssthresh = pcb->mss * 10;
     pcb->connected = connected;
@@ -703,7 +708,7 @@ void tcp_slowtmr(struct tcp_pcb *pcb)
                                     pcb->remote_ip, pcb->is_ipv6);
 
                 ++pcb_remove;
-                err = ERR_ABRT;
+                err = ERR_TIMEOUT;
                 ++pcb_reset;
             }
 #if LWIP_TCP_KEEPALIVE
@@ -783,39 +788,14 @@ void tcp_slowtmr(struct tcp_pcb *pcb)
 }
 
 /**
- * Is called every slow_tmr_interval and process data previously
- * "refused" by upper layer (application) and sends delayed ACKs.
- *
+ * Is called every slow_tmr_interval/2 and process data previously * and sends delayed ACKs.
  * Automatically called from tcp_tmr().
  */
 void tcp_fasttmr(struct tcp_pcb *pcb)
 {
     if (pcb != NULL && PCB_IN_ACTIVE_STATE(pcb)) {
-        /* If there is data which was previously "refused" by upper layer */
-        while (pcb->refused_data !=
-               NULL) { // 'while' instead of 'if' because windows scale uses large pbuf
-            struct pbuf *rest;
-            /* Notify again application with data previously received. */
-            err_t err;
-            pbuf_split_64k(pcb->refused_data, &rest);
-            LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_fasttmr: notify kept packet\n"));
-            TCP_EVENT_RECV(pcb, pcb->refused_data, ERR_OK, err);
-            if (err == ERR_OK) {
-                pcb->refused_data = rest;
-            } else {
-                if (rest) {
-                    pbuf_cat(pcb->refused_data, rest); /* undo splitting */
-                }
-                if (err == ERR_ABRT) {
-                    /* if err == ERR_ABRT, 'pcb' is already deallocated */
-                    pcb = NULL;
-                }
-                break;
-            }
-        }
-
         /* send delayed ACKs */
-        if (pcb && (pcb->flags & TF_ACK_DELAY)) {
+        if (pcb->flags & TF_ACK_DELAY) {
             LWIP_DEBUGF(TCP_DEBUG, ("tcp_fasttmr: delayed ACK\n"));
             tcp_ack_now(pcb);
             tcp_output(pcb);
@@ -917,7 +897,7 @@ err_t tcp_recv_null(void *arg, struct tcp_pcb *pcb, struct pbuf *p, err_t err)
 {
     LWIP_UNUSED_ARG(arg);
     if (p != NULL) {
-        tcp_recved(pcb, (u32_t)p->tot_len);
+        tcp_recved(pcb, (u32_t)p->tot_len, true);
         pbuf_free(p);
     } else if (err == ERR_OK) {
         return tcp_close(pcb);
@@ -931,11 +911,8 @@ void tcp_pcb_init(struct tcp_pcb *pcb, u8_t prio, void *container)
 
     memset(pcb, 0, sizeof(*pcb));
     pcb->my_container = container;
-    pcb->max_snd_buff = TCP_SND_BUF;
     pcb->is_last_seg_dropped = false;
     pcb->prio = prio;
-    pcb->snd_buf = pcb->max_snd_buff;
-    pcb->snd_queuelen = 0;
     pcb->snd_scale = 0;
     pcb->rcv_scale = 0;
     pcb->rcv_wnd = TCP_WND_SCALED(pcb);
@@ -946,9 +923,8 @@ void tcp_pcb_init(struct tcp_pcb *pcb, u8_t prio, void *container)
     pcb->ttl = TCP_TTL;
     pcb->is_ipv6 = false;
 
-    u16_t snd_mss = pcb->advtsd_mss = tcp_initial_mss(pcb);
-    UPDATE_PCB_BY_MSS(pcb, snd_mss);
-    pcb->max_unsent_len = pcb->max_tcp_snd_queuelen;
+    pcb->advtsd_mss = tcp_initial_mss(pcb);
+    pcb->mss = pcb->advtsd_mss;
     pcb->user_timeout_ms = 0;
     pcb->ticks_since_data_sent = -1;
     pcb->rto = 3000 / slow_tmr_interval;
@@ -996,7 +972,6 @@ void tcp_pcb_init(struct tcp_pcb *pcb, u8_t prio, void *container)
     pcb->is_in_input = 0;
     pcb->enable_ts_opt = enable_ts_option;
     pcb->seg_alloc = NULL;
-    pcb->pbuf_alloc = NULL;
 }
 
 /**
@@ -1008,8 +983,6 @@ void tcp_pcb_recycle(struct tcp_pcb *pcb)
     u32_t iss;
 
     pcb->flags = 0;
-    pcb->max_snd_buff = TCP_SND_BUF;
-    pcb->snd_buf = pcb->max_snd_buff;
     pcb->user_timeout_ms = 0;
     pcb->ticks_since_data_sent = -1;
     pcb->rto = 3000 / slow_tmr_interval;
@@ -1037,69 +1010,41 @@ void tcp_pcb_recycle(struct tcp_pcb *pcb)
     pcb->keep_cnt_sent = 0;
     pcb->quickack = 0;
     pcb->is_in_input = 0;
-    pcb->snd_queuelen = 0;
     pcb->snd_scale = 0;
     pcb->rcv_scale = 0;
     pcb->last_unsent = NULL;
     pcb->last_unacked = NULL;
-    pcb->unsent_oversize = 0;
     if (pcb->seg_alloc != NULL) {
         tcp_tx_seg_free(pcb, pcb->seg_alloc);
         pcb->seg_alloc = NULL;
     }
-    if (pcb->pbuf_alloc) {
-        tcp_tx_pbuf_free(pcb, pcb->pbuf_alloc);
-        pcb->pbuf_alloc = NULL;
-    }
-    if (pcb->refused_data) {
-        pbuf_free(pcb->refused_data);
-        pcb->refused_data = NULL;
-    }
 }
 
-struct pbuf *tcp_tx_pbuf_alloc(struct tcp_pcb *pcb, u16_t length, pbuf_type type, pbuf_desc *desc,
+struct pbuf *tcp_tx_pbuf_alloc(struct tcp_pcb *pcb, u32_t length, pbuf_type type, pbuf_desc *desc,
                                struct pbuf *p_buff)
 {
     struct pbuf *p;
 
-    if (!pcb->pbuf_alloc || pcb->pbuf_alloc->type != type) {
-
-        // pbuf_alloc is not valid, we should allocate a new pbuf.
-        p = external_tcp_tx_pbuf_alloc(pcb, type, desc, p_buff);
-        if (!p) {
-            return NULL;
-        }
-
-        p->next = NULL;
-        p->type = type;
-        /* set reference count */
-        p->ref = 1;
-        /* set flags */
-        p->flags = 0;
-    } else {
-        // pbuf_alloc is valid, we dont need to allocate a new pbuf element.
-        p = pcb->pbuf_alloc;
-        pcb->pbuf_alloc = NULL;
+    p = external_tcp_tx_pbuf_alloc(pcb, type, desc, p_buff);
+    if (!p) {
+        return NULL;
     }
 
-    /* Set up internal structure of the pbuf. */
+    p->next = NULL;
+    p->type = type;
+    p->ref = 1;
+    p->flags = 0;
     p->len = p->tot_len = length;
 
     return p;
 }
 
-// Release preallocated buffers
 void tcp_tx_preallocted_buffers_free(struct tcp_pcb *pcb)
 {
     if (pcb->seg_alloc) {
         tcp_tx_seg_free(pcb, pcb->seg_alloc);
         pcb->seg_alloc = NULL;
     }
-
-    if (pcb->pbuf_alloc) {
-        tcp_tx_pbuf_free(pcb, pcb->pbuf_alloc);
-        pcb->pbuf_alloc = NULL;
-    }
 }
 
 void tcp_tx_pbuf_free(struct tcp_pcb *pcb, struct pbuf *p)
@@ -1146,11 +1091,11 @@ void tcp_recv(struct tcp_pcb *pcb, tcp_recv_fn recv)
  * has been successfully delivered to the remote host.
  *
  * @param pcb tcp_pcb to set the sent callback
- * @param sent callback function to call for this pcb when data is successfully sent
+ * @param acked_cb callback function to call for this pcb when data is acknowledged
  */
-void tcp_sent(struct tcp_pcb *pcb, tcp_sent_fn sent)
+void tcp_acked(struct tcp_pcb *pcb, tcp_acked_fn acked_cb)
 {
-    pcb->sent = sent;
+    pcb->acked_cb = acked_cb;
 }
 
 /**
@@ -1239,11 +1184,6 @@ void tcp_pcb_purge(struct tcp_pcb *pcb)
 
         LWIP_DEBUGF(TCP_DEBUG, ("tcp_pcb_purge\n"));
 
-        if (pcb->refused_data != NULL) {
-            LWIP_DEBUGF(TCP_DEBUG, ("tcp_pcb_purge: data left on ->refused_data\n"));
-            pbuf_free(pcb->refused_data);
-            pcb->refused_data = NULL;
-        }
         if (pcb->unsent != NULL) {
             LWIP_DEBUGF(TCP_DEBUG, ("tcp_pcb_purge: not all data sent\n"));
         }
@@ -1265,10 +1205,7 @@ void tcp_pcb_purge(struct tcp_pcb *pcb)
         tcp_tx_segs_free(pcb, pcb->unsent);
         tcp_tx_segs_free(pcb, pcb->unacked);
         pcb->unacked = pcb->unsent = NULL;
-        pcb->last_unsent = NULL;
-#if TCP_OVERSIZE
-        pcb->unsent_oversize = 0;
-#endif /* TCP_OVERSIZE */
+        pcb->last_unacked = pcb->last_unsent = NULL;
 #if TCP_CC_ALGO_MOD
         cc_destroy(pcb);
 #endif
