@@ -89,8 +89,7 @@ void set_tmr_resolution(u32_t v);
     ((((tpcb)->unacked == NULL) || (((tpcb)->flags & TF_NODELAY) && tcp_nodelay_treshold(tpcb)) || \
       ((tpcb)->flags & TF_INFR) ||                                                                 \
       (((tpcb)->unsent != NULL) &&                                                                 \
-       (((tpcb)->unsent->next != NULL) || ((tpcb)->unsent->len >= (tpcb)->mss))) ||                \
-      ((tcp_sndbuf(tpcb) <= 0) || (tcp_sndqueuelen(tpcb) >= (tpcb)->snd_queuelen_max)))            \
+       (((tpcb)->unsent->next != NULL) || ((tpcb)->unsent->len >= (tpcb)->mss))))                  \
          ? 1                                                                                       \
          : 0)
 #define tcp_output_nagle(tpcb) (tcp_do_output_nagle(tpcb) ? tcp_output(tpcb) : ERR_OK)
@@ -100,9 +99,6 @@ void set_tmr_resolution(u32_t v);
 #define TCP_SEQ_GT(a, b)  ((s32_t)((u32_t)(a) - (u32_t)(b)) > 0)
 #define TCP_SEQ_GEQ(a, b) ((s32_t)((u32_t)(a) - (u32_t)(b)) >= 0)
 /* is b<=a<=c? */
-#if 0 /* see bug #10548 */
-#define TCP_SEQ_BETWEEN(a, b, c) ((c) - (b) >= (a) - (b))
-#endif
 #define TCP_SEQ_BETWEEN(a, b, c) (TCP_SEQ_GEQ(a, b) && TCP_SEQ_LEQ(a, c))
 #define TCP_FIN                  0x01U
 #define TCP_SYN                  0x02U
@@ -223,10 +219,10 @@ PACK_STRUCT_END
             (pcb)->accepted_pcb((newpcb));                                                         \
     } while (0)
 
-#define TCP_EVENT_SENT(pcb, space, ret)                                                            \
+#define TCP_EVENT_ACKED(pcb, acked, ret)                                                           \
     do {                                                                                           \
-        if ((pcb)->sent != NULL)                                                                   \
-            (ret) = (pcb)->sent((pcb)->callback_arg, (pcb), (space));                              \
+        if ((pcb)->acked_cb != NULL)                                                               \
+            (ret) = (pcb)->acked_cb((pcb)->callback_arg, (pcb), (acked));                          \
         else                                                                                       \
             (ret) = ERR_OK;                                                                        \
     } while (0)
@@ -273,13 +269,12 @@ struct tcp_seg {
     u32_t len; /* the TCP length of this segment should allow >64K size */
 
     u8_t flags;
-#define TF_SEG_OPTS_MSS       (u8_t)0x01U /* Include MSS option. */
-#define TF_SEG_OPTS_TS        (u8_t)0x02U /* Include timestamp option. */
-#define TF_SEG_OPTS_WNDSCALE  (u8_t)0x08U /* Include window scaling option */
-#define TF_SEG_OPTS_DUMMY_MSG (u8_t) TCP_WRITE_DUMMY /* Include dummy send option */
-#define TF_SEG_OPTS_TSO       (u8_t) TCP_WRITE_TSO /* Use TSO send mode */
-#define TF_SEG_OPTS_NOMERGE   (u8_t)0x40U /* Don't merge with other segments */
-#define TF_SEG_OPTS_ZEROCOPY  (u8_t) TCP_WRITE_ZEROCOPY /* Use zerocopy send mode */
+#define TF_SEG_OPTS_MSS      (u8_t)0x01U /* Include MSS option. */
+#define TF_SEG_OPTS_TS       (u8_t)0x02U /* Include timestamp option. */
+#define TF_SEG_OPTS_WNDSCALE (u8_t)0x08U /* Include window scaling option */
+#define TF_SEG_OPTS_TSO      (u8_t) TCP_WRITE_TSO /* Use TSO send mode */
+#define TF_SEG_OPTS_NOMERGE  (u8_t)0x40U /* Don't merge with other segments */
+#define TF_SEG_OPTS_ZEROCOPY (u8_t) TCP_WRITE_ZEROCOPY /* Use zerocopy send mode */
 
     u8_t tcp_flags; /* Cached TCP flags for outgoing segments */
 
@@ -293,8 +288,6 @@ struct tcp_seg {
     u32_t l2_l3_tcphdr_zc[25] __attribute__((aligned(4)));
 };
 
-#define LWIP_IS_DUMMY_SEGMENT(seg) (seg->flags & TF_SEG_OPTS_DUMMY_MSG)
-
 #if LWIP_TCP_TIMESTAMPS
 #define LWIP_TCP_OPT_LEN_TS 12U
 #endif
