@@ -99,7 +99,7 @@ static inline void fill_parsed_ip_hdr(const void *payload, parsed_ip_hdr_t *iphd
     const u8_t *view_8bit = (const u8_t *)payload;
     const u16_t *view_16bit = (const u16_t *)payload;
 
-    iphdr->is_ipv6 = (view_8bit[0] >> 4U) == IPV6_VERSION;
+    iphdr->is_ipv6 = (view_8bit[0] >> 4U) == XLIO_IPV6_VERSION;
     if (iphdr->is_ipv6) {
         iphdr->src = (void *)&view_8bit[8];
         iphdr->dest = (void *)&view_8bit[24];
@@ -202,7 +202,7 @@ void L3_level_tcp_input(struct pbuf *p, struct tcp_pcb *pcb)
                        called when new send buffer space is available, we call it
                        now. */
                     if (pcb->acked > 0) {
-                        TCP_EVENT_SENT(pcb, pcb->acked, err);
+                        TCP_EVENT_ACKED(pcb, pcb->acked, err);
                         if (err == ERR_ABRT) {
                             goto aborted;
                         }
@@ -365,7 +365,7 @@ static void tcp_listen_input(struct tcp_pcb *pcb, tcp_in_data *in_data)
         npcb->ssthresh = npcb->snd_wnd;
 #if TCP_CALCULATE_EFF_SEND_MSS
         // mss can be changed by tcp_parseopt, need to take the MIN
-        UPDATE_PCB_BY_MSS(npcb, LWIP_MIN(npcb->mss, npcb->advtsd_mss));
+        npcb->mss = LWIP_MIN(npcb->mss, npcb->advtsd_mss);
 #endif /* TCP_CALCULATE_EFF_SEND_MSS */
 
         /* Register the new PCB so that we can begin sending segments
@@ -410,7 +410,7 @@ static err_t tcp_pcb_reuse(struct tcp_pcb *pcb, tcp_in_data *in_data)
     pcb->ssthresh = pcb->snd_wnd;
 
     // mss can be changed by tcp_parseopt, need to take the MIN
-    UPDATE_PCB_BY_MSS(pcb, LWIP_MIN(pcb->mss, pcb->advtsd_mss));
+    pcb->mss = LWIP_MIN(pcb->mss, pcb->advtsd_mss);
     rc = pcb->syn_tw_handled_cb(pcb->listen_sock, pcb);
     if (rc != ERR_OK) {
         return rc;
@@ -563,7 +563,6 @@ static err_t tcp_process(struct tcp_pcb *pcb, tcp_in_data *in_data)
         /* received SYN ACK with expected sequence number? */
         if ((in_data->flags & TCP_ACK) && (in_data->flags & TCP_SYN) &&
             in_data->ackno == pcb->unacked->seqno + 1) {
-            // pcb->snd_buf++; SND_BUF_FOR_SYN_FIN
             pcb->rcv_nxt = in_data->seqno + 1;
             pcb->rcv_ann_right_edge = pcb->rcv_nxt;
             pcb->lastack = in_data->ackno;
@@ -575,7 +574,7 @@ static err_t tcp_process(struct tcp_pcb *pcb, tcp_in_data *in_data)
 
 #if TCP_CALCULATE_EFF_SEND_MSS
             // mss can be changed by tcp_parseopt, need to take the MIN
-            UPDATE_PCB_BY_MSS(pcb, LWIP_MIN(pcb->mss, tcp_send_mss(pcb)));
+            pcb->mss = LWIP_MIN(pcb->mss, tcp_send_mss(pcb));
 #endif /* TCP_CALCULATE_EFF_SEND_MSS */
 
             /* Set ssthresh again after changing pcb->mss (already set in tcp_connect
@@ -586,17 +585,13 @@ static err_t tcp_process(struct tcp_pcb *pcb, tcp_in_data *in_data)
 #else
             pcb->cwnd = ((pcb->cwnd == 1) ? (pcb->mss * 2) : pcb->mss);
 #endif
-            LWIP_ASSERT("pcb->snd_queuelen > 0", (pcb->snd_queuelen > 0));
-            --pcb->snd_queuelen;
-            LWIP_DEBUGF(
-                TCP_QLEN_DEBUG,
-                ("tcp_process: SYN-SENT --queuelen %" U16_F "\n", (u16_t)pcb->snd_queuelen));
             rseg = pcb->unacked;
             pcb->unacked = rseg->next;
 
             /* If there's nothing left to acknowledge, stop the retransmit
                timer, otherwise reset it to start again */
             if (pcb->unacked == NULL) {
+                pcb->last_unacked = NULL;
                 pcb->rtime = -1;
                 pcb->ticks_since_data_sent = -1;
             } else {
@@ -784,12 +779,11 @@ static void tcp_oos_insert_segment(struct tcp_pcb *pcb, struct tcp_seg *cseg, st
  * @param ackqno current ackqno
  * @return number of freed pbufs
  */
-static u32_t tcp_shrink_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t ackno)
+static void tcp_shrink_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t ackno)
 {
     struct pbuf *cur_p;
     struct pbuf *p;
     u32_t len;
-    u32_t count = 0;
     u8_t optflags = 0;
     u8_t optlen;
 
@@ -822,7 +816,7 @@ static u32_t tcp_shrink_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t
         p->payload = (u8_t *)p->payload + len;
         memmove(p->payload, seg->tcphdr, TCP_HLEN);
         seg->tcphdr = p->payload;
-        return count;
+        return;
     }
 
     cur_p = seg->p->next;
@@ -856,7 +850,6 @@ static u32_t tcp_shrink_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t
             } else {
                 pbuf_free(p);
             }
-            count++;
         }
     }
 
@@ -889,15 +882,12 @@ static u32_t tcp_shrink_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t
         } else {
             pbuf_free(p);
         }
-        count++;
     }
 
 #if TCP_TSO_DEBUG
     LWIP_DEBUGF(TCP_TSO_DEBUG | LWIP_DBG_TRACE,
-                ("tcp_shrink: count: %-5d unsent %s\n", count, _dump_seg(pcb->unsent)));
+                ("tcp_shrink: unsent %s\n", _dump_seg(pcb->unsent)));
 #endif /* TCP_TSO_DEBUG */
-
-    return count;
 }
 
 /**
@@ -909,10 +899,9 @@ static u32_t tcp_shrink_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t
  * @param ackqno current ackqno
  * @return number of freed pbufs
  */
-static u32_t tcp_shrink_zc_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t ackno)
+static void tcp_shrink_zc_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t ackno)
 {
     struct pbuf *p;
-    u32_t count = 0;
     u32_t len;
 
     assert(seg != NULL);
@@ -927,7 +916,6 @@ static u32_t tcp_shrink_zc_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32
         assert(seg->p != NULL);
         /* XXX Maybe we will need to free zeropcopy pbuf in different way */
         external_tcp_tx_pbuf_free(pcb, p);
-        ++count;
     }
     if (TCP_SEQ_GT(ackno, seg->seqno)) {
         len = ackno - seg->seqno;
@@ -938,8 +926,6 @@ static u32_t tcp_shrink_zc_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32
         seg->seqno = ackno;
     }
     seg->tcphdr->seqno = htonl(seg->seqno);
-
-    return count;
 }
 
 static void ack_partial_or_whole_segment(struct tcp_pcb *pcb, u32_t ackno, struct tcp_seg **seg)
@@ -957,10 +943,11 @@ static void ack_partial_or_whole_segment(struct tcp_pcb *pcb, u32_t ackno, struc
                 break;
             }
             // Ack partial TCP segment
-            u32_t removed = (*seg)->flags & TF_SEG_OPTS_ZEROCOPY
-                ? tcp_shrink_zc_segment(pcb, (*seg), ackno)
-                : tcp_shrink_segment(pcb, (*seg), ackno);
-            pcb->snd_queuelen -= removed;
+            if ((*seg)->flags & TF_SEG_OPTS_ZEROCOPY) {
+                tcp_shrink_zc_segment(pcb, (*seg), ackno);
+            } else {
+                tcp_shrink_segment(pcb, (*seg), ackno);
+            }
             break;
         }
 
@@ -972,7 +959,6 @@ static void ack_partial_or_whole_segment(struct tcp_pcb *pcb, u32_t ackno, struc
             pcb->acked--;
         }
 
-        pcb->snd_queuelen -= pbuf_clen(whole_seg_to_ack->p);
         tcp_tx_seg_free(pcb, whole_seg_to_ack);
     }
 }
@@ -1129,8 +1115,6 @@ static void tcp_receive(struct tcp_pcb *pcb, tcp_in_data *in_data)
             /* Update the send buffer space.*/
             pcb->acked = in_data->ackno - pcb->lastack;
 
-            pcb->snd_buf += pcb->acked;
-
             /* Reset the fast retransmit variables. */
             pcb->dupacks = 0;
             pcb->lastack = in_data->ackno;
@@ -1473,8 +1457,8 @@ static void tcp_receive(struct tcp_pcb *pcb, tcp_in_data *in_data)
                     if (TCPH_FLAGS(cseg->tcphdr) & TCP_FIN) {
                         LWIP_DEBUGF(TCP_INPUT_DEBUG, ("tcp_receive: dequeued FIN.\n"));
                         in_data->recv_flags |= TF_GOT_FIN;
-                        if (get_tcp_state(pcb) ==
-                            ESTABLISHED) { /* force passive close or we can move to active close */
+                        /* force passive close or we can move to active close */
+                        if (get_tcp_state(pcb) == ESTABLISHED) {
                             set_tcp_state(pcb, CLOSE_WAIT);
                         }
                     }
@@ -1706,10 +1690,11 @@ static bool tcp_parseopt_ts(u8_t *opts, u16_t opts_len, u32_t *tsval)
  */
 static void tcp_parseopt(struct tcp_pcb *pcb, tcp_in_data *in_data)
 {
-    u16_t c, max_c;
+    u16_t c;
+    u16_t max_c;
     u16_t mss;
-    u16_t snd_mss;
-    u8_t *opts, opt;
+    u8_t *opts;
+    u8_t opt;
 #if LWIP_TCP_TIMESTAMPS
     u32_t tsval;
 #endif
@@ -1743,8 +1728,7 @@ static void tcp_parseopt(struct tcp_pcb *pcb, tcp_in_data *in_data)
                     /* An MSS option with the right option length. */
                     mss = (opts[c + 2] << 8) | opts[c + 3];
                     /* Limit the mss to the configured TCP_MSS and prevent division by zero */
-                    snd_mss = ((mss > pcb->advtsd_mss) || (mss == 0)) ? pcb->advtsd_mss : mss;
-                    UPDATE_PCB_BY_MSS(pcb, snd_mss);
+                    pcb->mss = ((mss > pcb->advtsd_mss) || (mss == 0)) ? pcb->advtsd_mss : mss;
                 }
                 /* Advance to next option */
                 c += 0x04;
