@@ -241,13 +241,12 @@ static struct pbuf *tcp_pbuf_prealloc(u32_t length, struct tcp_pcb *pcb, pbuf_ty
     return p;
 }
 
-/** Checks if tcp_write is allowed or not (checks state, snd_buf and snd_queuelen).
+/** Checks if tcp_write is allowed or not ().
  *
  * @param pcb the tcp pcb to check for
- * @param len length of data to send (checked agains snd_buf)
  * @return ERR_OK if tcp_write is allowed to proceed, another err_t otherwise
  */
-static err_t tcp_write_checks(struct tcp_pcb *pcb, u32_t len)
+static err_t tcp_write_is_state_valid(struct tcp_pcb *pcb)
 {
     /* connection is in invalid state for data transmission? */
     if ((get_tcp_state(pcb) != ESTABLISHED) && (get_tcp_state(pcb) != CLOSE_WAIT) &&
@@ -255,17 +254,6 @@ static err_t tcp_write_checks(struct tcp_pcb *pcb, u32_t len)
         LWIP_DEBUGF(TCP_OUTPUT_DEBUG | LWIP_DBG_STATE | LWIP_DBG_LEVEL_SEVERE,
                     ("tcp_write() called in invalid state\n"));
         return ERR_CONN;
-    } else if (len == 0) {
-        return ERR_OK;
-    }
-
-    /* If total number of pbufs on the unsent/unacked queues exceeds the
-     * configured maximum, return an error */
-    if (pcb->snd_queuelen >= pcb->snd_queuelen_max) {
-        LWIP_DEBUGF(TCP_OUTPUT_DEBUG | 3,
-                    ("tcp_write: queue exceeds %" U32_F "\n", pcb->snd_queuelen_max));
-        pcb->flags |= TF_NAGLEMEMERR;
-        return ERR_MEM;
     }
     return ERR_OK;
 }
@@ -306,7 +294,6 @@ static inline u32_t tcp_xmit_size_goal(struct tcp_pcb *pcb, int use_max)
  * @param arg Pointer to the data to be enqueued for sending.
  * @param len Data length in bytes
  * @param apiflags combination of following flags:
- * - TCP_WRITE_DUMMY (0x10) indicates if the packet is a dummy packet
  * - TCP_WRITE_FILE (0x40) data should be taken from file
  * @param desc Additional metadata that allows later to check the data mkey/lkey.
  * @return ERR_OK if enqueued, another err_t on error
@@ -317,7 +304,6 @@ err_t tcp_write(struct tcp_pcb *pcb, const void *arg, u32_t len, u16_t apiflags,
     struct tcp_seg *prev_seg = NULL;
     struct tcp_seg *queue = NULL;
     u32_t pos = 0;
-    u32_t queuelen;
     u8_t optlen = 0;
     u8_t optflags = 0;
     err_t err;
@@ -332,7 +318,7 @@ err_t tcp_write(struct tcp_pcb *pcb, const void *arg, u32_t len, u16_t apiflags,
     off_t offset = 0;
     off_t offset_next = 0;
 
-    if (len < pcb->mss && !(apiflags & TCP_WRITE_DUMMY)) {
+    if (len < pcb->mss) {
         const u32_t byte_queued = pcb->snd_nxt - pcb->lastack;
         pcb->snd_sml_add = (pcb->unacked ? pcb->unacked->len : 0) + byte_queued;
     }
@@ -341,7 +327,7 @@ err_t tcp_write(struct tcp_pcb *pcb, const void *arg, u32_t len, u16_t apiflags,
                 ("tcp_write(pcb=%p, data=%p, len=%" U16_F ", apiflags=%" U16_F ")\n", (void *)pcb,
                  arg, len, (u16_t)apiflags));
 
-    err = tcp_write_checks(pcb, len);
+    err = tcp_write_is_state_valid(pcb);
     if (err != ERR_OK) {
         return err;
     }
@@ -351,10 +337,8 @@ err_t tcp_write(struct tcp_pcb *pcb, const void *arg, u32_t len, u16_t apiflags,
         optflags |= TF_SEG_OPTS_TS;
     }
 #endif /* LWIP_TCP_TIMESTAMPS */
-    optflags |= (apiflags & TCP_WRITE_DUMMY) ? TF_SEG_OPTS_DUMMY_MSG : 0;
     optlen = LWIP_TCP_OPT_LENGTH(optflags);
 
-    queuelen = pcb->snd_queuelen;
     mss_local = tcp_xmit_size_goal(pcb, 1);
     if (is_file) {
         offset = offset_next = *(__off64_t *)arg;
@@ -405,12 +389,6 @@ err_t tcp_write(struct tcp_pcb *pcb, const void *arg, u32_t len, u16_t apiflags,
         u32_t left = len - pos;
         u32_t seglen = LWIP_MIN(left, mss_local - optlen);
 
-        if (queuelen >= pcb->snd_queuelen_max) {
-            LWIP_DEBUGF(TCP_OUTPUT_DEBUG | 2,
-                        ("tcp_write: queue exceeds %" U32_F "\n", pcb->snd_queuelen_max));
-            goto memerr;
-        }
-
         p = tcp_pbuf_prealloc(seglen + optlen, pcb, PBUF_RAM, desc, NULL);
         if (!p) {
             LWIP_DEBUGF(TCP_OUTPUT_DEBUG | 2, ("tcp_write: could not allocate pbuf\n"));
@@ -456,7 +434,6 @@ err_t tcp_write(struct tcp_pcb *pcb, const void *arg, u32_t len, u16_t apiflags,
         prev_seg = seg;
 
         pos += seglen;
-        queuelen++; /* There is only one pbuf in the list */
     }
 
     /*
@@ -491,13 +468,6 @@ err_t tcp_write(struct tcp_pcb *pcb, const void *arg, u32_t len, u16_t apiflags,
 
     /* Finally update the pcb state. */
     pcb->snd_lbb += len;
-    pcb->snd_buf -= len;
-    pcb->snd_queuelen = queuelen;
-
-    LWIP_DEBUGF(TCP_QLEN_DEBUG, ("tcp_write: %" S16_F " (after enqueued)\n", pcb->snd_queuelen));
-    if (pcb->snd_queuelen != 0) {
-        LWIP_ASSERT("tcp_write: valid queue length", pcb->unacked != NULL || pcb->unsent != NULL);
-    }
 
     /* Set the PSH flag in the last segment that we enqueued. */
     if (enable_push_flag && seg != NULL && seg->tcphdr != NULL) {
@@ -514,11 +484,6 @@ memerr:
     if (queue != NULL) {
         tcp_tx_segs_free(pcb, queue);
     }
-    if (pcb->snd_queuelen != 0) {
-        LWIP_ASSERT("tcp_write: valid queue length", pcb->unacked != NULL || pcb->unsent != NULL);
-    }
-    LWIP_DEBUGF(TCP_QLEN_DEBUG | LWIP_DBG_STATE,
-                ("tcp_write: %" S16_F " (with mem err)\n", pcb->snd_queuelen));
     return ERR_MEM;
 }
 
@@ -546,7 +511,6 @@ err_t tcp_write_express(struct tcp_pcb *pcb, const struct iovec *iov, u32_t iovc
     u32_t seglen;
     u32_t last_seglen;
     u32_t total_len = 0;
-    u16_t queuelen = 0;
     u8_t optflags = TF_SEG_OPTS_ZEROCOPY;
 
     /*
@@ -602,7 +566,6 @@ err_t tcp_write_express(struct tcp_pcb *pcb, const struct iovec *iov, u32_t iovc
                 pbuf_cat(seg->p, p);
                 seg->len += p->tot_len;
                 pos += seglen;
-                queuelen++;
             }
         }
 
@@ -631,7 +594,6 @@ err_t tcp_write_express(struct tcp_pcb *pcb, const struct iovec *iov, u32_t iovc
             last = seg;
 
             pos += seglen;
-            queuelen++;
         }
 
         total_len += len;
@@ -663,8 +625,6 @@ err_t tcp_write_express(struct tcp_pcb *pcb, const struct iovec *iov, u32_t iovc
 
     /* Update the pcb state. */
     pcb->snd_lbb += total_len;
-    pcb->snd_buf -= total_len;
-    pcb->snd_queuelen += queuelen;
 
     /* TODO Move Minshall's logic to tcp_output(). */
     if (total_len < pcb->mss) {
@@ -721,21 +681,10 @@ err_t tcp_enqueue_flags(struct tcp_pcb *pcb, u8_t flags)
     u8_t optflags = 0;
     u8_t optlen = 0;
 
-    LWIP_DEBUGF(TCP_QLEN_DEBUG,
-                ("tcp_enqueue_flags: queuelen: %" U16_F "\n", (u16_t)pcb->snd_queuelen));
-
     LWIP_ASSERT(
         "tcp_enqueue_flags: need either TCP_SYN or TCP_FIN in flags (programmer violates API)",
         (flags & (TCP_SYN | TCP_FIN)) != 0);
 
-    // Check for max queuelen (FIN flag should always come through).
-    if ((pcb->snd_queuelen >= pcb->snd_queuelen_max) && ((flags & TCP_FIN) == 0)) {
-        LWIP_DEBUGF(TCP_OUTPUT_DEBUG | 3,
-                    ("tcp_enqueue_flags: queue exceeds %" U32_F "\n", pcb->snd_queuelen_max));
-        pcb->flags |= TF_NAGLEMEMERR;
-        return ERR_MEM;
-    }
-
     if (flags & TCP_SYN) {
         optflags = TF_SEG_OPTS_MSS;
         if (enable_wnd_scale && ((get_tcp_state(pcb) != SYN_RCVD) || (pcb->flags & TF_WND_SCALE))) {
@@ -787,18 +736,11 @@ err_t tcp_enqueue_flags(struct tcp_pcb *pcb, u8_t flags)
     /* SYN and FIN bump the sequence number */
     if (flags & (TCP_SYN | TCP_FIN)) {
         pcb->snd_lbb++;
-        /* optlen does not influence snd_buf */
-        pcb->snd_buf--;
     }
     if (flags & TCP_FIN) {
         pcb->flags |= TF_FIN;
     }
 
-    /* update number of segments on the queues */
-    pcb->snd_queuelen += pbuf_clen(seg->p);
-    LWIP_DEBUGF(TCP_QLEN_DEBUG,
-                ("tcp_enqueue_flags: %" S16_F " (after enqueued)\n", pcb->snd_queuelen));
-
     return ERR_OK;
 }
 
@@ -896,7 +838,7 @@ static void tcp_tso_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t wnd)
     /* Ignore retransmitted segments and special segments
      */
     if (TCP_SEQ_LT(seg->seqno, pcb->snd_nxt) ||
-        (seg->flags & (TF_SEG_OPTS_TSO | TF_SEG_OPTS_DUMMY_MSG | TF_SEG_OPTS_NOMERGE)) ||
+        (seg->flags & (TF_SEG_OPTS_TSO | TF_SEG_OPTS_NOMERGE)) ||
         ((TCPH_FLAGS(seg->tcphdr) & (~(TCP_ACK | TCP_PSH))) != 0)) {
         goto err;
     }
@@ -1128,9 +1070,6 @@ void tcp_split_segment(struct tcp_pcb *pcb, struct tcp_seg *seg, u32_t wnd)
             TCPH_SET_FLAG(newseg->tcphdr, TCP_PSH);
         }
 
-        /* Update number of buffer to be send */
-        pcb->snd_queuelen++;
-
         if (pcb->last_unsent == seg) {
             /* We have split the last unsent segment, update last_unsent */
             pcb->last_unsent = newseg;
@@ -1270,6 +1209,7 @@ err_t tcp_output(struct tcp_pcb *pcb)
     if (pcb->is_last_seg_dropped && pcb->unacked && !pcb->unacked->next) {
         /* Forcibly retransmit segment from the unacked queue if it was dropped
          * on the previous iteration.
+         * Disable the retransmission timer after the unacked queue is emptied.
          */
         pcb->is_last_seg_dropped = false;
         pcb->unacked->next = pcb->unsent;
@@ -1279,6 +1219,8 @@ err_t tcp_output(struct tcp_pcb *pcb)
             pcb->last_unsent = pcb->last_unacked;
         }
         pcb->last_unacked = NULL;
+        pcb->rtime = -1;
+        pcb->ticks_since_data_sent = -1;
     }
     seg = pcb->unsent;
 
@@ -1322,7 +1264,6 @@ err_t tcp_output(struct tcp_pcb *pcb)
 
         /* Split the segment in case of a small window */
         if ((NULL == pcb->unacked) && (wnd) && ((seg->len + seg->seqno - pcb->lastack) > wnd)) {
-            LWIP_ASSERT("tcp_output: no window for dummy packet", !LWIP_IS_DUMMY_SEGMENT(seg));
             tcp_split_segment(pcb, seg, wnd);
         }
 
@@ -1338,7 +1279,7 @@ err_t tcp_output(struct tcp_pcb *pcb)
              *   either seg->next != NULL or pcb->unacked == NULL;
              *   RST is no sent using tcp_write/tcp_output.
              */
-            if ((tcp_do_output_nagle(pcb) == 0) && !LWIP_IS_DUMMY_SEGMENT(seg) &&
+            if ((tcp_do_output_nagle(pcb) == 0) &&
                 ((pcb->flags & (TF_NAGLEMEMERR | TF_FIN)) == 0)) {
                 if (pcb->snd_sml_snt > (pcb->unacked ? pcb->unacked->len : 0)) {
                     pcb->flags &= ~(TF_ACK_NOW); // TODO bug #3574064: check if maybe we do want to
@@ -1368,11 +1309,6 @@ err_t tcp_output(struct tcp_pcb *pcb)
             ++i;
 #endif /* TCP_CWND_DEBUG */
 
-            // Send ack now if the packet is a dummy packet
-            if (LWIP_IS_DUMMY_SEGMENT(seg) && (pcb->flags & (TF_ACK_DELAY | TF_ACK_NOW))) {
-                tcp_send_empty_ack(pcb);
-            }
-
             if (get_tcp_state(pcb) != SYN_SENT) {
                 TCPH_SET_FLAG(seg->tcphdr, TCP_ACK);
                 pcb->flags &= ~(TF_ACK_DELAY | TF_ACK_NOW);
@@ -1393,45 +1329,37 @@ err_t tcp_output(struct tcp_pcb *pcb)
 
             pcb->unsent = seg->next;
             snd_nxt = seg->seqno + TCP_SEGLEN(seg);
-            if (TCP_SEQ_LT(pcb->snd_nxt, snd_nxt) && !LWIP_IS_DUMMY_SEGMENT(seg)) {
+            if (TCP_SEQ_LT(pcb->snd_nxt, snd_nxt)) {
                 pcb->snd_nxt = snd_nxt;
             }
             /* put segment on unacknowledged list if length > 0 */
             if (TCP_SEGLEN(seg) > 0) {
                 seg->next = NULL;
-                // unroll dummy segment
-                if (LWIP_IS_DUMMY_SEGMENT(seg)) {
-                    pcb->snd_lbb -= seg->len;
-                    pcb->snd_buf += seg->len;
-                    pcb->snd_queuelen -= pbuf_clen(seg->p);
-                    tcp_tx_seg_free(pcb, seg);
+                /* unacked list is empty? */
+                if (pcb->unacked == NULL) {
+                    pcb->unacked = seg;
+                    pcb->last_unacked = seg;
+                    /* unacked list is not empty? */
                 } else {
-                    /* unacked list is empty? */
-                    if (pcb->unacked == NULL) {
-                        pcb->unacked = seg;
-                        pcb->last_unacked = seg;
-                        /* unacked list is not empty? */
-                    } else {
-                        /* In the case of fast retransmit, the packet should not go to the tail
-                         * of the unacked queue, but rather somewhere before it. We need to check
-                         * for this case. -STJ Jul 27, 2004 */
-                        useg = pcb->last_unacked;
-                        if (TCP_SEQ_LT(seg->seqno, useg->seqno)) {
-                            /* add segment to before tail of unacked list, keeping the list sorted
-                             */
-                            struct tcp_seg **cur_seg = &(pcb->unacked);
-                            while (*cur_seg && TCP_SEQ_LT((*cur_seg)->seqno, seg->seqno)) {
-                                cur_seg = &((*cur_seg)->next);
-                            }
-                            LWIP_ASSERT("Value of last_unacked is invalid",
-                                        *cur_seg != pcb->last_unacked->next);
-                            seg->next = (*cur_seg);
-                            (*cur_seg) = seg;
-                        } else {
-                            /* add segment to tail of unacked list */
-                            useg->next = seg;
-                            pcb->last_unacked = seg;
+                    /* In the case of fast retransmit, the packet should not go to the tail
+                     * of the unacked queue, but rather somewhere before it. We need to check
+                     * for this case. -STJ Jul 27, 2004 */
+                    useg = pcb->last_unacked;
+                    if (TCP_SEQ_LT(seg->seqno, useg->seqno)) {
+                        /* add segment to before tail of unacked list, keeping the list sorted
+                         */
+                        struct tcp_seg **cur_seg = &(pcb->unacked);
+                        while (*cur_seg && TCP_SEQ_LT((*cur_seg)->seqno, seg->seqno)) {
+                            cur_seg = &((*cur_seg)->next);
                         }
+                        LWIP_ASSERT("Value of last_unacked is invalid",
+                                    *cur_seg != pcb->last_unacked->next);
+                        seg->next = (*cur_seg);
+                        (*cur_seg) = seg;
+                    } else {
+                        /* add segment to tail of unacked list */
+                        useg->next = seg;
+                        pcb->last_unacked = seg;
                     }
                 }
                 /* do not queue empty segments on the unacked list */
@@ -1462,11 +1390,6 @@ err_t tcp_output(struct tcp_pcb *pcb)
         pcb->seg_alloc = tcp_create_segment(pcb, NULL, 0, 0, 0);
     }
 
-    if (!pcb->pbuf_alloc) {
-        // Fetch pbuf for the next packet.
-        pcb->pbuf_alloc = tcp_tx_pbuf_alloc(pcb, 0, PBUF_RAM, NULL, NULL);
-    }
-
     return rc == ERR_WOULDBLOCK ? ERR_OK : rc;
 }
 
@@ -1496,9 +1419,8 @@ static err_t tcp_output_segment(struct tcp_seg *seg, struct tcp_pcb *pcb)
         seg->tcphdr->wnd = htons(TCPWND_MIN16(RCV_WND_SCALE(pcb, pcb->rcv_ann_wnd)));
     }
 
-    if (!LWIP_IS_DUMMY_SEGMENT(seg)) {
-        pcb->rcv_ann_right_edge = pcb->rcv_nxt + pcb->rcv_ann_wnd;
-    }
+    pcb->rcv_ann_right_edge = pcb->rcv_nxt + pcb->rcv_ann_wnd;
+
     /* Add any requested options.  NB MSS option is only set on SYN
        packets, so ignore it here */
     LWIP_ASSERT("seg->tcphdr not aligned", ((uintptr_t)(seg->tcphdr + 1) % 4) == 0);
@@ -1517,9 +1439,7 @@ static err_t tcp_output_segment(struct tcp_seg *seg, struct tcp_pcb *pcb)
     }
 
 #if LWIP_TCP_TIMESTAMPS
-    if (!LWIP_IS_DUMMY_SEGMENT(seg)) {
-        pcb->ts_lastacksent = pcb->rcv_nxt;
-    }
+    pcb->ts_lastacksent = pcb->rcv_nxt;
 
     if (seg->flags & TF_SEG_OPTS_TS) {
         tcp_build_timestamp_option(pcb, opts);
@@ -1539,22 +1459,21 @@ static err_t tcp_output_segment(struct tcp_seg *seg, struct tcp_pcb *pcb)
     }
 
     /* Set retransmission timer running if it is not currently enabled */
-    if (!LWIP_IS_DUMMY_SEGMENT(seg)) {
-        if (pcb->rtime == -1) {
-            pcb->rtime = 0;
-        }
-
-        if (pcb->ticks_since_data_sent == -1) {
-            pcb->ticks_since_data_sent = 0;
-        }
-
-        if (pcb->rttest == 0) {
-            pcb->rttest = tcp_ticks;
-            pcb->rtseq = seg->seqno;
-
-            LWIP_DEBUGF(TCP_RTO_DEBUG, ("tcp_output_segment: rtseq %" U32_F "\n", pcb->rtseq));
-        }
+    if (pcb->rtime == -1) {
+        pcb->rtime = 0;
     }
+
+    if (pcb->ticks_since_data_sent == -1) {
+        pcb->ticks_since_data_sent = 0;
+    }
+
+    if (pcb->rttest == 0) {
+        pcb->rttest = tcp_ticks;
+        pcb->rtseq = seg->seqno;
+
+        LWIP_DEBUGF(TCP_RTO_DEBUG, ("tcp_output_segment: rtseq %" U32_F "\n", pcb->rtseq));
+    }
+
     LWIP_DEBUGF(TCP_OUTPUT_DEBUG,
                 ("tcp_output_segment: %" U32_F ":%" U32_F "\n", htonl(seg->tcphdr->seqno),
                  htonl(seg->tcphdr->seqno) + seg->len));
@@ -1580,7 +1499,6 @@ static err_t tcp_output_segment(struct tcp_seg *seg, struct tcp_pcb *pcb)
     }
 
     u16_t flags = 0;
-    flags |= seg->flags & TF_SEG_OPTS_DUMMY_MSG;
     flags |= seg->flags & TF_SEG_OPTS_TSO;
     flags |= (TCP_SEQ_LT(seg->seqno, pcb->snd_nxt) ? TCP_WRITE_REXMIT : 0);
     flags |= seg->flags & TF_SEG_OPTS_ZEROCOPY;
@@ -1641,6 +1559,10 @@ void tcp_rst(u32_t seqno, u32_t ackno, u16_t local_port, u16_t remote_port, stru
 
     pcb->ip_output(p, NULL, pcb, 0);
     tcp_tx_pbuf_free(pcb, p);
+
+    // Don't send delayed ACKs after RST
+    pcb->flags &= ~TF_ACK_DELAY;
+
     LWIP_DEBUGF(TCP_RST_DEBUG, ("tcp_rst: seqno %" U32_F " ackno %" U32_F ".\n", seqno, ackno));
 }
 
