@@ -49,6 +49,14 @@ dst_entry::dst_entry(const sock_addr &dst, uint16_t src_port, socket_data &sock_
     init_members();
 }
 
+/**
+ * @brief Destructor for dst_entry class
+ *
+ * Cleans up all resources associated with the dst_entry object
+ *
+ * @note The coverity[UNCAUGHT_EXCEPT] was added as it's a False Positive
+ */
+// coverity[UNCAUGHT_EXCEPT]
 dst_entry::~dst_entry()
 {
     dst_logdbg("%s", to_str().c_str());
@@ -414,10 +422,8 @@ bool dst_entry::conf_l2_hdr_and_snd_wqe_eth()
                                               get_sge_lst_4_not_inline_send(), 1);
     m_p_send_wqe_handler->init_wqe(m_fragmented_send_wqe, get_sge_lst_4_not_inline_send(), 1);
 
-    const net_device_val_eth *netdevice_eth =
-        dynamic_cast<const net_device_val_eth *>(m_p_net_dev_val);
     BULLSEYE_EXCLUDE_BLOCK_START
-    if (netdevice_eth) {
+    if (m_p_net_dev_val) {
         BULLSEYE_EXCLUDE_BLOCK_END
         const L2_address *src = m_p_net_dev_val->get_l2_address();
         const L2_address *dst = m_p_neigh_val->get_l2_address();
@@ -425,11 +431,11 @@ bool dst_entry::conf_l2_hdr_and_snd_wqe_eth()
         BULLSEYE_EXCLUDE_BLOCK_START
         if (src && dst) {
             BULLSEYE_EXCLUDE_BLOCK_END
-            configure_eth_headers(m_header, *src, *dst, netdevice_eth->get_vlan());
+            configure_eth_headers(m_header, *src, *dst, m_p_net_dev_val->get_vlan());
             init_sge();
             ret_val = true;
         } else {
-            dst_logerr("Can't build proper L2 header, L2 address is not available");
+            dst_logerr("Unable to build L2 header, L2 address unresolved src=%p, dst=%p", src, dst);
         }
     } else {
         dst_logerr("Dynamic cast failed, can't build proper L2 header");
@@ -485,7 +491,8 @@ bool dst_entry::offloaded_according_to_rules()
     return ret_val;
 }
 
-bool dst_entry::prepare_to_send(struct xlio_rate_limit_t &rate_limit, bool skip_rules)
+bool dst_entry::prepare_to_send(struct xlio_rate_limit_t &rate_limit, bool skip_rules,
+                                bool skip_resolve_ring)
 {
     bool resolved = false;
     m_slow_path_lock.lock();
@@ -507,7 +514,7 @@ bool dst_entry::prepare_to_send(struct xlio_rate_limit_t &rate_limit, bool skip_
             m_max_udp_payload_size =
                 get_route_mtu() - ((get_sa_family() == AF_INET) ? IP_HLEN : IPV6_HLEN);
             m_max_ip_payload_size = m_max_udp_payload_size & ~0x7;
-            if (resolve_ring()) {
+            if (skip_resolve_ring || resolve_ring()) {
                 b_is_offloaded = true;
                 modify_ratelimit(rate_limit);
                 if (resolve_neigh()) {
@@ -519,22 +526,12 @@ bool dst_entry::prepare_to_send(struct xlio_rate_limit_t &rate_limit, bool skip_
                         dst_logdbg("peer L2 address: %s",
                                    m_p_neigh_val->get_l2_address()->to_str().c_str());
                     }
-                    configure_headers();
-                    m_id = m_p_ring->generate_id(m_p_net_dev_val->get_l2_address()->get_address(),
-                                                 m_p_neigh_val->get_l2_address()->get_address(),
-                                                 /* if vlan, use vlan proto */
-                                                 ((ethhdr *)(m_header->m_actual_hdr_addr))->h_proto,
-                                                 htons(ETH_P_IP), m_pkt_src_ip, m_dst_ip,
-                                                 m_src_port, m_dst_port);
-                    if (m_p_tx_mem_buf_desc_list) {
-                        m_p_ring->mem_buf_tx_release(m_p_tx_mem_buf_desc_list, true);
-                        m_p_tx_mem_buf_desc_list = nullptr;
+
+                    if (!skip_resolve_ring) {
+                        configure_headers();
+                        generate_id();
+                        resolved = true;
                     }
-                    if (m_p_zc_mem_buf_desc_list) {
-                        m_p_ring->mem_buf_tx_release(m_p_zc_mem_buf_desc_list, true);
-                        m_p_zc_mem_buf_desc_list = nullptr;
-                    }
-                    resolved = true;
                 }
             }
         }
@@ -553,6 +550,15 @@ bool dst_entry::prepare_to_send(struct xlio_rate_limit_t &rate_limit, bool skip_
     return m_b_is_offloaded;
 }
 
+void dst_entry::generate_id()
+{
+    m_id = m_p_ring->generate_id(m_p_net_dev_val->get_l2_address()->get_address(),
+                                 m_p_neigh_val->get_l2_address()->get_address(),
+                                 /* if vlan, use vlan proto */
+                                 ((ethhdr *)(m_header->m_actual_hdr_addr))->h_proto,
+                                 htons(ETH_P_IP), m_pkt_src_ip, m_dst_ip, m_src_port, m_dst_port);
+}
+
 bool dst_entry::try_migrate_ring_tx(lock_base &socket_lock)
 {
     bool ret = false;
@@ -598,7 +604,7 @@ void dst_entry::do_ring_migration_tx(lock_base &socket_lock, resource_allocation
     new_key->set_user_id_key(new_calc_id);
     m_slow_path_lock.unlock();
     socket_lock.unlock();
-
+    /* coverity[sleep] */
     ring *new_ring = m_p_net_dev_val->reserve_ring(new_key);
     if (!new_ring) {
         socket_lock.lock();
@@ -689,12 +695,10 @@ ssize_t dst_entry::pass_pkt_to_neigh(const iovec *p_iov, size_t sz_iov, uint32_t
     // For IPv6 - packet_id will be taken from neigh_send_data
     configure_ip_header(m_header_neigh, (uint16_t)(packet_id & 0xffff));
 
-    const L2_address *dummy =
-        m_p_net_dev_val->get_l2_address(); // Real MAC addresses will be filled by neigh
-    const net_device_val_eth *netdevice_eth =
-        dynamic_cast<const net_device_val_eth *>(m_p_net_dev_val);
-    if (m_p_neigh_entry && netdevice_eth && dummy) {
-        configure_eth_headers(m_header_neigh, *dummy, *dummy, netdevice_eth->get_vlan());
+    // Real MAC addresses will be filled by neigh
+    const L2_address *dummy = m_p_net_dev_val->get_l2_address();
+    if (m_p_neigh_entry && dummy) {
+        configure_eth_headers(m_header_neigh, *dummy, *dummy, m_p_net_dev_val->get_vlan());
 
         neigh_send_data n_send_info(const_cast<iovec *>(p_iov), sz_iov, m_header_neigh,
                                     get_route_mtu(), packet_id);
@@ -730,6 +734,11 @@ bool dst_entry::alloc_neigh_val(transport_type_t tranport)
     return ret_val;
 }
 
+mem_buf_desc_t *dst_entry::get_tx_buffer()
+{
+    return m_p_ring->mem_buf_tx_get(m_id, false, PBUF_RAM, 1, true);
+}
+
 void dst_entry::return_buffers_pool()
 {
     int count;
