@@ -13,6 +13,7 @@
 #include "sockinfo.h"
 #include "sockinfo_udp.h"
 #include "sockinfo_tcp.h"
+#include "event/entity_context.h"
 #include "iomux/epfd_info.h"
 
 #undef MODULE_NAME
@@ -57,9 +58,6 @@ fd_collection::fd_collection()
 
     m_p_cq_channel_map = new cq_channel_info *[m_n_fd_map_size];
     memset(m_p_cq_channel_map, 0, m_n_fd_map_size * sizeof(cq_channel_info *));
-
-    m_p_tap_map = new ring_tap *[m_n_fd_map_size];
-    memset(m_p_tap_map, 0, m_n_fd_map_size * sizeof(ring_tap *));
 }
 
 fd_collection::~fd_collection()
@@ -78,9 +76,6 @@ fd_collection::~fd_collection()
     delete[] m_p_cq_channel_map;
     m_p_cq_channel_map = nullptr;
 
-    delete[] m_p_tap_map;
-    m_p_tap_map = nullptr;
-
     m_epfd_lst.clear_without_cleanup();
     m_pending_to_remove_lst.clear_without_cleanup();
 }
@@ -160,11 +155,6 @@ void fd_collection::clear()
             m_p_cq_channel_map[fd] = nullptr;
             fdcoll_logdbg("destroyed cq_channel_fd=%d", fd);
         }
-
-        if (m_p_tap_map[fd]) {
-            m_p_tap_map[fd] = nullptr;
-            fdcoll_logdbg("destroyed tapfd=%d", fd);
-        }
     }
 
     unlock();
@@ -358,29 +348,6 @@ int fd_collection::addepfd(int epfd, int size)
     return 0;
 }
 
-int fd_collection::addtapfd(int tapfd, ring_tap *p_ring)
-{
-    fdcoll_logfunc("tapfd=%d, p_ring=%p", tapfd, p_ring);
-
-    if (!is_valid_fd(tapfd)) {
-        return -1;
-    }
-
-    lock();
-
-    if (get_tapfd(tapfd)) {
-        fdcoll_logwarn("[tapfd=%d] already exist in the collection (ring %p)", tapfd,
-                       get_tapfd(tapfd));
-        return -1;
-    }
-
-    m_p_tap_map[tapfd] = p_ring;
-
-    unlock();
-
-    return 0;
-}
-
 int fd_collection::add_cq_channel_fd(int cq_ch_fd, ring *p_ring)
 {
     fdcoll_logfunc("cq_ch_fd=%d", cq_ch_fd);
@@ -448,6 +415,10 @@ int fd_collection::del_sockfd(int fd, bool is_for_udp_pool /*=false*/)
     p_sfd_api = get_sockfd(fd);
 
     if (p_sfd_api) {
+        if (safe_mce_sys().worker_threads && handle_worker_threads_mode_close(fd, p_sfd_api)) {
+            return 0;
+        }
+
         // TCP socket need some timer to before it can be deleted,
         // in order to gracefuly terminate TCP connection
         // so we have to stages:
@@ -482,6 +453,53 @@ int fd_collection::del_sockfd(int fd, bool is_for_udp_pool /*=false*/)
     return ret_val;
 }
 
+bool fd_collection::handle_worker_threads_mode_close(int fd, sockinfo *p_sfd_api)
+{
+    // Clear the socket from the fd_collection
+    clear_socket(fd);
+
+    if (p_sfd_api->get_protocol() == PROTO_TCP) {
+        sockinfo_tcp *tcp_si = static_cast<sockinfo_tcp *>(p_sfd_api);
+        if (tcp_si->get_entity_context()) {
+            // Incoming/outgoing sockets - delegate to entity context and return
+            handle_socket_close_job_worker_threads_mode(tcp_si);
+            return true; // Should return from del_sockfd
+        } else if (tcp_si->get_listen_context()) {
+            // RSS listen socket - send close jobs to children
+            handle_listen_socket_close_worker_threads_mode(tcp_si);
+            // Main listen socket - Fall through to legacy close flow below
+        }
+        // Other TCP sockets (e.g., created via socket() but not bound/connected/listening) - Fall
+        // through to legacy close flow below
+    }
+    // Non-TCP sockets - Fall through to legacy close flow below
+    return false; // Should continue with legacy flow
+}
+
+void fd_collection::handle_listen_socket_close_worker_threads_mode(sockinfo_tcp *listen_si)
+{
+    // Reset counters before sending close jobs
+    listen_si->get_listen_context()->reset_counters();
+
+    // Send close jobs to all children
+    size_t num_children = listen_si->get_listen_context()->get_listen_rss_children_size();
+    for (size_t i = 0; i < num_children; ++i) {
+        sockinfo_tcp *child = listen_si->get_listen_context()->get_listen_rss_child(i);
+        assert(child);
+        handle_socket_close_job_worker_threads_mode(child);
+    }
+    // Wait for all children to finish
+    listen_si->get_listen_context()->wait_for_rss_children_ready();
+}
+
+void fd_collection::handle_socket_close_job_worker_threads_mode(sockinfo_tcp *si)
+{
+    // Send close job to socket's entity context
+    assert(si->get_entity_context());
+    si->get_entity_context()->add_job(
+        entity_context::job_desc {entity_context::JOB_TYPE_SOCK_CLOSE, 0, si, nullptr, 0U, 0U});
+}
+
 int fd_collection::del_epfd(int fd, bool b_cleanup /*=false*/)
 {
     return del(fd, b_cleanup, m_p_epfd_map);
@@ -499,17 +517,6 @@ int fd_collection::del_cq_channel_fd(int fd, bool b_cleanup /*=false*/)
     return del(fd, b_cleanup, m_p_cq_channel_map);
 }
 
-void fd_collection::del_tapfd(int fd)
-{
-    if (!is_valid_fd(fd)) {
-        return;
-    }
-
-    lock();
-    m_p_tap_map[fd] = nullptr;
-    unlock();
-}
-
 template <typename cls> int fd_collection::del(int fd, bool b_cleanup, cls **map_type)
 {
     fdcoll_logfunc("fd=%d%s", fd,
