@@ -10,7 +10,7 @@
 #include <netdb.h>
 #include <linux/sockios.h>
 #include <cinttypes>
-
+#include "event/entity_context.h"
 #include "utils/bullseye.h"
 #include "vlogger/vlogger.h"
 #include "util/if.h"
@@ -72,20 +72,20 @@ const char *sockinfo::setsockopt_so_opt_to_str(int opt)
 
 sockinfo::sockinfo(int fd, int domain, bool use_ring_locks)
     : m_skip_cq_poll_in_rx(safe_mce_sys().skip_poll_in_rx == SKIP_POLL_IN_RX_ENABLE)
-    , m_fd_context((void *)((uintptr_t)fd))
+    , m_app_lock(multilock::create_new_lock(MULTILOCK_RECURSIVE, "app_sock_lock"))
     , m_family(domain)
     , m_fd(fd)
     , m_rx_num_buffs_reuse(safe_mce_sys().rx_bufs_batch)
     , m_is_ipv6only(safe_mce_sys().sysctl_reader.get_ipv6_bindv6only())
+    , m_n_uc_ttl_hop_lim(m_family == AF_INET
+                             ? safe_mce_sys().sysctl_reader.get_net_ipv4_ttl()
+                             : safe_mce_sys().sysctl_reader.get_net_ipv6_hop_limit())
     , m_lock_rcv(MULTILOCK_RECURSIVE, MODULE_NAME "::m_lock_rcv")
     , m_lock_snd(MODULE_NAME "::m_lock_snd")
     , m_so_bindtodevice_ip(ip_address::any_addr(), domain)
     , m_rx_ring_map_lock(MODULE_NAME "::m_rx_ring_map_lock")
     , m_ring_alloc_log_rx(safe_mce_sys().ring_allocation_logic_rx, use_ring_locks)
     , m_ring_alloc_log_tx(safe_mce_sys().ring_allocation_logic_tx, use_ring_locks)
-    , m_n_uc_ttl_hop_lim(m_family == AF_INET
-                             ? safe_mce_sys().sysctl_reader.get_net_ipv4_ttl()
-                             : safe_mce_sys().sysctl_reader.get_net_ipv6_hop_limit())
 {
     m_rx_epfd = SYSCALL(epoll_create, 128);
     if (unlikely(m_rx_epfd == -1)) {
@@ -94,7 +94,6 @@ sockinfo::sockinfo(int fd, int domain, bool use_ring_locks)
     m_sock_wakeup_pipe.wakeup_set_epoll_fd(m_rx_epfd);
     if (m_fd == SOCKET_FAKE_FD) {
         m_fd = m_rx_epfd;
-        m_fd_context = (void *)((uintptr_t)m_fd);
     }
 
     m_ring_alloc_logic_rx = ring_allocation_logic_rx(get_fd(), m_ring_alloc_log_rx);
@@ -105,12 +104,18 @@ sockinfo::sockinfo(int fd, int domain, bool use_ring_locks)
     memset(&m_so_ratelimit, 0, sizeof(xlio_rate_limit_t));
     set_flow_tag(m_fd + 1);
 
-    atomic_set(&m_zckey, 0);
-
     m_connected.set_sa_family(m_family);
     m_bound.set_sa_family(m_family);
 }
 
+/**
+ * @brief Destructor for sockinfo class
+ *
+ * Cleans up all resources associated with the sockinfo object
+ *
+ * @note The coverity[UNCAUGHT_EXCEPT] was added as it's a False Positive
+ */
+// coverity[UNCAUGHT_EXCEPT]
 sockinfo::~sockinfo()
 {
     m_state = SOCKINFO_DESTROYING;
@@ -127,22 +132,12 @@ sockinfo::~sockinfo()
         SYSCALL(close, m_rx_epfd);
     }
 
-    while (!m_error_queue.empty()) {
-        mem_buf_desc_t *buff = m_error_queue.get_and_pop_front();
-        if (buff->m_flags & mem_buf_desc_t::CLONED) {
-            delete buff;
-        } else {
-            si_logerr("Detected invalid element in socket error queue as %p with flags 0x%x", buff,
-                      buff->m_flags);
-        }
-    }
-
     if (m_p_socket_stats) {
         xlio_stats_instance_remove_socket_block(m_p_socket_stats);
         sock_stats::instance().return_stats_obj(m_p_socket_stats);
     }
 
-    bool toclose = safe_mce_sys().deferred_close && m_fd >= 0;
+    bool toclose = (safe_mce_sys().deferred_close || is_xlio_socket()) && m_fd >= 0;
 
 #if defined(DEFINED_NGINX)
     if (g_p_app->type == APP_NGINX) {
@@ -152,7 +147,10 @@ sockinfo::~sockinfo()
 #endif
 
     if (toclose) {
-        SYSCALL(close, m_fd);
+        int rc = SYSCALL(close, m_fd);
+        if (rc != 0) {
+            si_logdbg("close(fd=%d) failed with errno=%d", m_fd, errno);
+        }
     }
 }
 
@@ -180,39 +178,6 @@ void sockinfo::socket_stats_init()
     m_p_socket_stats->sa_family = m_family;
 }
 
-ring_ec *sockinfo::pop_next_ec()
-{
-    if (likely(m_socketxtreme_ec_first)) {
-        ring_ec *temp = m_socketxtreme_ec_first;
-        m_socketxtreme_ec_first = m_socketxtreme_ec_first->next;
-        if (likely(!m_socketxtreme_ec_first)) { // We likely to have a single ec most of the time.
-            m_socketxtreme_ec_last = nullptr;
-        }
-
-        return temp;
-    }
-
-    return nullptr;
-}
-
-ring_ec *sockinfo::clear_ecs()
-{
-    ring_ec *temp = m_socketxtreme_ec_first;
-    m_socketxtreme_ec_first = m_socketxtreme_ec_last = nullptr;
-    return temp;
-}
-
-void sockinfo::add_ec(ring_ec *ec)
-{
-    memset(&ec->completion, 0, sizeof(ec->completion));
-    if (likely(!m_socketxtreme_ec_last)) {
-        m_socketxtreme_ec_last = m_socketxtreme_ec_first = ec;
-    } else {
-        m_socketxtreme_ec_last->next = ec;
-        m_socketxtreme_ec_last = ec;
-    }
-}
-
 void sockinfo::set_blocking(bool is_blocked)
 {
     si_logdbg("set socket to %s mode", is_blocked ? "blocked" : "non-blocking");
@@ -435,16 +400,6 @@ int sockinfo::setsockopt(int __level, int __optname, const void *__optval, sockl
 
     if (__level == SOL_SOCKET) {
         switch (__optname) {
-        case SO_XLIO_USER_DATA:
-            if (__optlen == sizeof(m_fd_context)) {
-                m_fd_context = *(void **)__optval;
-                ret = SOCKOPT_INTERNAL_XLIO_SUPPORT;
-            } else {
-                ret = SOCKOPT_NO_XLIO_SUPPORT;
-                errno = EINVAL;
-            }
-            break;
-
         case SO_REUSEADDR:
             if (__optval && __optlen == sizeof(int)) {
                 m_reuseaddr = *(int *)__optval;
@@ -722,14 +677,6 @@ int sockinfo::getsockopt(int __level, int __optname, void *__optval, socklen_t *
     switch (__level) {
     case SOL_SOCKET:
         switch (__optname) {
-        case SO_XLIO_USER_DATA:
-            if (*__optlen == sizeof(m_fd_context)) {
-                *(void **)__optval = m_fd_context;
-                ret = 0;
-            } else {
-                errno = EINVAL;
-            }
-            break;
         case SO_MAX_PACING_RATE:
             if (*__optlen == sizeof(struct xlio_rate_limit_t)) {
                 *(struct xlio_rate_limit_t *)__optval = m_so_ratelimit;
@@ -955,9 +902,7 @@ bool sockinfo::attach_receiver(flow_tuple_with_local_if &flow_key)
     // Registered as receiver successfully
     si_logdbg("Attached %s to ring %p", flow_key.to_str().c_str(), p_nd_resources->p_ring);
 
-    /* Verify 5 tuple over 3 tuple
-     * and replace flow rule with the strongest
-     */
+    // Verify 5 tuple over 3 tuple and replace flow rule with the strongest
     if (flow_key.is_5_tuple()) {
         // Check and remove lesser 3 tuple
         flow_tuple_with_local_if flow_key_3t(
@@ -973,7 +918,7 @@ bool sockinfo::attach_receiver(flow_tuple_with_local_if &flow_key)
     return true;
 }
 
-bool sockinfo::detach_receiver(flow_tuple_with_local_if &flow_key)
+bool sockinfo::detach_receiver(flow_tuple_with_local_if &flow_key, rfs_rule **rule_extract)
 {
     si_logdbg("Unregistering receiver: %s", flow_key.to_str().c_str());
 
@@ -994,7 +939,7 @@ bool sockinfo::detach_receiver(flow_tuple_with_local_if &flow_key)
 
     // Detach tuple
     unlock_rx_q();
-    p_ring->detach_flow(flow_key, this);
+    p_ring->detach_flow(flow_key, this, rule_extract);
     lock_rx_q();
 
     // Un-map flow from local map
@@ -1037,6 +982,7 @@ net_device_resources_t *sockinfo::create_nd_resources(const ip_addr &ip_local)
             key = m_ring_alloc_logic_rx.create_new_key(ip_local);
         }
         m_rx_ring_map_lock.unlock();
+        /* coverity[sleep] */
         nd_resources.p_ring = nd_resources.p_ndv->reserve_ring(key);
         m_rx_migration_lock.unlock();
         lock_rx_q();
@@ -1136,6 +1082,7 @@ void sockinfo::do_rings_migration_rx(resource_allocation_key &old_key)
         net_device_resources_t *p_nd_resources = &(rx_nd_iter->second);
         ring *p_old_ring = p_nd_resources->p_ring;
         unlock_rx_q();
+        /* coverity[sleep] */
         ring *new_ring = p_nd_resources->p_ndv->reserve_ring(new_key);
         if (new_ring == p_old_ring) {
             rc = p_nd_resources->p_ndv->release_ring(&old_key);
@@ -1201,7 +1148,7 @@ void sockinfo::do_rings_migration_rx(resource_allocation_key &old_key)
             si_logdbg("Detaching %s from ring %p", flow_key.to_str().c_str(), p_old_ring);
             // Detach tuple
             unlock_rx_q();
-            p_old_ring->detach_flow(flow_key, this);
+            p_old_ring->detach_flow(flow_key, this, nullptr);
             lock_rx_q();
             rx_del_ring_cb(p_old_ring);
 
@@ -1270,7 +1217,7 @@ void sockinfo::do_rings_migration_rx(resource_allocation_key &old_key)
 
 void sockinfo::consider_rings_migration_rx()
 {
-    if (m_ring_alloc_logic_rx.is_logic_support_migration()) {
+    if (!m_entity_context && m_ring_alloc_logic_rx.is_logic_support_migration()) {
         if (!m_rx_migration_lock.trylock()) {
             if (m_ring_alloc_logic_rx.should_migrate_ring()) {
                 ring_alloc_logic_attr old_key(*m_ring_alloc_logic_rx.get_key());
@@ -1289,9 +1236,10 @@ int sockinfo::add_epoll_context(epfd_info *epfd)
     m_rx_ring_map_lock.lock();
     lock_rx_q();
 
-    if (!m_econtext && !safe_mce_sys().enable_socketxtreme) {
+    if (!m_econtext) {
         // This socket is not registered to any epfd
         m_econtext = epfd;
+        set_epoll_event_flags_thread(0);
     } else {
         // Currently XLIO does not support more then 1 epfd listed
         errno = (m_econtext == epfd) ? EEXIST : ENOMEM;
@@ -1672,9 +1620,6 @@ void sockinfo::rx_del_ring_cb(ring *p_ring)
             delete p_ring_info;
 
             if (m_p_rx_ring == base_ring) {
-                // Ring should not have completion events related closed socket in wait list
-                m_p_rx_ring->socketxtreme_ec_clear_sock(this);
-
                 if (m_rx_ring_map.size() == 1) {
                     m_p_rx_ring = m_rx_ring_map.begin()->first;
                 } else {
@@ -1945,13 +1890,6 @@ void sockinfo::destructor_helper()
     m_p_connected_dst_entry = nullptr;
 }
 
-int sockinfo::register_callback_ctx(xlio_recv_callback_t callback, void *context)
-{
-    m_rx_callback = callback;
-    m_rx_callback_context = context;
-    return 0;
-}
-
 int sockinfo::modify_ratelimit(dst_entry *p_dst_entry, struct xlio_rate_limit_t &rate_limit)
 {
     if (m_ring_alloc_log_tx.get_ring_alloc_logic() == RING_LOGIC_PER_SOCKET ||
@@ -1975,57 +1913,6 @@ int sockinfo::modify_ratelimit(dst_entry *p_dst_entry, struct xlio_rate_limit_t
     return -1;
 }
 
-int sockinfo::get_rings_num()
-{
-    size_t count = 0;
-    size_t num_rx_channel_fds;
-
-    ring *tx_ring = m_p_connected_dst_entry ? m_p_connected_dst_entry->get_ring() : nullptr;
-    if (tx_ring) {
-        (void)tx_ring->get_rx_channel_fds(count);
-    }
-
-    for (auto pair : m_rx_ring_map) {
-        if (tx_ring == pair.first) {
-            continue;
-        }
-        (void)pair.first->get_rx_channel_fds(num_rx_channel_fds);
-        count += num_rx_channel_fds;
-    }
-    return static_cast<int>(count);
-}
-
-int sockinfo::get_rings_fds(int *ring_fds, int ring_fds_sz)
-{
-    size_t num_rx_channel_fds;
-    int *channel_fds;
-    int index = 0;
-
-    /*
-     * We return RX channels for the TX ring to make it consistent and comparable with the RX
-     * rings. The channels are used only as indirect pointers to the rings, therefore, this
-     * doesn't introduce any functionality issues.
-     */
-    ring *tx_ring = m_p_connected_dst_entry ? m_p_connected_dst_entry->get_ring() : nullptr;
-    if (tx_ring) {
-        channel_fds = tx_ring->get_rx_channel_fds(num_rx_channel_fds);
-        for (size_t i = 0; i < num_rx_channel_fds && index < ring_fds_sz; ++i) {
-            ring_fds[index++] = channel_fds[i];
-        }
-    }
-
-    for (auto pair : m_rx_ring_map) {
-        if (tx_ring == pair.first) {
-            continue;
-        }
-        channel_fds = pair.first->get_rx_channel_fds(num_rx_channel_fds);
-        for (size_t i = 0; i < num_rx_channel_fds && index < ring_fds_sz; ++i) {
-            ring_fds[index++] = channel_fds[i];
-        }
-    }
-    return index;
-}
-
 int sockinfo::setsockopt_kernel(int __level, int __optname, const void *__optval,
                                 socklen_t __optlen, int supported, bool allow_privileged)
 {
@@ -2113,7 +2000,8 @@ void sockinfo::process_timestamps(mem_buf_desc_t *p_desc)
     }
 }
 
-void sockinfo::handle_recv_timestamping(struct cmsg_state *cm_state)
+void sockinfo::handle_recv_timestamping(struct cmsg_state *cm_state,
+                                        timestamps_t *packet_timestamps)
 {
     struct {
         struct timespec systime;
@@ -2123,7 +2011,6 @@ void sockinfo::handle_recv_timestamping(struct cmsg_state *cm_state)
 
     memset(&tsing, 0, sizeof(tsing));
 
-    timestamps_t *packet_timestamps = get_socket_timestamps();
     struct timespec *packet_systime = &packet_timestamps->sw;
 
     // Only fill in SO_TIMESTAMPNS if both requested.
@@ -2155,30 +2042,6 @@ void sockinfo::handle_recv_timestamping(struct cmsg_state *cm_state)
     insert_cmsg(cm_state, SOL_SOCKET, SO_TIMESTAMPING, &tsing, sizeof(tsing));
 }
 
-void sockinfo::handle_recv_errqueue(struct cmsg_state *cm_state)
-{
-    mem_buf_desc_t *buff = nullptr;
-
-    m_error_queue_lock.lock();
-    if (m_error_queue.empty()) {
-        m_error_queue_lock.unlock();
-        return;
-    }
-    buff = m_error_queue.get_and_pop_front();
-    m_error_queue_lock.unlock();
-
-    if (!(buff->m_flags & mem_buf_desc_t::CLONED)) {
-        si_logerr("Detected invalid element in socket error queue as %p with flags 0x%x", buff,
-                  buff->m_flags);
-        return;
-    }
-
-    insert_cmsg(cm_state, 0, IP_RECVERR, &buff->ee, sizeof(buff->ee));
-    cm_state->mhdr->msg_flags |= MSG_ERRQUEUE;
-
-    delete buff;
-}
-
 void sockinfo::insert_cmsg(struct cmsg_state *cm_state, int level, int type, void *data, int len)
 {
     if (!cm_state->cmhdr || cm_state->mhdr->msg_flags & MSG_CTRUNC) {
@@ -2213,7 +2076,7 @@ void sockinfo::insert_cmsg(struct cmsg_state *cm_state, int level, int type, voi
     }
 }
 
-void sockinfo::handle_cmsg(struct msghdr *msg, int flags)
+void sockinfo::handle_cmsg(struct msghdr *msg)
 {
     struct cmsg_state cm_state;
 
@@ -2225,10 +2088,7 @@ void sockinfo::handle_cmsg(struct msghdr *msg, int flags)
         handle_ip_pktinfo(&cm_state);
     }
     if (m_b_rcvtstamp || m_n_tsing_flags) {
-        handle_recv_timestamping(&cm_state);
-    }
-    if (flags & MSG_ERRQUEUE) {
-        handle_recv_errqueue(&cm_state);
+        handle_recv_timestamping(&cm_state, get_socket_timestamps());
     }
 
     cm_state.mhdr->msg_controllen = cm_state.cmsg_bytes_consumed;
@@ -2269,12 +2129,6 @@ ssize_t sockinfo::tx_os(const tx_call_t call_type, const iovec *p_iov, const ssi
 {
     errno = 0;
 
-    // Ignore dummy messages for OS
-    if (unlikely(IS_DUMMY_PACKET(__flags))) {
-        errno = EINVAL;
-        return -1;
-    }
-
     switch (call_type) {
     case TX_WRITE:
         __log_info_func("calling os transmit with orig write");
@@ -2331,3 +2185,18 @@ bool sockinfo::skip_os_select()
     // os (i.e. TRUE...)
     return (!safe_mce_sys().select_poll_os_ratio);
 }
+
+void sockinfo::rx_handle_cmsg(struct msghdr *msg, mem_buf_desc_t *out_buf)
+{
+    struct cmsg_state cm_state;
+
+    cm_state.mhdr = msg;
+    cm_state.cmhdr = CMSG_FIRSTHDR(msg);
+    cm_state.cmsg_bytes_consumed = 0;
+
+    if (m_b_rcvtstamp || m_n_tsing_flags) {
+        handle_recv_timestamping(&cm_state, &out_buf->rx.timestamps);
+    }
+
+    cm_state.mhdr->msg_controllen = cm_state.cmsg_bytes_consumed;
+}
