@@ -79,8 +79,7 @@
 #define MSG_ZEROCOPY 0x4000000
 #endif
 
-#define IS_DUMMY_PACKET(flags)            (flags & XLIO_SND_FLAGS_DUMMY)
-#define NOTIFY_ON_EVENTS(context, events) context->set_events(events)
+#define NOTIFY_ON_EVENTS(context, events) context->insert_epoll_event(events)
 
 #define IF_STATS(x)                                                                                \
     if (unlikely(m_p_socket_stats)) {                                                              \
@@ -118,10 +117,6 @@ enum {
 
 typedef enum { RX_READ = 23, RX_READV, RX_RECV, RX_RECVFROM, RX_RECVMSG } rx_call_t;
 
-enum {
-    TX_FLAG_NO_PARTIAL_WRITE = 1 << 0,
-};
-
 enum fd_type_t {
     FD_TYPE_SOCKET = 0,
     FD_TYPE_PIPE,
@@ -182,9 +177,6 @@ struct ring_info_t {
 // Used attributes can be of different types TX_FILE, TX_WRITE, TX_WRITEV, TX_SEND, TX_SENDTO,
 // TX_SENDMSG
 struct xlio_tx_call_attr_t {
-    tx_call_t opcode;
-    unsigned xlio_flags;
-
     struct _attr {
         struct iovec *iov;
         ssize_t sz_iov;
@@ -195,6 +187,7 @@ struct xlio_tx_call_attr_t {
     } attr;
 
     pbuf_desc priv;
+    tx_call_t opcode;
 
     ~xlio_tx_call_attr_t() {};
     void clear(void)
@@ -203,7 +196,6 @@ struct xlio_tx_call_attr_t {
         memset(&attr, 0, sizeof(attr));
         memset(&priv, 0, sizeof(priv));
         priv.attr = PBUF_DESC_NONE;
-        xlio_flags = 0;
     }
 
     xlio_tx_call_attr_t() { clear(); }
@@ -217,7 +209,10 @@ typedef std::unordered_map<ring *, ring_info_t *> rx_ring_map_t;
 // see route.c in Linux kernel
 const uint8_t ip_tos2prio[16] = {0, 0, 0, 0, 2, 2, 2, 2, 6, 6, 6, 6, 4, 4, 4, 4};
 
+// Forward declarations
 class epfd_info;
+class poll_group;
+class entity_context;
 
 class sockinfo {
 public:
@@ -258,7 +253,10 @@ public:
     virtual bool rx_input_cb(mem_buf_desc_t *p_rx_pkt_mem_buf_desc_info,
                              void *pv_fd_ready_array) = 0;
 
+    virtual void rx_data_recvd(uint32_t tot_size) = 0;
     virtual ssize_t tx(xlio_tx_call_attr_t &tx_arg) = 0;
+    virtual void tx_thread_commit(mem_buf_desc_t *buf_list, uint32_t offset, uint32_t size,
+                                  int flags) = 0;
     virtual bool is_readable(uint64_t *p_poll_sn, fd_array_t *p_fd_array = nullptr) = 0;
     virtual bool is_writeable() = 0;
     virtual bool is_errorable(int *errors) = 0;
@@ -272,6 +270,8 @@ public:
     virtual int accept4(struct sockaddr *__addr, socklen_t *__addrlen, int __flags) = 0;
     virtual int bind(const sockaddr *__addr, socklen_t __addrlen) = 0;
     virtual int connect(const sockaddr *__to, socklen_t __tolen) = 0;
+    virtual void connect_entity_context() = 0;
+    virtual void set_entity_context(entity_context *ctx) = 0;
     virtual int getsockname(sockaddr *__name, socklen_t *__namelen) = 0;
     virtual int getpeername(sockaddr *__name, socklen_t *__namelen) = 0;
     virtual int setsockopt(int __level, int __optname, __const void *__optval,
@@ -281,7 +281,6 @@ public:
     virtual bool is_incoming() = 0;
     virtual bool is_closable() = 0;
     virtual void statistics_print(vlog_levels_t log_level = VLOG_DEBUG) = 0;
-    virtual int register_callback(xlio_recv_callback_t callback, void *context) = 0;
     virtual int fcntl(int __cmd, unsigned long int __arg);
     virtual int fcntl64(int __cmd, unsigned long int __arg);
     virtual int ioctl(unsigned long int __request, unsigned long int __arg);
@@ -291,9 +290,6 @@ public:
                        int *p_flags = 0, sockaddr *__from = nullptr, socklen_t *__fromlen = nullptr,
                        struct msghdr *__msg = nullptr) = 0;
 
-    virtual int recvfrom_zcopy_free_packets(struct xlio_recvfrom_zcopy_packet_t *pkts,
-                                            size_t count) = 0;
-
     // Instructing the socket to immediately sample/un-sample the OS in receive flow
     virtual void set_immediate_os_sample() = 0;
     virtual void unset_immediate_os_sample() = 0;
@@ -308,16 +304,12 @@ public:
     inline bool set_flow_tag(uint32_t flow_tag_id);
     inline void sock_pop_descs_rx_ready(descq_t *cache);
 
-    // Socketxtreme related.
-    ring_ec *pop_next_ec();
-    ring_ec *clear_ecs();
-    void add_ec(ring_ec *ec);
-    ring_ec *get_last_ec() { return m_socketxtreme_ec_last; }
-    bool has_next_ec() { return (m_socketxtreme_ec_first != nullptr); }
-    sockinfo *get_ec_ring_list_next() { return m_socketxtreme_ring_list_next; }
-    void set_ec_ring_list_next(sockinfo *sock) { m_socketxtreme_ring_list_next = sock; }
-
-    bool has_epoll_context() { return (!safe_mce_sys().enable_socketxtreme && m_econtext); }
+    entity_context *get_entity_context() const { return m_entity_context; }
+    uint32_t get_epoll_event_flags() { return m_epoll_event_flags; }
+    uint32_t get_epoll_event_flags_thread() const { return m_epoll_event_flags_thread; }
+    void set_epoll_event_flags(uint32_t events) { m_epoll_event_flags = events; }
+    void set_epoll_event_flags_thread(uint32_t events) { m_epoll_event_flags_thread = events; }
+    bool has_epoll_context() { return (!!m_econtext); }
     bool get_rx_pkt_ready_list_count() const { return m_n_rx_pkt_ready_list_count; }
     int get_fd() const { return m_fd; };
     sa_family_t get_family() { return m_family; }
@@ -333,10 +325,7 @@ public:
     rfs *get_rfs_ptr() const { return m_rfs_ptr; }
     void set_rfs_ptr(rfs *r) { m_rfs_ptr = r; }
     void destructor_helper();
-    int get_rings_fds(int *ring_fds, int ring_fds_sz);
-    int get_rings_num();
     bool validate_and_convert_mapped_ipv4(sock_addr &sock) const;
-    int register_callback_ctx(xlio_recv_callback_t callback, void *context);
     void consider_rings_migration_rx();
     int add_epoll_context(epfd_info *epfd);
     void remove_epoll_context(epfd_info *epfd);
@@ -358,6 +347,11 @@ public:
     void set_rx_num_buffs_reuse(int val) { m_rx_num_buffs_reuse = val; }
 #endif
 #endif
+
+    // XLIO Ultra API
+    bool is_xlio_socket() const { return m_is_xlio_socket; }
+    poll_group *get_poll_group() const { return m_p_group; }
+
 protected:
     static const char *setsockopt_so_opt_to_str(int opt);
 
@@ -375,9 +369,8 @@ protected:
                                                int &rx_pkt_ready_list_idx) = 0;
     virtual timestamps_t *get_socket_timestamps() = 0;
     virtual void update_socket_timestamps(timestamps_t *ts) = 0;
-    virtual void post_dequeue(bool release_buff) = 0;
+    virtual void post_dequeue() = 0;
     virtual int os_epoll_wait(epoll_event *ep_events, int maxevents);
-    virtual int zero_copy_rx(iovec *p_iov, mem_buf_desc_t *pdesc, int *p_flags) = 0;
     virtual void handle_ip_pktinfo(struct cmsg_state *cm_state) = 0;
     virtual bool try_un_offloading(); // un-offload the socket if possible
 
@@ -390,9 +383,6 @@ protected:
 
     inline void set_rx_reuse_pending(bool is_pending = true);
     inline void reuse_buffer(mem_buf_desc_t *buff);
-    inline xlio_socketxtreme_completion_t *set_events_socketxtreme(uint64_t events,
-                                                                   bool full_transaction);
-    inline void set_events(uint64_t events);
     inline void save_strq_stats(uint32_t packet_strides);
 
     inline int dequeue_packet(iovec *p_iov, ssize_t sz_iov, sockaddr *__from, socklen_t *__fromlen,
@@ -405,7 +395,7 @@ protected:
     void save_stats_rx_offload(int nbytes);
     void socket_stats_init();
     bool attach_receiver(flow_tuple_with_local_if &flow_key);
-    bool detach_receiver(flow_tuple_with_local_if &flow_key);
+    bool detach_receiver(flow_tuple_with_local_if &flow_key, rfs_rule **rule_extract = nullptr);
     net_device_resources_t *create_nd_resources(const ip_addr &ip_local);
     bool destroy_nd_resources(const ip_addr &ip_local);
     void do_rings_migration_rx(resource_allocation_key &old_key);
@@ -422,16 +412,17 @@ protected:
     int set_sockopt_prio(__const void *__optval, socklen_t __optlen);
     bool ipv6_set_addr_sel_pref(int val);
     int ipv6_get_addr_sel_pref();
-    inline void handle_recv_timestamping(struct cmsg_state *cm_state);
-    inline void handle_recv_errqueue(struct cmsg_state *cm_state);
+    inline void handle_recv_timestamping(struct cmsg_state *cm_state,
+                                         timestamps_t *packet_timestamps);
     void insert_cmsg(struct cmsg_state *cm_state, int level, int type, void *data, int len);
-    void handle_cmsg(struct msghdr *msg, int flags);
+    void handle_cmsg(struct msghdr *msg);
     void process_timestamps(mem_buf_desc_t *p_desc);
     void add_cqfd_to_sock_rx_epfd(ring *p_ring);
     void remove_cqfd_from_sock_rx_epfd(ring *p_ring);
     int os_wait_sock_rx_epfd(epoll_event *ep_events, int maxevents);
     void insert_epoll_event(uint64_t events);
     int handle_exception_flow();
+    void rx_handle_cmsg(struct msghdr *msg, mem_buf_desc_t *out_buf);
 
     // Attach to all relevant rings for offloading receive flows - always used from slow path
     // According to bounded information we need to attach to all UC relevant flows
@@ -458,65 +449,42 @@ protected:
     sockinfo_state m_state = SOCKINFO_OPENED; // socket current state
     uint8_t m_n_tsing_flags = 0U;
     bool m_b_rcvtstamp = false;
-    bool m_b_zc = false;
+    bool m_b_pktinfo = false;
     bool m_b_blocking = true;
     bool m_b_rcvtstampns = false;
     bool m_skip_cq_poll_in_rx;
     rfs *m_rfs_ptr = nullptr;
     socket_stats_t *m_p_socket_stats = nullptr;
     ring *m_p_rx_ring = nullptr; // used in TCP/UDP
-    ring_ec *m_socketxtreme_ec_first = nullptr;
-    ring_ec *m_socketxtreme_ec_last = nullptr;
+    epfd_info *m_econtext = nullptr;
+    entity_context *m_entity_context = nullptr;
 
     // End of first cache line
 
-    sockinfo *m_socketxtreme_ring_list_next = nullptr;
-    void *m_fd_context; // Context data stored with socket
-    mem_buf_desc_t *m_last_zcdesc = nullptr;
+    uint32_t m_epoll_event_flags = 0U;
+    uint32_t m_epoll_event_flags_thread = 0U;
+    int m_n_rx_pkt_ready_list_count = 0;
+    size_t m_rx_pkt_ready_offset = 0U;
+    size_t m_rx_ready_byte_count = 0U;
+    multilock m_app_lock;
 
-    /* Socket error queue that keeps local errors and internal data required
-     * to provide notification ability.
-     */
-    descq_t m_error_queue;
-    lock_spin_simple m_error_queue_lock;
-
-    /* TX zcopy counter
-     * The notification itself for tx zcopy operation is a simple scalar value.
-     * Each socket maintains an internal unsigned 32-bit counter.
-     * Each send call with MSG_ZEROCOPY that successfully sends data increments
-     * the counter. The counter is not incremented on failure or if called with
-     * length zero.
-     * The counter counts system call invocations, not bytes.
-     * It wraps after UINT_MAX calls.
-     */
-    atomic_t m_zckey;
-
-    // End of second cache line
-
-    epfd_info *m_econtext = nullptr;
+public:
+    list_node<sockinfo, sockinfo::ep_ready_fd_node_offset> ep_ready_fd_node;
+    epoll_fd_rec m_fd_rec;
     wakeup_pipe m_sock_wakeup_pipe;
+
+    // End of second cache 8 bytes ago
+
+    list_node<sockinfo, sockinfo::socket_fd_list_node_offset> socket_fd_list_node;
+    list_node<sockinfo, sockinfo::ep_info_fd_node_offset> ep_info_fd_node;
+    list_node<sockinfo, sockinfo::pending_to_remove_node_offset> pending_to_remove_node;
+
+protected:
     int m_rx_epfd;
     in_protocol_t m_protocol = PROTO_UNDEFINED;
     sa_family_t m_family;
-
-public:
-    list_node<sockinfo, sockinfo::socket_fd_list_node_offset> socket_fd_list_node;
-    list_node<sockinfo, sockinfo::ep_ready_fd_node_offset> ep_ready_fd_node;
-    list_node<sockinfo, sockinfo::ep_info_fd_node_offset> ep_info_fd_node;
-    list_node<sockinfo, sockinfo::pending_to_remove_node_offset> pending_to_remove_node;
-    epoll_fd_rec m_fd_rec;
-    uint32_t m_epoll_event_flags = 0U;
-
-protected:
-    int m_fd; // identification information <socket fd>
-    /**
-     * list of pending ready packet on the Rx,
-     * each element is a pointer to the ib_conn_mgr that holds this ready rx datagram
-     */
-    size_t m_rx_pkt_ready_offset = 0U;
-    size_t m_rx_ready_byte_count = 0U;
     buff_info_t m_rx_reuse_buff; // used in TCP instead of m_rx_ring_map
-    int m_n_rx_pkt_ready_list_count = 0;
+    int m_fd; // identification information <socket fd>
     int m_rx_num_buffs_reuse;
     // used to periodically return buffers, even if threshold was not reached
     bool m_rx_reuse_buf_pending = false;
@@ -524,11 +492,10 @@ protected:
     bool m_rx_reuse_buf_postponed = false;
     bool m_reuseaddr = false; // to track setsockopt with SO_REUSEADDR
     bool m_reuseport = false; // to track setsockopt with SO_REUSEPORT
-    bool m_b_pktinfo = false;
     bool m_bind_no_port = false;
     bool m_is_ipv6only;
     uint8_t m_src_sel_flags = 0U;
-
+    uint8_t m_n_uc_ttl_hop_lim;
     multilock m_lock_rcv;
     lock_mutex m_lock_snd;
     lock_mutex m_rx_migration_lock;
@@ -543,13 +510,22 @@ protected:
     loops_timer m_loops_timer;
     ring_alloc_logic_attr m_ring_alloc_log_rx;
     ring_alloc_logic_attr m_ring_alloc_log_tx;
-    // Callback function pointer to support XLIO extra API (xlio_extra.h)
-    xlio_recv_callback_t m_rx_callback = nullptr;
-    void *m_rx_callback_context = nullptr; // user context
     struct xlio_rate_limit_t m_so_ratelimit;
     uint32_t m_pcp = 0U;
     uint32_t m_flow_tag_id = 0U; // Flow Tag for this socket
-    uint8_t m_n_uc_ttl_hop_lim;
+
+    /*
+     * XLIO Ultra API
+     * TODO Move other Ultra API fields from sockinfo_tcp to sockinfo
+     * TODO Move the fields to proper cold/hot sections in the final version.
+     */
+
+    // Polling group associated with this socket
+    poll_group *m_p_group = nullptr;
+    // Flag indicating if this is an XLIO socket
+    bool m_is_xlio_socket = false;
+    // Flag indicating if this is an XLIO socket terminat CB was called
+    bool m_is_xlio_socket_terminated = false;
 
 public:
 #if defined(DEFINED_NGINX) || defined(DEFINED_ENVOY)
@@ -605,36 +581,6 @@ void sockinfo::sock_pop_descs_rx_ready(descq_t *cache)
     unlock_rx_q();
 }
 
-xlio_socketxtreme_completion_t *sockinfo::set_events_socketxtreme(uint64_t events,
-                                                                  bool full_transaction)
-{
-    bool always_new =
-        ((events & (XLIO_SOCKETXTREME_PACKET | XLIO_SOCKETXTREME_NEW_CONNECTION_ACCEPTED)) != 0U);
-    xlio_socketxtreme_completion_t &completion =
-        m_p_rx_ring->socketxtreme_start_ec_operation(this, always_new);
-    completion.user_data = (uint64_t)m_fd_context;
-    completion.events |= events;
-
-    if (full_transaction) {
-        m_p_rx_ring->socketxtreme_end_ec_operation();
-        return nullptr;
-    }
-
-    return &completion;
-}
-
-void sockinfo::set_events(uint64_t events)
-{
-    /* Collect all events if rx ring is enabled */
-    if (safe_mce_sys().enable_socketxtreme) {
-        if (m_state == SOCKINFO_OPENED) {
-            set_events_socketxtreme(events, true);
-        }
-    } else {
-        insert_epoll_event(events);
-    }
-}
-
 void sockinfo::save_strq_stats(uint32_t packet_strides)
 {
     if (unlikely(m_p_socket_stats)) {
@@ -652,7 +598,6 @@ int sockinfo::dequeue_packet(iovec *p_iov, ssize_t sz_iov, sockaddr *__from, soc
     mem_buf_desc_t *pdesc;
     int total_rx = 0;
     uint32_t nbytes, pos;
-    bool release_buff = true;
 
     bool is_peek = in_flags & MSG_PEEK;
     int rx_pkt_ready_list_idx = 1;
@@ -674,62 +619,52 @@ int sockinfo::dequeue_packet(iovec *p_iov, ssize_t sz_iov, sockaddr *__from, soc
         }
     }
 
-    if (in_flags & MSG_XLIO_ZCOPY) {
-        release_buff = false;
-        total_rx = zero_copy_rx(p_iov, pdesc, p_out_flags);
-        if (unlikely(total_rx < 0)) {
-            return -1;
-        }
-        m_rx_pkt_ready_offset = 0;
-    } else {
 #ifdef DEFINED_UTLS
-        uint8_t tls_type = pdesc->rx.tls_type;
+    uint8_t tls_type = pdesc->rx.tls_type;
 #endif /* DEFINED_UTLS */
-        for (int i = 0; i < sz_iov && pdesc; i++) {
-            pos = 0;
-            while (pos < p_iov[i].iov_len && pdesc) {
+    for (int i = 0; i < sz_iov && pdesc; i++) {
+        pos = 0;
+        while (pos < p_iov[i].iov_len && pdesc) {
 #ifdef DEFINED_UTLS
-                if (unlikely(pdesc->rx.tls_type != tls_type)) {
-                    break;
-                }
+            if (unlikely(pdesc->rx.tls_type != tls_type)) {
+                break;
+            }
 #endif /* DEFINED_UTLS */
-                nbytes = p_iov[i].iov_len - pos;
-                if (nbytes > bytes_left) {
-                    nbytes = bytes_left;
+            nbytes = p_iov[i].iov_len - pos;
+            if (nbytes > bytes_left) {
+                nbytes = bytes_left;
+            }
+            memcpy((char *)(p_iov[i].iov_base) + pos, iov_base, nbytes);
+            pos += nbytes;
+            total_rx += nbytes;
+            m_rx_pkt_ready_offset += nbytes;
+            bytes_left -= nbytes;
+            iov_base = (uint8_t *)iov_base + nbytes;
+            if (m_b_rcvtstamp || m_n_tsing_flags) {
+                update_socket_timestamps(&pdesc->rx.timestamps);
+            }
+            if (bytes_left <= 0) {
+                if (unlikely(is_peek)) {
+                    pdesc = get_next_desc_peek(pdesc, rx_pkt_ready_list_idx);
+                } else {
+                    pdesc = get_next_desc(pdesc);
                 }
-                memcpy((char *)(p_iov[i].iov_base) + pos, iov_base, nbytes);
-                pos += nbytes;
-                total_rx += nbytes;
-                m_rx_pkt_ready_offset += nbytes;
-                bytes_left -= nbytes;
-                iov_base = (uint8_t *)iov_base + nbytes;
-                if (m_b_rcvtstamp || m_n_tsing_flags) {
-                    update_socket_timestamps(&pdesc->rx.timestamps);
-                }
-                if (bytes_left <= 0) {
-                    if (unlikely(is_peek)) {
-                        pdesc = get_next_desc_peek(pdesc, rx_pkt_ready_list_idx);
-                    } else {
-                        pdesc = get_next_desc(pdesc);
-                    }
-                    m_rx_pkt_ready_offset = 0;
-                    if (pdesc) {
-                        iov_base = pdesc->rx.frag.iov_base;
-                        bytes_left = pdesc->rx.frag.iov_len;
-                    }
+                m_rx_pkt_ready_offset = 0;
+                if (pdesc) {
+                    iov_base = pdesc->rx.frag.iov_base;
+                    bytes_left = pdesc->rx.frag.iov_len;
                 }
             }
         }
     }
 
     if (unlikely(is_peek)) {
-        m_rx_pkt_ready_offset =
-            rx_pkt_ready_offset; // if MSG_PEEK is on, m_rx_pkt_ready_offset must be zero-ed
-        // save_stats_rx_offload(total_rx); //TODO??
+        // if MSG_PEEK is on, m_rx_pkt_ready_offset must be zero-ed
+        m_rx_pkt_ready_offset = rx_pkt_ready_offset;
     } else {
         IF_STATS(m_p_socket_stats->n_rx_ready_byte_count -= total_rx);
         m_rx_ready_byte_count -= total_rx;
-        post_dequeue(release_buff);
+        post_dequeue();
         save_stats_rx_offload(total_rx);
     }
 
