@@ -19,6 +19,12 @@
 #define si_ulp_loginfo __log_info_info
 #define si_ulp_logerr  __log_info_err
 
+#define TLS_STATS(x) ((x)->get_sock_stats()->tls_counters)
+#define IF_STATS_OB(o, x)                                                                          \
+    if (unlikely((o)->get_sock_stats())) {                                                         \
+        (x);                                                                                       \
+    }
+
 /*inline*/
 ring *sockinfo_tcp_ops::get_tx_ring()
 {
@@ -55,6 +61,11 @@ bool sockinfo_tcp_ops::handle_send_ret(ssize_t ret, struct tcp_seg *seg)
     return true;
 }
 
+ssize_t sockinfo_tcp_ops_thread::tx(xlio_tx_call_attr_t &tx_arg)
+{
+    return m_p_sock->tcp_tx_thread(tx_arg);
+}
+
 #ifdef DEFINED_UTLS
 
 #include <openssl/evp.h>
@@ -74,6 +85,8 @@ struct xlio_tls_api {
                               const unsigned char *);
     int (*EVP_EncryptUpdate)(EVP_CIPHER_CTX *, unsigned char *, int *, const unsigned char *, int);
     int (*EVP_EncryptFinal_ex)(EVP_CIPHER_CTX *, unsigned char *, int *);
+    const char *(*OpenSSL_version)(int t);
+    unsigned long (*OpenSSL_version_num)();
 };
 
 static struct xlio_tls_api *g_tls_api = nullptr;
@@ -104,12 +117,20 @@ void xlio_tls_api_setup()
     XLIO_TLS_API_FIND(EVP_EncryptInit_ex);
     XLIO_TLS_API_FIND(EVP_EncryptUpdate);
     XLIO_TLS_API_FIND(EVP_EncryptFinal_ex);
+    XLIO_TLS_API_FIND(OpenSSL_version);
+    XLIO_TLS_API_FIND(OpenSSL_version_num);
     if (s_tls_api.EVP_CIPHER_CTX_new && s_tls_api.EVP_CIPHER_CTX_free &&
         s_tls_api.EVP_CIPHER_CTX_reset && s_tls_api.EVP_aes_128_gcm && s_tls_api.EVP_aes_256_gcm &&
         s_tls_api.EVP_DecryptInit_ex && s_tls_api.EVP_DecryptUpdate &&
         s_tls_api.EVP_CIPHER_CTX_ctrl && s_tls_api.EVP_DecryptFinal_ex &&
         s_tls_api.EVP_EncryptInit_ex && s_tls_api.EVP_EncryptUpdate &&
         s_tls_api.EVP_EncryptFinal_ex) {
+        if (s_tls_api.OpenSSL_version) {
+            vlog_printf(VLOG_DEBUG, "Found OpenSSL version: %s\n", s_tls_api.OpenSSL_version(0));
+        } else if (s_tls_api.OpenSSL_version_num) {
+            vlog_printf(VLOG_DEBUG, "Found OpenSSL version: 0x%lx",
+                        s_tls_api.OpenSSL_version_num());
+        }
         g_tls_api = &s_tls_api;
     }
 }
@@ -708,7 +729,6 @@ ssize_t sockinfo_tcp_ops_tls::tx(xlio_tx_call_attr_t &tx_arg)
 
     tls_arg.opcode = TX_FILE; /* Not to use hugepage zerocopy path */
     tls_arg.attr.flags = MSG_ZEROCOPY;
-    tls_arg.xlio_flags = TX_FLAG_NO_PARTIAL_WRITE;
     tls_arg.attr.iov = tls_iov;
     tls_arg.attr.sz_iov = is_zerocopy ? 3 : 1;
     tls_arg.priv.attr = PBUF_DESC_MDESC;
@@ -782,10 +802,10 @@ ssize_t sockinfo_tcp_ops_tls::tx(xlio_tx_call_attr_t &tx_arg)
         retry:
             if (!block_this_run) {
                 m_p_sock->rx_poll_on_tx_if_needed();
-                ret2 = m_p_sock->tcp_tx_express(tls_arg.attr.iov, tls_arg.attr.sz_iov, 0,
-                                                XLIO_EXPRESS_OP_TYPE_FILE_ZEROCOPY,
-                                                reinterpret_cast<void *>(rec));
-
+                ret2 = m_p_sock->tcp_tx_express(
+                    tls_arg.attr.iov, tls_arg.attr.sz_iov, 0,
+                    XLIO_EXPRESS_OP_TYPE_FILE_ZEROCOPY | XLIO_EXPRESS_MSG_SND_BUF,
+                    reinterpret_cast<void *>(rec));
             } else {
                 ret2 = m_p_sock->tcp_tx(tls_arg);
             }
@@ -1200,13 +1220,6 @@ int sockinfo_tcp_ops_tls::tls_rx_encrypt(struct pbuf *plist)
         return TLS_DECRYPT_INTERNAL;
     }
 
-    /* Set authentication tag. TODO We can avoid copy if the tag doesn't cross a pbuf boundary. */
-    copy_by_offset(buf, m_rx_offset + m_rx_rec_len - TLS_RECORD_TAG_LEN, TLS_RECORD_TAG_LEN);
-    ret = g_tls_api->EVP_CIPHER_CTX_ctrl(tls_ctx, EVP_CTRL_GCM_SET_TAG, TLS_RECORD_TAG_LEN, buf);
-    if (unlikely(!ret)) {
-        return TLS_DECRYPT_INTERNAL;
-    }
-
     /* Additional data for AEAD */
     if (is_rx_tls13()) {
         copy_by_offset(buf, m_rx_offset, 3);
@@ -1404,19 +1417,25 @@ check_single_record:
         if (decrypted_nr == 0 || tls_decrypted == TLS_RX_DECRYPTED) {
             /* Case #2 and #3. */
             ret = tls_rx_decrypt(pres);
+            IF_STATS_OB(m_p_sock,
+                        decrypted_nr == 0 ? ++(TLS_STATS(m_p_sock).n_tls_rx_records_full_enc)
+                                          : ++(TLS_STATS(m_p_sock).n_tls_rx_records_head_enc));
         } else if (tls_decrypted == TLS_RX_AUTH_FAIL) {
             /* Case #1. */
             ret = TLS_DECRYPT_BAD_MAC;
+            IF_STATS_OB(m_p_sock, ++(TLS_STATS(m_p_sock).n_tls_rx_records_hw_auth_fail));
         } else {
             /* Case #4 and #5. */
             switch (((mem_buf_desc_t *)pres)->rx.tls_decrypted) {
             case TLS_RX_RESYNC:
                 /* Fallthrough */
             case TLS_RX_ENCRYPTED:
-                ret = tls_rx_decrypt(pres);
+                ret = tls_rx_decrypt(pres); // Case #5
+                IF_STATS_OB(m_p_sock, ++(TLS_STATS(m_p_sock).n_tls_rx_records_mix_enc));
                 /* Fallthrough */
             case TLS_RX_DECRYPTED:
-                ret = ret ?: tls_rx_encrypt(pres) ?: tls_rx_decrypt(pres);
+                ret = ret ?: tls_rx_encrypt(pres) ?: tls_rx_decrypt(pres); // Case #4
+                IF_STATS_OB(m_p_sock, ++(TLS_STATS(m_p_sock).n_tls_rx_records_tail_enc));
                 break;
             default:
                 /* Unexpected case. */
@@ -1424,13 +1443,6 @@ check_single_record:
                 break;
             }
         }
-
-        /* Statistics */
-        if (unlikely(m_p_sock->get_sock_stats())) {
-            m_p_sock->get_sock_stats()->tls_counters.n_tls_rx_records_enc += !!(decrypted_nr == 0);
-            m_p_sock->get_sock_stats()->tls_counters.n_tls_rx_records_partial +=
-                !!(decrypted_nr != 0);
-        }
     }
 
     /* Handle decryption failures. */
@@ -1438,6 +1450,7 @@ check_single_record:
         terminate_session_fatal(ret == TLS_DECRYPT_BAD_MAC ? TLS_BAD_RECORD_MAC
                                                            : TLS_INTERNAL_ERROR);
         m_refused_data = pres;
+        IF_STATS_OB(m_p_sock, ++(TLS_STATS(m_p_sock).n_tls_rx_records_sw_dec_fail));
         return ERR_OK;
     }
 
@@ -1471,7 +1484,7 @@ check_single_record:
 
     ++m_next_recno_rx;
 
-    tcp_recved(m_p_sock->get_pcb(), m_tls_rec_overhead);
+    tcp_recved(m_p_sock->get_pcb(), m_tls_rec_overhead, true);
     if (likely(pres)) {
         assert(pres->tot_len == (m_rx_rec_len - m_tls_rec_overhead));
         err = sockinfo_tcp::rx_lwip_cb((void *)m_p_sock, m_p_sock->get_pcb(), pres, ERR_OK);
