@@ -7,34 +7,32 @@
 #ifndef SYS_VARS_H
 #define SYS_VARS_H
 
-#include <stdio.h>
-#include <sched.h>
-#include <string>
 #include <netinet/in.h>
+#include <sched.h>
+#include <stdio.h>
+#include <string>
 
-#include "vtypes.h"
 #include "config.h"
+#include "vtypes.h"
 
+#include "core/config/config_registry.h"
 #include "core/ib/base/verbs_extra.h"
-#include "core/util/sysctl_reader.h"
 #include "core/util/agent_def.h"
+#include "core/util/sysctl_reader.h"
 #include "core/xlio_extra.h"
 
+extern bool g_use_new_config;
+
 typedef enum {
     MCE_SPEC_NONE = 0,
-    MCE_SPEC_SOCKPERF_ULTRA_LATENCY,
-    MCE_SPEC_SOCKPERF_LATENCY,
+    MCE_SPEC_ULTRA_LATENCY,
+    MCE_SPEC_LATENCY,
     MCE_SPEC_NGINX,
     MCE_SPEC_NGINX_DPU,
     MCE_SPEC_NVME_BF3,
     MCE_SPEC_ALL /* last element */
 } xlio_spec_t;
 
-enum {
-    IOCTL_USER_ALLOC_TX = (1 << 0),
-    IOCTL_USER_ALLOC_RX = (1 << 1),
-};
-
 typedef void *(*alloc_t)(size_t);
 typedef void (*free_t)(void *);
 
@@ -117,8 +115,6 @@ typedef enum { MTU_FOLLOW_INTERFACE = 0 } mtu_mode_t;
 typedef enum {
     CTL_THREAD_DISABLE = 0,
     CTL_THREAD_DELEGATE_TCP_TIMERS,
-    CTL_THREAD_WITH_WAKEUP,
-    CTL_THREAD_NO_WAKEUP,
     CTL_THREAD_LAST
 } tcp_ctl_thread_t;
 
@@ -174,18 +170,10 @@ OPTIONS_FROM_TO_STR_DEF;
 } // namespace option_3
 
 namespace option_tcp_ctl_thread {
-typedef enum {
-    CTL_THREAD_DISABLE = 0,
-    CTL_THREAD_DELEGATE_TCP_TIMERS,
-    CTL_THREAD_WITH_WAKEUP,
-    CTL_THREAD_NO_WAKEUP,
-    CTL_THREAD_LAST
-} mode_t;
+typedef enum { CTL_THREAD_DISABLE = 0, CTL_THREAD_DELEGATE_TCP_TIMERS, CTL_THREAD_LAST } mode_t;
 OPTIONS_FROM_TO_STR_DEF;
 } // namespace option_tcp_ctl_thread
 
-#define tcp_ctl_thread_on(var) ((var) > option_tcp_ctl_thread::CTL_THREAD_DELEGATE_TCP_TIMERS)
-
 namespace option_alloc_type {
 typedef enum {
     ANON = 0,
@@ -206,11 +194,18 @@ typedef enum {
 } alloc_mode_t;
 
 ////////////////////////////////////////////////////////////////////////////////
+
+#define SYS_VAR_EXCEPTION_HANDLING    "XLIO_EXCEPTION_HANDLING"
+#define CONFIG_VAR_EXCEPTION_HANDLING "core.exception_handling"
+
 class xlio_exception_handling {
 public:
     static const char *getName() { return "Exception handling mode"; }
 
-    static const char *getSysVar() { return "XLIO_EXCEPTION_HANDLING"; }
+    static const char *getSysVar()
+    {
+        return g_use_new_config ? CONFIG_VAR_EXCEPTION_HANDLING : SYS_VAR_EXCEPTION_HANDLING;
+    }
 
     typedef enum {
         MODE_FIRST = -3,
@@ -301,24 +296,28 @@ public:
     enum hyper_t { HYPER_NONE = 0, HYPER_XEN, HYPER_KVM, HYPER_MSHV, HYPER_VMWARE };
 
 public:
-    void get_env_params();
+    void get_params();
+    bool is_threads_mode() const { return worker_threads > 0; }
 
     // Update parameters for multi-process applications
     void update_multi_process_params();
-
+    void fixup_params();
     char *app_name;
     char app_id[MAX_APP_ID_LENGHT];
 
     uint32_t mce_spec;
 
-    bool print_report;
+    option_3::mode_t print_report;
     bool quick_start;
     vlog_levels_t log_level;
     uint32_t log_details;
     char log_filename[PATH_MAX];
     char stats_filename[PATH_MAX];
     char stats_shmem_dirname[PATH_MAX];
-    char conf_filename[PATH_MAX];
+
+    // can be used both for legacy config file path and transport control rules
+    char acceleration_rules[PATH_MAX];
+
     char service_notify_dir[PATH_MAX];
     bool service_enable;
     bool log_colors;
@@ -333,7 +332,6 @@ public:
     int ring_migration_ratio_rx;
     int ring_limit_per_interface;
     int ring_dev_mem_tx;
-    int tcp_max_syn_rate;
 
     size_t zc_cache_threshold;
     uint32_t tx_buf_size;
@@ -360,9 +358,10 @@ public:
     uint32_t rx_ready_byte_min_limit;
     uint32_t rx_prefetch_bytes;
     uint32_t rx_prefetch_bytes_before_poll;
-    uint32_t rx_cq_drain_rate_nsec; // If enabled this will cause the Rx to drain all wce in CQ
-                                    // before returning to user, Else (Default: Disbaled) it will
-                                    // return when first ready packet is in socket queue
+    uint32_t rx_cq_drain_rate_nsec; // If enabled this will cause the Rx to drain
+                                    // all wce in CQ before returning to user,
+                                    // Else (Default: Disbaled) it will return
+                                    // when first ready packet is in socket queue
     uint32_t rx_delta_tsc_between_cq_polls;
 
     uint32_t strq_stride_num_per_rwqe;
@@ -380,7 +379,6 @@ public:
     bool mc_force_flowtag;
 
     int32_t select_poll_num;
-    bool select_poll_os_force;
     uint32_t select_poll_os_ratio;
     uint32_t select_skip_os_fd_check;
     bool select_handle_cpu_usage_stats;
@@ -399,9 +397,9 @@ public:
     bool cq_keep_qp_full;
     uint32_t qp_compensation_level;
     uint32_t max_tso_sz;
-    size_t user_huge_page_size;
 
     bool offloaded_sockets;
+    uint16_t worker_threads;
     uint32_t timer_resolution_msec;
     uint32_t tcp_timer_resolution_msec;
     option_tcp_ctl_thread::mode_t tcp_ctl_thread;
@@ -427,21 +425,19 @@ public:
     char internal_thread_cpuset[FILENAME_MAX];
     char internal_thread_affinity_str[FILENAME_MAX];
     cpu_set_t internal_thread_affinity;
-    bool internal_thread_arm_cq_enabled;
     skip_poll_in_rx_t skip_poll_in_rx;
     multilock_t multilock;
-
-    bool enable_socketxtreme;
     option_3::mode_t enable_tso;
     option_3::mode_t enable_lro;
     option_3::mode_t enable_strq_env;
 #ifdef DEFINED_UTLS
     bool enable_utls_rx;
     bool enable_utls_tx;
-    // DEK cache size high-watermark. Max number of DEKs to be stored in the cache.
+    // DEK cache size high-watermark. Max number of DEKs to be stored in the
+    // cache.
     size_t utls_high_wmark_dek_cache_size;
-    // DEK cache size low-watermark. Min number of available DEKs required in the cache
-    // to perform Crypto-Sync and reuse.
+    // DEK cache size low-watermark. Min number of available DEKs required in the
+    // cache to perform Crypto-Sync and reuse.
     size_t utls_low_wmark_dek_cache_size;
 #endif /* DEFINED_UTLS */
     uint32_t timer_netlink_update_msec;
@@ -452,14 +448,14 @@ public:
     uint32_t neigh_num_err_retries;
 
     sysctl_reader_t &sysctl_reader;
-    // Workaround for #3440429: postpone close(2) to the socket destructor, so the sockfd is closed
-    // after the rfs rule is destroyed. Otherwise, flow_tag or TCP port can be reused too early.
+    // Workaround for #3440429: postpone close(2) to the socket destructor, so the
+    // sockfd is closed after the rfs rule is destroyed. Otherwise, flow_tag or
+    // TCP port can be reused too early.
     bool deferred_close;
     bool tcp_abort_on_close;
     bool rx_poll_on_tx_tcp;
     bool rx_cq_wait_ctrl;
     hyper_t hypervisor;
-    bool trigger_dummy_send_getsockname;
 #if defined(DEFINED_NGINX)
     int nginx_udp_socket_pool_size;
     int nginx_udp_socket_pool_rx_num_buffs_reuse;
@@ -475,35 +471,63 @@ public:
     uint32_t tcp_send_buffer_size;
     uint32_t tx_segs_ring_batch_tcp;
     uint32_t tx_segs_pool_batch_tcp;
-    /* This field should be used to store and use data for XLIO_EXTRA_API_IOCTL */
     struct {
-        struct {
-            uint8_t flags;
-            alloc_t memalloc;
-            free_t memfree;
-        } user_alloc;
-    } m_ioctl;
+        alloc_t memalloc;
+        free_t memfree;
+    } user_alloc;
 
 private:
+    void get_app_name();
+    void get_env_params();
+    void initialize_base_variables(const config_registry &registry);
+    void read_hypervisor_info();
+    void configure_striding_rq(const config_registry &registry);
+    void configure_running_mode(const config_registry &registry);
+    void detect_application_profile(const config_registry &registry);
+    void apply_spec_profile_optimizations();
+    void apply_ultra_latency_profile();
+    void apply_latency_profile();
+    void apply_nginx_profile();
+    void apply_nvme_bf3_profile();
+    void configure_monitor(const config_registry &registry);
+    void configure_buffer_allocation(const config_registry &registry);
+    void configure_tcp_parameters(const config_registry &registry);
+    void configure_buffer_sizes(const config_registry &registry);
+    void configure_polling_mechanism(const config_registry &registry);
+    void configure_completion_queue(const config_registry &registry);
+    void configure_thread_affinity(const config_registry &registry);
+    void configure_memory_limits(const config_registry &registry);
+    void configure_multicast_settings(const config_registry &registry);
+    void configure_network_protocols(const config_registry &registry);
+    void configure_application_specifics(const config_registry &registry);
+    void configure_syscall_behavior(const config_registry &registry);
+    void configure_network_timing(const config_registry &registry);
+    void apply_config_from_registry();
     void print_xlio_load_failure_msg();
     int list_to_cpuset(char *cpulist, cpu_set_t *cpu_set);
     int hex_to_cpuset(char *start, cpu_set_t *cpu_set);
     int env_to_cpuset(char *orig_start, cpu_set_t *cpu_set);
-    void read_env_variable_with_pid(char *mce_sys_name, size_t mce_sys_max_size, char *env_ptr);
+    void read_env_variable_with_pid(char *mce_sys_name, size_t mce_sys_max_size,
+                                    const char *env_ptr);
     bool check_cpuinfo_flag(const char *flag);
     bool cpuid_hv();
     const char *cpuid_hv_vendor();
     void read_hv();
-    void read_strq_strides_num();
-    void read_strq_stride_size_bytes();
+    void read_strq_strides_num(const config_registry &registry);
+    void read_strq_stride_size_bytes(const config_registry &registry);
+    void legacy_read_strq_strides_num();
+    void legacy_read_strq_stride_size_bytes();
 
     // prevent unautothrized creation of objects
+    // coverity[CTOR_DTOR_LEAK] suppression for app_name member is needed because Coverity
+    // incorrectly flags it as a memory leak
     mce_sys_var()
         : sysctl_reader(sysctl_reader_t::instance())
-        , m_ioctl {}
+        , user_alloc {}
     {
         // coverity[uninit_member]
-        get_env_params();
+        // coverity[CTOR_DTOR_LEAK]
+        get_params();
     }
     mce_sys_var(const mce_sys_var &);
     mce_sys_var &operator=(const mce_sys_var &);
@@ -511,6 +535,10 @@ private:
 
 extern mce_sys_var &safe_mce_sys();
 
+/*
+ * This block consists of library specific configuration
+ * environment variables
+ */
 /*
  * This block consists of library specific configuration
  * environment variables
@@ -579,7 +607,6 @@ extern mce_sys_var &safe_mce_sys();
 
 #define SYS_VAR_SELECT_CPU_USAGE_STATS "XLIO_CPU_USAGE_STATS"
 #define SYS_VAR_SELECT_NUM_POLLS       "XLIO_SELECT_POLL"
-#define SYS_VAR_SELECT_POLL_OS_FORCE   "XLIO_SELECT_POLL_OS_FORCE"
 #define SYS_VAR_SELECT_POLL_OS_RATIO   "XLIO_SELECT_POLL_OS_RATIO"
 #define SYS_VAR_SELECT_SKIP_OS         "XLIO_SELECT_SKIP_OS"
 
@@ -597,8 +624,8 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_CQ_KEEP_QP_FULL           "XLIO_CQ_KEEP_QP_FULL"
 #define SYS_VAR_QP_COMPENSATION_LEVEL     "XLIO_QP_COMPENSATION_LEVEL"
 #define SYS_VAR_MAX_TSO_SIZE              "XLIO_MAX_TSO_SIZE"
-#define SYS_VAR_USER_HUGE_PAGE_SIZE       "XLIO_USER_HUGE_PAGE_SIZE"
 #define SYS_VAR_OFFLOADED_SOCKETS         "XLIO_OFFLOADED_SOCKETS"
+#define SYS_VAR_WORKER_THREADS            "XLIO_WORKER_THREADS"
 #define SYS_VAR_TIMER_RESOLUTION_MSEC     "XLIO_TIMER_RESOLUTION_MSEC"
 #define SYS_VAR_TCP_TIMER_RESOLUTION_MSEC "XLIO_TCP_TIMER_RESOLUTION_MSEC"
 #define SYS_VAR_TCP_CTL_THREAD            "XLIO_TCP_CTL_THREAD"
@@ -630,13 +657,10 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_SRC_PORT_STRIDE "XLIO_SRC_PORT_STRIDE"
 #define SYS_VAR_DISTRIBUTE_CQ   "XLIO_DISTRIBUTE_CQ"
 #endif
-#define SYS_VAR_TCP_MAX_SYN_RATE "XLIO_TCP_MAX_SYN_RATE"
-#define SYS_VAR_MSS              "XLIO_MSS"
-#define SYS_VAR_TCP_CC_ALGO      "XLIO_TCP_CC_ALGO"
-#define SYS_VAR_SPEC             "XLIO_SPEC"
-
-#define SYS_VAR_SOCKETXTREME "XLIO_SOCKETXTREME"
-#define SYS_VAR_TSO          "XLIO_TSO"
+#define SYS_VAR_MSS         "XLIO_MSS"
+#define SYS_VAR_TCP_CC_ALGO "XLIO_TCP_CC_ALGO"
+#define SYS_VAR_SPEC        "XLIO_SPEC"
+#define SYS_VAR_TSO         "XLIO_TSO"
 #ifdef DEFINED_UTLS
 #define SYS_VAR_UTLS_RX                        "XLIO_UTLS_RX"
 #define SYS_VAR_UTLS_TX                        "XLIO_UTLS_TX"
@@ -648,7 +672,6 @@ extern mce_sys_var &safe_mce_sys();
 
 #define SYS_VAR_INTERNAL_THREAD_AFFINITY "XLIO_INTERNAL_THREAD_AFFINITY"
 #define SYS_VAR_INTERNAL_THREAD_CPUSET   "XLIO_INTERNAL_THREAD_CPUSET"
-#define SYS_VAR_INTERNAL_THREAD_ARM_CQ   "XLIO_INTERNAL_THREAD_ARM_CQ"
 
 #define SYS_VAR_NETLINK_TIMER_MSEC "XLIO_NETLINK_TIMER"
 
@@ -656,20 +679,172 @@ extern mce_sys_var &safe_mce_sys();
 #define SYS_VAR_NEIGH_UC_ARP_DELAY_MSEC "XLIO_NEIGH_UC_ARP_DELAY_MSEC"
 #define SYS_VAR_NEIGH_NUM_ERR_RETRIES   "XLIO_NEIGH_NUM_ERR_RETRIES"
 
-#define SYS_VAR_DEFERRED_CLOSE                 "XLIO_DEFERRED_CLOSE"
-#define SYS_VAR_TCP_ABORT_ON_CLOSE             "XLIO_TCP_ABORT_ON_CLOSE"
-#define SYS_VAR_RX_POLL_ON_TX_TCP              "XLIO_RX_POLL_ON_TX_TCP"
-#define SYS_VAR_RX_CQ_WAIT_CTRL                "XLIO_RX_CQ_WAIT_CTRL"
-#define SYS_VAR_TRIGGER_DUMMY_SEND_GETSOCKNAME "XLIO_TRIGGER_DUMMY_SEND_GETSOCKNAME"
-#define SYS_VAR_TCP_SEND_BUFFER_SIZE           "XLIO_TCP_SEND_BUFFER_SIZE"
-#define SYS_VAR_SKIP_POLL_IN_RX                "XLIO_SKIP_POLL_IN_RX"
-#define SYS_VAR_MULTILOCK                      "XLIO_MULTILOCK"
+#define SYS_VAR_DEFERRED_CLOSE       "XLIO_DEFERRED_CLOSE"
+#define SYS_VAR_TCP_ABORT_ON_CLOSE   "XLIO_TCP_ABORT_ON_CLOSE"
+#define SYS_VAR_RX_POLL_ON_TX_TCP    "XLIO_RX_POLL_ON_TX_TCP"
+#define SYS_VAR_RX_CQ_WAIT_CTRL      "XLIO_RX_CQ_WAIT_CTRL"
+#define SYS_VAR_TCP_SEND_BUFFER_SIZE "XLIO_TCP_SEND_BUFFER_SIZE"
+#define SYS_VAR_SKIP_POLL_IN_RX      "XLIO_SKIP_POLL_IN_RX"
+#define SYS_VAR_MULTILOCK            "XLIO_MULTILOCK"
+
+#define CONFIG_VAR_PRINT_REPORT        "monitor.exit_report"
+#define CONFIG_VAR_LOG_LEVEL           "monitor.log.level"
+#define CONFIG_VAR_LOG_DETAILS         "monitor.log.details"
+#define CONFIG_VAR_LOG_FILENAME        "monitor.log.file_path"
+#define CONFIG_VAR_STATS_FILENAME      "monitor.stats.file_path"
+#define CONFIG_VAR_STATS_SHMEM_DIRNAME "monitor.stats.shmem_dir"
+#define CONFIG_VAR_SERVICE_DIR         "core.daemon.dir"
+#define CONFIG_VAR_SERVICE_ENABLE      "core.daemon.enable"
+#define CONFIG_VAR_LOG_COLORS          "monitor.log.colors"
+#define CONFIG_VAR_APPLICATION_ID      "acceleration_control.app_id"
+#define CONFIG_VAR_HANDLE_SIGINTR      "core.signals.sigint.exit"
+#define CONFIG_VAR_HANDLE_SIGSEGV      "core.signals.sigsegv.backtrace"
+#define CONFIG_VAR_STATS_FD_NUM        "monitor.stats.fd_num"
+#define CONFIG_VAR_QUICK_START         "core.quick_init"
+
+#define CONFIG_VAR_RING_ALLOCATION_LOGIC_TX "performance.rings.tx.allocation_logic"
+#define CONFIG_VAR_RING_ALLOCATION_LOGIC_RX "performance.rings.rx.allocation_logic"
+#define CONFIG_VAR_RING_MIGRATION_RATIO_TX  "performance.rings.tx.migration_ratio"
+#define CONFIG_VAR_RING_MIGRATION_RATIO_RX  "performance.rings.rx.migration_ratio"
+#define CONFIG_VAR_RING_LIMIT_PER_INTERFACE "performance.rings.max_per_interface"
+#define CONFIG_VAR_RING_DEV_MEM_TX          "performance.rings.tx.max_on_device_memory"
+
+#define CONFIG_VAR_ZC_CACHE_THRESHOLD    "core.syscall.sendfile_cache_limit"
+#define CONFIG_VAR_TX_BUF_SIZE           "performance.buffers.tx.buf_size"
+#define CONFIG_VAR_TCP_NODELAY_TRESHOLD  "network.protocols.tcp.nodelay.byte_threshold"
+#define CONFIG_VAR_TX_NUM_WRE            "performance.rings.tx.ring_elements_count"
+#define CONFIG_VAR_TX_NUM_WRE_TO_SIGNAL  "performance.rings.tx.completion_batch_size"
+#define CONFIG_VAR_TX_MAX_INLINE         "performance.rings.tx.max_inline_size"
+#define CONFIG_VAR_TX_MC_LOOPBACK        "network.multicast.mc_loopback"
+#define CONFIG_VAR_TX_NONBLOCKED_EAGAINS "performance.polling.nonblocking_eagain"
+#define CONFIG_VAR_TX_PREFETCH_BYTES     "performance.buffers.tx.prefetch_size"
+#define CONFIG_VAR_TX_BUFS_BATCH_TCP     "performance.rings.tx.tcp_buffer_batch"
+#define CONFIG_VAR_TX_SEGS_BATCH_TCP     "performance.buffers.tcp_segments.socket_batch_size"
+
+#define CONFIG_VAR_STRQ                            "hardware_features.striding_rq.enable"
+#define CONFIG_VAR_STRQ_NUM_STRIDES                "hardware_features.striding_rq.strides_num"
+#define CONFIG_VAR_STRQ_STRIDE_SIZE_BYTES          "hardware_features.striding_rq.stride_size"
+#define CONFIG_VAR_STRQ_STRIDES_COMPENSATION_LEVEL "performance.rings.rx.spare_strides"
+
+#define CONFIG_VAR_RX_BUF_SIZE                   "performance.buffers.rx.buf_size"
+#define CONFIG_VAR_RX_NUM_WRE                    "performance.rings.rx.ring_elements_count"
+#define CONFIG_VAR_RX_NUM_WRE_TO_POST_RECV       "performance.rings.rx.post_batch_size"
+#define CONFIG_VAR_RX_NUM_POLLS                  "performance.polling.blocking_rx_poll_usec"
+#define CONFIG_VAR_RX_NUM_POLLS_INIT             "performance.polling.offload_transition_poll_count"
+#define CONFIG_VAR_RX_UDP_POLL_OS_RATIO          "performance.polling.rx_kernel_fd_attention_level"
+#define CONFIG_VAR_HW_TS_CONVERSION_MODE         "network.timing.hw_ts_conversion"
+#define CONFIG_VAR_RX_POLL_YIELD                 "performance.polling.yield_on_poll"
+#define CONFIG_VAR_RX_BYTE_MIN_LIMIT             "performance.override_rcvbuf_limit"
+#define CONFIG_VAR_RX_PREFETCH_BYTES             "performance.buffers.rx.prefetch_size"
+#define CONFIG_VAR_RX_PREFETCH_BYTES_BEFORE_POLL "performance.buffers.rx.prefetch_before_poll"
+#define CONFIG_VAR_RX_CQ_DRAIN_RATE_NSEC         "performance.completion_queue.rx_drain_rate_nsec"
+#define CONFIG_VAR_GRO_STREAMS_MAX               "performance.max_gro_streams"
+#define CONFIG_VAR_DISABLE_FLOW_TAG              "performance.steering_rules.disable_flowtag"
+#define CONFIG_VAR_TCP_2T_RULES                  "performance.steering_rules.tcp.2t_rules"
+#define CONFIG_VAR_TCP_3T_RULES                  "performance.steering_rules.tcp.3t_rules"
+#define CONFIG_VAR_UDP_3T_RULES                  "performance.steering_rules.udp.3t_rules"
+#define CONFIG_VAR_ETH_MC_L2_ONLY_RULES          "performance.steering_rules.udp.only_mc_l2_rules"
+#define CONFIG_VAR_MC_FORCE_FLOWTAG              "network.multicast.mc_flowtag_acceleration"
+#define CONFIG_VAR_TX_SEGS_RING_BATCH_TCP        "performance.buffers.tcp_segments.ring_batch_size"
+#define CONFIG_VAR_TX_SEGS_POOL_BATCH_TCP        "performance.buffers.tcp_segments.pool_batch_size"
+
+#define CONFIG_VAR_SELECT_CPU_USAGE_STATS "monitor.stats.cpu_usage"
+#define CONFIG_VAR_SELECT_NUM_POLLS       "performance.polling.iomux.poll_usec"
+#define CONFIG_VAR_SELECT_POLL_OS_RATIO   "performance.polling.iomux.poll_os_ratio"
+#define CONFIG_VAR_SELECT_SKIP_OS         "performance.polling.iomux.skip_os"
+
+#define CONFIG_VAR_CQ_MODERATION_ENABLE "performance.completion_queue.interrupt_moderation.enable"
+#define CONFIG_VAR_CQ_MODERATION_COUNT                                                             \
+    "performance.completion_queue.interrupt_moderation.packet_count"
+#define CONFIG_VAR_CQ_MODERATION_PERIOD_USEC                                                       \
+    "performance.completion_queue.interrupt_moderation.period_usec"
+#define CONFIG_VAR_CQ_AIM_MAX_COUNT                                                                \
+    "performance.completion_queue.interrupt_moderation.adaptive_count"
+#define CONFIG_VAR_CQ_AIM_MAX_PERIOD_USEC                                                          \
+    "performance.completion_queue.interrupt_moderation.adaptive_period_usec"
+#define CONFIG_VAR_CQ_AIM_INTERVAL_MSEC                                                            \
+    "performance.completion_queue.interrupt_moderation.adaptive_change_"                           \
+    "frequency_msec"
+#define CONFIG_VAR_CQ_AIM_INTERRUPTS_RATE_PER_SEC                                                  \
+    "performance.completion_queue.interrupt_moderation.adaptive_interrupt_per_sec"
+
+#define CONFIG_VAR_CQ_POLL_BATCH_MAX         "performance.polling.max_rx_poll_batch"
+#define CONFIG_VAR_PROGRESS_ENGINE_INTERVAL  "performance.completion_queue.periodic_drain_msec"
+#define CONFIG_VAR_PROGRESS_ENGINE_WCE_MAX   "performance.completion_queue.periodic_drain_max_cqes"
+#define CONFIG_VAR_CQ_KEEP_QP_FULL           "performance.completion_queue.keep_full"
+#define CONFIG_VAR_QP_COMPENSATION_LEVEL     "performance.rings.rx.spare_buffers"
+#define CONFIG_VAR_MAX_TSO_SIZE              "hardware_features.tcp.tso.max_size"
+#define CONFIG_VAR_WORKER_THREADS            "performance.threading.worker_threads"
+#define CONFIG_VAR_OFFLOADED_SOCKETS         "acceleration_control.default_acceleration"
+#define CONFIG_VAR_TIMER_RESOLUTION_MSEC     "performance.threading.internal_handler.timer_msec"
+#define CONFIG_VAR_TCP_TIMER_RESOLUTION_MSEC "network.protocols.tcp.timer_msec"
+#define CONFIG_VAR_TCP_CTL_THREAD            "performance.threading.internal_handler.behavior"
+#define CONFIG_VAR_TCP_TIMESTAMP_OPTION      "network.protocols.tcp.timestamps"
+#define CONFIG_VAR_TCP_NODELAY               "network.protocols.tcp.nodelay.enable"
+#define CONFIG_VAR_TCP_QUICKACK              "network.protocols.tcp.quickack"
+#define CONFIG_VAR_TCP_PUSH_FLAG             "network.protocols.tcp.push"
+#define CONFIG_VAR_AVOID_SYS_CALLS_ON_TCP_FD "core.syscall.avoid_ctl_syscalls"
+#define CONFIG_VAR_ALLOW_PRIVILEGED_SOCK_OPT "core.syscall.allow_privileged_sockopt"
+#define CONFIG_VAR_WAIT_AFTER_JOIN_MSEC      "network.multicast.wait_after_join_msec"
+#define CONFIG_VAR_BUFFER_BATCHING_MODE      "performance.buffers.batching_mode"
+#define CONFIG_VAR_MEM_ALLOC_TYPE            "core.resources.hugepages.enable"
+#define CONFIG_VAR_MEMORY_LIMIT              "core.resources.memory_limit"
+#define CONFIG_VAR_MEMORY_LIMIT_USER         "core.resources.external_memory_limit"
+#define CONFIG_VAR_HEAP_METADATA_BLOCK       "core.resources.heap_metadata_block_size"
+#define CONFIG_VAR_HUGEPAGE_SIZE             "core.resources.hugepages.size"
+#define CONFIG_VAR_FORK                      "core.syscall.fork_support"
+#define CONFIG_VAR_CLOSE_ON_DUP2             "core.syscall.dup2_close_fd"
+#define CONFIG_VAR_MTU                       "network.protocols.ip.mtu"
+#if defined(DEFINED_NGINX)
+#define CONFIG_VAR_NGINX_WORKERS_NUM                 "applications.nginx.workers_num"
+#define CONFIG_VAR_NGINX_UDP_POOL_SIZE               "applications.nginx.udp_pool_size"
+#define CONFIG_VAR_NGINX_UDP_POOL_RX_NUM_BUFFS_REUSE "applications.nginx.udp_socket_pool_reuse"
+#endif
+#if defined(DEFINED_ENVOY)
+#define CONFIG_VAR_ENVOY_WORKERS_NUM "XLIO_ENVOY_WORKERS_NUM"
+#endif /* DEFINED_ENVOY */
+#if defined(DEFINED_NGINX) || defined(DEFINED_ENVOY)
+#define CONFIG_VAR_SRC_PORT_STRIDE "applications.nginx.src_port_stride"
+#define CONFIG_VAR_DISTRIBUTE_CQ   "applications.nginx.distribute_cq"
+#endif
+#define CONFIG_VAR_MSS         "network.protocols.tcp.mss"
+#define CONFIG_VAR_TCP_CC_ALGO "network.protocols.tcp.congestion_control"
+#define CONFIG_VAR_SPEC        "profiles.spec"
+
+#define CONFIG_VAR_TSO "hardware_features.tcp.tso.enable"
+#ifdef DEFINED_UTLS
+#define CONFIG_VAR_UTLS_RX "hardware_features.tcp.tls_offload.rx_enable"
+#define CONFIG_VAR_UTLS_TX "hardware_features.tcp.tls_offload.tx_enable"
+#define CONFIG_VAR_UTLS_HIGH_WMARK_DEK_CACHE_SIZE                                                  \
+    "hardware_features.tcp.tls_offload.dek_cache_max_size"
+#define CONFIG_VAR_UTLS_LOW_WMARK_DEK_CACHE_SIZE                                                   \
+    "hardware_features.tcp.tls_offload.dek_cache_min_size"
+#endif /* DEFINED_UTLS */
+
+#define CONFIG_VAR_LRO "hardware_features.tcp.lro"
+
+#define CONFIG_VAR_INTERNAL_THREAD_AFFINITY "performance.threading.cpu_affinity"
+#define CONFIG_VAR_INTERNAL_THREAD_CPUSET   "performance.threading.cpuset"
+
+#define CONFIG_VAR_NETLINK_TIMER_MSEC "network.neighbor.update_interval_msec"
+
+#define CONFIG_VAR_NEIGH_UC_ARP_QUATA      "network.neighbor.arp.uc_retries"
+#define CONFIG_VAR_NEIGH_UC_ARP_DELAY_MSEC "network.neighbor.arp.uc_delay_msec"
+#define CONFIG_VAR_NEIGH_NUM_ERR_RETRIES   "network.neighbor.errors_before_reset"
+
+#define CONFIG_VAR_DEFERRED_CLOSE       "core.syscall.deferred_close"
+#define CONFIG_VAR_TCP_ABORT_ON_CLOSE   "network.protocols.tcp.linger_0"
+#define CONFIG_VAR_RX_POLL_ON_TX_TCP    "performance.polling.rx_poll_on_tx_tcp"
+#define CONFIG_VAR_RX_CQ_WAIT_CTRL      "performance.polling.rx_cq_wait_ctrl"
+#define CONFIG_VAR_TCP_SEND_BUFFER_SIZE "network.protocols.tcp.wmem"
+#define CONFIG_VAR_SKIP_POLL_IN_RX      "performance.polling.skip_cq_on_rx"
+#define CONFIG_VAR_MULTILOCK            "performance.threading.mutex_over_spinlock"
 
 /*
  * This block consists of default values for library specific
  * configuration variables
  */
-#define MCE_DEFAULT_PRINT_REPORT             (false)
+#define MCE_DEFAULT_PRINT_REPORT             (option_3::AUTO)
 #define MCE_DEFAULT_TCP_SEND_BUFFER_SIZE     (1024 * 1024)
 #define MCE_DEFAULT_LOG_FILE                 ("")
 #define MCE_DEFAULT_CONF_FILE                ("/etc/libxlio.conf")
@@ -690,7 +865,6 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_RING_MIGRATION_RATIO_RX  (-1)
 #define MCE_DEFAULT_RING_LIMIT_PER_INTERFACE (0)
 #define MCE_DEFAULT_RING_DEV_MEM_TX          (0)
-#define MCE_DEFAULT_TCP_MAX_SYN_RATE         (0)
 #define MCE_DEFAULT_TCP_NODELAY_TRESHOLD     (0)
 #define MCE_DEFAULT_ZC_CACHE_THRESHOLD       (10LU * 1024 * 1024 * 1024) // 10GB
 #define MCE_DEFAULT_TX_BUF_SIZE              (0)
@@ -736,7 +910,6 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_ETH_MC_L2_ONLY_RULES          (false)
 #define MCE_DEFAULT_MC_FORCE_FLOWTAG              (false)
 #define MCE_DEFAULT_SELECT_NUM_POLLS              (100000)
-#define MCE_DEFAULT_SELECT_POLL_OS_FORCE          (0)
 #define MCE_DEFAULT_SELECT_POLL_OS_RATIO          (10)
 #define MCE_DEFAULT_SELECT_SKIP_OS                (4)
 #define MCE_DEFAULT_SELECT_CPU_USAGE_STATS        (false)
@@ -755,8 +928,6 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_PROGRESS_ENGINE_INTERVAL_MSEC  (10)
 #define MCE_DEFAULT_PROGRESS_ENGINE_WCE_MAX        (10000)
 #define MCE_DEFAULT_CQ_KEEP_QP_FULL                (true)
-#define MCE_DEFAULT_USER_HUGE_PAGE_SIZE            (2 * 1024 * 1024)
-#define MCE_DEFAULT_INTERNAL_THREAD_ARM_CQ_ENABLED (false)
 #define MCE_DEFAULT_QP_FORCE_MC_ATTACH             (false)
 #define MCE_DEFAULT_OFFLOADED_SOCKETS              (true)
 #define MCE_DEFAULT_TIMER_RESOLUTION_MSEC          (10)
@@ -775,7 +946,7 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_MEMORY_LIMIT_USER              (0)
 #define MCE_DEFAULT_HEAP_METADATA_BLOCK            (32LU * 1024 * 1024)
 #define MCE_DEFAULT_HUGEPAGE_SIZE                  (0)
-#define MCE_MAX_HUGEPAGE_SIZE                      (1ULL << 63ULL)
+#define MCE_MAX_HUGEPAGE_SIZE                      (1ULL << 63ULL) - 1
 #define MCE_DEFAULT_FORK_SUPPORT                   (true)
 #define MCE_DEFAULT_CLOSE_ON_DUP2                  (true)
 #define MCE_DEFAULT_MTU                            (0)
@@ -785,9 +956,9 @@ extern mce_sys_var &safe_mce_sys();
 #endif
 #if defined(DEFINED_NGINX) || defined(DEFINED_ENVOY)
 #define MCE_DEFAULT_APP_WORKERS_NUM (0)
-#define MCE_DEFAULT_SRC_PORT_STRIDE (2)
 #define MCE_DEFAULT_DISTRIBUTE_CQ   (false)
 #endif
+#define MCE_DEFAULT_SRC_PORT_STRIDE              (2)
 #define MCE_DEFAULT_MSS                          (0)
 #define MCE_DEFAULT_LWIP_CC_ALGO_MOD             (0)
 #define MCE_DEFAULT_INTERNAL_THREAD_AFFINITY     (-1)
@@ -810,9 +981,9 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_CQ_ADAPTIVE_MODERATION_DISABLED (0)
 #define MCE_MIN_CQ_POLL_BATCH               (1)
 #define MCE_MAX_CQ_POLL_BATCH               (32768)
-#define MCE_DEFAULT_SOCKETXTREME            (false)
 #define MCE_DEFAULT_TSO                     (option_3::AUTO)
 #define MCE_DEFAULT_MAX_TSO_SIZE            (256 * 1024)
+#define MCE_DEFAULT_WORKER_THREADS          (0)
 #ifdef DEFINED_UTLS
 #define MCE_DEFAULT_UTLS_RX                        (false)
 #define MCE_DEFAULT_UTLS_TX                        (true)
@@ -820,15 +991,14 @@ extern mce_sys_var &safe_mce_sys();
 #define MCE_DEFAULT_UTLS_LOW_WMARK_DEK_CACHE_SIZE  (512)
 #endif /* DEFINED_UTLS */
 
-#define MCE_DEFAULT_LRO                            (option_3::AUTO)
-#define MCE_DEFAULT_DEFERRED_CLOSE                 (false)
-#define MCE_DEFAULT_TCP_ABORT_ON_CLOSE             (false)
-#define MCE_DEFAULT_RX_POLL_ON_TX_TCP              (false)
-#define MCE_DEFAULT_TRIGGER_DUMMY_SEND_GETSOCKNAME (false)
-#define MCE_DEFAULT_RX_CQ_WAIT_CTRL                (false)
-#define MCE_ALIGNMENT                              ((unsigned long)63)
-#define MCE_DEFAULT_SKIP_POLL_IN_RX                (SKIP_POLL_IN_RX_DISABLE)
-#define MCE_DEFAULT_MULTILOCK                      (MULTILOCK_SPIN)
+#define MCE_DEFAULT_LRO                (option_3::AUTO)
+#define MCE_DEFAULT_DEFERRED_CLOSE     (false)
+#define MCE_DEFAULT_TCP_ABORT_ON_CLOSE (false)
+#define MCE_DEFAULT_RX_POLL_ON_TX_TCP  (false)
+#define MCE_DEFAULT_RX_CQ_WAIT_CTRL    (false)
+#define MCE_ALIGNMENT                  ((unsigned long)63)
+#define MCE_DEFAULT_SKIP_POLL_IN_RX    (SKIP_POLL_IN_RX_DISABLE)
+#define MCE_DEFAULT_MULTILOCK          (MULTILOCK_SPIN)
 
 /*
  * This block consists of auxiliary constants
@@ -840,7 +1010,6 @@ extern mce_sys_var &safe_mce_sys();
 #define NUM_TX_WRE_TO_SIGNAL_MAX            64
 #define NUM_RX_WRE_TO_POST_RECV_MAX         1024
 #define MAX_MLX5_CQ_SIZE_ITEMS              4194304
-#define TCP_MAX_SYN_RATE_TOP_LIMIT          100000
 #define DEFAULT_MC_TTL                      64
 #define DEFAULT_MC_HOP_LIMIT                1
 #define IFTYPE_PARAM_FILE                   "/sys/class/net/%s/type"
@@ -861,12 +1030,9 @@ extern mce_sys_var &safe_mce_sys();
 #define OPER_STATE_PARAM_FILE          "/sys/class/net/%s/operstate"
 #define FLOW_STEERING_MGM_ENTRY_SIZE_PARAM_FILE                                                    \
     "/sys/module/mlx4_core/parameters/log_num_mgm_entry_size"
-#define VIRTUAL_DEVICE_FOLDER    "/sys/devices/virtual/net/%s/"
-#define BOND_DEVICE_FILE         "/proc/net/bonding/%s"
-#define NETVSC_DEVICE_CLASS_FILE "/sys/class/net/%s/device/class_id"
-#define NETVSC_DEVICE_LOWER_FILE "/sys/class/net/%s/lower_%s/ifindex"
-#define NETVSC_DEVICE_UPPER_FILE "/sys/class/net/%s/upper_%s/ifindex"
-#define NETVSC_ID                "{f8615163-df3e-46c5-913f-f2d2f965ed0e}\n"
+#define VIRTUAL_DEVICE_FOLDER  "/sys/devices/virtual/net/%s/"
+#define BOND_DEVICE_FILE       "/proc/net/bonding/%s"
+#define BOND_DEVICE_UPPER_FILE "/sys/class/net/%s/upper_%s/ifindex"
 
 #define MAX_STATS_FD_NUM   1024U
 #define MAX_WINDOW_SCALING 14
