@@ -164,8 +164,8 @@ int get_base_interface_name(const char *if_name, char *base_ifname, size_t sz_ba
     return 0;
 }
 
-void print_roce_lag_warnings(const char *interface, char *disable_path /* = NULL */,
-                             const char *port1 /* = NULL */, const char *port2 /* = NULL */)
+void print_roce_lag_warnings(const char *interface, const char *port1 /* = NULL */,
+                             const char *port2 /* = NULL */)
 {
     vlog_printf(VLOG_WARNING,
                 "**********************************************************************************"
@@ -179,15 +179,7 @@ void print_roce_lag_warnings(const char *interface, char *disable_path /* = NULL
         vlog_printf(VLOG_WARNING, "* Unexpected behaviour may occur during runtime.\n");
     } else {
         vlog_printf(VLOG_WARNING, "* Interface %s will not be offloaded.\n", interface);
-        vlog_printf(VLOG_WARNING,
-                    "* " PRODUCT_NAME " cannot offload the device while RoCE LAG is enabled.\n");
-    }
-
-    vlog_printf(VLOG_WARNING, "* Please refer to " PRODUCT_NAME " Release Notes for more info\n");
-
-    if (disable_path) {
-        vlog_printf(VLOG_WARNING, "* In order to disable RoCE LAG please use:\n");
-        vlog_printf(VLOG_WARNING, "* echo 0 > %s\n", disable_path);
+        vlog_printf(VLOG_WARNING, "* Cannot offload the device; ib_ctx is not found.\n");
     }
     vlog_printf(VLOG_WARNING,
                 "**********************************************************************************"
@@ -1034,7 +1026,7 @@ bool get_bond_name(IN const char *ifname, OUT char *bond_name, IN int sz)
     }
 
     for (ifa = ifaddr; ifa; ifa = ifa->ifa_next) {
-        snprintf(upper_path, sizeof(upper_path), NETVSC_DEVICE_UPPER_FILE, base_ifname,
+        snprintf(upper_path, sizeof(upper_path), BOND_DEVICE_UPPER_FILE, base_ifname,
                  ifa->ifa_name);
         int fd = SYSCALL(open, upper_path, O_RDONLY);
         if (fd >= 0) {
@@ -1083,56 +1075,6 @@ bool check_bond_roce_lag_exist(OUT char *bond_roce_lag_path, int sz, IN const ch
     return false;
 }
 
-bool get_netvsc_slave(IN const char *ifname, OUT char *slave_name, OUT unsigned int &slave_flags)
-{
-    char netvsc_path[256];
-    char base_ifname[IFNAMSIZ];
-    get_base_interface_name(ifname, base_ifname, sizeof(base_ifname));
-    struct ifaddrs *ifaddr, *ifa;
-    bool ret = false;
-
-    if (getifaddrs(&ifaddr) == -1) {
-        __log_err("getifaddrs() failed (errno = %d %m)", errno);
-        return ret;
-    }
-
-    for (ifa = ifaddr; ifa; ifa = ifa->ifa_next) {
-        snprintf(netvsc_path, sizeof(netvsc_path), NETVSC_DEVICE_LOWER_FILE, base_ifname,
-                 ifa->ifa_name);
-        int fd = SYSCALL(open, netvsc_path, O_RDONLY);
-        if (fd >= 0) {
-            SYSCALL(close, fd);
-            memcpy(slave_name, ifa->ifa_name, IFNAMSIZ);
-            slave_flags = ifa->ifa_flags;
-            __log_dbg("Found slave_name = %s, slave_flags = %u", slave_name, slave_flags);
-            ret = true;
-            break;
-        }
-    }
-
-    freeifaddrs(ifaddr);
-
-    return ret;
-}
-
-bool check_netvsc_device_exist(const char *ifname)
-{
-    int ret = -1;
-    char device_path[256] = {0};
-    char base_ifname[IFNAMSIZ];
-    get_base_interface_name(ifname, base_ifname, sizeof(base_ifname));
-    sprintf(device_path, NETVSC_DEVICE_CLASS_FILE, base_ifname);
-    char sys_res[1024] = {0};
-    if ((ret = priv_read_file(device_path, sys_res, sizeof(sys_res) - 1, VLOG_FUNC)) > 0) {
-        sys_res[ret] = '\0';
-        if (strcmp(sys_res, NETVSC_ID) == 0) {
-            return true;
-        }
-    }
-
-    return false;
-}
-
 /*
  * this function will work only for kernel  > 3.14 or RH7.2 and higher
  */
@@ -1359,6 +1301,12 @@ loops_timer::loops_timer()
     ts_clear(&m_current);
 }
 
+loops_timer::loops_timer(int timeout_msec)
+    : loops_timer()
+{
+    m_timeout_msec = timeout_msec;
+}
+
 void loops_timer::start()
 {
     ts_clear(&m_start);
@@ -1391,29 +1339,3 @@ uint32_t fd2inode(int fd)
     int rc = fstat(fd, &buf);
     return rc == 0 ? buf.st_ino : 0; // no inode is 0
 }
-
-///////////////////////////////////////////
-xlio_error::xlio_error(const char *_message, const char *_function, const char *_filename,
-                       int _lineno, int _errnum) throw()
-    : message(_message)
-    , function(_function)
-    , filename(_filename)
-    , lineno(_lineno)
-    , errnum(_errnum)
-{
-    snprintf(formatted_message, sizeof(formatted_message),
-             "xlio_error <%s> (errno=%d %s) in %s:%d\n", message, errnum, strerror(errnum),
-             filename, lineno);
-    formatted_message[sizeof(formatted_message) - 1] = '\0';
-}
-
-xlio_error::~xlio_error() throw()
-{
-}
-
-const char *xlio_error::what() const throw()
-{
-    return formatted_message;
-}
-
-///////////////////////////////////////////
