@@ -12,17 +12,31 @@
 #include <string.h>
 #include <ifaddrs.h>
 #include <linux/if_ether.h>
+#include <limits.h>
 #include <exception>
+#include <cstdint>
+#include <cstddef>
 #include "vtypes.h"
 #include "utils/rdtsc.h"
 #include "vlogger/vlogger.h"
 #include "core/proto/mem_buf_desc.h"
-#include "core/util/xlio_stats.h"
+#include "core/util/xlio_exception.h"
 
 #ifndef ARRAY_SIZE
 #define ARRAY_SIZE(arr) (sizeof(arr) / sizeof(arr[0]))
 #endif /* ARRAY_SIZE */
 
+#define FILE_NAME_MAX_SIZE (NAME_MAX + 1)
+
+#define PADDING(x)     uint8_t __explicit_padding[x]
+#define CACHELINE_SIZE 64
+#define BOUNDARY_SIZE_ASSERT(type, boundary)                                                       \
+    static_assert(                                                                                 \
+        (offsetof(type, __explicit_padding) + sizeof(type::__explicit_padding)) % (boundary) == 0, \
+        #type " explicit padding is not aligned to a boundary of " #boundary ". "                  \
+              "Adjust PADDING() to correct the alignment.")
+#define CACHELINE_BOUNDARY_SIZE_ASSERT(type) BOUNDARY_SIZE_ASSERT(type, CACHELINE_SIZE)
+
 /**
  * Check if file type is regular
  */
@@ -254,9 +268,8 @@ size_t get_vlan_base_name_from_ifname(const char *ifname, char *base_ifname, siz
 /* Upon success - returns the actual address len in bytes; Upon error - returns zero*/
 size_t get_local_ll_addr(const char *ifname, unsigned char *addr, int addr_len, bool is_broadcast);
 
-/* Print warning while RoCE Lag is enabled */
-void print_roce_lag_warnings(const char *interface, char *disable_path = nullptr,
-                             const char *port1 = nullptr, const char *port2 = nullptr);
+void print_roce_lag_warnings(const char *interface, const char *port1 = nullptr,
+                             const char *port2 = nullptr);
 
 /*Print a warning to the user when there was an error registering memory*/
 void print_warning_rlimit_memlock(size_t length, int error);
@@ -268,8 +281,6 @@ bool get_bond_slaves_name_list(IN const char *bond_name, OUT char *slaves_list,
 bool check_bond_roce_lag_exist(OUT char *bond_roce_lag_path, int sz, IN const char *slave_name);
 bool check_device_exist(const char *ifname, const char *path);
 bool check_device_name_ib_name(const char *ifname, const char *ibname);
-bool check_netvsc_device_exist(const char *ifname);
-bool get_netvsc_slave(IN const char *ifname, OUT char *slave_name, OUT unsigned int &slave_flags);
 bool get_interface_oper_state(IN const char *interface_name, OUT char *slaves_list, IN int sz);
 
 bool validate_user_has_cap_net_raw_privliges();
@@ -413,6 +424,7 @@ static inline void create_mgid_from_ipv4_mc_ip(uint8_t *mgid, uint16_t pkey, uin
 class loops_timer {
 public:
     loops_timer();
+    loops_timer(int timeout_msec);
     void start();
     int time_left_msec();
     void set_timeout_msec(int timeout_msec) { m_timeout_msec = timeout_msec; }
@@ -460,74 +472,6 @@ private:
 // 32 bit inodes This should be safe for 'proc' filesytem and for standard filesystems
 uint32_t fd2inode(int fd);
 
-/**
- * @class xlio_error
- *
- * base class for xlio exceptions classes.
- * Note: xlio code should NOT catch xlio_error; xlio code should only catch exceptions of derived
- * classes
- */
-class xlio_error : public std::exception {
-    char formatted_message[512];
-
-public:
-    const char *const message;
-    const char *const function;
-    const char *const filename;
-    const int lineno;
-    const int errnum;
-
-    /**
-     * Create an object that contains const members for all the given arguments, plus a formatted
-     * message that will be available thru the 'what()' method of base class.
-     *
-     * The formatted_message will look like this:
-     * 		"xlio_error <create internal epoll> (errno=24 Too many open files) in
-     * sock/sockinfo.cpp:61" catcher can print it to log like this: fdcoll_loginfo("recovering from
-     * %s", e.what());
-     */
-    xlio_error(const char *_message, const char *_function, const char *_filename, int _lineno,
-               int _errnum) throw();
-
-    virtual ~xlio_error() throw();
-
-    virtual const char *what() const throw();
-};
-
-/**
- * @class xlio_exception
- * NOTE: ALL exceptions that can be caught by XLIO should be derived of this class
- */
-class xlio_exception : public xlio_error {
-public:
-    xlio_exception(const char *_message, const char *_function, const char *_filename, int _lineno,
-                   int _errnum) throw()
-        : xlio_error(_message, _function, _filename, _lineno, _errnum)
-    {
-    }
-};
-
-#define create_xlio_exception_class(clsname, basecls)                                              \
-    class clsname : public basecls {                                                               \
-    public:                                                                                        \
-        clsname(const char *_message, const char *_function, const char *_filename, int _lineno,   \
-                int _errnum) throw()                                                               \
-            : basecls(_message, _function, _filename, _lineno, _errnum)                            \
-        {                                                                                          \
-        }                                                                                          \
-    }
-
-create_xlio_exception_class(xlio_unsupported_api, xlio_error);
-
-#define throw_xlio_exception(msg)                                                                  \
-    throw xlio_exception(msg, __PRETTY_FUNCTION__, __FILE__, __LINE__, errno)
-// uses for throwing  something that is derived from xlio_error and has similar CTOR; msg will
-// automatically be class name
-#define xlio_throw_object(_class)                                                                  \
-    throw _class(#_class, __PRETTY_FUNCTION__, __FILE__, __LINE__, errno)
-#define xlio_throw_object_with_msg(_class, _msg)                                                   \
-    throw _class(_msg, __PRETTY_FUNCTION__, __FILE__, __LINE__, errno)
-
 /* Rounding up to nearest power of 2 */
 static inline uint32_t align32pow2(uint32_t x)
 {
