@@ -391,6 +391,12 @@ enum ctx_device {
 	YUNSILICON_ANDES	= 35,
 	YUNSILICON_DIAMOND	= 36,
 	YUNSILICON_DIAMOND_NEXT	= 37,
+	NBL_LEONIS		= 38,
+	NBL_DRACO		= 39,
+	NBL_DF200		= 40,
+	NBL_RNIC400		= 41,
+	PENSANDO_ALL		= 42,
+	CONNECTX10		= 43,
 };
 
 /* Units for rate limiter */
@@ -670,11 +676,16 @@ struct perftest_parameters {
 	int 			ah_allocated;
 	int				use_write_with_imm;
 	int				use_unsolicited_write;
-	int				use_ddp;
-	int				no_ddp;
+	int				use_enhanced_reorder;
+	int				no_enhanced_reorder;
 	int				connectionless;
 	uint16_t			cqe_poll;
 	int				use_cqe_poll;
+	int				tph_mem_type;
+	int				cpu_id;
+	int				processing_hints;
+	int				dynamic_cqe_poll;
+	int				sig_offload;
 };
 
 struct report_options {
