@@ -42,6 +42,7 @@ extern int rshim_log_level;
 extern bool rshim_daemon_mode;
 extern int rshim_drop_mode;
 extern bool rshim_force_mode;
+extern int rshim_boot_timeout;
 extern int rshim_usb_timeout;
 extern int rshim_usb_reset_delay;
 extern bool rshim_has_usb_reset_delay;
@@ -643,6 +644,7 @@ static inline void rshim_usb_poll(bool blocking)
 /* PCIe & PCIe livefish backend APIs. */
 #ifdef HAVE_RSHIM_PCIE
 int rshim_pcie_init(void);
+void rshim_pcie_exit(void);
 int rshim_pcie_lf_init(void);
 void rshim_pcie_check(rshim_backend_t *bd);
 #else
@@ -651,6 +653,10 @@ static inline int rshim_pcie_init(void)
   return -1;
 }
 
+static inline void rshim_pcie_exit(void)
+{
+}
+
 static inline int rshim_pcie_lf_init(void)
 {
   return -1;
