@@ -87,15 +87,17 @@ typedef enum { e_K = 1024, e_M = 1048576 } units_t;
 #define FORMAT_RING_INTERRUPT  "%-20s %zu / %zu [requests/received] %-3s\n"
 #define FORMAT_RING_MODERATION "%-20s %u / %u [frames/usec period] %-3s\n"
 #define FORMAT_RING_DM_STATS   "%-20s %zu / %zu / %zu [kilobytes/packets/oob] %-3s\n"
-#define FORMAT_RING_TAP_NAME   "%-20s %s\n"
 #define FORMAT_RING_MASTER     "%-20s %p\n"
+#define FORMAT_RING_RX_TLS     "%-20s %u / %u / %u [contexts/resyncs/auth-fail] %-3s\n"
+#define FORMAT_RING_TX_TLS     "%-20s %u / %u [contexts/resyncs] %-3s\n"
+#define FORMAT_ENTCTX          "%-15s"
 
 #define INTERVAL                1
 #define BYTES_TRAFFIC_UNIT      e_K
 #define SCREEN_SIZE             24
 #define MAX_BUFF_SIZE           256
 #define PRINT_DETAILS_MODES_NUM 2
-#define VIEW_MODES_NUM          5
+#define VIEW_MODES_NUM          6
 #define DEFAULT_DELAY_SEC       1
 #define DEFAULT_CYCLES          0
 #define DEFAULT_VIEW_MODE       e_basic
@@ -105,6 +107,7 @@ typedef enum { e_K = 1024, e_M = 1048576 } units_t;
 #define INIT_XLIO_LOG_DETAILS   -1
 #define NANO_TO_MICRO(n)        (((n) + 500) / 1000)
 #define SEC_TO_MICRO(n)         ((n)*1000000)
+#define INFO_TABS               "\t\t\t\t"
 #define TIME_DIFF_in_MICRO(start, end)                                                             \
     (SEC_TO_MICRO((end).tv_sec - (start).tv_sec) + (NANO_TO_MICRO((end).tv_nsec - (start).tv_nsec)))
 
@@ -140,9 +143,12 @@ void usage(const char *argv0)
     printf("  -i, --interval=<n>\t\tPrint report every <n> seconds\n");
     printf("  -c, --cycles=<n>\t\tDo <n> report print cycles and exit, use 0 value for infinite "
            "(default)\n");
-    printf("  -v, --view=<1|2|3|4|5>\tSet view type:1- basic info,2- extra info,3- full info,4- mc "
-           "groups,5- similar to 'netstat -tunaep'\n");
-    printf("  -d, --details=<1|2>\t\tSet details mode:1- to see totals,2- to see deltas\t\t\n");
+    printf("  -v, --view=<1|2|3|4|5|6>\tSet view type:\n" INFO_TABS "1 - Basic info\n" INFO_TABS
+           "2 - Extra info\n" INFO_TABS "3 - Full info\n" INFO_TABS
+           "4 - Multicast groups\n" INFO_TABS "5 - Show as 'netstat -tunaep'\n" INFO_TABS
+           "6 - Entity Context info\n");
+    printf("  -d, --details=<1|2>\t\tSet details mode:\n" INFO_TABS "1 - Totals\n" INFO_TABS
+           "2 - Deltas\n");
     printf("  -z, --zero\t\t\tZero counters\n");
     printf(
         "  -l, --log_level=<level>\tSet " PRODUCT_NAME
@@ -216,7 +222,6 @@ void update_delta_stat(socket_stats_t *p_curr_stat, socket_stats_t *p_prev_stat)
         delay;
     p_prev_stat->n_rx_ready_pkt_count = p_curr_stat->n_rx_ready_pkt_count;
     p_prev_stat->counters.n_rx_ready_pkt_max = p_curr_stat->counters.n_rx_ready_pkt_max;
-    p_prev_stat->n_rx_zcopy_pkt_count = p_curr_stat->n_rx_zcopy_pkt_count;
     p_prev_stat->strq_counters.n_strq_total_strides =
         (p_curr_stat->strq_counters.n_strq_total_strides -
          p_prev_stat->strq_counters.n_strq_total_strides) /
@@ -241,13 +246,25 @@ void update_delta_stat(socket_stats_t *p_curr_stat, socket_stats_t *p_prev_stat)
     p_prev_stat->tls_counters.n_tls_rx_records =
         (p_curr_stat->tls_counters.n_tls_rx_records - p_prev_stat->tls_counters.n_tls_rx_records) /
         delay;
-    p_prev_stat->tls_counters.n_tls_rx_records_enc =
-        (p_curr_stat->tls_counters.n_tls_rx_records_enc -
-         p_prev_stat->tls_counters.n_tls_rx_records_enc) /
+    p_prev_stat->tls_counters.n_tls_rx_records_full_enc =
+        (p_curr_stat->tls_counters.n_tls_rx_records_full_enc -
+         p_prev_stat->tls_counters.n_tls_rx_records_full_enc) /
         delay;
-    p_prev_stat->tls_counters.n_tls_rx_records_partial =
-        (p_curr_stat->tls_counters.n_tls_rx_records_partial -
-         p_prev_stat->tls_counters.n_tls_rx_records_partial) /
+    p_prev_stat->tls_counters.n_tls_rx_records_head_enc =
+        (p_curr_stat->tls_counters.n_tls_rx_records_head_enc -
+         p_prev_stat->tls_counters.n_tls_rx_records_head_enc) /
+        delay;
+    p_prev_stat->tls_counters.n_tls_rx_records_tail_enc =
+        (p_curr_stat->tls_counters.n_tls_rx_records_tail_enc -
+         p_prev_stat->tls_counters.n_tls_rx_records_tail_enc) /
+        delay;
+    p_prev_stat->tls_counters.n_tls_rx_records_mix_enc =
+        (p_curr_stat->tls_counters.n_tls_rx_records_mix_enc -
+         p_prev_stat->tls_counters.n_tls_rx_records_mix_enc) /
+        delay;
+    p_prev_stat->tls_counters.n_tls_rx_records_hw_auth_fail =
+        (p_curr_stat->tls_counters.n_tls_rx_records_hw_auth_fail -
+         p_prev_stat->tls_counters.n_tls_rx_records_hw_auth_fail) /
         delay;
     p_prev_stat->tls_counters.n_tls_rx_bytes =
         (p_curr_stat->tls_counters.n_tls_rx_bytes - p_prev_stat->tls_counters.n_tls_rx_bytes) /
@@ -328,66 +345,53 @@ void update_delta_ring_stat(ring_stats_t *p_curr_ring_stats, ring_stats_t *p_pre
             (p_curr_ring_stats->n_tx_pkt_count - p_prev_ring_stats->n_tx_pkt_count) / delay;
         p_prev_ring_stats->n_tx_retransmits =
             (p_curr_ring_stats->n_tx_retransmits - p_prev_ring_stats->n_tx_retransmits) / delay;
-        p_prev_ring_stats->simple.n_tx_dropped_wqes =
-            (p_curr_ring_stats->simple.n_tx_dropped_wqes -
-             p_prev_ring_stats->simple.n_tx_dropped_wqes) /
-            delay;
-        p_prev_ring_stats->simple.n_tx_num_bufs =
-            (p_curr_ring_stats->simple.n_tx_num_bufs - p_prev_ring_stats->simple.n_tx_num_bufs) /
-            delay;
-        p_prev_ring_stats->simple.n_zc_num_bufs =
-            (p_curr_ring_stats->simple.n_zc_num_bufs - p_prev_ring_stats->simple.n_zc_num_bufs) /
-            delay;
+        p_prev_ring_stats->n_tx_dropped_wqes =
+            (p_curr_ring_stats->n_tx_dropped_wqes - p_prev_ring_stats->n_tx_dropped_wqes) / delay;
+        p_prev_ring_stats->n_tx_num_bufs =
+            (p_curr_ring_stats->n_tx_num_bufs - p_prev_ring_stats->n_tx_num_bufs) / delay;
+        p_prev_ring_stats->n_zc_num_bufs =
+            (p_curr_ring_stats->n_zc_num_bufs - p_prev_ring_stats->n_zc_num_bufs) / delay;
 #ifdef DEFINED_UTLS
         p_prev_ring_stats->n_tx_tls_contexts =
             (p_curr_ring_stats->n_tx_tls_contexts - p_prev_ring_stats->n_tx_tls_contexts) / delay;
         p_prev_ring_stats->n_rx_tls_contexts =
             (p_curr_ring_stats->n_rx_tls_contexts - p_prev_ring_stats->n_rx_tls_contexts) / delay;
+        p_prev_ring_stats->n_tx_tls_resyncs =
+            (p_curr_ring_stats->n_tx_tls_resyncs - p_prev_ring_stats->n_tx_tls_resyncs) / delay;
+        p_prev_ring_stats->n_rx_tls_resyncs =
+            (p_curr_ring_stats->n_rx_tls_resyncs - p_prev_ring_stats->n_rx_tls_resyncs) / delay;
+        p_prev_ring_stats->n_rx_tls_auth_fail =
+            (p_curr_ring_stats->n_rx_tls_auth_fail - p_prev_ring_stats->n_rx_tls_auth_fail) / delay;
 #endif /* DEFINED_UTLS */
 
-        if (p_prev_ring_stats->n_type == RING_TAP) {
-            memcpy(p_prev_ring_stats->tap.s_tap_name, p_curr_ring_stats->tap.s_tap_name,
-                   sizeof(p_curr_ring_stats->tap.s_tap_name));
-            p_prev_ring_stats->tap.n_tap_fd = p_curr_ring_stats->tap.n_tap_fd;
-            p_prev_ring_stats->tap.n_rx_buffers = p_curr_ring_stats->tap.n_rx_buffers;
-            p_prev_ring_stats->tap.n_vf_plugouts =
-                (p_curr_ring_stats->tap.n_vf_plugouts - p_prev_ring_stats->tap.n_vf_plugouts);
-        } else {
-            p_prev_ring_stats->simple.n_tx_tso_pkt_count =
-                (p_curr_ring_stats->simple.n_tx_tso_pkt_count -
-                 p_prev_ring_stats->simple.n_tx_tso_pkt_count) /
-                delay;
-            p_prev_ring_stats->simple.n_tx_tso_byte_count =
-                (p_curr_ring_stats->simple.n_tx_tso_byte_count -
-                 p_prev_ring_stats->simple.n_tx_tso_byte_count) /
-                delay;
-            p_prev_ring_stats->simple.n_rx_interrupt_received =
-                (p_curr_ring_stats->simple.n_rx_interrupt_received -
-                 p_prev_ring_stats->simple.n_rx_interrupt_received) /
-                delay;
-            p_prev_ring_stats->simple.n_rx_interrupt_requests =
-                (p_curr_ring_stats->simple.n_rx_interrupt_requests -
-                 p_prev_ring_stats->simple.n_rx_interrupt_requests) /
-                delay;
-            p_prev_ring_stats->simple.n_rx_cq_moderation_count =
-                p_curr_ring_stats->simple.n_rx_cq_moderation_count;
-            p_prev_ring_stats->simple.n_rx_cq_moderation_period =
-                p_curr_ring_stats->simple.n_rx_cq_moderation_period;
-            p_prev_ring_stats->simple.n_tx_dev_mem_allocated =
-                p_curr_ring_stats->simple.n_tx_dev_mem_allocated;
-            p_prev_ring_stats->simple.n_tx_dev_mem_byte_count =
-                (p_curr_ring_stats->simple.n_tx_dev_mem_byte_count -
-                 p_prev_ring_stats->simple.n_tx_dev_mem_byte_count) /
-                delay;
-            p_prev_ring_stats->simple.n_tx_dev_mem_pkt_count =
-                (p_curr_ring_stats->simple.n_tx_dev_mem_pkt_count -
-                 p_prev_ring_stats->simple.n_tx_dev_mem_pkt_count) /
-                delay;
-            p_prev_ring_stats->simple.n_tx_dev_mem_oob =
-                (p_curr_ring_stats->simple.n_tx_dev_mem_oob -
-                 p_prev_ring_stats->simple.n_tx_dev_mem_oob) /
-                delay;
-        }
+        p_prev_ring_stats->n_rx_zc_migiration_drop = (p_curr_ring_stats->n_rx_zc_migiration_drop -
+                                                      p_prev_ring_stats->n_rx_zc_migiration_drop) /
+            delay;
+        p_prev_ring_stats->n_tx_tso_pkt_count =
+            (p_curr_ring_stats->n_tx_tso_pkt_count - p_prev_ring_stats->n_tx_tso_pkt_count) / delay;
+        p_prev_ring_stats->n_tx_tso_byte_count =
+            (p_curr_ring_stats->n_tx_tso_byte_count - p_prev_ring_stats->n_tx_tso_byte_count) /
+            delay;
+        p_prev_ring_stats->n_rx_interrupt_received = (p_curr_ring_stats->n_rx_interrupt_received -
+                                                      p_prev_ring_stats->n_rx_interrupt_received) /
+            delay;
+        p_prev_ring_stats->n_rx_interrupt_requests = (p_curr_ring_stats->n_rx_interrupt_requests -
+                                                      p_prev_ring_stats->n_rx_interrupt_requests) /
+            delay;
+        p_prev_ring_stats->n_rx_cq_moderation_count = p_curr_ring_stats->n_rx_cq_moderation_count;
+        p_prev_ring_stats->n_rx_cq_moderation_period = p_curr_ring_stats->n_rx_cq_moderation_period;
+        p_prev_ring_stats->n_tx_dev_mem_allocated = p_curr_ring_stats->n_tx_dev_mem_allocated;
+        p_prev_ring_stats->n_tx_dev_mem_byte_count = (p_curr_ring_stats->n_tx_dev_mem_byte_count -
+                                                      p_prev_ring_stats->n_tx_dev_mem_byte_count) /
+            delay;
+        p_prev_ring_stats->n_tx_dev_mem_pkt_count = (p_curr_ring_stats->n_tx_dev_mem_pkt_count -
+                                                     p_prev_ring_stats->n_tx_dev_mem_pkt_count) /
+            delay;
+        p_prev_ring_stats->n_tx_dev_mem_oob =
+            (p_curr_ring_stats->n_tx_dev_mem_oob - p_prev_ring_stats->n_tx_dev_mem_oob) / delay;
+        p_prev_ring_stats->n_rx_steering_rules =
+            (p_curr_ring_stats->n_rx_steering_rules - p_prev_ring_stats->n_rx_steering_rules) /
+            delay;
     }
 }
 
@@ -396,8 +400,11 @@ void update_delta_cq_stat(cq_stats_t *p_curr_cq_stats, cq_stats_t *p_prev_cq_sta
     int delay = user_params.interval;
     if (p_curr_cq_stats && p_prev_cq_stats) {
         p_prev_cq_stats->n_rx_drained_at_once_max = p_curr_cq_stats->n_rx_drained_at_once_max;
-        p_prev_cq_stats->n_rx_pkt_drop =
-            (p_curr_cq_stats->n_rx_pkt_drop - p_prev_cq_stats->n_rx_pkt_drop) / delay;
+        p_prev_cq_stats->n_rx_sw_pkt_drops =
+            (p_curr_cq_stats->n_rx_sw_pkt_drops - p_prev_cq_stats->n_rx_sw_pkt_drops) / delay;
+        p_prev_cq_stats->n_rx_hw_pkt_drops =
+            (p_curr_cq_stats->n_rx_hw_pkt_drops - p_prev_cq_stats->n_rx_hw_pkt_drops) / delay;
+
         p_prev_cq_stats->n_rx_sw_queue_len = p_curr_cq_stats->n_rx_sw_queue_len;
         p_prev_cq_stats->n_buffer_pool_len = p_curr_cq_stats->n_buffer_pool_len;
         p_prev_cq_stats->n_rx_lro_packets =
@@ -473,7 +480,7 @@ void print_ring_stats(ring_instance_block_t *p_ring_inst_arr)
             p_ring_stats = &p_ring_inst_arr[i].ring_stats;
             printf("======================================================\n");
 
-            printf("\t%s=[%u]\n", ring_type_str[p_ring_stats->n_type], i);
+            printf("\tETH=[%u]\n", i);
 
             if (p_ring_stats->p_ring_master) {
                 printf(FORMAT_RING_MASTER, "Master:", p_ring_stats->p_ring_master);
@@ -491,64 +498,54 @@ void print_ring_stats(ring_instance_block_t *p_ring_inst_arr)
                        post_fix);
             }
 
-            if (p_ring_stats->simple.n_tx_dropped_wqes) {
-                printf(FORMAT_STATS_64bit,
-                       "TX Dropped Send Reqs:", p_ring_stats->simple.n_tx_dropped_wqes, post_fix);
+            if (p_ring_stats->n_tx_dropped_wqes) {
+                printf(FORMAT_STATS_64bit, "TX Dropped Send Reqs:", p_ring_stats->n_tx_dropped_wqes,
+                       post_fix);
             }
 
-#ifdef DEFINED_UTLS
-            if (p_ring_stats->n_tx_tls_contexts) {
-                printf(FORMAT_STATS_64bit,
-                       "TLS TX Context Setups:", (uint64_t)p_ring_stats->n_tx_tls_contexts,
-                       post_fix);
+            if (p_ring_stats->n_rx_zc_migiration_drop) {
+                printf(FORMAT_STATS_32bit,
+                       "RX ZC Migration Drop:", p_ring_stats->n_rx_zc_migiration_drop);
             }
-            if (p_ring_stats->n_rx_tls_contexts) {
-                printf(FORMAT_STATS_64bit,
-                       "TLS RX Context Setups:", (uint64_t)p_ring_stats->n_rx_tls_contexts,
-                       post_fix);
+#ifdef DEFINED_UTLS
+            if (p_ring_stats->n_tx_tls_contexts || p_ring_stats->n_tx_tls_resyncs) {
+                printf(FORMAT_RING_TX_TLS, "HW TLS TX:", p_ring_stats->n_tx_tls_contexts,
+                       p_ring_stats->n_tx_tls_resyncs, post_fix);
+            }
+            if (p_ring_stats->n_rx_tls_contexts || p_ring_stats->n_rx_tls_resyncs ||
+                p_ring_stats->n_rx_tls_auth_fail) {
+                printf(FORMAT_RING_RX_TLS, "HW TLS RX:", p_ring_stats->n_rx_tls_contexts,
+                       p_ring_stats->n_rx_tls_resyncs, p_ring_stats->n_rx_tls_auth_fail, post_fix);
             }
 #endif /* DEFINED_UTLS */
 
-            if (p_ring_stats->n_type == RING_TAP) {
-                printf(FORMAT_STATS_32bit, "Rx Buffers:", p_ring_stats->tap.n_rx_buffers);
-                if (p_ring_stats->tap.n_vf_plugouts) {
-                    printf(FORMAT_STATS_32bit, "VF Plugouts:", p_ring_stats->tap.n_vf_plugouts);
-                }
-                printf(FORMAT_STATS_32bit, "Tap fd:", p_ring_stats->tap.n_tap_fd);
-                printf(FORMAT_RING_TAP_NAME, "Tap Device:", p_ring_stats->tap.s_tap_name);
-            } else {
-                if (p_ring_stats->simple.n_tx_tso_pkt_count ||
-                    p_ring_stats->simple.n_tx_tso_byte_count) {
-                    printf(FORMAT_RING_PACKETS, "TSO Offload:",
-                           p_ring_stats->simple.n_tx_tso_byte_count / BYTES_TRAFFIC_UNIT,
-                           p_ring_stats->simple.n_tx_tso_pkt_count, post_fix);
-                }
-                if (p_ring_stats->simple.n_rx_interrupt_requests ||
-                    p_ring_stats->simple.n_rx_interrupt_received) {
-                    printf(FORMAT_RING_INTERRUPT,
-                           "Interrupts:", p_ring_stats->simple.n_rx_interrupt_requests,
-                           p_ring_stats->simple.n_rx_interrupt_received, post_fix);
-                }
-                if (p_ring_stats->simple.n_rx_cq_moderation_count ||
-                    p_ring_stats->simple.n_rx_cq_moderation_period) {
-                    printf(FORMAT_RING_MODERATION,
-                           "Moderation:", p_ring_stats->simple.n_rx_cq_moderation_count,
-                           p_ring_stats->simple.n_rx_cq_moderation_period, post_fix);
-                }
-                if (p_ring_stats->simple.n_tx_dev_mem_allocated) {
-                    printf(FORMAT_STATS_32bit,
-                           "Dev Mem Alloc:", p_ring_stats->simple.n_tx_dev_mem_allocated);
-                    printf(FORMAT_RING_DM_STATS, "Dev Mem Stats:",
-                           p_ring_stats->simple.n_tx_dev_mem_byte_count / BYTES_TRAFFIC_UNIT,
-                           p_ring_stats->simple.n_tx_dev_mem_pkt_count,
-                           p_ring_stats->simple.n_tx_dev_mem_oob, post_fix);
-                }
-
-                printf(FORMAT_STATS_32bit,
-                       "TX buffers inflight:", p_ring_stats->simple.n_tx_num_bufs);
-                printf(FORMAT_STATS_32bit,
-                       "TX ZC buffers inflight:", p_ring_stats->simple.n_zc_num_bufs);
+            if (p_ring_stats->n_tx_tso_pkt_count || p_ring_stats->n_tx_tso_byte_count) {
+                printf(FORMAT_RING_PACKETS,
+                       "TSO Offload:", p_ring_stats->n_tx_tso_byte_count / BYTES_TRAFFIC_UNIT,
+                       p_ring_stats->n_tx_tso_pkt_count, post_fix);
             }
+            if (p_ring_stats->n_rx_interrupt_requests || p_ring_stats->n_rx_interrupt_received) {
+                printf(FORMAT_RING_INTERRUPT, "Interrupts:", p_ring_stats->n_rx_interrupt_requests,
+                       p_ring_stats->n_rx_interrupt_received, post_fix);
+            }
+            if (p_ring_stats->n_rx_cq_moderation_count || p_ring_stats->n_rx_cq_moderation_period) {
+                printf(FORMAT_RING_MODERATION,
+                       "Moderation:", p_ring_stats->n_rx_cq_moderation_count,
+                       p_ring_stats->n_rx_cq_moderation_period, post_fix);
+            }
+            if (p_ring_stats->n_tx_dev_mem_allocated) {
+                printf(FORMAT_STATS_32bit, "Dev Mem Alloc:", p_ring_stats->n_tx_dev_mem_allocated);
+                printf(FORMAT_RING_DM_STATS,
+                       "Dev Mem Stats:", p_ring_stats->n_tx_dev_mem_byte_count / BYTES_TRAFFIC_UNIT,
+                       p_ring_stats->n_tx_dev_mem_pkt_count, p_ring_stats->n_tx_dev_mem_oob,
+                       post_fix);
+            }
+            if (p_ring_stats->n_rx_steering_rules) {
+                printf(FORMAT_STATS_32bit, "RX steering rules:", p_ring_stats->n_rx_steering_rules);
+            }
+
+            printf(FORMAT_STATS_32bit, "TX buffers inflight:", p_ring_stats->n_tx_num_bufs);
+            printf(FORMAT_STATS_32bit, "TX ZC buffers inflight:", p_ring_stats->n_zc_num_bufs);
         }
     }
     printf("======================================================\n");
@@ -569,7 +566,10 @@ void print_cq_stats(cq_instance_block_t *p_cq_inst_arr)
             p_cq_stats = &p_cq_inst_arr[i].cq_stats;
             printf("======================================================\n");
             printf("\tCQ=[%u]\n", i);
-            printf(FORMAT_STATS_64bit, "Packets dropped:", p_cq_stats->n_rx_pkt_drop, post_fix);
+            printf(FORMAT_STATS_64bit, "SW RX Packets dropped:", p_cq_stats->n_rx_sw_pkt_drops,
+                   post_fix);
+            printf(FORMAT_STATS_64bit, "HW RX Packets dropped:", p_cq_stats->n_rx_hw_pkt_drops,
+                   post_fix);
             printf(FORMAT_STATS_32bit, "Packets queue len:", p_cq_stats->n_rx_sw_queue_len);
             printf(FORMAT_STATS_32bit, "Drained max:", p_cq_stats->n_rx_drained_at_once_max);
             printf(FORMAT_STATS_32bit, "Buffer pool size:", p_cq_stats->n_buffer_pool_len);
@@ -608,6 +608,51 @@ void print_cq_stats(cq_instance_block_t *p_cq_inst_arr)
     printf("======================================================\n");
 }
 
+void print_entity_context_stats(entity_context_instance_block_t *p_ent_ctx_inst_arr)
+{
+    char post_fix[3] = "";
+    if (user_params.print_details_mode == e_deltas) {
+        strcpy(post_fix, "/s");
+    }
+
+    printf("======================================================================================="
+           "============\n");
+    printf("Entity  | CPUTime                  | Job Queue     | Sockets                           "
+           "    | RSS   \n");
+    printf("Context | Idle | PollHit | JobProc | Max   | Avg   | Total   | Added   | Removed | Delt"
+           "a   | Childs\n");
+    printf("---------------------------------------------------------------------------------------"
+           "------------\n");
+
+    for (int i = 0; i < NUM_OF_SUPPORTED_ENTITY_CTX; i++) {
+        // coverity[missing_lock:FALSE] /* Turn off coverity missing_lock check*/
+        if (p_ent_ctx_inst_arr[i].b_enabled) {
+            entity_context_stats_t &p_ent_ctx_stats = p_ent_ctx_inst_arr[i].entity_ctx_stats;
+            int64_t tot_time = p_ent_ctx_stats.hit_poll_time + p_ent_ctx_stats.idle_time +
+                p_ent_ctx_stats.job_proc_time + 1U;
+
+            uint8_t poll_hit_time =
+                static_cast<uint8_t>((p_ent_ctx_stats.hit_poll_time * 100.0) / tot_time);
+            uint8_t job_time =
+                static_cast<uint8_t>((p_ent_ctx_stats.job_proc_time * 100.0) / tot_time);
+            uint8_t idle_time = 100U - poll_hit_time - job_time;
+
+            printf(
+                "%7d | %3" PRIu8 "%% | %6" PRIu8 "%% | %6" PRIu8 "%% | %5" PRIu32 " | %5" PRIu32
+                " | %7" PRIu32 " | %7G"
+                " | %7G | %7G | %" PRIu32 "\n",
+                i, idle_time, poll_hit_time, job_time, p_ent_ctx_stats.job_queue_size_max,
+                p_ent_ctx_stats.job_queue_size_acc / std::max(p_ent_ctx_stats.job_queue_hits, 1U),
+                p_ent_ctx_inst_arr[i].tot_socket,
+                static_cast<double>(p_ent_ctx_stats.socket_num_added),
+                static_cast<double>(p_ent_ctx_stats.socket_num_removed),
+                static_cast<double>(p_ent_ctx_stats.socket_num_added -
+                                    p_ent_ctx_stats.socket_num_removed),
+                p_ent_ctx_stats.listen_rsschild_num);
+        }
+    }
+}
+
 void print_bpool_stats(bpool_instance_block_t *p_bpool_inst_arr)
 {
     bpool_stats_t *p_bpool_stats = NULL;
@@ -1101,6 +1146,48 @@ void show_ring_stats(ring_instance_block_t *p_curr_ring_blocks,
     }
 }
 
+void show_entity_context_stats(entity_context_instance_block_t *p_curr_entctx_blocks,
+                               entity_context_instance_block_t *p_prev_entctx_blocks)
+{
+    if (unlikely(!p_curr_entctx_blocks || !p_prev_entctx_blocks)) {
+        return;
+    }
+
+    for (int i = 0; i < NUM_OF_SUPPORTED_ENTITY_CTX; i++) {
+        int delay = user_params.interval;
+        entity_context_stats_t &curr_entctx_stats = p_curr_entctx_blocks[i].entity_ctx_stats;
+        entity_context_stats_t &prev_entctx_stats = p_prev_entctx_blocks[i].entity_ctx_stats;
+
+        prev_entctx_stats.idle_time =
+            (curr_entctx_stats.idle_time - prev_entctx_stats.idle_time) / delay;
+        prev_entctx_stats.hit_poll_time =
+            (curr_entctx_stats.hit_poll_time - prev_entctx_stats.hit_poll_time) / delay;
+        prev_entctx_stats.job_proc_time =
+            (curr_entctx_stats.job_proc_time - prev_entctx_stats.job_proc_time) / delay;
+        prev_entctx_stats.job_queue_hits =
+            (curr_entctx_stats.job_queue_hits - prev_entctx_stats.job_queue_hits) / delay;
+        prev_entctx_stats.job_queue_size_acc =
+            (curr_entctx_stats.job_queue_size_acc - prev_entctx_stats.job_queue_size_acc) / delay;
+        prev_entctx_stats.job_queue_size_max = curr_entctx_stats.job_queue_size_max;
+        prev_entctx_stats.listen_rsschild_num = curr_entctx_stats.listen_rsschild_num;
+        p_prev_entctx_blocks[i].tot_socket = static_cast<uint32_t>(
+            curr_entctx_stats.socket_num_added - curr_entctx_stats.socket_num_removed);
+
+        if (user_params.print_details_mode == e_totals) {
+            prev_entctx_stats.socket_num_added = curr_entctx_stats.socket_num_added;
+            prev_entctx_stats.socket_num_removed = curr_entctx_stats.socket_num_removed;
+        } else {
+            prev_entctx_stats.socket_num_added =
+                (curr_entctx_stats.socket_num_added - prev_entctx_stats.socket_num_added) / delay;
+            prev_entctx_stats.socket_num_removed =
+                (curr_entctx_stats.socket_num_removed - prev_entctx_stats.socket_num_removed) /
+                delay;
+        }
+    }
+
+    print_entity_context_stats(p_prev_entctx_blocks);
+}
+
 void show_cq_stats(cq_instance_block_t *p_curr_cq_blocks, cq_instance_block_t *p_prev_cq_blocks)
 {
     switch (user_params.print_details_mode) {
@@ -1394,6 +1481,8 @@ void stats_reader_handler(sh_mem_t *p_sh_mem, int pid)
     cq_instance_block_t curr_cq_blocks[NUM_OF_SUPPORTED_CQS];
     ring_instance_block_t prev_ring_blocks[NUM_OF_SUPPORTED_RINGS];
     ring_instance_block_t curr_ring_blocks[NUM_OF_SUPPORTED_RINGS];
+    entity_context_instance_block_t prev_entctx_blocks[NUM_OF_SUPPORTED_ENTITY_CTX];
+    entity_context_instance_block_t curr_entctx_blocks[NUM_OF_SUPPORTED_ENTITY_CTX];
     bpool_instance_block_t prev_bpool_blocks[NUM_OF_SUPPORTED_BPOOLS];
     bpool_instance_block_t curr_bpool_blocks[NUM_OF_SUPPORTED_BPOOLS];
     global_instance_block_t prev_global_blocks[NUM_OF_SUPPORTED_GLOBALS];
@@ -1439,6 +1528,10 @@ void stats_reader_handler(sh_mem_t *p_sh_mem, int pid)
     memset((void *)curr_cq_blocks, 0, sizeof(cq_instance_block_t) * NUM_OF_SUPPORTED_CQS);
     memset((void *)prev_ring_blocks, 0, sizeof(ring_instance_block_t) * NUM_OF_SUPPORTED_RINGS);
     memset((void *)curr_ring_blocks, 0, sizeof(ring_instance_block_t) * NUM_OF_SUPPORTED_RINGS);
+    memset((void *)prev_entctx_blocks, 0,
+           sizeof(entity_context_instance_block_t) * NUM_OF_SUPPORTED_ENTITY_CTX);
+    memset((void *)curr_entctx_blocks, 0,
+           sizeof(entity_context_instance_block_t) * NUM_OF_SUPPORTED_ENTITY_CTX);
     memset((void *)prev_bpool_blocks, 0, sizeof(bpool_instance_block_t) * NUM_OF_SUPPORTED_BPOOLS);
     memset((void *)curr_bpool_blocks, 0, sizeof(bpool_instance_block_t) * NUM_OF_SUPPORTED_BPOOLS);
     for (int i = 0; i < NUM_OF_SUPPORTED_GLOBALS; i++) {
@@ -1452,6 +1545,9 @@ void stats_reader_handler(sh_mem_t *p_sh_mem, int pid)
     memset(&prev_iomux_blocks, 0, sizeof(prev_iomux_blocks));
     memset(&curr_iomux_blocks, 0, sizeof(curr_iomux_blocks));
 
+    memcpy((void *)prev_entctx_blocks, (void *)p_sh_mem->ent_ctx_inst_arr,
+           NUM_OF_SUPPORTED_ENTITY_CTX * sizeof(entity_context_instance_block_t));
+
     if (user_params.print_details_mode == e_deltas) {
         memcpy((void *)prev_instance_blocks, (void *)p_sh_mem->skt_inst_arr,
                p_sh_mem->max_skt_inst_num * sizeof(socket_instance_block_t));
@@ -1519,6 +1615,12 @@ void stats_reader_handler(sh_mem_t *p_sh_mem, int pid)
                                 p_sh_mem->max_skt_inst_num);
             goto out;
             break;
+        case e_entctx:
+            memcpy((void *)curr_entctx_blocks, (void *)p_sh_mem->ent_ctx_inst_arr,
+                   NUM_OF_SUPPORTED_ENTITY_CTX * sizeof(entity_context_instance_block_t));
+            show_entity_context_stats(curr_entctx_blocks, prev_entctx_blocks);
+            memcpy((void *)prev_entctx_blocks, (void *)curr_entctx_blocks,
+                   NUM_OF_SUPPORTED_ENTITY_CTX * sizeof(entity_context_instance_block_t));
         default:
             break;
         }
@@ -1782,24 +1884,24 @@ void zero_ring_stats(ring_stats_t *p_ring_stats)
     p_ring_stats->n_tx_pkt_count = 0;
     p_ring_stats->n_tx_byte_count = 0;
     p_ring_stats->n_tx_retransmits = 0;
+    p_ring_stats->n_tx_tso_pkt_count = 0;
+    p_ring_stats->n_tx_tso_byte_count = 0;
+    p_ring_stats->n_rx_interrupt_received = 0;
+    p_ring_stats->n_rx_interrupt_requests = 0;
+    p_ring_stats->n_tx_dropped_wqes = 0;
+    p_ring_stats->n_tx_num_bufs = 0;
+    p_ring_stats->n_zc_num_bufs = 0;
 #ifdef DEFINED_UTLS
     p_ring_stats->n_tx_tls_contexts = 0;
     p_ring_stats->n_rx_tls_contexts = 0;
+    p_ring_stats->n_tx_tls_resyncs = 0;
+    p_ring_stats->n_rx_tls_resyncs = 0;
+    p_ring_stats->n_rx_tls_auth_fail = 0;
 #endif /* DEFINED_UTLS */
-    if (p_ring_stats->n_type == RING_TAP) {
-        p_ring_stats->tap.n_vf_plugouts = 0;
-    } else {
-        p_ring_stats->simple.n_tx_tso_pkt_count = 0;
-        p_ring_stats->simple.n_tx_tso_byte_count = 0;
-        p_ring_stats->simple.n_rx_interrupt_received = 0;
-        p_ring_stats->simple.n_rx_interrupt_requests = 0;
-        p_ring_stats->simple.n_tx_dropped_wqes = 0;
-        p_ring_stats->simple.n_tx_dev_mem_byte_count = 0;
-        p_ring_stats->simple.n_tx_dev_mem_pkt_count = 0;
-        p_ring_stats->simple.n_tx_dev_mem_oob = 0;
-        p_ring_stats->simple.n_tx_num_bufs = 0;
-        p_ring_stats->simple.n_zc_num_bufs = 0;
-    }
+    p_ring_stats->n_rx_zc_migiration_drop = 0;
+    p_ring_stats->n_tx_dev_mem_byte_count = 0;
+    p_ring_stats->n_tx_dev_mem_pkt_count = 0;
+    p_ring_stats->n_tx_dev_mem_oob = 0;
 }
 
 void zero_cq_stats(cq_stats_t *p_cq_stats)
@@ -1807,6 +1909,11 @@ void zero_cq_stats(cq_stats_t *p_cq_stats)
     memset(p_cq_stats, 0, sizeof(*p_cq_stats));
 }
 
+void zero_ent_ctx_stats(entity_context_stats_t *p_ent_ctx_stats)
+{
+    memset(p_ent_ctx_stats, 0, sizeof(*p_ent_ctx_stats));
+}
+
 void zero_bpool_stats(bpool_stats_t *p_bpool_stats)
 {
     p_bpool_stats->n_buffer_pool_size = 0;
