@@ -422,11 +422,13 @@ tcp_listen_input(struct tcp_pcb_listen *pcb, tcp_in_data* in_data)
 
     /* Send a SYN|ACK together with the MSS option. */
     rc = tcp_enqueue_flags(npcb, TCP_SYN | TCP_ACK);
-    if (rc != ERR_OK) {
+    if (rc == ERR_OK) {
+      rc = tcp_output(npcb);
+    }else{
       tcp_abandon(npcb, 0);
-      return rc;
     }
-    return tcp_output(npcb);
+    TCP_EVENT_ACCEPTED_PCB(pcb, npcb);
+    return rc;
   }
   return ERR_OK;
 }
@@ -575,6 +577,7 @@ tcp_process(struct tcp_pcb *pcb, tcp_in_data* in_data)
       /* If there's nothing left to acknowledge, stop the retransmit
          timer, otherwise reset it to start again */
       if(pcb->unacked == NULL) {
+        pcb->last_unacked = NULL;
         pcb->rtime = -1;
       } else {
         pcb->rtime = 0;
@@ -1103,6 +1106,7 @@ tcp_receive(struct tcp_pcb *pcb, tcp_in_data* in_data)
       /* If there's nothing left to acknowledge, stop the retransmit
          timer, otherwise reset it to start again */
       if(pcb->unacked == NULL) {
+        pcb->last_unacked = NULL;
         if (persist) {
           /* start persist timer */
           pcb->persist_cnt = 0;
