@@ -1,33 +1,7 @@
 /*
- * Copyright (c) 2001-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
- *
- * This software is available to you under a choice of one of two
- * licenses.  You may choose to be licensed under the terms of the GNU
- * General Public License (GPL) Version 2, available from the file
- * COPYING in the main directory of this source tree, or the
- * BSD license below:
- *
- *     Redistribution and use in source and binary forms, with or
- *     without modification, are permitted provided that the following
- *     conditions are met:
- *
- *      - Redistributions of source code must retain the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer.
- *
- *      - Redistributions in binary form must reproduce the above
- *        copyright notice, this list of conditions and the following
- *        disclaimer in the documentation and/or other materials
- *        provided with the distribution.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
+ * Copyright (c) 2001-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: GPL-2.0-only or BSD-2-Clause
  */
 
 #ifndef VMA_EXTRA_H
@@ -53,6 +27,38 @@
 #define SO_VMA_FLOW_TAG         2820
 #define SO_VMA_SHUTDOWN_RX      2821
 
+enum {
+	/* cmsg_level is SOL_SOCKET as protocol independent option
+	 * cmsg_data has data in vma_cmsg_ioctl_user_alloc_t format
+	 */
+	CMSG_VMA_IOCTL_USER_ALLOC = 2900
+};
+
+/**
+ * @brief This structure as an argument for vma_ioctl() call with
+ *  @ref CMSG_VMA_IOCTL_USER_ALLOC to use user provided functions for
+ *  internal pool allocation.
+ * 
+ * @note CMSG_VMA_IOCTL_USER_ALLOC must be called before the library
+ * initialization that is done during first call of following functions
+ * as socket(), epoll_create(), epoll_create1(), pipe().
+ *
+ * @param flags - set a internal memory pool to apply.
+ * @param memalloc - ponter to the function to allocate memory.
+ * @param memfree - ponter to the function to free previously allocated memory.
+ */
+
+enum {
+	VMA_IOCTL_USER_ALLOC_FLAG_TX = (1 << 0),
+	VMA_IOCTL_USER_ALLOC_FLAG_RX = (1 << 1)
+};
+
+struct __attribute__ ((packed)) vma_cmsg_ioctl_user_alloc_t {
+	uint8_t flags;
+	void* (*memalloc)(size_t);
+	void (*memfree)(void *);
+};
+
 /*
  * Flags for Dummy send API
  */
@@ -366,6 +372,7 @@ typedef enum {
 	VMA_EXTRA_API_REGISTER_MEMORY_ON_RING        = (1 << 17),
 	VMA_EXTRA_API_DEREGISTER_MEMORY_ON_RING      = (1 << 18),
 	VMA_EXTRA_API_MODIFY_RING                    = (1 << 20),
+	VMA_EXTRA_API_IOCTL                          = (1 << 21),
 } vma_extra_api_mask;
 
 /** 
@@ -711,15 +718,36 @@ struct __attribute__ ((packed)) vma_api_t {
 	/**
 	 * Used to identify which methods were initialized by VMA as part of vma_get_api().
 	 * The value content is based on vma_extra_api_mask enum.
+	 * Order of fields in this structure should not be changed to keep abi compatibility.
 	 */
 	uint64_t vma_extra_supported_mask;
 
+	/**
+	 * This function allows to communicate with library using extendable protocol
+	 * based on struct cmshdr.
+	 *
+	 * Ancillary data is a sequence of cmsghdr structures with appended data.
+	 * The sequence of cmsghdr structures should never be accessed directly.
+	 * Instead, use only the following macros: CMSG_ALIGN, CMSG_SPACE, CMSG_DATA,
+	 * CMSG_LEN.
+	 *
+	 * @param cmsg_hdr - point to control message
+	 * @param cmsg_len - the byte count of the ancillary data,
+	 *                   which contains the size of the structure header.
+	 *
+	 * @return -1 on failure and 0 on success
+	 */
+	int (*ioctl)(void *cmsg_hdr, size_t cmsg_len);
 };
 
 /**
  * Retrieve VMA extended API.
+ * This function can be called as an alternative to getsockopt() call
+ * when library is preloaded using LD_PRELOAD
+ * getsockopt() call should be used in case application loads library
+ * using dlopen()/dlsym().
  *
- * @return Pointer to the VMA Extended Socket API, of NULL if VMA not found. 
+ * @return Pointer to the VMA Extended Socket API, of NULL if VMA not found.
  */
 static inline struct vma_api_t* vma_get_api()
 {
