@@ -96,7 +96,7 @@
 #include <linux/slab.h>
 #endif
 #include <linux/spinlock.h>
-#include <asm/unaligned.h>
+#include <linux/unaligned.h>
 #include <rdma/ib_verbs.h>
 #include <rdma/rdma_cm.h>
 
@@ -119,25 +119,18 @@ svc_rdma_next_recv_ctxt(struct list_head *list)
 					rc_list);
 }
 
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
-static void svc_rdma_recv_cid_init(struct svcxprt_rdma *rdma,
-				   struct rpc_rdma_cid *cid)
-{
-	cid->ci_queue_id = rdma->sc_rq_cq->res.id;
-	cid->ci_completion_id = atomic_inc_return(&rdma->sc_completion_ids);
-}
-
 static struct svc_rdma_recv_ctxt *
 svc_rdma_recv_ctxt_alloc(struct svcxprt_rdma *rdma)
 {
+	int node = ibdev_to_node(rdma->sc_cm_id->device);
 	struct svc_rdma_recv_ctxt *ctxt;
 	dma_addr_t addr;
 	void *buffer;
 
-	ctxt = kmalloc(sizeof(*ctxt), GFP_KERNEL);
+	ctxt = kzalloc_node(sizeof(*ctxt), GFP_KERNEL, node);
 	if (!ctxt)
 		goto fail0;
-	buffer = kmalloc(rdma->sc_max_req_size, GFP_KERNEL);
+	buffer = kmalloc_node(rdma->sc_max_req_size, GFP_KERNEL, node);
 	if (!buffer)
 		goto fail1;
 	addr = ib_dma_map_single(rdma->sc_pd->device, buffer,
@@ -162,7 +155,7 @@ svc_rdma_recv_ctxt_alloc(struct svcxprt_rdma *rdma)
 	ctxt->rc_recv_sge.length = rdma->sc_max_req_size;
 	ctxt->rc_recv_sge.lkey = rdma->sc_pd->local_dma_lkey;
 	ctxt->rc_recv_buf = buffer;
-	ctxt->rc_temp = false;
+	svc_rdma_cc_init(rdma, &ctxt->rc_cc);
 	return ctxt;
 
 fail2:
@@ -181,7 +174,6 @@ static void svc_rdma_recv_ctxt_destroy(struct svcxprt_rdma *rdma,
 	kfree(ctxt->rc_recv_buf);
 	kfree(ctxt);
 }
-#endif
 
 /**
  * svc_rdma_recv_ctxts_destroy - Release all recv_ctxt's for an xprt
@@ -191,86 +183,56 @@ static void svc_rdma_recv_ctxt_destroy(struct svcxprt_rdma *rdma,
 void svc_rdma_recv_ctxts_destroy(struct svcxprt_rdma *rdma)
 {
 	struct svc_rdma_recv_ctxt *ctxt;
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 	struct llist_node *node;
 
 	while ((node = llist_del_first(&rdma->sc_recv_ctxts))) {
 		ctxt = llist_entry(node, struct svc_rdma_recv_ctxt, rc_node);
 		svc_rdma_recv_ctxt_destroy(rdma, ctxt);
-#else
-	while ((ctxt = svc_rdma_next_recv_ctxt(&rdma->sc_recv_ctxts))) {
-		list_del(&ctxt->rc_list);
-		kfree(ctxt);
-#endif
 	}
 }
 
-#ifdef HAVE_SVC_RDMA_PCL
 /**
  * svc_rdma_recv_ctxt_get - Allocate a recv_ctxt
  * @rdma: controlling svcxprt_rdma
  *
  * Returns a recv_ctxt or (rarely) NULL if none are available.
  */
+#ifdef HAVE_SVC_RDMA_PCL
+struct svc_rdma_recv_ctxt *svc_rdma_recv_ctxt_get(struct svcxprt_rdma *rdma)
+{
+	struct svc_rdma_recv_ctxt *ctxt;
+	struct llist_node *node;
+
+	node = llist_del_first(&rdma->sc_recv_ctxts);
+	if (!node)
+		return NULL;
+
+	ctxt = llist_entry(node, struct svc_rdma_recv_ctxt, rc_node);
+	ctxt->rc_page_count = 0;
+	return ctxt;
+}
+#else
 struct svc_rdma_recv_ctxt *svc_rdma_recv_ctxt_get(struct svcxprt_rdma *rdma)
-#else
-static struct svc_rdma_recv_ctxt *
-svc_rdma_recv_ctxt_get(struct svcxprt_rdma *rdma)
-#endif
 {
 	struct svc_rdma_recv_ctxt *ctxt;
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 	struct llist_node *node;
 
 	node = llist_del_first(&rdma->sc_recv_ctxts);
 	if (!node)
 		goto out_empty;
 	ctxt = llist_entry(node, struct svc_rdma_recv_ctxt, rc_node);
-#else
-	spin_lock(&rdma->sc_recv_lock);
-	ctxt = svc_rdma_next_recv_ctxt(&rdma->sc_recv_ctxts);
-	if (!ctxt)
-		goto out_empty;
-	list_del(&ctxt->rc_list);
-	spin_unlock(&rdma->sc_recv_lock);
-#endif
- 
+
 out:
-#ifndef HAVE_SVC_FILL_WRITE_VECTOR
-	ctxt->rc_recv_wr.num_sge = 0;
-#endif
 	ctxt->rc_page_count = 0;
-#ifndef HAVE_SVC_RDMA_PCL
 	ctxt->rc_read_payload_length = 0;
-#endif
 	return ctxt;
 
 out_empty:
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 	ctxt = svc_rdma_recv_ctxt_alloc(rdma);
-#else
-	spin_unlock(&rdma->sc_recv_lock);
-
-	ctxt = kmalloc(sizeof(*ctxt), GFP_KERNEL);
-#endif
 	if (!ctxt)
 		return NULL;
 	goto out;
 }
-
-#ifndef HAVE_SVC_FILL_WRITE_VECTOR
-static void svc_rdma_recv_ctxt_unmap(struct svcxprt_rdma *rdma,
-				     struct svc_rdma_recv_ctxt *ctxt)
-{
-	struct ib_device *device = rdma->sc_cm_id->device;
-	int i;
-
-	for (i = 0; i < ctxt->rc_recv_wr.num_sge; i++)
-		ib_dma_unmap_page(device,
-				  ctxt->rc_sges[i].addr,
-				  ctxt->rc_sges[i].length,
-				  DMA_FROM_DEVICE);
-}
 #endif
 
 /**
@@ -282,10 +244,19 @@ static void svc_rdma_recv_ctxt_unmap(struct svcxprt_rdma *rdma,
 void svc_rdma_recv_ctxt_put(struct svcxprt_rdma *rdma,
 			    struct svc_rdma_recv_ctxt *ctxt)
 {
-#ifndef HAVE_SVC_RDMA_PCL
-	unsigned int i;
+#ifndef HAVE_RELEASE_PAGES_IN_MM_H
+	int i;
+#endif
 
-	for (i = 0; i < ctxt->rc_page_count; i++)
+	svc_rdma_cc_release(rdma, &ctxt->rc_cc, DMA_FROM_DEVICE);
+
+	/* @rc_page_count is normally zero here, but error flows
+	 * can leave pages in @rc_pages.
+	 */
+#ifdef HAVE_RELEASE_PAGES_IN_MM_H
+	release_pages(ctxt->rc_pages, ctxt->rc_page_count);
+#else
+	for (i = 0; i < ctxt->rc_page_count; ++i)
 		put_page(ctxt->rc_pages[i]);
 #endif
 
@@ -296,16 +267,7 @@ void svc_rdma_recv_ctxt_put(struct svcxprt_rdma *rdma,
 	pcl_free(&ctxt->rc_reply_pcl);
 #endif
 
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
-	if (!ctxt->rc_temp)
-		llist_add(&ctxt->rc_node, &rdma->sc_recv_ctxts);
-	else
-		svc_rdma_recv_ctxt_destroy(rdma, ctxt);
-#else
-	spin_lock(&rdma->sc_recv_lock);
-	list_add(&ctxt->rc_list, &rdma->sc_recv_ctxts);
-	spin_unlock(&rdma->sc_recv_lock);
-#endif
+	llist_add(&ctxt->rc_node, &rdma->sc_recv_ctxts);
 }
 
 #ifdef HAVE_SVC_RDMA_RELEASE_RQST
@@ -353,7 +315,7 @@ void svc_rdma_release_ctxt(struct svc_xprt *xprt, void *vctxt)
 
 #ifdef HAVE_SVC_RDMA_PCL
 static bool svc_rdma_refresh_recvs(struct svcxprt_rdma *rdma,
-				   unsigned int wanted, bool temp)
+				   unsigned int wanted)
 {
 	const struct ib_recv_wr *bad_wr = NULL;
 	struct svc_rdma_recv_ctxt *ctxt;
@@ -370,9 +332,8 @@ static bool svc_rdma_refresh_recvs(struct svcxprt_rdma *rdma,
 			break;
 
 #ifdef HAVE_TRACE_RPCRDMA_H
-		trace_svcrdma_post_recv(ctxt);
+		trace_svcrdma_post_recv(&ctxt->rc_cid);
 #endif
-		ctxt->rc_temp = temp;
 		ctxt->rc_recv_wr.next = recv_chain;
 		recv_chain = &ctxt->rc_recv_wr;
 #ifdef HAVE_SVCXPRT_RDMA_SC_PENDING_RECVS
@@ -380,7 +341,7 @@ static bool svc_rdma_refresh_recvs(struct svcxprt_rdma *rdma,
 #endif
 	}
 	if (!recv_chain)
-		return false;
+		return true;
 
 	ret = ib_post_recv(rdma->sc_qp, recv_chain, &bad_wr);
 	if (ret)
@@ -402,14 +363,13 @@ err_free:
 	return false;
 }
 #else
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 static int __svc_rdma_post_recv(struct svcxprt_rdma *rdma,
 				struct svc_rdma_recv_ctxt *ctxt)
 {
 	int ret;
 
 #ifdef HAVE_TRACE_RPCRDMA_H
-	trace_svcrdma_post_recv(ctxt);
+	trace_svcrdma_post_recv(&ctxt->rc_cid);
 #endif
 	ret = ib_post_recv(rdma->sc_qp, &ctxt->rc_recv_wr, NULL);
 	if (ret)
@@ -435,99 +395,49 @@ static int svc_rdma_post_recv(struct svcxprt_rdma *rdma)
 		return -ENOMEM;
 	return __svc_rdma_post_recv(rdma, ctxt);
 }
-#else
-static int svc_rdma_post_recv(struct svcxprt_rdma *rdma)
-{
-	struct ib_device *device = rdma->sc_cm_id->device;
-	struct svc_rdma_recv_ctxt *ctxt;
-	int sge_no, buflen, ret;
-	struct page *page;
-	dma_addr_t pa;
-
-	ctxt = svc_rdma_recv_ctxt_get(rdma);
-	if (!ctxt)
-		return -ENOMEM;
-
-	buflen = 0;
-	ctxt->rc_cqe.done = svc_rdma_wc_receive;
-	for (sge_no = 0; buflen < rdma->sc_max_req_size; sge_no++) {
-		if (sge_no >= rdma->sc_max_send_sges) {
-			pr_err("svcrdma: Too many sges (%d)\n", sge_no);
-			goto err_put_ctxt;
-		}
-
-		page = alloc_page(GFP_KERNEL);
-		if (!page)
-			goto err_put_ctxt;
-		ctxt->rc_pages[sge_no] = page;
-		ctxt->rc_page_count++;
-
-		pa = ib_dma_map_page(device, ctxt->rc_pages[sge_no],
-				     0, PAGE_SIZE, DMA_FROM_DEVICE);
-		if (ib_dma_mapping_error(device, pa))
-			goto err_put_ctxt;
-		ctxt->rc_sges[sge_no].addr = pa;
-		ctxt->rc_sges[sge_no].length = PAGE_SIZE;
-		ctxt->rc_sges[sge_no].lkey = rdma->sc_pd->local_dma_lkey;
-		ctxt->rc_recv_wr.num_sge++;
-
-		buflen += PAGE_SIZE;
-	}
-	ctxt->rc_recv_wr.next = NULL;
-	ctxt->rc_recv_wr.sg_list = &ctxt->rc_sges[0];
-	ctxt->rc_recv_wr.wr_cqe = &ctxt->rc_cqe;
-
-#ifdef HAVE_TRACE_RPCRDMA_H
-	trace_svcrdma_post_recv(ctxt);
-#endif
-	ret = ib_post_recv(rdma->sc_qp, &ctxt->rc_recv_wr, NULL);
-	if (ret)
-		goto err_post;
-	return 0;
-
-err_put_ctxt:
-	svc_rdma_recv_ctxt_unmap(rdma, ctxt);
-	svc_rdma_recv_ctxt_put(rdma, ctxt);
-	return -ENOMEM;
-
-err_post:
-	svc_rdma_recv_ctxt_unmap(rdma, ctxt);
-#ifdef HAVE_TRACE_RPCRDMA_H
-	trace_svcrdma_rq_post_err(rdma, ret);
-#endif
-	svc_rdma_recv_ctxt_put(rdma, ctxt);
-	return ret;
-}
-#endif /* end of HAVE_SVC_FILL_WRITE_VECTOR */
 #endif /* end of HAVE_SVC_RDMA_PCL */
- 
+
 /**
  * svc_rdma_post_recvs - Post initial set of Recv WRs
  * @rdma: fresh svcxprt_rdma
  *
- * Returns true if successful, otherwise false.
+ * Return values:
+ *   %true: Receive Queue initialization successful
+ *   %false: memory allocation or DMA error
  */
 bool svc_rdma_post_recvs(struct svcxprt_rdma *rdma)
 {
 #ifdef HAVE_SVC_RDMA_PCL
-	return svc_rdma_refresh_recvs(rdma, rdma->sc_max_requests, true);
+	unsigned int total;
+
+	/* For each credit, allocate enough recv_ctxts for one
+	 * posted Receive and one RPC in process.
+	 */
+#ifdef HAVE_SVCXPRT_RDMA_SC_PENDING_RECVS
+	total = (rdma->sc_max_requests * 2) + rdma->sc_recv_batch;
 #else
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
-	struct svc_rdma_recv_ctxt *ctxt;
+	total = (rdma->sc_max_requests * 2);
 #endif
+	while (total--) {
+		struct svc_rdma_recv_ctxt *ctxt;
+
+		ctxt = svc_rdma_recv_ctxt_alloc(rdma);
+		if (!ctxt)
+			return false;
+		llist_add(&ctxt->rc_node, &rdma->sc_recv_ctxts);
+	}
+
+	return svc_rdma_refresh_recvs(rdma, rdma->sc_max_requests);
+#else
+	struct svc_rdma_recv_ctxt *ctxt;
 	unsigned int i;
 	int ret;
 
 	for (i = 0; i < rdma->sc_max_requests; i++) {
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 		ctxt = svc_rdma_recv_ctxt_get(rdma);
 		if (!ctxt)
 			return false;
-		ctxt->rc_temp = true;
 		ret = __svc_rdma_post_recv(rdma, ctxt);
-#else
-		ret = svc_rdma_post_recv(rdma);
-#endif
 		if (ret)
 			return false;
 	}
@@ -553,9 +463,6 @@ static void svc_rdma_wc_receive(struct ib_cq *cq, struct ib_wc *wc)
 
 	/* WARNING: Only wc->wr_cqe and wc->status are reliable */
 	ctxt = container_of(cqe, struct svc_rdma_recv_ctxt, rc_cqe);
-#ifndef HAVE_SVC_FILL_WRITE_VECTOR
-	svc_rdma_recv_ctxt_unmap(rdma, ctxt);
-#endif
 
 	if (wc->status != IB_WC_SUCCESS)
 		goto flushed;
@@ -575,10 +482,10 @@ static void svc_rdma_wc_receive(struct ib_cq *cq, struct ib_wc *wc)
 	 */
 #ifdef HAVE_SVCXPRT_RDMA_SC_PENDING_RECVS
 	if (rdma->sc_pending_recvs < rdma->sc_max_requests)
-		if (!svc_rdma_refresh_recvs(rdma, rdma->sc_recv_batch, false))
+		if (!svc_rdma_refresh_recvs(rdma, rdma->sc_recv_batch))
 			goto dropped;
 #else
-	if (!svc_rdma_refresh_recvs(rdma, 1, false))
+	if (!svc_rdma_refresh_recvs(rdma, 1))
 		goto dropped;
 #endif
 #else
@@ -626,13 +533,10 @@ void svc_rdma_flush_recv_queues(struct svcxprt_rdma *rdma)
 {
 	struct svc_rdma_recv_ctxt *ctxt;
 
-#ifndef HAVE_SVC_RDMA_PCL
 	while ((ctxt = svc_rdma_next_recv_ctxt(&rdma->sc_read_complete_q))) {
 		list_del(&ctxt->rc_list);
 		svc_rdma_recv_ctxt_put(rdma, ctxt);
 	}
-#endif
-
 	while ((ctxt = svc_rdma_next_recv_ctxt(&rdma->sc_rq_dto_q))) {
 		list_del(&ctxt->rc_list);
 		svc_rdma_recv_ctxt_put(rdma, ctxt);
@@ -642,7 +546,6 @@ void svc_rdma_flush_recv_queues(struct svcxprt_rdma *rdma)
 static void svc_rdma_build_arg_xdr(struct svc_rqst *rqstp,
 				   struct svc_rdma_recv_ctxt *ctxt)
 {
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 	struct xdr_buf *arg = &rqstp->rq_arg;
 
 	arg->head[0].iov_base = ctxt->rc_recv_buf;
@@ -653,58 +556,6 @@ static void svc_rdma_build_arg_xdr(struct svc_rqst *rqstp,
 	arg->page_base = 0;
 	arg->buflen = ctxt->rc_byte_len;
 	arg->len = ctxt->rc_byte_len;
-#else
-	struct page *page;
-	int sge_no;
-	u32 len;
-
-	/* The reply path assumes the Call's transport header resides
-	 * in rqstp->rq_pages[0].
-	 */
-	page = ctxt->rc_pages[0];
-	put_page(rqstp->rq_pages[0]);
-	rqstp->rq_pages[0] = page;
-
-	/* Set up the XDR head */
-	rqstp->rq_arg.head[0].iov_base = page_address(page);
-	rqstp->rq_arg.head[0].iov_len =
-		min_t(size_t, ctxt->rc_byte_len, ctxt->rc_sges[0].length);
-	rqstp->rq_arg.len = ctxt->rc_byte_len;
-	rqstp->rq_arg.buflen = ctxt->rc_byte_len;
-
-	/* Compute bytes past head in the SGL */
-	len = ctxt->rc_byte_len - rqstp->rq_arg.head[0].iov_len;
-
-	/* If data remains, store it in the pagelist */
-	rqstp->rq_arg.page_len = len;
-	rqstp->rq_arg.page_base = 0;
-
-	sge_no = 1;
-	while (len && sge_no < ctxt->rc_recv_wr.num_sge) {
-		page = ctxt->rc_pages[sge_no];
-		put_page(rqstp->rq_pages[sge_no]);
-		rqstp->rq_pages[sge_no] = page;
-		len -= min_t(u32, len, ctxt->rc_sges[sge_no].length);
-		sge_no++;
-	}
-	ctxt->rc_hdr_count = sge_no;
-	rqstp->rq_respages = &rqstp->rq_pages[sge_no];
-	rqstp->rq_next_page = rqstp->rq_respages + 1;
-
-	/* If not all pages were used from the SGL, free the remaining ones */
-	while (sge_no < ctxt->rc_recv_wr.num_sge) {
-		page = ctxt->rc_pages[sge_no++];
-		put_page(page);
-	}
-
-	/* @ctxt's pages have all been released or moved to @rqstp->rq_pages.
-	 */
-	ctxt->rc_page_count = 0;
-
-	/* Set up tail */
-	rqstp->rq_arg.tail[0].iov_base = NULL;
-	rqstp->rq_arg.tail[0].iov_len = 0;
-#endif
 }
 
 #ifndef HAVE_SVC_RDMA_PCL
@@ -803,7 +654,13 @@ static bool xdr_check_write_chunk(struct svc_rdma_recv_ctxt *rctxt)
 	if (xdr_stream_decode_u32(&rctxt->rc_stream, &segcount))
 		return false;
 
-	/* A bogus segcount causes this buffer overflow check to fail. */
+	/* Before trusting the segcount value enough to use it in
+	 * a computation, perform a simple range check. This is an
+	 * arbitrary but sensible limit (ie, not architectural).
+	 */
+	if (unlikely(segcount > RPCSVC_MAXPAGES))
+		return false;
+
 	p = xdr_inline_decode(&rctxt->rc_stream,
 			      segcount * rpcrdma_segment_maxsz * sizeof(*p));
 	return p != NULL;
@@ -941,16 +798,13 @@ static bool xdr_check_write_chunk(struct svc_rdma_recv_ctxt *rctxt, u32 maxlen)
  */
 static bool xdr_check_write_list(struct svc_rdma_recv_ctxt *rctxt)
 {
-#ifndef HAVE_SVC_RDMA_PCL
-	u32 chcount = 0;
-#endif
+#ifdef HAVE_SVC_RDMA_PCL
 	__be32 *p;
 
 	p = xdr_inline_decode(&rctxt->rc_stream, sizeof(*p));
 	if (!p)
 		return false;
 
-#ifdef HAVE_SVC_RDMA_PCL
 	if (!xdr_count_write_chunks(rctxt, p))
 		return false;
 	if (!pcl_alloc_write(rctxt, &rctxt->rc_write_pcl, p))
@@ -959,6 +813,13 @@ static bool xdr_check_write_list(struct svc_rdma_recv_ctxt *rctxt)
 	rctxt->rc_cur_result_payload = pcl_first_chunk(&rctxt->rc_write_pcl);
 	return true;
 #else
+	u32 chcount = 0;
+	__be32 *p;
+
+	p = xdr_inline_decode(&rctxt->rc_stream, sizeof(*p));
+	if (!p)
+		return false;
+
 	rctxt->rc_write_list = p;
 	while (xdr_item_is_present(p)) {
 		if (!xdr_check_write_chunk(rctxt, MAX_BYTES_WRITE_CHUNK))
@@ -1077,11 +938,7 @@ static void svc_rdma_get_inv_rkey(struct svcxprt_rdma *rdma,
 		return;
 
 	inv_rkey = xdr_zero;
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 	p = ctxt->rc_recv_buf;
-#else
-	p = page_address(ctxt->rc_pages[0]);
-#endif
 	p += rpcrdma_fixed_maxsz;
 
 	/* Read list */
@@ -1224,37 +1081,6 @@ out_inval:
 	return -EINVAL;
 }
 
-#ifndef HAVE_SVC_RDMA_PCL
-static void rdma_read_complete(struct svc_rqst *rqstp,
-			       struct svc_rdma_recv_ctxt *head)
-{
-	int page_no;
-
-	/* Move Read chunk pages to rqstp so that they will be released
-	 * when svc_process is done with them.
-	 */
-	for (page_no = 0; page_no < head->rc_page_count; page_no++) {
-		put_page(rqstp->rq_pages[page_no]);
-		rqstp->rq_pages[page_no] = head->rc_pages[page_no];
-	}
-	head->rc_page_count = 0;
-
-	/* Point rq_arg.pages past header */
-	rqstp->rq_arg.pages = &rqstp->rq_pages[head->rc_hdr_count];
-	rqstp->rq_arg.page_len = head->rc_arg.page_len;
-
-	/* rq_respages starts after the last arg page */
-	rqstp->rq_respages = &rqstp->rq_pages[page_no];
-	rqstp->rq_next_page = rqstp->rq_respages + 1;
-
-	/* Rebuild rq_arg head and tail. */
-	rqstp->rq_arg.head[0] = head->rc_arg.head[0];
-	rqstp->rq_arg.tail[0] = head->rc_arg.tail[0];
-	rqstp->rq_arg.len = head->rc_arg.len;
-	rqstp->rq_arg.buflen = head->rc_arg.buflen;
-}
-#endif
-
 static void svc_rdma_send_error(struct svcxprt_rdma *rdma,
 				struct svc_rdma_recv_ctxt *rctxt,
 				int status)
@@ -1276,11 +1102,7 @@ static void svc_rdma_send_error(struct svcxprt_rdma *rdma,
 static bool svc_rdma_is_reverse_direction_reply(struct svc_xprt *xprt,
 						struct svc_rdma_recv_ctxt *rctxt)
 {
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 	__be32 *p = rctxt->rc_recv_buf;
-#else
-	__be32 *p = page_address(rctxt->rc_pages[0]);
-#endif
 
 	if (!xprt->xpt_bc_xprt)
 		return false;
@@ -1334,6 +1156,160 @@ static bool svc_rdma_is_backchannel_reply(struct svc_xprt *xprt,
 }
 #endif
 
+#ifdef HAVE_SVC_RDMA_PCL
+/* Finish constructing the RPC Call message in rqstp::rq_arg.
+ *
+ * The incoming RPC/RDMA message is an RDMA_MSG type message
+ * with a single Read chunk (only the upper layer data payload
+ * was conveyed via RDMA Read).
+ */
+static void svc_rdma_read_complete_one(struct svc_rqst *rqstp,
+				       struct svc_rdma_recv_ctxt *ctxt)
+{
+	struct svc_rdma_chunk *chunk = pcl_first_chunk(&ctxt->rc_read_pcl);
+	struct xdr_buf *buf = &rqstp->rq_arg;
+	unsigned int length;
+
+	/* Split the Receive buffer between the head and tail
+	 * buffers at Read chunk's position. XDR roundup of the
+	 * chunk is not included in either the pagelist or in
+	 * the tail.
+	 */
+	buf->tail[0].iov_base = buf->head[0].iov_base + chunk->ch_position;
+	buf->tail[0].iov_len = buf->head[0].iov_len - chunk->ch_position;
+	buf->head[0].iov_len = chunk->ch_position;
+
+	/* Read chunk may need XDR roundup (see RFC 8166, s. 3.4.5.2).
+	 *
+	 * If the client already rounded up the chunk length, the
+	 * length does not change. Otherwise, the length of the page
+	 * list is increased to include XDR round-up.
+	 *
+	 * Currently these chunks always start at page offset 0,
+	 * thus the rounded-up length never crosses a page boundary.
+	 */
+	buf->pages = &rqstp->rq_pages[0];
+	length = xdr_align_size(chunk->ch_length);
+	buf->page_len = length;
+	buf->len += length;
+	buf->buflen += length;
+}
+
+/* Finish constructing the RPC Call message in rqstp::rq_arg.
+ *
+ * The incoming RPC/RDMA message is an RDMA_MSG type message
+ * with payload in multiple Read chunks and no PZRC.
+ */
+static void svc_rdma_read_complete_multiple(struct svc_rqst *rqstp,
+					    struct svc_rdma_recv_ctxt *ctxt)
+{
+	struct xdr_buf *buf = &rqstp->rq_arg;
+
+	buf->len += ctxt->rc_readbytes;
+	buf->buflen += ctxt->rc_readbytes;
+
+	buf->head[0].iov_base = page_address(rqstp->rq_pages[0]);
+	buf->head[0].iov_len = min_t(size_t, PAGE_SIZE, ctxt->rc_readbytes);
+	buf->pages = &rqstp->rq_pages[1];
+	buf->page_len = ctxt->rc_readbytes - buf->head[0].iov_len;
+}
+
+/* Finish constructing the RPC Call message in rqstp::rq_arg.
+ *
+ * The incoming RPC/RDMA message is an RDMA_NOMSG type message
+ * (the RPC message body was conveyed via RDMA Read).
+ */
+static void svc_rdma_read_complete_pzrc(struct svc_rqst *rqstp,
+					struct svc_rdma_recv_ctxt *ctxt)
+{
+	struct xdr_buf *buf = &rqstp->rq_arg;
+
+	buf->len += ctxt->rc_readbytes;
+	buf->buflen += ctxt->rc_readbytes;
+
+	buf->head[0].iov_base = page_address(rqstp->rq_pages[0]);
+	buf->head[0].iov_len = min_t(size_t, PAGE_SIZE, ctxt->rc_readbytes);
+	buf->pages = &rqstp->rq_pages[1];
+	buf->page_len = ctxt->rc_readbytes - buf->head[0].iov_len;
+}
+
+static noinline void svc_rdma_read_complete(struct svc_rqst *rqstp,
+					    struct svc_rdma_recv_ctxt *ctxt)
+{
+	unsigned int i;
+
+	/* Transfer the Read chunk pages into @rqstp.rq_pages, replacing
+	 * the rq_pages that were already allocated for this rqstp.
+	 */
+#ifdef HAVE_RELEASE_PAGES_IN_MM_H
+	release_pages(rqstp->rq_respages, ctxt->rc_page_count);
+#else
+	for (i = 0; i < ctxt->rc_page_count; ++i)
+		put_page(rqstp->rq_respages[i]);
+#endif
+	for (i = 0; i < ctxt->rc_page_count; i++)
+		rqstp->rq_pages[i] = ctxt->rc_pages[i];
+
+	/* Update @rqstp's result send buffer to start after the
+	 * last page in the RDMA Read payload.
+	 */
+	rqstp->rq_respages = &rqstp->rq_pages[ctxt->rc_page_count];
+	rqstp->rq_next_page = rqstp->rq_respages + 1;
+
+	/* Prevent svc_rdma_recv_ctxt_put() from releasing the
+	 * pages in ctxt::rc_pages a second time.
+	 */
+	ctxt->rc_page_count = 0;
+
+	/* Finish constructing the RPC Call message. The exact
+	 * procedure for that depends on what kind of RPC/RDMA
+	 * chunks were provided by the client.
+	 */
+	rqstp->rq_arg = ctxt->rc_saved_arg;
+	if (pcl_is_empty(&ctxt->rc_call_pcl)) {
+		if (ctxt->rc_read_pcl.cl_count == 1)
+			svc_rdma_read_complete_one(rqstp, ctxt);
+		else
+			svc_rdma_read_complete_multiple(rqstp, ctxt);
+	} else {
+		svc_rdma_read_complete_pzrc(rqstp, ctxt);
+	}
+
+#ifdef HAVE_TRACE_RPCRDMA_H
+	trace_svcrdma_read_finished(&ctxt->rc_cid);
+#endif
+}
+#else
+static void rdma_read_complete(struct svc_rqst *rqstp,
+                   struct svc_rdma_recv_ctxt *head)
+{
+	int page_no;
+
+	/* Move Read chunk pages to rqstp so that they will be released
+	 * when svc_process is done with them.
+	 */
+	for (page_no = 0; page_no < head->rc_page_count; page_no++) {
+		put_page(rqstp->rq_pages[page_no]);
+		rqstp->rq_pages[page_no] = head->rc_pages[page_no];
+	}
+	head->rc_page_count = 0;
+
+	/* Point rq_arg.pages past header */
+	rqstp->rq_arg.pages = &rqstp->rq_pages[head->rc_hdr_count];
+	rqstp->rq_arg.page_len = head->rc_arg.page_len;
+
+	/* rq_respages starts after the last arg page */
+	rqstp->rq_respages = &rqstp->rq_pages[page_no];
+	rqstp->rq_next_page = rqstp->rq_respages + 1;
+
+	/* Rebuild rq_arg head and tail. */
+	rqstp->rq_arg.head[0] = head->rc_arg.head[0];
+	rqstp->rq_arg.tail[0] = head->rc_arg.tail[0];
+	rqstp->rq_arg.len = head->rc_arg.len;
+	rqstp->rq_arg.buflen = head->rc_arg.buflen;
+}
+#endif
+
 /**
  * svc_rdma_recvfrom - Receive an RPC call
  * @rqstp: request structure into which to receive an RPC Call
@@ -1351,9 +1327,6 @@ static bool svc_rdma_is_backchannel_reply(struct svc_xprt *xprt,
  *
  * The next ctxt is removed from the "receive" lists.
  *
- * - If the ctxt completes a Read, then finish assembling the Call
- *   message and return the number of bytes in the message.
- *
  * - If the ctxt completes a Receive, then construct the Call
  *   message from the contents of the Receive buffer.
  *
@@ -1362,7 +1335,8 @@ static bool svc_rdma_is_backchannel_reply(struct svc_xprt *xprt,
  *     in the message.
  *
  *   - If there are Read chunks in this message, post Read WRs to
- *     pull that payload and return 0.
+ *     pull that payload. When the Read WRs complete, build the
+ *     full message and return the number of bytes in it.
  */
 int svc_rdma_recvfrom(struct svc_rqst *rqstp)
 {
@@ -1375,29 +1349,31 @@ int svc_rdma_recvfrom(struct svc_rqst *rqstp)
 #endif
 	int ret;
 
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
-	/* Prevent svc_xprt_release from releasing pages in rq_pages
-	 * if we return 0 or an error.
+	/* Prevent svc_xprt_release() from releasing pages in rq_pages
+	 * when returning 0 or an error.
 	 */
 	rqstp->rq_respages = rqstp->rq_pages;
 	rqstp->rq_next_page = rqstp->rq_respages;
-#endif
 
 #ifdef HAVE_SVC_RDMA_RELEASE_RQST
 	rqstp->rq_xprt_ctxt = NULL;
 #endif
 
-	ctxt = NULL;
 	spin_lock(&rdma_xprt->sc_rq_dto_lock);
-#ifndef HAVE_SVC_RDMA_PCL
 	ctxt = svc_rdma_next_recv_ctxt(&rdma_xprt->sc_read_complete_q);
 	if (ctxt) {
 		list_del(&ctxt->rc_list);
 		spin_unlock(&rdma_xprt->sc_rq_dto_lock);
+#ifdef HAVE_SVC_RDMA_PCL
+#ifdef HAVE_SVC_XPRT_RECEIVED
+		svc_xprt_received(xprt);
+#endif
+		svc_rdma_read_complete(rqstp, ctxt);
+#else
 		rdma_read_complete(rqstp, ctxt);
+#endif
 		goto complete;
 	}
-#endif
 	ctxt = svc_rdma_next_recv_ctxt(&rdma_xprt->sc_rq_dto_q);
 	if (ctxt)
 		list_del(&ctxt->rc_list);
@@ -1410,29 +1386,25 @@ int svc_rdma_recvfrom(struct svc_rqst *rqstp)
 	/* Unblock the transport for the next receive */
 	svc_xprt_received(xprt);
 #endif
-
 	if (!ctxt)
 		return 0;
 
 	percpu_counter_inc(&svcrdma_stat_recv);
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 	ib_dma_sync_single_for_cpu(rdma_xprt->sc_pd->device,
 				   ctxt->rc_recv_sge.addr, ctxt->rc_byte_len,
 				   DMA_FROM_DEVICE);
-#endif
 	svc_rdma_build_arg_xdr(rqstp, ctxt);
 
 #ifndef HAVE_SVC_RDMA_PCL
 	p = (__be32 *)rqstp->rq_arg.head[0].iov_base;
 #endif
-
 	ret = svc_rdma_xdr_decode_req(&rqstp->rq_arg, ctxt);
 	if (ret < 0)
 		goto out_err;
 	if (ret == 0)
 		goto out_drop;
 #ifdef HAVE_SVC_RQST_RQ_XPRT_HLEN
- 	rqstp->rq_xprt_hlen = ret;
+	rqstp->rq_xprt_hlen = ret;
 #endif
 
 #ifdef HAVE_SVC_RDMA_PCL
@@ -1446,21 +1418,15 @@ int svc_rdma_recvfrom(struct svc_rqst *rqstp)
 
 #ifdef HAVE_SVC_RDMA_PCL
 	if (!pcl_is_empty(&ctxt->rc_read_pcl) ||
-	    !pcl_is_empty(&ctxt->rc_call_pcl)) {
-		ret = svc_rdma_process_read_list(rdma_xprt, rqstp, ctxt);
-		if (ret < 0)
-			goto out_readfail;
-	}
+	    !pcl_is_empty(&ctxt->rc_call_pcl))
+		goto out_readlist;
 #else
 	p += rpcrdma_fixed_maxsz;
 	if (*p != xdr_zero)
 		goto out_readchunk;
 #endif
 
-#ifndef HAVE_SVC_RDMA_PCL
 complete:
-#endif
-
 	rqstp->rq_xprt_ctxt = ctxt;
 	rqstp->rq_prot = IPPROTO_MAX;
 	svc_xprt_copy_addrs(rqstp, xprt);
@@ -1483,14 +1449,34 @@ out_err:
 	return 0;
 
 #ifdef HAVE_SVC_RDMA_PCL
-out_readfail:
+out_readlist:
+	/* This @rqstp is about to be recycled. Save the work
+	 * already done constructing the Call message in rq_arg
+	 * so it can be restored when the RDMA Reads have
+	 * completed.
+	 */
+	ctxt->rc_saved_arg = rqstp->rq_arg;
+
+	ret = svc_rdma_process_read_list(rdma_xprt, rqstp, ctxt);
+	if (ret < 0) {
+		if (ret == -EINVAL)
+			svc_rdma_send_error(rdma_xprt, ctxt, ret);
+		svc_rdma_recv_ctxt_put(rdma_xprt, ctxt);
+#ifdef HAVE_SVC_XPRT_DEFERRED_CLOSE
+		svc_xprt_deferred_close(xprt);
+#else
+		set_bit(XPT_CLOSE, &xprt->xpt_flags);
+#endif
+		return ret;
+	}
+	return 0;
 #else
 out_postfail:
-#endif
 	if (ret == -EINVAL)
 		svc_rdma_send_error(rdma_xprt, ctxt, ret);
 	svc_rdma_recv_ctxt_put(rdma_xprt, ctxt);
 	return ret;
+#endif
 
 out_backchannel:
 	svc_rdma_handle_bc_reply(rqstp, ctxt);
