@@ -100,7 +100,7 @@
  */
 
 #include <linux/spinlock.h>
-#include <asm/unaligned.h>
+#include <linux/unaligned.h>
 
 #include <rdma/ib_verbs.h>
 #include <rdma/rdma_cm.h>
@@ -115,28 +115,20 @@
 
 static void svc_rdma_wc_send(struct ib_cq *cq, struct ib_wc *wc);
 
-static void svc_rdma_send_cid_init(struct svcxprt_rdma *rdma,
-				   struct rpc_rdma_cid *cid)
-{
-	cid->ci_queue_id = rdma->sc_sq_cq->res.id;
-	cid->ci_completion_id = atomic_inc_return(&rdma->sc_completion_ids);
-}
-
 static struct svc_rdma_send_ctxt *
 svc_rdma_send_ctxt_alloc(struct svcxprt_rdma *rdma)
 {
+	int node = ibdev_to_node(rdma->sc_cm_id->device);
 	struct svc_rdma_send_ctxt *ctxt;
 	dma_addr_t addr;
 	void *buffer;
-	size_t size;
 	int i;
 
-	size = sizeof(*ctxt);
-	size += rdma->sc_max_send_sges * sizeof(struct ib_sge);
-	ctxt = kmalloc(size, GFP_KERNEL);
+	ctxt = kzalloc_node(struct_size(ctxt, sc_sges, rdma->sc_max_send_sges),
+			    GFP_KERNEL, node);
 	if (!ctxt)
 		goto fail0;
-	buffer = kmalloc(rdma->sc_max_req_size, GFP_KERNEL);
+	buffer = kmalloc_node(rdma->sc_max_req_size, GFP_KERNEL, node);
 	if (!buffer)
 		goto fail1;
 	addr = ib_dma_map_single(rdma->sc_pd->device, buffer,
@@ -146,6 +138,7 @@ svc_rdma_send_ctxt_alloc(struct svcxprt_rdma *rdma)
 
 	svc_rdma_send_cid_init(rdma, &ctxt->sc_cid);
 
+	ctxt->sc_rdma = rdma;
 	ctxt->sc_send_wr.next = NULL;
 	ctxt->sc_send_wr.wr_cqe = &ctxt->sc_cqe;
 	ctxt->sc_send_wr.sg_list = ctxt->sc_sges;
@@ -203,10 +196,11 @@ struct svc_rdma_send_ctxt *svc_rdma_send_ctxt_get(struct svcxprt_rdma *rdma)
 
 	spin_lock(&rdma->sc_send_lock);
 	node = llist_del_first(&rdma->sc_send_ctxts);
+	spin_unlock(&rdma->sc_send_lock);
 	if (!node)
 		goto out_empty;
+
 	ctxt = llist_entry(node, struct svc_rdma_send_ctxt, sc_node);
-	spin_unlock(&rdma->sc_send_lock);
 
 out:
 	rpcrdma_set_xdrlen(&ctxt->sc_hdrbuf, 0);
@@ -218,19 +212,67 @@ out:
 			ctxt->sc_xprt_buf);
 #endif
 
+#ifdef HAVE_SVC_RDMA_PCL
+	svc_rdma_cc_init(rdma, &ctxt->sc_reply_info.wi_cc);
+	ctxt->sc_wr_chain = &ctxt->sc_send_wr;
+#endif
 	ctxt->sc_send_wr.num_sge = 0;
 	ctxt->sc_cur_sge_no = 0;
 	ctxt->sc_page_count = 0;
+	ctxt->sc_sqecount = 1;
+
 	return ctxt;
 
 out_empty:
-	spin_unlock(&rdma->sc_send_lock);
 	ctxt = svc_rdma_send_ctxt_alloc(rdma);
 	if (!ctxt)
 		return NULL;
 	goto out;
 }
 
+#ifdef HAVE_SVC_RDMA_PCL
+static void svc_rdma_send_ctxt_release(struct svcxprt_rdma *rdma,
+				       struct svc_rdma_send_ctxt *ctxt)
+{
+	struct ib_device *device = rdma->sc_cm_id->device;
+	unsigned int i;
+
+	svc_rdma_reply_chunk_release(rdma, ctxt);
+
+#ifdef HAVE_RELEASE_PAGES_IN_MM_H
+	if (ctxt->sc_page_count)
+		release_pages(ctxt->sc_pages, ctxt->sc_page_count);
+#else
+	for (i = 0; i < ctxt->sc_page_count; ++i)
+		put_page(ctxt->sc_pages[i]);
+#endif
+
+	/* The first SGE contains the transport header, which
+	 * remains mapped until @ctxt is destroyed.
+	 */
+	for (i = 1; i < ctxt->sc_send_wr.num_sge; i++) {
+#ifdef HAVE_TRACE_RPCRDMA_H
+		trace_svcrdma_dma_unmap_page(&ctxt->sc_cid,
+					     ctxt->sc_sges[i].addr,
+					     ctxt->sc_sges[i].length);
+#endif
+		ib_dma_unmap_page(device,
+				  ctxt->sc_sges[i].addr,
+				  ctxt->sc_sges[i].length,
+				  DMA_TO_DEVICE);
+	}
+
+	llist_add(&ctxt->sc_node, &rdma->sc_send_ctxts);
+}
+
+static void svc_rdma_send_ctxt_put_async(struct work_struct *work)
+{
+	struct svc_rdma_send_ctxt *ctxt;
+
+	ctxt = container_of(work, struct svc_rdma_send_ctxt, sc_work);
+	svc_rdma_send_ctxt_release(ctxt->sc_rdma, ctxt);
+}
+
 /**
  * svc_rdma_send_ctxt_put - Return send_ctxt to free list
  * @rdma: controlling svcxprt_rdma
@@ -240,6 +282,13 @@ out_empty:
  */
 void svc_rdma_send_ctxt_put(struct svcxprt_rdma *rdma,
 			    struct svc_rdma_send_ctxt *ctxt)
+{
+	INIT_WORK(&ctxt->sc_work, svc_rdma_send_ctxt_put_async);
+	queue_work(svcrdma_wq, &ctxt->sc_work);
+}
+#else
+void svc_rdma_send_ctxt_put(struct svcxprt_rdma *rdma,
+		struct svc_rdma_send_ctxt *ctxt)
 {
 	struct ib_device *device = rdma->sc_cm_id->device;
 	unsigned int i;
@@ -257,18 +306,19 @@ void svc_rdma_send_ctxt_put(struct svcxprt_rdma *rdma,
 	 */
 	for (i = 1; i < ctxt->sc_send_wr.num_sge; i++) {
 		ib_dma_unmap_page(device,
-				  ctxt->sc_sges[i].addr,
-				  ctxt->sc_sges[i].length,
-				  DMA_TO_DEVICE);
+			ctxt->sc_sges[i].addr,
+			ctxt->sc_sges[i].length,
+			DMA_TO_DEVICE);
 #ifdef HAVE_TRACE_RPCRDMA_H
-		trace_svcrdma_dma_unmap_page(rdma,
-					     ctxt->sc_sges[i].addr,
-					     ctxt->sc_sges[i].length);
+		trace_svcrdma_dma_unmap_page(&ctxt->sc_cid,
+			ctxt->sc_sges[i].addr,
+			ctxt->sc_sges[i].length);
 #endif
 	}
 
 	llist_add(&ctxt->sc_node, &rdma->sc_send_ctxts);
 }
+#endif
 
 /**
  * svc_rdma_wake_send_waiters - manage Send Queue accounting
@@ -299,13 +349,13 @@ static void svc_rdma_wc_send(struct ib_cq *cq, struct ib_wc *wc)
 	struct svc_rdma_send_ctxt *ctxt =
 		container_of(cqe, struct svc_rdma_send_ctxt, sc_cqe);
 
-	svc_rdma_wake_send_waiters(rdma, 1);
+	svc_rdma_wake_send_waiters(rdma, ctxt->sc_sqecount);
 
 	if (unlikely(wc->status != IB_WC_SUCCESS))
 		goto flushed;
 
 #ifdef HAVE_TRACE_RPCRDMA_H
-	trace_svcrdma_wc_send(wc, &ctxt->sc_cid);
+	trace_svcrdma_wc_send(&ctxt->sc_cid);
 #endif
 	svc_rdma_send_ctxt_put(rdma, ctxt);
 	return;
@@ -326,6 +376,94 @@ flushed:
 #endif
 }
 
+#ifdef HAVE_SVC_RDMA_PCL
+/**
+ * svc_rdma_post_send - Post a WR chain to the Send Queue
+ * @rdma: transport context
+ * @ctxt: WR chain to post
+ *
+ * Copy fields in @ctxt to stack variables in order to guarantee
+ * that these values remain available after the ib_post_send() call.
+ * In some error flow cases, svc_rdma_wc_send() releases @ctxt.
+ *
+ * Note there is potential for starvation when the Send Queue is
+ * full because there is no order to when waiting threads are
+ * awoken. The transport is typically provisioned with a deep
+ * enough Send Queue that SQ exhaustion should be a rare event.
+ *
+ * Return values:
+ *   %0: @ctxt's WR chain was posted successfully
+ *   %-ENOTCONN: The connection was lost
+ */
+int svc_rdma_post_send(struct svcxprt_rdma *rdma,
+		       struct svc_rdma_send_ctxt *ctxt)
+{
+	struct ib_send_wr *first_wr = ctxt->sc_wr_chain;
+	struct ib_send_wr *send_wr = &ctxt->sc_send_wr;
+	const struct ib_send_wr *bad_wr = first_wr;
+#ifdef HAVE_TRACE_RPCRDMA_H
+	struct rpc_rdma_cid cid = ctxt->sc_cid;
+#endif
+	int ret, sqecount = ctxt->sc_sqecount;
+
+	might_sleep();
+
+	/* Sync the transport header buffer */
+	ib_dma_sync_single_for_device(rdma->sc_pd->device,
+				      send_wr->sg_list[0].addr,
+				      send_wr->sg_list[0].length,
+				      DMA_TO_DEVICE);
+
+	/* If the SQ is full, wait until an SQ entry is available */
+	while (!test_bit(XPT_CLOSE, &rdma->sc_xprt.xpt_flags)) {
+		if (atomic_sub_return(sqecount, &rdma->sc_sq_avail) < 0) {
+			svc_rdma_wake_send_waiters(rdma, sqecount);
+
+			/* When the transport is torn down, assume
+			 * ib_drain_sq() will trigger enough Send
+			 * completions to wake us. The XPT_CLOSE test
+			 * above should then cause the while loop to
+			 * exit.
+			 */
+			percpu_counter_inc(&svcrdma_stat_sq_starve);
+#ifdef HAVE_TRACE_RPCRDMA_H
+			trace_svcrdma_sq_full(rdma, &cid);
+#endif
+			wait_event(rdma->sc_send_wait,
+				   atomic_read(&rdma->sc_sq_avail) > 0);
+#ifdef HAVE_TRACE_RPCRDMA_H
+			trace_svcrdma_sq_retry(rdma, &cid);
+#endif
+			continue;
+		}
+
+#ifdef HAVE_TRACE_RPCRDMA_H
+		trace_svcrdma_post_send(ctxt);
+#endif
+		ret = ib_post_send(rdma->sc_qp, first_wr, &bad_wr);
+		if (ret) {
+#ifdef HAVE_TRACE_RPCRDMA_H
+			trace_svcrdma_sq_post_err(rdma, &cid, ret);
+#endif
+#ifdef HAVE_SVC_XPRT_DEFERRED_CLOSE
+			svc_xprt_deferred_close(&rdma->sc_xprt);
+
+			/* If even one WR was posted, there will be a
+			 * Send completion that bumps sc_sq_avail.
+			 */
+			if (bad_wr == first_wr) {
+				svc_rdma_wake_send_waiters(rdma, sqecount);
+				break;
+			}
+#else
+			set_bit(XPT_CLOSE, &rdma->sc_xprt.xpt_flags);
+#endif
+		}
+		return 0;
+	}
+	return -ENOTCONN;
+}
+#else
 /**
  * svc_rdma_send - Post a single Send WR
  * @rdma: transport on which to post the WR
@@ -339,26 +477,28 @@ int svc_rdma_send(struct svcxprt_rdma *rdma, struct svc_rdma_send_ctxt *ctxt)
 	struct ib_send_wr *wr = &ctxt->sc_send_wr;
 	int ret;
 
+	might_sleep();
+
 	/* Sync the transport header buffer */
 	ib_dma_sync_single_for_device(rdma->sc_pd->device,
-				      wr->sg_list[0].addr,
-				      wr->sg_list[0].length,
-				      DMA_TO_DEVICE);
+			wr->sg_list[0].addr,
+			wr->sg_list[0].length,
+			DMA_TO_DEVICE);
 
 	/* If the SQ is full, wait until an SQ entry is available */
 	while (1) {
 		if ((atomic_dec_return(&rdma->sc_sq_avail) < 0)) {
 			percpu_counter_inc(&svcrdma_stat_sq_starve);
 #ifdef HAVE_TRACE_RPCRDMA_H
-			trace_svcrdma_sq_full(rdma);
+			trace_svcrdma_sq_full(rdma, &ctxt->sc_cid);
 #endif
 			atomic_inc(&rdma->sc_sq_avail);
 			wait_event(rdma->sc_send_wait,
-				   atomic_read(&rdma->sc_sq_avail) > 1);
+			atomic_read(&rdma->sc_sq_avail) > 1);
 			if (test_bit(XPT_CLOSE, &rdma->sc_xprt.xpt_flags))
 				return -ENOTCONN;
 #ifdef HAVE_TRACE_RPCRDMA_H
-			trace_svcrdma_sq_retry(rdma);
+			trace_svcrdma_sq_retry(rdma, &ctxt->sc_cid);
 #endif
 			continue;
 		}
@@ -373,7 +513,7 @@ int svc_rdma_send(struct svcxprt_rdma *rdma, struct svc_rdma_send_ctxt *ctxt)
 	}
 
 #ifdef HAVE_TRACE_RPCRDMA_H
-	trace_svcrdma_sq_post_err(rdma, ret);
+	trace_svcrdma_sq_post_err(rdma, &ctxt->sc_cid, ret);
 #endif
 #ifdef HAVE_SVC_XPRT_DEFERRED_CLOSE
 	svc_xprt_deferred_close(&rdma->sc_xprt);
@@ -383,6 +523,7 @@ int svc_rdma_send(struct svcxprt_rdma *rdma, struct svc_rdma_send_ctxt *ctxt)
 #endif
 	return ret;
 }
+#endif
 
 /**
  * svc_rdma_encode_read_list - Encode RPC Reply's Read chunk list
@@ -399,7 +540,6 @@ static ssize_t svc_rdma_encode_read_list(struct svc_rdma_send_ctxt *sctxt)
 	return xdr_stream_encode_item_absent(&sctxt->sc_stream);
 }
 
-
 #ifdef HAVE_SVC_RDMA_PCL
 /**
  * svc_rdma_encode_write_segment - Encode one Write segment
@@ -542,6 +682,7 @@ static ssize_t svc_rdma_encode_write_chunk(__be32 *src,
 }
 #endif
 
+#ifdef HAVE_SVC_RDMA_PCL
 /**
  * svc_rdma_encode_write_list - Encode RPC Reply's Write chunk list
  * @rctxt: Reply context with information about the RPC Call
@@ -552,23 +693,12 @@ static ssize_t svc_rdma_encode_write_chunk(__be32 *src,
  *   that was consumed by the Reply's Write list
  *   %-EMSGSIZE on XDR buffer overflow
  */
-#ifdef HAVE_SVC_RDMA_PCL
 static ssize_t svc_rdma_encode_write_list(struct svc_rdma_recv_ctxt *rctxt,
 					  struct svc_rdma_send_ctxt *sctxt)
-
 {
 	struct svc_rdma_chunk *chunk;
 	ssize_t len, ret;
-#else
-static ssize_t
-svc_rdma_encode_write_list(const struct svc_rdma_recv_ctxt *rctxt,
-			   struct svc_rdma_send_ctxt *sctxt,
-			   unsigned int length)
-{
-	ssize_t len, ret;
-#endif
 
-#ifdef HAVE_SVC_RDMA_PCL
 	len = 0;
 	pcl_for_each_chunk(chunk, &rctxt->rc_write_pcl) {
 		ret = svc_rdma_encode_write_chunk(sctxt, chunk);
@@ -576,12 +706,6 @@ svc_rdma_encode_write_list(const struct svc_rdma_recv_ctxt *rctxt,
 			return ret;
 		len += ret;
 	}
-#else
-	ret = svc_rdma_encode_write_chunk(rctxt->rc_write_list, sctxt, length);
-	if (ret < 0)
-		return ret;
-	len = ret;
-#endif
 
 	/* Terminate the Write list */
 	ret = xdr_stream_encode_item_absent(&sctxt->sc_stream);
@@ -590,6 +714,27 @@ svc_rdma_encode_write_list(const struct svc_rdma_recv_ctxt *rctxt,
 
 	return len + ret;
 }
+#else
+static ssize_t
+svc_rdma_encode_write_list(const struct svc_rdma_recv_ctxt *rctxt,
+			   struct svc_rdma_send_ctxt *sctxt,
+			   unsigned int length)
+{
+	ssize_t len, ret;
+
+	ret = svc_rdma_encode_write_chunk(rctxt->rc_write_list, sctxt, length);
+	if (ret < 0)
+		return ret;
+	len = ret;
+
+	/* Terminate the Write list */
+	ret = xdr_stream_encode_item_absent(&sctxt->sc_stream);
+	if (ret < 0)
+		return ret;
+
+	return len + ret;
+}
+#endif
 
 /**
  * svc_rdma_encode_reply_chunk - Encode RPC Reply's Reply chunk
@@ -655,29 +800,17 @@ static int svc_rdma_page_dma_map(void *data, struct page *page,
 	struct svc_rdma_map_data *args = data;
 	struct svcxprt_rdma *rdma = args->md_rdma;
 	struct svc_rdma_send_ctxt *ctxt = args->md_ctxt;
-#else
-static int svc_rdma_dma_map_page(struct svcxprt_rdma *rdma,
-				 struct svc_rdma_send_ctxt *ctxt,
-				 struct page *page,
-				 unsigned long offset,
-				 unsigned int len)
-{
-#endif
 	struct ib_device *dev = rdma->sc_cm_id->device;
 	dma_addr_t dma_addr;
 
-#ifdef HAVE_SVC_RDMA_PCL
 	++ctxt->sc_cur_sge_no;
-#endif
 
 	dma_addr = ib_dma_map_page(dev, page, offset, len, DMA_TO_DEVICE);
 	if (ib_dma_mapping_error(dev, dma_addr))
 		goto out_maperr;
 
 #ifdef HAVE_TRACE_RPCRDMA_H
-#ifdef HAVE_SVC_RDMA_PCL
-	trace_svcrdma_dma_map_page(rdma, dma_addr, len);
-#endif
+	trace_svcrdma_dma_map_page(&ctxt->sc_cid, dma_addr, len);
 #endif
 	ctxt->sc_sges[ctxt->sc_cur_sge_no].addr = dma_addr;
 	ctxt->sc_sges[ctxt->sc_cur_sge_no].length = len;
@@ -686,12 +819,33 @@ static int svc_rdma_dma_map_page(struct svcxprt_rdma *rdma,
 
 out_maperr:
 #ifdef HAVE_TRACE_RPCRDMA_H
-#ifdef HAVE_SVC_RDMA_PCL
-	trace_svcrdma_dma_map_err(rdma, dma_addr, len);
-#endif
+	trace_svcrdma_dma_map_err(&ctxt->sc_cid, dma_addr, len);
 #endif
 	return -EIO;
 }
+#else
+static int svc_rdma_dma_map_page(struct svcxprt_rdma *rdma,
+				 struct svc_rdma_send_ctxt *ctxt,
+				 struct page *page,
+				 unsigned long offset,
+				 unsigned int len)
+{
+	struct ib_device *dev = rdma->sc_cm_id->device;
+	dma_addr_t dma_addr;
+
+	dma_addr = ib_dma_map_page(dev, page, offset, len, DMA_TO_DEVICE);
+	if (ib_dma_mapping_error(dev, dma_addr))
+		goto out_maperr;
+
+	ctxt->sc_sges[ctxt->sc_cur_sge_no].addr = dma_addr;
+	ctxt->sc_sges[ctxt->sc_cur_sge_no].length = len;
+	ctxt->sc_send_wr.num_sge++;
+	return 0;
+
+out_maperr:
+	return -EIO;
+}
+#endif
 
 #ifdef HAVE_SVC_RDMA_PCL
 /**
@@ -811,14 +965,12 @@ static int svc_rdma_xb_count_sges(const struct xdr_buf *xdr,
 	args->pd_length += xdr->len;
 	return 0;
 }
-#endif
 
-#ifdef HAVE_SVC_RDMA_PCL
 /**
  * svc_rdma_pull_up_needed - Determine whether to use pull-up
  * @rdma: controlling transport
  * @sctxt: send_ctxt for the Send WR
- * @rctxt: Write and Reply chunks provided by client
+ * @write_pcl: Write chunk list provided by client
  * @xdr: xdr_buf containing RPC message to transmit
  *
  * Returns:
@@ -827,7 +979,7 @@ static int svc_rdma_xb_count_sges(const struct xdr_buf *xdr,
  */
 static bool svc_rdma_pull_up_needed(const struct svcxprt_rdma *rdma,
 				    const struct svc_rdma_send_ctxt *sctxt,
-				    const struct svc_rdma_recv_ctxt *rctxt,
+				    const struct svc_rdma_pcl *write_pcl,
 				    const struct xdr_buf *xdr)
 {
 	/* Resources needed for the transport header */
@@ -837,7 +989,7 @@ static bool svc_rdma_pull_up_needed(const struct svcxprt_rdma *rdma,
 	};
 	int ret;
 
-	ret = pcl_process_nonpayloads(&rctxt->rc_write_pcl, xdr,
+	ret = pcl_process_nonpayloads(write_pcl, xdr,
 				      svc_rdma_xb_count_sges, &args);
 	if (ret < 0)
 		return false;
@@ -893,7 +1045,7 @@ static int svc_rdma_xb_linearize(const struct xdr_buf *xdr,
  * svc_rdma_pull_up_reply_msg - Copy Reply into a single buffer
  * @rdma: controlling transport
  * @sctxt: send_ctxt for the Send WR; xprt hdr is already prepared
- * @rctxt: Write and Reply chunks provided by client
+ * @write_pcl: Write chunk list provided by client
  * @xdr: prepared xdr_buf containing RPC message
  *
  * The device is not capable of sending the reply directly.
@@ -908,7 +1060,7 @@ static int svc_rdma_xb_linearize(const struct xdr_buf *xdr,
  */
 static int svc_rdma_pull_up_reply_msg(const struct svcxprt_rdma *rdma,
 				      struct svc_rdma_send_ctxt *sctxt,
-				      const struct svc_rdma_recv_ctxt *rctxt,
+				      const struct svc_rdma_pcl *write_pcl,
 				      const struct xdr_buf *xdr)
 {
 	struct svc_rdma_pullup_data args = {
@@ -916,7 +1068,7 @@ static int svc_rdma_pull_up_reply_msg(const struct svcxprt_rdma *rdma,
 	};
 	int ret;
 
-	ret = pcl_process_nonpayloads(&rctxt->rc_write_pcl, xdr,
+	ret = pcl_process_nonpayloads(write_pcl, xdr,
 				      svc_rdma_xb_linearize, &args);
 	if (ret < 0)
 		return ret;
@@ -1045,10 +1197,12 @@ static int svc_rdma_pull_up_reply_msg(struct svcxprt_rdma *rdma,
 }
 #endif
 
+#ifdef HAVE_SVC_RDMA_PCL
 /* svc_rdma_map_reply_msg - DMA map the buffer holding RPC message
  * @rdma: controlling transport
  * @sctxt: send_ctxt for the Send WR
- * @rctxt: Write and Reply chunks provided by client
+ * @write_pcl: Write chunk list provided by client
+ * @reply_pcl: Reply chunk provided by client
  * @xdr: prepared xdr_buf containing RPC message
  *
  * Returns:
@@ -1060,26 +1214,46 @@ static int svc_rdma_pull_up_reply_msg(struct svcxprt_rdma *rdma,
  */
 int svc_rdma_map_reply_msg(struct svcxprt_rdma *rdma,
 			   struct svc_rdma_send_ctxt *sctxt,
-			   const struct svc_rdma_recv_ctxt *rctxt,
-#ifdef HAVE_SVC_RDMA_PCL
+			   const struct svc_rdma_pcl *write_pcl,
+			   const struct svc_rdma_pcl *reply_pcl,
 			   const struct xdr_buf *xdr)
-#else
-			   struct xdr_buf *xdr)
-#endif
 {
-#ifdef HAVE_SVC_RDMA_PCL
 	struct svc_rdma_map_data args = {
 		.md_rdma	= rdma,
 		.md_ctxt	= sctxt,
 	};
+
+	/* Set up the (persistently-mapped) transport header SGE. */
+	sctxt->sc_send_wr.num_sge = 1;
+	sctxt->sc_sges[0].length = sctxt->sc_hdrbuf.len;
+
+	/* If there is a Reply chunk, nothing follows the transport
+	 * header, so there is nothing to map.
+	 */
+	if (!pcl_is_empty(reply_pcl))
+		return 0;
+
+	/* For pull-up, svc_rdma_send() will sync the transport header.
+	 * No additional DMA mapping is necessary.
+	 */
+	if (svc_rdma_pull_up_needed(rdma, sctxt, write_pcl, xdr))
+		return svc_rdma_pull_up_reply_msg(rdma, sctxt, write_pcl, xdr);
+
+	return pcl_process_nonpayloads(write_pcl, xdr,
+				       svc_rdma_xb_dma_map, &args);
+}
 #else
+int svc_rdma_map_reply_msg(struct svcxprt_rdma *rdma,
+			   struct svc_rdma_send_ctxt *sctxt,
+			   const struct svc_rdma_recv_ctxt *rctxt,
+			   struct xdr_buf *xdr)
+{
 	unsigned int len, remaining;
 	unsigned long page_off;
 	struct page **ppages;
 	unsigned char *base;
 	u32 xdr_pad;
 	int ret;
-#endif
 
 	/* Set up the (persistently-mapped) transport header SGE. */
 	sctxt->sc_send_wr.num_sge = 1;
@@ -1088,11 +1262,7 @@ int svc_rdma_map_reply_msg(struct svcxprt_rdma *rdma,
 	/* If there is a Reply chunk, nothing follows the transport
 	 * header, and we're done here.
 	 */
-#ifdef HAVE_SVC_RDMA_PCL
-	if (!pcl_is_empty(&rctxt->rc_reply_pcl))
-#else
 	if (rctxt && rctxt->rc_reply_chunk)
-#endif
 		return 0;
 
 	/* For pull-up, svc_rdma_send() will sync the transport header.
@@ -1101,10 +1271,6 @@ int svc_rdma_map_reply_msg(struct svcxprt_rdma *rdma,
 	if (svc_rdma_pull_up_needed(rdma, sctxt, rctxt, xdr))
 		return svc_rdma_pull_up_reply_msg(rdma, sctxt, rctxt, xdr);
 
-#ifdef HAVE_SVC_RDMA_PCL
-	return pcl_process_nonpayloads(&rctxt->rc_write_pcl, xdr,
-				       svc_rdma_xb_dma_map, &args);
-#else
 	++sctxt->sc_cur_sge_no;
 	ret = svc_rdma_dma_map_buf(rdma, sctxt,
 				   xdr->head[0].iov_base,
@@ -1157,8 +1323,8 @@ tail:
 	}
 
 	return 0;
-#endif
 }
+#endif
 
 /* The svc_rqst and all resources it owns are released as soon as
  * svc_rdma_sendto returns. Transfer pages under I/O to the ctxt
@@ -1184,16 +1350,10 @@ static void svc_rdma_save_io_pages(struct svc_rqst *rqstp,
  * in sc_sges[0], and the RPC xdr_buf is prepared in following sges.
  *
  * Depending on whether a Write list or Reply chunk is present,
- * the server may send all, a portion of, or none of the xdr_buf.
+ * the server may Send all, a portion of, or none of the xdr_buf.
  * In the latter case, only the transport header (sc_sges[0]) is
  * transmitted.
  *
- * RDMA Send is the last step of transmitting an RPC reply. Pages
- * involved in the earlier RDMA Writes are here transferred out
- * of the rqstp and into the sctxt's page array. These pages are
- * DMA unmapped by each Write completion, but the subsequent Send
- * completion finally releases these pages.
- *
  * Assumptions:
  * - The Reply's transport header will never be larger than a page.
  */
@@ -1202,22 +1362,35 @@ static int svc_rdma_send_reply_msg(struct svcxprt_rdma *rdma,
 				   const struct svc_rdma_recv_ctxt *rctxt,
 				   struct svc_rqst *rqstp)
 {
+	struct ib_send_wr *send_wr = &sctxt->sc_send_wr;
 	int ret;
 
+#ifdef HAVE_SVC_RDMA_PCL
+	ret = svc_rdma_map_reply_msg(rdma, sctxt, &rctxt->rc_write_pcl,
+				     &rctxt->rc_reply_pcl, &rqstp->rq_res);
+#else
 	ret = svc_rdma_map_reply_msg(rdma, sctxt, rctxt, &rqstp->rq_res);
+#endif
 	if (ret < 0)
 		return ret;
 
+	/* Transfer pages involved in RDMA Writes to the sctxt's
+	 * page array. Completion handling releases these pages.
+	 */
 	svc_rdma_save_io_pages(rqstp, sctxt);
 
 	if (rctxt->rc_inv_rkey) {
-		sctxt->sc_send_wr.opcode = IB_WR_SEND_WITH_INV;
-		sctxt->sc_send_wr.ex.invalidate_rkey = rctxt->rc_inv_rkey;
+		send_wr->opcode = IB_WR_SEND_WITH_INV;
+		send_wr->ex.invalidate_rkey = rctxt->rc_inv_rkey;
 	} else {
-		sctxt->sc_send_wr.opcode = IB_WR_SEND;
+		send_wr->opcode = IB_WR_SEND;
 	}
 
+#ifdef HAVE_SVC_RDMA_PCL
+	return svc_rdma_post_send(rdma, sctxt);
+#else
 	return svc_rdma_send(rdma, sctxt);
+#endif
 }
 
 /**
@@ -1240,13 +1413,7 @@ void svc_rdma_send_error_msg(struct svcxprt_rdma *rdma,
 			     struct svc_rdma_recv_ctxt *rctxt,
 			     int status)
 {
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 	__be32 *rdma_argp = rctxt->rc_recv_buf;
-#else
-	struct svc_rqst *rqstp =
-			container_of((void *)rctxt, struct svc_rqst, rq_xprt_ctxt);
-	__be32 *rdma_argp = page_address(rqstp->rq_pages[0]);
-#endif
 	__be32 *p;
 
 	rpcrdma_set_xdrlen(&sctxt->sc_hdrbuf, 0);
@@ -1296,9 +1463,12 @@ void svc_rdma_send_error_msg(struct svcxprt_rdma *rdma,
 	sctxt->sc_send_wr.num_sge = 1;
 	sctxt->sc_send_wr.opcode = IB_WR_SEND;
 	sctxt->sc_sges[0].length = sctxt->sc_hdrbuf.len;
+#ifdef HAVE_SVC_RDMA_PCL
+	if (svc_rdma_post_send(rdma, sctxt))
+#else
 	if (svc_rdma_send(rdma, sctxt))
+#endif
 		goto put_ctxt;
-
 	return;
 
 put_ctxt:
@@ -1323,18 +1493,15 @@ int svc_rdma_sendto(struct svc_rqst *rqstp)
 	struct svcxprt_rdma *rdma =
 		container_of(xprt, struct svcxprt_rdma, sc_xprt);
 	struct svc_rdma_recv_ctxt *rctxt = rqstp->rq_xprt_ctxt;
-#ifdef HAVE_SVC_FILL_WRITE_VECTOR
 	__be32 *rdma_argp = rctxt->rc_recv_buf;
+	struct svc_rdma_send_ctxt *sctxt;
+#ifdef HAVE_SVC_RDMA_PCL
+	unsigned int rc_size;
 #else
-	__be32 *rdma_argp = page_address(rqstp->rq_pages[0]);
-#endif
-#ifndef HAVE_SVC_RDMA_PCL
 	__be32 *wr_lst = rctxt->rc_write_list;
 	__be32 *rp_ch = rctxt->rc_reply_chunk;
 	struct xdr_buf *xdr = &rqstp->rq_res;
 #endif
-	struct svc_rdma_send_ctxt *sctxt;
-	unsigned int rc_size;
 	__be32 *p;
 	int ret;
 
@@ -1356,11 +1523,20 @@ int svc_rdma_sendto(struct svc_rqst *rqstp)
 		goto put_ctxt;
 
 #ifdef HAVE_SVC_RDMA_PCL
-	ret = svc_rdma_send_reply_chunk(rdma, rctxt, &rqstp->rq_res);
+	ret = svc_rdma_send_write_list(rdma, rctxt, &rqstp->rq_res);
 	if (ret < 0)
-		goto reply_chunk;
+		goto put_ctxt;
+
+	rc_size = 0;
+	if (!pcl_is_empty(&rctxt->rc_reply_pcl)) {
+		ret = svc_rdma_prepare_reply_chunk(rdma, &rctxt->rc_write_pcl,
+						   &rctxt->rc_reply_pcl, sctxt,
+						   &rqstp->rq_res);
+		if (ret < 0)
+			goto reply_chunk;
+		rc_size = ret;
+	}
 #endif
-	rc_size = ret;
 
 	*p++ = *rdma_argp;
 	*p++ = *(rdma_argp + 1);
@@ -1415,10 +1591,10 @@ int svc_rdma_sendto(struct svc_rqst *rqstp)
 			goto put_ctxt;
 	}
 #endif
+
 	ret = svc_rdma_send_reply_msg(rdma, sctxt, rctxt, rqstp);
 	if (ret < 0)
 		goto put_ctxt;
-
 #ifdef HAVE_SVC_RDMA_RELEASE_RQST
 	return 0;
 #else
@@ -1466,21 +1642,6 @@ drop_connection:
 #endif
 }
 
-/**
- * svc_rdma_result_payload - special processing for a result payload
- * @rqstp: svc_rqst to operate on
- * @offset: payload's byte offset in @xdr
- * @length: size of payload, in bytes
- *
- * Return values:
- *   %0 if successful or nothing needed to be done
- *   %-EMSGSIZE on XDR buffer overflow
- *   %-E2BIG if the payload was larger than the Write chunk
- *   %-EINVAL if client provided too many segments
- *   %-ENOMEM if rdma_rw context pool was exhausted
- *   %-ENOTCONN if posting failed (connection is lost)
- *   %-EIO if rdma_rw initialization failed (DMA mapping, etc)
- */
 #ifdef HAVE_XPO_READ_PAYLOAD
 int svc_rdma_read_payload(struct svc_rqst *rqstp, unsigned int offset,
 			    unsigned int length)
@@ -1498,6 +1659,20 @@ int svc_rdma_read_payload(struct svc_rqst *rqstp, unsigned int offset,
 }
 #endif
 
+/**
+ * svc_rdma_result_payload - special processing for a result payload
+ * @rqstp: RPC transaction context
+ * @offset: payload's byte offset in @rqstp->rq_res
+ * @length: size of payload, in bytes
+ *
+ * Assign the passed-in result payload to the current Write chunk,
+ * and advance to cur_result_payload to the next Write chunk, if
+ * there is one.
+ *
+ * Return values:
+ *   %0 if successful or nothing needed to be done
+ *   %-E2BIG if the payload was larger than the Write chunk
+ */
 #ifdef HAVE_XPO_RESULT_PAYLOAD
 #ifdef HAVE_SVC_RDMA_PCL
 int svc_rdma_result_payload(struct svc_rqst *rqstp, unsigned int offset,
@@ -1505,29 +1680,17 @@ int svc_rdma_result_payload(struct svc_rqst *rqstp, unsigned int offset,
 {
 	struct svc_rdma_recv_ctxt *rctxt = rqstp->rq_xprt_ctxt;
 	struct svc_rdma_chunk *chunk;
-	struct svcxprt_rdma *rdma;
-	struct xdr_buf subbuf;
-	int ret;
 
 	chunk = rctxt->rc_cur_result_payload;
 	if (!length || !chunk)
 		return 0;
 	rctxt->rc_cur_result_payload =
 		pcl_next_chunk(&rctxt->rc_write_pcl, chunk);
+
 	if (length > chunk->ch_length)
 		return -E2BIG;
-
 	chunk->ch_position = offset;
 	chunk->ch_payload_length = length;
-
-	if (xdr_buf_subsegment(&rqstp->rq_res, &subbuf, offset, length))
-		return -EMSGSIZE;
-
-	rdma = container_of(rqstp->rq_xprt, struct svcxprt_rdma, sc_xprt);
-	ret = svc_rdma_send_write_chunk(rdma, chunk, &subbuf);
-	if (ret < 0)
-		return ret;
-
 	return 0;
 }
 #else
