@@ -216,289 +216,6 @@ TEST_F(tcp_sockopt, ti_2_tcp_congestion)
     }
 }
 
-/**
- * @test tcp_sockopt.ti_1_getsockopt_tcp_info
- * @brief
- *    getsockopt(TCP_INFO).
- * @details
- */
-TEST_F(tcp_sockopt, ti_3_setsockopt_isolate)
-{
-    struct xlio_api_t *xlio_api = xlio_get_api();
-    pid_t pid;
-
-    SKIP_TRUE(xlio_api != NULL, "XLIO API not found. Run the test under XLIO.");
-    SKIP_TRUE(server_addr.addr.sa_family == AF_INET && client_addr.addr.sa_family == AF_INET,
-              "This test supports only IPv4");
-
-    auto compare_rings_ne = [&](int *arr1, int arr1_nr, int *arr2, int arr2_nr) {
-        // Whether arr1 and arr2 don't overlap (contain different rings)
-        for (int i = 0; i < arr1_nr; ++i) {
-            for (int j = 0; j < arr2_nr; ++j) {
-                ASSERT_NE(arr1[i], arr2[j]);
-            }
-        }
-    };
-    auto compare_rings_contains = [&](int *arr1, int arr1_nr, int *arr2, int arr2_nr) {
-        // Whether arr1 contains all arr2
-        for (int i = 0; i < arr2_nr; ++i) {
-            bool contains = false;
-            for (int j = 0; j < arr1_nr; ++j) {
-                contains = contains || (arr2[i] == arr1[j]);
-            }
-            ASSERT_TRUE(contains);
-        }
-    };
-
-    auto test_client = [&]() {
-        char buf[64];
-        sockaddr_store_t addr;
-        sockaddr_store_t local_addr;
-        ssize_t len;
-        int sock;
-        int sock2;
-        int val = SO_XLIO_ISOLATE_SAFE;
-        int valdef = SO_XLIO_ISOLATE_DEFAULT;
-        int rc;
-
-        sock = tcp_base::sock_create();
-        ASSERT_LE(0, sock);
-        sock2 = tcp_base::sock_create();
-        ASSERT_LE(0, sock2);
-
-        rc = setsockopt(sock, SOL_SOCKET, SO_XLIO_ISOLATE, &val, sizeof(val));
-        ASSERT_EQ(0, rc);
-        rc = setsockopt(sock, SOL_SOCKET, SO_XLIO_ISOLATE, &valdef, sizeof(valdef));
-        ASSERT_EQ(-1, rc);
-        ASSERT_EQ(EINVAL, errno);
-
-        memcpy(&local_addr, &client_addr, sizeof(local_addr));
-        sys_set_port((struct sockaddr *)&local_addr, 0);
-        rc = bind(sock, (struct sockaddr *)&local_addr, sizeof(local_addr));
-        ASSERT_EQ(0, rc);
-        rc = bind(sock2, (struct sockaddr *)&local_addr, sizeof(local_addr));
-        ASSERT_EQ(0, rc);
-
-        memcpy(&addr, &server_addr, sizeof(addr));
-        sys_set_port((struct sockaddr *)&addr, 8080);
-        rc = connect(sock, (struct sockaddr *)&addr, sizeof(addr));
-        ASSERT_EQ(0, rc);
-
-        sys_set_port((struct sockaddr *)&addr, 8081);
-        rc = connect(sock2, (struct sockaddr *)&addr, sizeof(addr));
-        ASSERT_EQ(0, rc);
-
-        rc = setsockopt(sock2, SOL_SOCKET, SO_XLIO_ISOLATE, &val, sizeof(val));
-        ASSERT_EQ(-1, rc);
-        ASSERT_EQ(EINVAL, errno);
-
-        int ring_fds[3];
-        int ring_fds2[3];
-        int ring_fds_nr;
-        int ring_fds2_nr;
-        rc = xlio_api->get_socket_rings_num(sock);
-        ASSERT_LT(0, rc);
-        ASSERT_GE((int)ARRAY_SIZE(ring_fds), rc);
-        rc = xlio_api->get_socket_rings_num(sock2);
-        ASSERT_LT(0, rc);
-        ASSERT_GE((int)ARRAY_SIZE(ring_fds2), rc);
-        ring_fds_nr = xlio_api->get_socket_rings_fds(sock, ring_fds, ARRAY_SIZE(ring_fds));
-        ring_fds2_nr = xlio_api->get_socket_rings_fds(sock2, ring_fds2, ARRAY_SIZE(ring_fds2));
-        compare_rings_ne(ring_fds, ring_fds_nr, ring_fds2, ring_fds2_nr);
-        ASSERT_TRUE(!testing::Test::HasFailure());
-
-        len = write(sock, HELLO_STR, sizeof(HELLO_STR));
-        ASSERT_LT(0, len);
-        EXPECT_EQ(static_cast<ssize_t>(sizeof(HELLO_STR)), len);
-
-        do {
-            len = read(sock, buf, sizeof(buf));
-        } while (len == -1 && errno == EINTR);
-        ASSERT_LT(0, len);
-        EXPECT_LE(static_cast<ssize_t>(sizeof(HELLO_STR)), len);
-        EXPECT_EQ(0, strncmp(HELLO_STR, buf, sizeof(HELLO_STR)));
-
-        usleep(100);
-
-        rc = close(sock);
-        ASSERT_EQ(0, rc);
-        rc = close(sock2);
-        ASSERT_EQ(0, rc);
-    };
-
-    auto test_server = [&]() {
-        char buf[64];
-        sockaddr_store_t addr;
-        sockaddr_store_t peer_addr;
-        socklen_t socklen;
-        ssize_t len;
-        int sock;
-        int sock2;
-        int sock3;
-        int sock_in;
-        int sock_in2;
-        int val = SO_XLIO_ISOLATE_SAFE;
-        int rc;
-
-        /*
-         * Socket create
-         */
-
-        sock = tcp_base::sock_create();
-        ASSERT_LE(0, sock);
-        sock2 = tcp_base::sock_create();
-        ASSERT_LE(0, sock2);
-        sock3 = tcp_base::sock_create();
-        ASSERT_LE(0, sock3);
-
-        rc = setsockopt(sock, SOL_SOCKET, SO_XLIO_ISOLATE, &val, sizeof(val));
-        ASSERT_EQ(0, rc);
-
-        /*
-         * Socket bind
-         */
-
-        memcpy(&addr, &server_addr, sizeof(addr));
-        sys_set_port((struct sockaddr *)&addr, 8080);
-        rc = bind(sock, (struct sockaddr *)&addr, sizeof(addr));
-        ASSERT_EQ(0, rc);
-
-        sys_set_port((struct sockaddr *)&addr, 8081);
-        rc = bind(sock2, (struct sockaddr *)&addr, sizeof(addr));
-        ASSERT_EQ(0, rc);
-
-        sys_set_port((struct sockaddr *)&addr, 8082);
-        rc = bind(sock3, (struct sockaddr *)&addr, sizeof(addr));
-        ASSERT_EQ(0, rc);
-
-        rc = setsockopt(sock2, SOL_SOCKET, SO_XLIO_ISOLATE, &val, sizeof(val));
-        ASSERT_EQ(0, rc);
-
-        /*
-         * Socket listen
-         */
-
-        rc = listen(sock, 5);
-        ASSERT_EQ(0, rc);
-
-        rc = listen(sock2, 5);
-        ASSERT_EQ(0, rc);
-
-        rc = listen(sock3, 5);
-        ASSERT_EQ(0, rc);
-
-        rc = setsockopt(sock3, SOL_SOCKET, SO_XLIO_ISOLATE, &val, sizeof(val));
-        ASSERT_EQ(-1, rc);
-        ASSERT_EQ(EINVAL, errno);
-
-        /*
-         * Check rings for listen sockets
-         */
-
-        int ring_fds[3];
-        int ring_fds2[3];
-        int ring_fds3[3];
-        int ring_fds_nr;
-        int ring_fds2_nr;
-        int ring_fds3_nr;
-        ring_fds_nr = xlio_api->get_socket_rings_fds(sock, ring_fds, ARRAY_SIZE(ring_fds));
-        ASSERT_EQ(1, ring_fds_nr);
-        ring_fds2_nr = xlio_api->get_socket_rings_fds(sock2, ring_fds2, ARRAY_SIZE(ring_fds2));
-        ASSERT_EQ(1, ring_fds2_nr);
-        ring_fds3_nr = xlio_api->get_socket_rings_fds(sock3, ring_fds3, ARRAY_SIZE(ring_fds3));
-        ASSERT_EQ(1, ring_fds3_nr);
-        ASSERT_EQ(ring_fds[0], ring_fds2[0]);
-        ASSERT_NE(ring_fds[0], ring_fds3[0]);
-
-        // Notify client to proceed with connect()
-        barrier_fork(pid);
-
-        /*
-         * Socket accept
-         */
-
-        do {
-            socklen = sizeof(peer_addr);
-            sock_in = accept(sock, (struct sockaddr *)&peer_addr, &socklen);
-        } while (sock_in == -1 && errno == EINTR);
-        ASSERT_LE(0, sock_in);
-        log_trace("Accepted connection: fd=%d from %s\n", sock_in,
-                  sys_addr2str((struct sockaddr *)&peer_addr));
-
-        do {
-            socklen = sizeof(peer_addr);
-            sock_in2 = accept(sock2, (struct sockaddr *)&peer_addr, &socklen);
-        } while (sock_in2 == -1 && errno == EINTR);
-        ASSERT_LE(0, sock_in2);
-        log_trace("Accepted connection: fd=%d from %s\n", sock_in2,
-                  sys_addr2str((struct sockaddr *)&peer_addr));
-
-        rc = xlio_api->get_socket_rings_num(sock_in);
-        ASSERT_LT(0, rc);
-        ASSERT_GE((int)ARRAY_SIZE(ring_fds2), rc);
-        rc = xlio_api->get_socket_rings_num(sock_in2);
-        ASSERT_LT(0, rc);
-        ASSERT_GE((int)ARRAY_SIZE(ring_fds3), rc);
-
-        ring_fds2_nr = xlio_api->get_socket_rings_fds(sock_in, ring_fds2, ARRAY_SIZE(ring_fds2));
-        ring_fds3_nr = xlio_api->get_socket_rings_fds(sock_in2, ring_fds3, ARRAY_SIZE(ring_fds3));
-        // Incoming TCP sockets inherit ring allocation logic from their parents
-        compare_rings_contains(ring_fds2, ring_fds2_nr, ring_fds, ring_fds_nr);
-        ASSERT_TRUE(!testing::Test::HasFailure());
-        compare_rings_contains(ring_fds3, ring_fds3_nr, ring_fds, ring_fds_nr);
-        ASSERT_TRUE(!testing::Test::HasFailure());
-
-        /*
-         * Socket read / write
-         */
-
-        len = write(sock_in, HELLO_STR, sizeof(HELLO_STR));
-        ASSERT_LT(0, len);
-        EXPECT_EQ(static_cast<ssize_t>(sizeof(HELLO_STR)), len);
-
-        do {
-            len = read(sock_in, buf, sizeof(buf));
-        } while (len == -1 && errno == EINTR);
-        ASSERT_LT(0, len);
-        EXPECT_LE(static_cast<ssize_t>(sizeof(HELLO_STR)), len);
-        EXPECT_EQ(0, strncmp(HELLO_STR, buf, sizeof(HELLO_STR)));
-
-        /*
-         * Socket close
-         */
-
-        usleep(100);
-
-        rc = close(sock_in);
-        ASSERT_EQ(0, rc);
-        rc = close(sock_in2);
-        ASSERT_EQ(0, rc);
-        rc = close(sock);
-        ASSERT_EQ(0, rc);
-        rc = close(sock2);
-        ASSERT_EQ(0, rc);
-        rc = close(sock3);
-        ASSERT_EQ(0, rc);
-    };
-
-    pid = fork();
-    if (0 == pid) { /* I am the child */
-        barrier_fork(pid);
-        test_client();
-        /* This exit is very important, otherwise the fork
-         * keeps running and may duplicate other tests.
-         */
-        exit(testing::Test::HasFailure());
-    } else { /* I am the parent */
-        test_server();
-        if (testing::Test::HasFailure()) {
-            usleep(500);
-            kill(pid, SIGTERM);
-        }
-        ASSERT_EQ(0, wait_fork(pid));
-    }
-}
-
 class tcp_set_get_sockopt : public ::testing::Test {
 protected:
     void SetUp() override
@@ -608,7 +325,7 @@ struct reusable_cleanable_test_socket {
         EXPECT_EQ(result, 0) << "setsockopt failed to set reuse addr";
 
         struct timeval tv;
-        tv.tv_sec = 5;
+        tv.tv_sec = 20;
         tv.tv_usec = 0;
         setsockopt(m_fd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
     }
@@ -641,9 +358,11 @@ struct ipc {
     {
         if (m_pipe[ReadSide] != -1) {
             close(m_pipe[ReadSide]);
+            m_pipe[ReadSide] = -1;
         }
         if (m_pipe[WriteSide] != -1) {
             close(m_pipe[WriteSide]);
+            m_pipe[WriteSide] = -1;
         }
     }
     bool wait_peer()
@@ -883,6 +602,9 @@ TEST_P(tcp_with_fifo, accepted_socket_inherits_the_setsockopt_param)
 
         auto result = setsockopt(listen_fd, level, optname, &value, sizeof(value));
         EXPECT_EQ(result, 0) << "setsockopt failed to set the value";
+        if (result < 0) {
+            EXPECT_EQ(errno, 0) << "setsockopt failed with errno";
+        }
 
         m_ipc_server_to_client.signal_to_peer();
 
@@ -909,11 +631,9 @@ TEST_P(tcp_with_fifo, accepted_socket_inherits_the_setsockopt_param)
                        sizeof(gtest_conf.client_addr)),
                   0);
         m_ipc_server_to_client.wait_peer();
-
         auto result = connect(client_fd, (struct sockaddr *)&gtest_conf.server_addr,
                               sizeof(gtest_conf.server_addr));
         EXPECT_EQ(result, 0);
-
         m_ipc_server_to_client.wait_peer();
 
         // This exit stops the process from inerfering with other tests.
