@@ -56,6 +56,9 @@
 #include <linux/sunrpc/rpc_rdma_cid.h> 	/* completion IDs */
 #include <linux/sunrpc/rpc_rdma.h> 	/* RPC/RDMA protocol */
 #include <linux/sunrpc/xprtrdma.h> 	/* xprt parameters */
+#ifdef HAVE_RPCRDMA_RN_REGISTER
+#include <linux/sunrpc/rdma_rn.h>	/* removal notifications */
+#endif
 
 #define RDMA_RESOLVE_TIMEOUT	(5000)	/* 5 seconds */
 #define RDMA_CONNECT_RETRY_MAX	(2)	/* retries if no listener backlog */
@@ -92,6 +95,9 @@ struct rpcrdma_ep {
 	struct rpcrdma_connect_private
 				re_cm_private;
 	struct rdma_conn_param	re_remote_cma;
+#ifdef HAVE_RPCRDMA_RN_REGISTER
+	struct rpcrdma_notification	re_rn;
+#endif
 	int			re_receive_count;
 	unsigned int		re_max_requests; /* depends on device */
 	unsigned int		re_inline_send;	/* negotiated */
@@ -198,7 +204,6 @@ struct rpcrdma_rep {
 	__be32			rr_proc;
 	int			rr_wc_flags;
 	u32			rr_inv_rkey;
-	bool			rr_temp;
 	struct rpcrdma_regbuf	*rr_rdmabuf;
 	struct rpcrdma_xprt	*rr_rxprt;
 	struct rpc_rqst		*rr_rqst;
@@ -314,9 +319,7 @@ enum {
 struct rpcrdma_buffer;
 struct rpcrdma_req {
 	struct list_head	rl_list;
-#ifdef HAVE_RPC_XPRT_OPS_FREE_SLOT
 	struct rpc_rqst		rl_slot;
-#endif
 	struct rpcrdma_rep	*rl_reply;
 	struct xdr_stream	rl_stream;
 	struct xdr_buf		rl_hdrbuf;
@@ -326,10 +329,6 @@ struct rpcrdma_req {
 	struct rpcrdma_regbuf	*rl_sendbuf;	/* rq_snd_buf */
 	struct rpcrdma_regbuf	*rl_recvbuf;	/* rq_rcv_buf */
 
-#if !defined(HAVE_RPC_XPRT_OPS_FREE_SLOT) || !defined(HAVE_XPRT_PIN_RQST)
-	__be32			    rl_xid;
-#endif
-
 	struct list_head	rl_all;
 	struct kref		rl_kref;
 
@@ -338,22 +337,10 @@ struct rpcrdma_req {
 	struct rpcrdma_mr_seg	rl_segments[RPCRDMA_MAX_SEGS];
 };
 
-#ifndef HAVE_RPC_XPRT_OPS_FREE_SLOT
-static inline void
-rpcrdma_set_xprtdata(struct rpc_rqst *rqst, struct rpcrdma_req *req)
-{
-	rqst->rq_xprtdata = req;
-}
-#endif
-
 static inline struct rpcrdma_req *
 rpcr_to_rdmar(const struct rpc_rqst *rqst)
 {
-#ifdef HAVE_RPC_XPRT_OPS_FREE_SLOT
 	return container_of(rqst, struct rpcrdma_req, rl_slot);
-#else
-	return rqst->rq_xprtdata;
-#endif
 }
 
 static inline void
@@ -392,10 +379,6 @@ struct rpcrdma_buffer {
 	unsigned long		rb_sc_last;
 	struct rpcrdma_sendctx	**rb_sc_ctxs;
 
-#ifndef HAVE_XPRT_PIN_RQST
-	struct list_head	rb_pending;
-#endif
-
 	struct list_head	rb_allreqs;
 	struct list_head	rb_all_mrs;
 	struct list_head	rb_all_reps;
@@ -411,38 +394,6 @@ struct rpcrdma_buffer {
 	struct work_struct	rb_refresh_worker;
 };
 
-#ifndef HAVE_XPRT_PIN_RQST
-static inline void
-rpcrdma_insert_req(struct rpcrdma_buffer *buffers, struct rpcrdma_req *req)
-{
-	spin_lock(&buffers->rb_lock);
-	if (list_empty(&req->rl_list))
-		list_add_tail(&req->rl_list, &buffers->rb_pending);
-	spin_unlock(&buffers->rb_lock);
-}
-
-static inline struct rpcrdma_req *
-rpcrdma_lookup_req_locked(struct rpcrdma_buffer *buffers, __be32 xid)
-{
-	struct rpcrdma_req *pos;
-
-	list_for_each_entry(pos, &buffers->rb_pending, rl_list)
-	if (pos->rl_xid == xid)
-		return pos;
-	return NULL;
-}
-
-static inline void
-rpcrdma_remove_req(struct rpcrdma_buffer *buffers, struct rpcrdma_req *req)
-{
-	spin_lock(&buffers->rb_lock);
-	list_del(&req->rl_list);
-	spin_unlock(&buffers->rb_lock);
-}
-
-void rpcrdma_recv_buffer_put_locked(struct rpcrdma_rep *rep);
-#endif
-
 /*
  * Statistics for RPCRDMA
  */
@@ -532,7 +483,7 @@ void rpcrdma_flush_disconnect(struct rpcrdma_xprt *r_xprt, struct ib_wc *wc);
 int rpcrdma_xprt_connect(struct rpcrdma_xprt *r_xprt);
 void rpcrdma_xprt_disconnect(struct rpcrdma_xprt *r_xprt);
 
-void rpcrdma_post_recvs(struct rpcrdma_xprt *r_xprt, int needed, bool temp);
+void rpcrdma_post_recvs(struct rpcrdma_xprt *r_xprt, int needed);
 
 /*
  * Buffer calls - xprtrdma/verbs.c
@@ -642,21 +593,6 @@ static inline void rpcrdma_set_xdrlen(struct xdr_buf *xdr, size_t len)
 	xdr->len = len;
 }
 
-#ifndef HAVE_XDR_STREAM_REMAINING
-/**
- * xdr_stream_remaining - Return the number of bytes remaining in the stream
- * @xdr: pointer to struct xdr_stream
- *
- * Return value:
- *   Number of bytes remaining in @xdr before xdr->end
- */
-static inline size_t
-xdr_stream_remaining(const struct xdr_stream *xdr)
-{
-	return xdr->nwords << 2;
-}
-#endif
-
 /* RPC/RDMA module init - xprtrdma/transport.c
  */
 extern unsigned int xprt_rdma_max_inline_read;
@@ -679,7 +615,6 @@ size_t xprt_rdma_bc_maxpayload(struct rpc_xprt *);
 #ifdef HAVE_RPC_XPRT_OPS_BC_NUM_SLOTS
 unsigned int xprt_rdma_bc_max_slots(struct rpc_xprt *);
 #endif
-int rpcrdma_bc_post_recv(struct rpcrdma_xprt *, unsigned int);
 void rpcrdma_bc_receive_call(struct rpcrdma_xprt *, struct rpcrdma_rep *);
 int xprt_rdma_bc_send_reply(struct rpc_rqst *rqst);
 void xprt_rdma_bc_free_rqst(struct rpc_rqst *);
